/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
* @class AccountRestoreXWizard
* @contructor AccountRestoreXWizard
* @param ZaApp parent
* @param ZaApp app
* this is the wizard dialog for restoring a single account
* @author Greg Solovyev
**/
function AccountRestoreXWizard (parent) {
	ZaXWizardDialog.call(this, parent, null, com_zimbra_backuprestore.BNR_Restore_WizTitle, "700px", "300px", null);

    this.TAB_INDEX = 0;
    AccountRestoreXWizard.GENTERAL_STEP = ++ this.TAB_INDEX;      // 1
    AccountRestoreXWizard.SERVER_STEP = ++ this.TAB_INDEX;        // 2
    AccountRestoreXWizard.LOOKFOR_LABEL_STEP = ++ this.TAB_INDEX; // 3
    AccountRestoreXWizard.SELECT_LABEL_STEP = ++ this.TAB_INDEX;  // 4
    AccountRestoreXWizard.RESTORE_STEP = ++ this.TAB_INDEX;       // 5

	this.stepChoices = [
        {
            label: com_zimbra_backuprestore.BNR_AccountNameStep_Title,
            value: AccountRestoreXWizard.GENTERAL_STEP
        },
        {
            label: com_zimbra_backuprestore.BNR_SelectServersStep_Title,
            value: AccountRestoreXWizard.SERVER_STEP
        },
        {
            label: com_zimbra_backuprestore.BNR_LookingForLabelsStep_Title,
            value: AccountRestoreXWizard.LOOKFOR_LABEL_STEP
        },
        {
            label: com_zimbra_backuprestore.BNR_SelectLabelsStep_Title,
            value: AccountRestoreXWizard.SELECT_LABEL_STEP
        },
        {
            label: com_zimbra_backuprestore.BNR_RestoringStepTitle,
            value: AccountRestoreXWizard.RESTORE_STEP
        }
    ];
    this._lastStep = this.stepChoices.length;
	this.initForm(ZaRestore.myXModel,this.getMyXForm());	
	this._helpURL = AccountRestoreXWizard.helpURL;		
}

AccountRestoreXWizard.prototype = new ZaXWizardDialog;
AccountRestoreXWizard.prototype.constructor = AccountRestoreXWizard;
AccountRestoreXWizard.prototype.miniType = 2;
AccountRestoreXWizard.helpURL = location.pathname + "help/admin/html/managing_accounts/restoring_a_mailbox.htm?locid=" + AjxEnv.DEFAULT_LOCALE;
AccountRestoreXWizard.labelChoices = new XFormChoices([], XFormChoices.OBJECT_LIST, "label", "label");

AccountRestoreXWizard.prototype.handleXFormChange =
function () {
    if (this._containedObject[ZaModel.currentStep] != this._lastStep) {
         this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
    }else{
        this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
        this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
    }
}
/**
* @method setObject sets the object contained in the view
* @param entry - ZaRestore object to display
**/
AccountRestoreXWizard.prototype.setObject =
function(entry) {
	this._containedObject = entry;
	if(!this._containedObject.restoreRequest[ZaRestore.A_toServer])
		this._containedObject.restoreRequest[ZaRestore.A_toServer] = ZaApp.getInstance().getServerList().getArray()[0].id;
	this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] || 1;
    if(entry._uuid) {
        this._containedObject._uuid = entry._uuid;
    }

	this._localXForm.setInstance(this._containedObject);		
}
/**
* Overwritten methods that control wizard's flow (open, go next,go previous, finish)
**/
AccountRestoreXWizard.prototype.popup = 
function (loc) {
	ZaXWizardDialog.prototype.popup.call(this, loc);
	this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
	this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
	this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);	
}

/**
* Callback function invoked by Asynchronous CSFE command when "RestoreRequest" call returns
**/
AccountRestoreXWizard.prototype.restoreCallBack = 
function (ix,respObj) {
	if(!respObj) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"AccountRestoreXWizard.prototype.restoreCallBack");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.restoreCallBack");
		this.goPrev();
		return;
	}	
	var instance = 	this._localXForm.getInstance();
	var accounts = instance[ZaRestore.A_accounts];
	var cnt = accounts.length;		
	if(respObj.isException && respObj.isException()) {
		var ex = respObj.getException();
		accounts[ix][ZaRestore.A_status] = ZaRestore.A_status_ERROR;
		var msg = com_zimbra_backuprestore.ERROR_RESTORING;
		accounts[ix][ZaModel.ErrorMessage] = msg;
		accounts._version++;
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
	} else {
		
		var resp;
		if(respObj.getResponse) {
			resp = respObj.getResponse().Body.RestoreResponse;
		} else if(respObj.Body.RestoreResponse) {
			resp = respObj.Body.RestoreResponse;
		}	
		if(resp) {	
			accounts[ix][ZaRestore.A_status] = resp[ZaRestore.A_status];
			switch(resp[ZaRestore.A_status]) {
				case ZaRestore.A_status_OK:
					accounts[ix][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreSuccess,[instance.accounts[ix][ZaRestore.A_accountName]]);									
				break;
				case ZaRestore.A_status_INTERRUPTED:
					accounts[ix][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreInterrupted,[instance.accounts[ix][ZaRestore.A_accountName]]);					
				break;				
				case ZaRestore.A_status_ABORTING:
					accounts[ix][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreAborting,[instance.accounts[ix][ZaRestore.A_accountName]]);					
				break;		
				case ZaRestore.A_status_ERROR:
					accounts[ix][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreFailed,[instance.accounts[ix][ZaRestore.A_accountName]]);					
					accounts[ix][ZaModel.ErrorMessage] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreFailed,[instance.accounts[ix][ZaRestore.A_accountName]]);					
				break;						
			}
			accounts[ix][ZaModel.ErrorMessage] = "";
			accounts._version++;
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
		}
	}
	//this._localXForm.refresh();
	ix++;
	if(ix < cnt) {
		this.restoreAccount(ix);
	} else {
		this.doneRestoring();
	}
}

/**
* Callback function invoked by Asynchronous CSFE command when "RestoreRequest" call returns
**/
AccountRestoreXWizard.prototype.restoreAllCallBack = 
function (respObj) {
	if(!respObj) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"AccountRestoreXWizard.prototype.restoreAllCallBack");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.restoreAllCallBack");
		this.goPrev();
		return;
	}	
	var instance = 	this._localXForm.getInstance();
	if(respObj.isException && respObj.isException()) {
		var ex = respObj.getException();
		//instance[ZaRestore.A_status] = ZaRestore.A_status_ERROR;
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status,ZaRestore.A_status_ERROR);
		var msg = com_zimbra_backuprestore.ERROR_RESTORING;
		//instance[ZaModel.ErrorMessage] = msg;
		this._localXForm.getModel().setInstanceValue(instance,ZaModel.ErrorMessage,msg);
	} else {
		
		var resp;
		if(respObj.getResponse) {
			resp = respObj.getResponse().Body.RestoreResponse;
		} else if(respObj.Body.RestoreResponse) {
			resp = respObj.Body.RestoreResponse;
		}	
		if(resp) {
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status,resp[ZaRestore.A_status]);	
			//instance[ZaRestore.A_status] = resp[ZaRestore.A_status];
			switch(resp[ZaRestore.A_status]) {
				case ZaRestore.A_status_OK:
					this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_RestoreAllSuccess);
					//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_RestoreAllSuccess;									
				break;
				case ZaRestore.A_status_INTERRUPTED:
					this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_RestoreAllInterrupted);
					//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_RestoreAllInterrupted;					
				break;				
				case ZaRestore.A_status_ABORTING:
					this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_RestoreAllAborting);
					//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_RestoreAllAborting;					
				break;		
				case ZaRestore.A_status_ERROR:
					this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_RestoreAllFailed);
					//this._localXForm.getModel().setInstanceValue(instance,ZaModel.ErrorMessage,com_zimbra_backuprestore.BNR_RestoreAllFailed);
					//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_RestoreAllFailed;					
					//instance[ZaModel.ErrorMessage] = com_zimbra_backuprestore.BNR_RestoreAllFailed;					
				break;						
			}
			this._localXForm.getModel().setInstanceValue(instance,ZaModel.ErrorMessage,"");
			//instance[ZaModel.ErrorMessage] = "";
		}
	}
	//this._localXForm.refresh();
	this.doneRestoring();
}

AccountRestoreXWizard.prototype.restoreAll = function () {
	var instance = 	this._localXForm.getInstance();
	if(instance[ZaRestore.A_stop]) {
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status,ZaRestore.A_status_INTERRUPTED);
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_RestoreAllInterrupted);	
		//instance[ZaRestore.A_status] = ZaRestore.A_status_INTERRUPTED;
		//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_RestoreAllInterrupted;
		//this._localXForm.refresh();
		return;
	}
		
	var restoreRequest = instance.restoreRequest;
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status,ZaRestore.A_status_RESTORING);
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_status_msg,com_zimbra_backuprestore.BNR_AllRestoring);
	//instance[ZaRestore.A_status] = ZaRestore.A_status_RESTORING;
	//instance[ZaRestore.A_status_msg] = com_zimbra_backuprestore.BNR_AllRestoring;
	this._localXForm.refresh();

	var callBack = new AjxCallback(this, this.restoreAllCallBack);
	ZaRestore.restoreAccount(restoreRequest, callBack);
}

AccountRestoreXWizard.prototype.restoreAccount = function (ix) {
	var instance = 	this._localXForm.getInstance();
	var accounts = instance[ZaRestore.A_accounts];
	var cnt = accounts.length;	
	if(instance[ZaRestore.A_stop]) {
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);	
		for(var i = ix; i < cnt; i++) {
			accounts[i][ZaRestore.A_status] = ZaRestore.A_status_INTERRUPTED;
			accounts[i][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_RestoreInterrupted,[accounts[ix][ZaRestore.A_accountName]]);
		}
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
		//this._localXForm.refresh();
		return;
	}
		
	while(ix < cnt && accounts[ix][ZaRestore.A_accountId] == null) {
		ix++;
	}
	if(ix<cnt) {
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_currentAccount,ix);
		//instance[ZaRestore.A_currentAccount] = ix;
		var restoreRequest = {};
		restoreRequest[ZaRestore.A_accountNames] = [accounts[ix][ZaRestore.A_accountName]];
		restoreRequest[ZaRestore.A_restoreAllAcc] = false;
		restoreRequest[ZaRestore.A_prefix] = accounts[ix][ZaRestore.A_prefix];
		restoreRequest[ZaRestore.A_target] = accounts[ix][ZaRestore.A_target];
		restoreRequest[ZaRestore.A_restoreMethod] = accounts[ix][ZaRestore.A_restoreMethod];
		restoreRequest[ZaRestore.A_restoreTo] = instance.restoreRequest[ZaRestore.A_restoreTo];
		restoreRequest[ZaRestore.A_restoreToTime] = instance.restoreRequest[ZaRestore.A_restoreToTime];
		restoreRequest[ZaRestore.A_toServer] = accounts[ix][ZaRestore.A_toServer];

		accounts[ix][ZaRestore.A_status] = ZaRestore.A_status_RESTORING;
		accounts[ix][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_Restoring,[accounts[ix][ZaRestore.A_accountName]]);
		accounts._version++;
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
		//instance[ZaRestore.A_accounts]._version++;
		//this._localXForm.refresh();

		var callBack = new AjxCallback(this, this.restoreCallBack, ix);
		ZaRestore.restoreAccount(restoreRequest, callBack);
	} else {
		this.doneRestoring();
	}
}

AccountRestoreXWizard.prototype.doneRestoring = function () {
	this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
	this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
	this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);		
}

AccountRestoreXWizard.prototype.startDiscovery = function () {
	this.serverArray = ZaServerBackup.getAll().getArray();
	var instance = 	this._localXForm.getInstance();
	var accounts = [];
	var accountNames = instance.restoreRequest[ZaRestore.A_accountNames];
	var cnt = accountNames.length;
	for(var i = 0; i < cnt; i++) {
		var account = {};
		account[ZaRestore.A_accountName] = accountNames[i];
		account[ZaRestore.A_restoreToTime] = instance.restoreRequest[ZaRestore.A_restoreToTime];
		account[ZaRestore.A_restoreTo] = instance.restoreRequest[ZaRestore.A_restoreTo];
		accounts.push(account);
	}
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_searchStatusMsg,com_zimbra_backuprestore.BNR_SearchingAccounts);
	//instance[ZaRestore.A_accounts] = accounts;
	//instance[ZaRestore.A_searchStatusMsg] = com_zimbra_backuprestore.BNR_SearchingAccounts;
	//this._localXForm.setInstance(instance);
	this.doAccountQuery(0);
}

AccountRestoreXWizard.prototype.doAccountQuery = function(ix) {
	try {
		var instance = 	this._localXForm.getInstance();
		var accountNames = instance.restoreRequest[ZaRestore.A_accountNames];
		var params = new Object();
		var dataCallback = new AjxCallback(this, this.searchAccountCallback, ix);
		params.types = [ZaSearch.ACCOUNTS, ZaSearch.RESOURCES];
		params.callback = dataCallback;
		params.sortBy = ZaAccount.A_name;
		params.query = ZaSearch.getSearchByNameQuery(accountNames[ix], params.types);
		ZaSearch.searchDirectory(params);
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.doAccountQuery");		
	}	
}

AccountRestoreXWizard.prototype.doBackupAccountQuery = function(serverIx) {
	if (serverIx >= this.serverArray.length) {
		//done
		
	} else {
		
		try {
			var instance = 	this._localXForm.getInstance();
			var	accountNames = instance.restoreRequest[ZaRestore.A_accountNames];
			var dataCallback = new AjxCallback(this, this.searchAccountBackupCallback, serverIx);
			var obj = {};
			obj[ZaBackup.A_target] = instance.restoreRequest[ZaBackup.A_target];
			if(instance.restoreRequest[ZaRestore.A_restoreTo] == ZaRestore.TO_LATEST) {
				obj[ZaBackup.A_type] = ZaBackupLabel.TYPE_FULL;
			} else if(instance.restoreRequest[ZaRestore.A_restoreTo] == ZaRestore.TO_PIT) {
				obj[ZaBackup.A_to] = instance.restoreRequest[ZaRestore.A_restoreToTime].getTime();
				obj[ZaBackup.A_type] = null;
			}
			obj[ZaRestore.A_accountNames] = accountNames;
			obj[ZaBackup.A_serverId] = this.serverArray[serverIx].id;
			obj[ZaBackup.A_backupListCount] = 1;
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_searchStatusMsg,AjxMessageFormat.format (com_zimbra_backuprestore.BNR_SearchingBackupsServ,[this.serverArray[serverIx][ZaServer.A_ServiceHostname]]));
			//instance[ZaRestore.A_searchStatusMsg] = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_SearchingBackupsServ,[this.serverArray[serverIx][ZaServer.A_ServiceHostname]]);
			//this._localXForm.setInstance(instance);			
			ZaBackup.queryAccountBackup(obj,dataCallback);
			
		} catch (ex) {
			ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.doBackupAccountQuery");		
		}	
	}
}

AccountRestoreXWizard.prototype.searchAccountBackupCallback = function (serverIx, resp) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "AccountRestoreXWizard.prototype.searchAccountBackupCallback"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse().Body.BackupAccountQueryResponse;
			var accounts = response.account;
			var cnt = accounts.length;
			var instAccounts = instance[ZaRestore.A_accounts];
			for(var i = 0; i < cnt; i ++) {
				if(accounts[i] && accounts[i].name && accounts[i].backup) {
					var backup = accounts[i].backup[0];
					if(instAccounts[i][ZaRestore.A_accountName].toLowerCase() == accounts[i].name.toLowerCase()) {
						instAccounts[i][ZaRestore.A_accountId] = backup[ZaRestore.A_accountId];
						instAccounts[i][ZaRestore.A_toServer] = this.serverArray[serverIx].id;
						instAccounts[i][ZaRestore.A_target] = instance.restoreRequest[ZaRestore.A_target];
						instAccounts[i][ZaRestore.A_prefix] = instance.restoreRequest[ZaRestore.A_prefix];						
						instAccounts[i][ZaRestore.A_restoreMethod] = instance.restoreRequest[ZaRestore.A_restoreMethod];
						if(instAccounts[i][ZaRestore.A_accountId] != instance[ZaRestore.A_accounts][i][ZaRestore.A_currentAccountId] && instance[ZaRestore.A_accounts][i][ZaRestore.A_currentAccountId] != null) {
							instAccounts[i][ZaRestore.A_restoreMethod] = ZaRestore.CA;
						}
					}
				}
			}
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,instAccounts);
			var cnt = this.serverArray.length;
			serverIx++;
			if(serverIx<cnt) {
				this.doBackupAccountQuery(serverIx);
			} else {
				//done searching - show results
				//instance[ZaModel.currentStep] = instance[ZaModel.currentStep]+1;
				//this._localXForm.setInstance(instance);
				this.goNext();
				//find out if we found any backups
				var accountNames = instance.restoreRequest[ZaRestore.A_accountNames];
				var cnt = instance[ZaRestore.A_accounts].length;
				for (var i=0; i < cnt; i++) {
					if(instance[ZaRestore.A_accounts][i][ZaRestore.A_accountId]) {
						this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
						this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_Restore);
						break;
					}
				}
			}		
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.searchAccountBackupCallback");	
	}	
}

AccountRestoreXWizard.prototype.searchAccountCallback = function (ix, resp) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "AccountRestoreXWizard.prototype.searchAccountCallback"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse().Body.SearchDirectoryResponse;
			var list = new ZaItemList(null);	
			list.loadFromJS(response);	
			var arr = list.getArray();
			var accounts = instance[ZaRestore.A_accounts];
			if(arr[0] && ix !=null) {
				accounts[ix][ZaRestore.A_currentAccountId] = arr[0].id;
				//instance[ZaRestore.A_accounts][ix][ZaRestore.A_currentAccountId] = arr[0].id;
			}
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
			var accountNames = instance.restoreRequest[ZaRestore.A_accountNames];
			var cnt = accountNames.length;
			ix++;
			if(ix<cnt) {
				this.doAccountQuery(ix);
			} else {
				//done searching accounts, start searching backups
				//for accounts that exist - search 
				//instance[ZaRestore.A_searchStatusMsg] = com_zimbra_backuprestore.BNR_SearchingBackups;
				this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_searchStatusMsg,com_zimbra_backuprestore.BNR_SearchingBackups);
				//this._localXForm.setInstance(instance);
				this.doBackupAccountQuery(0);
				
			}
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.searchAccountCallback");	
	}		
}

AccountRestoreXWizard.prototype.doBackupQuery = function(serverIx) {
	if (serverIx >= this.serverArray.length) {
		//done
		
	} else {
		try {
			var instance = 	this._localXForm.getInstance();
			var	accountNames = ["all"];
			var dataCallback = new AjxCallback(this, this.searchBackupCallback, serverIx);
			var obj = {};
			obj[ZaBackup.A_target] = instance.restoreRequest[ZaBackup.A_target];
			if(instance.restoreRequest[ZaRestore.A_restoreTo] == ZaRestore.TO_LATEST) {
				obj[ZaBackup.A_type] = ZaBackupLabel.TYPE_FULL;
				obj[ZaBackup.A_backupListCount] = 1;
			} else if(instance.restoreRequest[ZaRestore.A_restoreTo] == ZaRestore.TO_PIT) {
				obj[ZaBackup.A_to] = instance.restoreRequest[ZaRestore.A_restoreToTime].getTime();
				obj[ZaBackup.A_type] = null;
				obj[ZaBackup.A_backupListCount] = -1; //need to check whether it has full backup before restoring, -1 means no limit
			}
			obj[ZaRestore.A_accountNames] = accountNames;
			obj[ZaBackup.A_serverId] = this.serverArray[serverIx].id;
			obj[ZaBackup.A_accountListCount] = 1;
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_searchStatusMsg,AjxMessageFormat.format (com_zimbra_backuprestore.BNR_SearchingBackupsServ,[this.serverArray[serverIx][ZaServer.A_ServiceHostname]]));			
			//instance[ZaRestore.A_searchStatusMsg] = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_SearchingBackupsServ,[this.serverArray[serverIx][ZaServer.A_ServiceHostname]]);
			//this._localXForm.setInstance(instance);			
			ZaBackup.queryBackups(obj,dataCallback);
			
		} catch (ex) {
			ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.doBackupQuery");		
		}	
	}
}

AccountRestoreXWizard.prototype.searchBackupCallback = function (serverIx, resp) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "AccountRestoreXWizard.prototype.searchAccountBackupCallback"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse();
			var backup = response.Body.BackupQueryResponse.backup;
			if(backup) {
				this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_foundAnyBackups,true);
				//instance[ZaRestore.A_foundAnyBackups] = true;
				//this._localXForm.setInstance(instance);

				var foundAnyFullBackups = false;
				if ( !AjxUtil.isArray(backup) ) {
					backup = [backup];
				}
				for ( var i = 0; i < backup.length; i++ ) {
					if (backup[i][ZaBackup.A_type] == ZaBackupLabel.TYPE_FULL) {
						foundAnyFullBackups = true; break;
					}
				}
				this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_foundAnyFullBackups,foundAnyFullBackups);

				if (foundAnyFullBackups) {
					this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
					this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_Restore);
				}
				this.goNext();
			} else {
				var cnt = this.serverArray.length;
				serverIx++;
				if(serverIx<cnt) {
					this.doBackupQuery(serverIx);
				} else {
					//done searching - show results
					//this._localXForm.setInstance(instance);
					this.goNext();
				}		
			}
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.prototype.searchAccountCallback");	
	}	
}

AccountRestoreXWizard.prototype.findBackupsAfter = function () {
	this.serverArray = ZaServerBackup.getAll().getArray();
	var instance = 	this._localXForm.getInstance();
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_searchStatusMsg,com_zimbra_backuprestore.BNR_SearchingAccounts);
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_foundAnyBackups,false);
	this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_foundAnyFullBackups,false);

	//this._localXForm.setInstance(instance);
	this.doBackupQuery(0);
}

AccountRestoreXWizard.prototype.goPrev = 
function() {
	if (this._containedObject[ZaModel.currentStep] == 2) {
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
	} else if(this._containedObject[ZaModel.currentStep] == 3){
		if(this._localXForm.hasErrors()){
                        this._localXForm.clearErrors();
                }
	} else if(this._containedObject[ZaModel.currentStep] == 4) {
		if(this._localXForm.hasErrors()){
			this._localXForm.clearErrors();
		}
		this._containedObject[ZaModel.currentStep]--;
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.NEXT_BUTTON].setText(AjxMsg._next);
	} else if(this._containedObject[ZaModel.currentStep] == 5 ) { 
		this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_Restore);
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
	}
	this.goPage(this._containedObject[ZaModel.currentStep] - 1);
}

AccountRestoreXWizard.prototype.goNext = 
function() {
	
	if (this._containedObject[ZaModel.currentStep] == 1) {
		var restoreRequest = this._containedObject.restoreRequest;
		// Remove empty account names from the array
		var accountNames = restoreRequest[ZaRestore.A_accountNames] = AjxUtil.collapseList(restoreRequest[ZaRestore.A_accountNames]);
		if (!restoreRequest[ZaRestore.A_restoreAllAcc] && (!accountNames || accountNames.length < 1)) {
			ZaApp.getInstance().getCurrentController().popupErrorDialog(com_zimbra_backuprestore.ERROR_EMAIL_ADDR_REQUIRED);
			return false;			
		}
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
	} else if(this._containedObject[ZaModel.currentStep] == 2 ) { //search backups
		var instance = 	this._localXForm.getInstance();
		if(!instance.restoreRequest[ZaRestore.A_restoreAllAcc]) {
			this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
			this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
			this.startDiscovery();
		}  else {
			this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
			this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
			this.findBackupsAfter();

/*			this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
			this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);	
			this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
			this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_Restore);		
			this._containedObject[ZaModel.currentStep]++;*/
		}
	} else if(this._containedObject[ZaModel.currentStep] == 3 ) { //review
		//this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
	} else if(this._containedObject[ZaModel.currentStep] == 4 ) { //restore
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
		var instance = 	this._localXForm.getInstance();
		this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_stop,false);
		//instance[ZaRestore.A_stop] = false;
		if(instance.restoreRequest[ZaRestore.A_restoreAllAcc]) {
			this.restoreAll();
		} else {
			var accounts = instance[ZaRestore.A_accounts];
			var cnt = accounts.length;
			for(var i =0; i < cnt; i++) {
				if(accounts[i][ZaRestore.A_accountId]) {
					accounts[i][ZaRestore.A_status] = ZaRestore.A_status_PENDING;
					accounts[i][ZaRestore.A_status_msg] = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_Waiting,[accounts[i][ZaRestore.A_accountName]]);
				}
			}
			this._localXForm.getModel().setInstanceValue(instance,ZaRestore.A_accounts,accounts);
			this.restoreAccount(0);	   
		}
		this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_Abort);
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);

	} else if(this._containedObject[ZaModel.currentStep] == 5 ) { //abort
		//this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
		this.abortRestore();
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
		return;
	}
	this.goPage(this._containedObject[ZaModel.currentStep] + 1);
}

AccountRestoreXWizard.prototype.abortRestore = function () {
	try {
		var instance = 	this._localXForm.getInstance();
		instance[ZaRestore.A_stop] = true;
		if(instance.restoreRequest[ZaRestore.A_restoreAllAcc]) {
			var restoreRequest = instance.restoreRequest;
			ZaRestore.abortRestore(restoreRequest, new AjxCallback(this, this.restoreAllCallBack));	
		} else {
			var restoreRequest = instance.accounts[instance[ZaRestore.A_currentAccount]];
			ZaRestore.abortRestore(restoreRequest, new AjxCallback(this, this.restoreCallBack,instance[ZaRestore.A_currentAccount]));	
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "AccountRestoreXWizard.abortRestore", null, false);
	}		   
}

AccountRestoreXWizard.isAccountIdTheSame = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0,this.refPath.indexOf(refPart)) : this.refPath + "/";
		if(baseRef && baseRef.length) {
			var accIdRef = baseRef + ZaRestore.A_accountId;
			var currenAccIdRef = baseRef + ZaRestore.A_currentAccountId;
			if(this.getInstanceValue(currenAccIdRef)!=null ) 
				return (this.getInstanceValue(accIdRef) == this.getInstanceValue(currenAccIdRef));
			else 
				return true;
		} else
			return true;
	} else
		return true;
}

AccountRestoreXWizard.isAccountIdNotTheSame = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0,this.refPath.indexOf(refPart)) : this.refPart + "/";
		if(baseRef && baseRef.length) {
			var accIdRef = baseRef + ZaRestore.A_accountId;
			var currenAccIdRef = baseRef + ZaRestore.A_currentAccountId;
			if(this.getInstanceValue(currenAccIdRef)!=null ) 
				return !((this.getInstanceValue(accIdRef) == this.getInstanceValue(currenAccIdRef)));
			else 
				return false;
		} else
			return false;
	} else
		return false;
}
AccountRestoreXWizard.accountBackupNotFound = function (refPart) { 
	return !AccountRestoreXWizard.accountBackupFound.call(this,refPart);
}

AccountRestoreXWizard.accountBackupFound = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0,this.refPath.indexOf(refPart)) : this.refPath + "/";
			
		if(baseRef && baseRef.length) {
			var accIdRef = baseRef + ZaRestore.A_accountId;

			return (this.getInstanceValue(accIdRef) != null);
		} else
			return false;
	} else
		return false;
}

AccountRestoreXWizard.getAccountStatus = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0, this.refPath.indexOf(refPart)) : this.refPath + "/";
			
		if(baseRef && baseRef.length) {
			var accStatusRef = baseRef + ZaRestore.A_status;
			return this.getInstanceValue(accStatusRef);
		} 
	} 
}

AccountRestoreXWizard.isStatusNormal = function () {
	return (this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_PENDING || this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_OK || this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_RESTORING);
}

AccountRestoreXWizard.isStatusAbortingOrInterrupted = function () {
	return (this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_ABORTING || this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_INTERRUPTED);
}

AccountRestoreXWizard.isStatusError = function () {
	return (this.getInstanceValue(ZaRestore.A_status) == ZaRestore.A_status_ERROR);
}

AccountRestoreXWizard.isLocalStatusNormal = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0, this.refPath.indexOf(refPart)) : this.refPath + "/";
			
		if(baseRef && baseRef.length) {
			var accStatusRef = baseRef + ZaRestore.A_status;
			return (this.getInstanceValue(accStatusRef) == ZaRestore.A_status_PENDING || this.getInstanceValue(accStatusRef) == ZaRestore.A_status_OK || this.getInstanceValue(accStatusRef) == ZaRestore.A_status_RESTORING);
		} else {
			return false;
		}
	} else
		return false;
}

AccountRestoreXWizard.isLocalAbortingOrInterrupted = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0, this.refPath.indexOf(refPart)) : this.refPath + "/";
			
		if(baseRef && baseRef.length) {
			var accStatusRef = baseRef + ZaRestore.A_status;
			return (this.getInstanceValue(accStatusRef) == ZaRestore.A_status_ABORTING || this.getInstanceValue(accStatusRef) == ZaRestore.A_status_INTERRUPTED);
		} else {
			return false;
		}
	} else
		return false;
}

AccountRestoreXWizard.isLocalStatusError = function (refPart) {
	if(this.refPath) {
		var instance = this.getInstance();
		var baseRef = refPart ? this.refPath.substring(0, this.refPath.indexOf(refPart)) : this.refPath + "/";
			
		if(baseRef && baseRef.length) {
			var accStatusRef = baseRef + ZaRestore.A_status;
			return (this.getInstanceValue(accStatusRef) == ZaRestore.A_status_ERROR);
		} else {
			return false;
		}
	} else
		return false;
}

AccountRestoreXWizard.noPITLabelsFound = function () {
	return (!this.getInstanceValue(ZaRestore.A_foundAnyFullBackups) && (this.getInstanceValue(ZaRestore.A_restoreTo) == ZaRestore.TO_PIT) && (this.getInstanceValue(ZaRestore.A_restoreAllAcc)));
}

AccountRestoreXWizard.noLatestLabelsFound = function () {
	return (!this.getInstanceValue(ZaRestore.A_foundAnyFullBackups) && (this.getInstanceValue(ZaRestore.A_restoreTo) == ZaRestore.TO_LATEST) && (this.getInstanceValue(ZaRestore.A_restoreAllAcc)));
}

AccountRestoreXWizard.isPrefixEnable = function () {
	var isEnable = true;
	if(this.getInstanceValue(ZaRestore.A_restoreMethod) != ZaRestore.CA){
		if(this.hasError()){
			this.clearError();
			this.setInstanceValue("restored_", ZaRestore.A_prefix);
			this.getForm().parent.setDirty(true);
		}
		isEnable = false;
	}
	return isEnable;
}

AccountRestoreXWizard.prototype.getMyXForm = function() {	
	var domainName;
	if(ZaSettings.DOMAINS_ENABLED) {
		domainName = ZaApp.getInstance().getGlobalConfig().attrs[ZaGlobalConfig.A_zimbraDefaultDomainName];
		if(!domainName)
			domainName = ZaApp.getInstance().getDomainList().getArray()[0].name;
	} else 
		domainName = ZaSettings.myDomainName;

	var emptyAlias = " @" + domainName;
	var xFormObject = {
        items: [
            {
                type: _OUTPUT_,
                colSpan: 2,
                valign: _TOP_,
                cssStyle: "white-space: normal",
                ref: ZaModel.currentStep,
                choices: this.stepChoices,
                valueChangeEventSources: [ZaModel.currentStep]
            },
            {
                type: _SEPARATOR_,
                align: _CENTER_,
                valign: _TOP_
            },
            {
                type: _SPACER_,
                align: _CENTER_,
                valign: _TOP_
            },
            {
                type: _SWITCH_,
                width: 650,
                valign: _TOP_,
                cssStyle: "white-space: normal",
                items: [
                    {
                        type: _CASE_,
                        numCols: 1,
                        caseKey: AccountRestoreXWizard.GENTERAL_STEP,
                        id: "restore_wiz_accounts_step",
                        align: _LEFT_,
                        valign: _TOP_,
                        items: [
                            {
                                type: _ZAWIZ_TOP_GROUPER_,
                                label: com_zimbra_backuprestore.BNR_SelectAccountsGrouper,
                                id: "restore_wiz_accounts_group",
                                numCols: 2,
                                items: [
                                    {
                                        ref: ZaRestore.A_restoreAllAcc,
                                        type: _RADIO_,
                                        groupname: "restore_all_acc_grp",
                                        label: com_zimbra_backuprestore.BNR_allAccounts,
                                        labelLocation: _RIGHT_,
                                        updateElement: function (value) {
                                            this.getElement().checked = value;
                                        },
                                        elementChanged: function (elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, true, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_skipDeletedAccounts,
                                        type: _CHECKBOX_,
                                        labelLocation: _RIGHT_,
                                        align: _RIGHT_,
                                        subLabel: "",
                                        label: com_zimbra_backuprestore.BNR_SkipDeletedAcc,
                                        trueValue: true,
                                        falseValue: false,
                                        enableDisableChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                true
                                            ]
                                        ],
                                        enableDisableChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                        visibilityChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_restoreAllAcc,
                                        type: _RADIO_,
                                        groupname: "restore_all_acc_grp",
                                        label: com_zimbra_backuprestore.BNR_SelectedAcc,
                                        updateElement: function (value) {
                                            this.getElement().checked = !value;
                                        },
                                        elementChanged: function (elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, false, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        type: _DWT_ALERT_,
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                false
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                        containerCssStyle: "padding-bottom:0px",
                                        style: DwtAlert.INFORMATION,
                                        iconVisible: true,
                                        align: _CENTER_,
                                        colSpan: 2,
                                        content: com_zimbra_backuprestore.BNR_AccountsNote
                                    },
                                    {
                                        ref: ZaRestore.A_accountNames,
                                        type: _REPEAT_,
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                false
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                        enableDisableChecks: [],
                                        label: com_zimbra_backuprestore.BNR_EmailAddress,
                                        labelLocation: _LEFT_,
                                        addButtonLabel: ZaMsg.NAD_AddAddress,
                                        align: _LEFT_,
                                        colSpan: "*",
                                        repeatInstance: emptyAlias,
                                        showAddButton: true,
                                        showRemoveButton: true,
                                        showAddOnNextRow: true,
                                        removeButtonLabel: ZaMsg.NAD_RemoveAddress,
                                        items: [
                                            {
                                                ref: ".",
                                                type: _DYNSELECT_,
                                                editable: true,
                                                width: "200px",
                                                inputSize: 30,
                                                label: null,
                                                dataFetcherClass: ZaSearch,
                                                dataFetcherTypes: [
                                                    ZaSearch.ACCOUNTS,
                                                    ZaSearch.RESOURCES
                                                ],
                                                dataFetcherAttrs: [
                                                    ZaItem.A_zimbraId,
                                                    ZaItem.A_cn,
                                                    ZaAccount.A_name,
                                                    ZaAccount.A_displayname,
                                                    ZaAccount.A_mail
                                                ],
                                                dataFetcherMethod: ZaSearch.prototype.dynSelectSearch,
                                                visibilityChecks: [],
												enableDisableChecks: [],
												autoCompleteEnabled: true
                                            }
                                        ]
                                    },
                                    {
                                        type: _CELLSPACER_,
                                        height: "20px",
                                        align: _CENTER_,
                                        colSpan:2
                                    },
                                    {
                                        ref: ZaRestore.A_ignoreRedoErrors,
                                        type: _CHECKBOX_,
                                        labelLocation: _RIGHT_,
                                        align: _RIGHT_,
                                        subLabel: "",
                                        label: com_zimbra_backuprestore.BNR_IgnoreRedoErrors,
                                        trueValue: true,
                                        falseValue: false,
                                        visibilityChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_skipDeleteOps,
                                        type: _CHECKBOX_,
                                        labelLocation: _RIGHT_,
                                        align: _RIGHT_,
                                        subLabel: "",
                                        label: com_zimbra_backuprestore.BNR_skipDeleteOps,
                                        trueValue: true,
                                        falseValue: false,
                                        visibilityChecks:[]
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        type: _CASE_,
                        numCols: 1,
                        caseKey: AccountRestoreXWizard.SERVER_STEP,
                        items: [
                            {
                                type: _ZAWIZ_TOP_GROUPER_,
                                label: com_zimbra_backuprestore.BNR_TimeOptionsGrouper,
                                id: "restore_wiz_time_group",
                                numCols: 2,
                                items: [
                                    {
                                        ref: ZaRestore.A_restoreTo,
                                        type: _RADIO_,
                                        groupname: "restore_to_grp",
                                        label: com_zimbra_backuprestore.BNR_toLatest,
                                        updateElement: function (value) {
                                            this.getElement().checked = (value == ZaRestore.TO_LATEST);
                                        },
                                        elementChanged: function (elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, ZaRestore.TO_LATEST, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_restoreTo,
                                        type: _RADIO_,
                                        groupname: "restore_to_grp",
                                        label: com_zimbra_backuprestore.BNR_toPIT,
                                        updateElement: function (value) {
                                            this.getElement().checked = (value == ZaRestore.TO_PIT);
                                        },
                                        elementChanged: function (elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, ZaRestore.TO_PIT, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_restoreToTime,
                                        type: _DWT_DATETIME_,
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreTo,
                                                ZaRestore.TO_PIT
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                        enableDisableChecks: [],
                                        bmolsnr: true,
                                        firstDayOfWeek: ZaZimbraAdmin.FIRST_DAY_OF_WEEK,
                                        label: com_zimbra_backuprestore.BNR_ChoosePIT,
                                        labelLocation: _LEFT_
                                    }
                                ]
                            },
                            {
                                type: _ZAWIZ_TOP_GROUPER_,
                                label: com_zimbra_backuprestore.BNR_NameOptionsGrouper,
                                id: "restore_wiz_name_group",
                                numCols: 2,
                                items: [
                                    {
                                        ref: ZaRestore.A_restoreMethod,
                                        type: _RADIO_,
                                        groupname: "restore_method_grp",
                                        label: com_zimbra_backuprestore.BNR_PreserveAccNames,
                                        updateElement: function (value) {
                                            this.getElement().checked = (value == ZaRestore.RA);
                                        },
                                        elementChanged: function(elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, ZaRestore.RA, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_restoreMethod,
                                        type: _RADIO_,
                                        groupname: "restore_method_grp",
                                        label: com_zimbra_backuprestore.BNR_UsePrefix,
                                        updateElement: function (value) {
                                            this.getElement().checked = (value == ZaRestore.CA);
                                        },
                                        elementChanged: function (elementValue, instanceValue, event) {
                                            this.getForm().itemChanged(this, ZaRestore.CA, event);
                                        },
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    },
                                    {
                                        ref: ZaRestore.A_prefix,
                                        type: _TEXTFIELD_,
                                        label: com_zimbra_backuprestore.BNR_Prefix,
                                        enableDisableChecks: [
                                            [
                                                AccountRestoreXWizard.isPrefixEnable
                                            ]
                                        ],
                                        enableDisableChangeEventSources: [ZaRestore.A_restoreMethod],
                                        visibilityChecks: []
                                    },
                                    {
                                        type: _DWT_ALERT_,
                                        ref: ZaRestore.A_prefix,
                                        valueChangeEventSources: [
                                            ZaRestore.A_prefix,
                                            ZaRestore.A_restoreMethod
                                        ],
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreMethod,
                                                ZaRestore.CA
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreMethod],
                                        containerCssStyle: "padding-bottom:0px",
                                        style: DwtAlert.INFORMATION,
                                        iconVisible: true,
                                        bmolsnr: true,
                                        align: _CENTER_,
                                        colSpan: 2,
                                        getDisplayValue: function (itemVal) {
                                            return AjxMessageFormat.format (
                                                com_zimbra_backuprestore.BNR_Restore_to_different,
                                                [itemVal]
                                            );
                                        }
                                    }
                                ]
                            },
                            {
                                type: _ZAWIZ_TOP_GROUPER_,
                                label: com_zimbra_backuprestore.BNR_ServerOptionsGrouper,
                                id: "restore_wiz_server_group",
                                numCols: 2,
                                visibilityChecks: [
                                    [
                                        XForm.checkInstanceValue,
                                        ZaRestore.A_restoreAllAcc,
                                        true
                                    ]
                                ],
                                visibilityChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                items: [
                                    {
                                        ref: ZaRestore.A_toServer,
                                        type: _OUTPUT_,
                                        label: com_zimbra_backuprestore.BNR_SelectServer,
                                        nowrap: false,
                                        labelWrap: true,
                                        choices: ZaServerBackup.getBackupServerListChoices()
                                    },
                                    {
                                        ref: ZaRestore.A_target,
                                        type: _TEXTFIELD_,
                                        label: com_zimbra_backuprestore.BNR_SelectPath,
                                        width: "250px",
                                        labelLocation: _LEFT_,
                                        visibilityChecks: [],
                                        enableDisableChecks: []
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        type: _CASE_,
                        id: "restore_wiz_find_searching_step",
                        numCols: 1,
                        caseKey: AccountRestoreXWizard.LOOKFOR_LABEL_STEP,
                        items: [
                            {
                                type: _DWT_ALERT_,
                                style: DwtAlert.INFORMATION,
                                iconVisible: true,
                                bmolsnr: true,
                                ref: ZaRestore.A_searchStatusMsg,
                                width: "100%",
                                align: _CENTER_,
                                valueChangeEventSources: [ZaRestore.A_searchStatusMsg]
                            }
                        ]
                    },
                    {
                        type: _CASE_,
                        id: "restore_wiz_review_step",
                        numCols: 1,
                        caseKey: AccountRestoreXWizard.SELECT_LABEL_STEP,
                        items: [
                            {
                                type: _ZAWIZ_TOP_GROUPER_,
                                colSizes: ["auto"],
                                numCols: 1,
                                label: com_zimbra_backuprestore.BNR_ReviewGrouper,
                                id: "restore_wiz_review_group",
                                items: [
                                    {
                                        type: _DWT_ALERT_,
                                        ref: ".",
                                        content: com_zimbra_backuprestore.BNR_NoBackupForPITFound,
                                        containerCssStyle: "padding-bottom:0px",
                                        style: DwtAlert.CRITICAL,
                                        iconVisible: true,
                                        bmolsnr: true,
                                        align: _CENTER_,
                                        width: "100%",
                                        visibilityChecks: [AccountRestoreXWizard.noPITLabelsFound],
                                        visibilityChangeEventSources: [
                                            ZaRestore.A_foundAnyFullBackups,
                                            ZaRestore.A_restoreTo,
                                            ZaRestore.A_restoreAllAcc
                                        ]
                                    },
                                    {
                                        type: _DWT_ALERT_,
                                        ref: ".",
                                        content: com_zimbra_backuprestore.BNR_NoBackupForFound,
                                        containerCssStyle: "padding-bottom:0px",
                                        style: DwtAlert.CRITICAL,
                                        iconVisible: true,
                                        bmolsnr: true,
                                        align: _CENTER_,
                                        width: "100%",
                                        visibilityChecks: [AccountRestoreXWizard.noLatestLabelsFound],
                                        visibilityChangeEventSources: [
                                            ZaRestore.A_foundAnyFullBackups,
                                            ZaRestore.A_restoreTo,
                                            ZaRestore.A_restoreAllAcc
                                        ]
                                    },
                                    {
                                        type: _ZAWIZ_TOP_GROUPER_,
                                        colSizes: ["auto"],
                                        numCols: 1,
                                        label: com_zimbra_backuprestore.BNR_TimeOptionsGrouper,
                                        id: "restore_wiz_review_time_group",
                                        items: [
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_restoreTo,
                                                        ZaRestore.TO_LATEST
                                                    ]
                                                ],
                                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                                items: [
                                                    {
                                                        ref: ZaRestore.A_restoreToTime,
                                                        label: null,
                                                        type: _OUTPUT_,
                                                        labelLocation: _NONE_,
                                                        getDisplayValue: function (itemVal) {
                                                            return com_zimbra_backuprestore.BNR_RestoretoLatest;
                                                        }
                                                    }
                                                ]
                                            },
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_restoreTo,
                                                        ZaRestore.TO_PIT
                                                    ]
                                                ],
                                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                                items: [
                                                    {
                                                        ref: ZaRestore.A_restoreToTime,
                                                        label: null,
                                                        type: _OUTPUT_,
                                                        labelLocation: _NONE_,
                                                        getDisplayValue: function (itemVal) {
                                                            return AjxMessageFormat.format (
                                                                com_zimbra_backuprestore.BNR_RestoreToTime,
                                                                [itemVal]
                                                            );
                                                        }
                                                    }
                                                ]
                                            }
                                        ]
                                    },
                                    {
                                        type: _ZAWIZ_TOP_GROUPER_,
                                        colSizes: ["auto"],
                                        numCols: 1,
                                        label: com_zimbra_backuprestore.BNR_AccountsGrouper,
                                        id: "restore_wiz_review_accounts_group",
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                false
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                        items: [
                                            {
                                                type: _REPEAT_,
                                                ref: ZaRestore.A_accounts,
                                                showRemoveButton: false,
                                                showAddButton: false,
                                                items: [
                                                    {
                                                        type: _ZA_PLAIN_GROUPER_,
                                                        items: [
                                                            {
                                                                type: _OUTPUT_,
                                                                ref: ZaRestore.A_accountName,
                                                                label: ZaMsg.NAD_AccountName,
                                                                valueChangeEventSources: [ZaRestore.A_accountName]
                                                            },
                                                            {
                                                                type: _OUTPUT_,
                                                                ref: ZaRestore.A_currentAccountId,
                                                                label: com_zimbra_backuprestore.BNR_current_acc_id,
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.isAccountIdNotTheSame,
                                                                        ZaRestore.A_currentAccountId
                                                                    ],
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        ZaRestore.A_currentAccountId
                                                                    ]
                                                                ],
                                                                valueChangeEventSources: [ZaRestore.A_currentAccountId]
                                                            },
                                                            {
                                                                type: _OUTPUT_,
                                                                ref: ZaRestore.A_accountId,
                                                                label: com_zimbra_backuprestore.BNR_old_acc_id,
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.isAccountIdNotTheSame,
                                                                        ZaRestore.A_accountId
                                                                    ],
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        ZaRestore.A_accountId
                                                                    ]
                                                                ],
                                                                valueChangeEventSources: [ZaRestore.A_accountId]
                                                            },
                                                            {
                                                                type: _OUTPUT_,
                                                                ref: ZaRestore.A_toServer,
                                                                label: com_zimbra_backuprestore.BNR_SelectServer,
                                                                nowrap: false,
                                                                labelWrap: true,
                                                                choices: ZaServerBackup.getBackupServerListChoices(),
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        ZaRestore.A_toServer
                                                                    ]
                                                                ],
                                                                valueChangeEventSources: [ZaRestore.A_toServer]
                                                            },
                                                            {
                                                                type: _TEXTFIELD_,
                                                                ref: ZaRestore.A_target,
                                                                label: com_zimbra_backuprestore.BNR_SelectPath,
                                                                width: "250px",
                                                                labelLocation: _LEFT_,
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        ZaRestore.A_toServer
                                                                    ]
                                                                ],
                                                                enableDisableChecks: [],
                                                                valueChangeEventSources: [ZaRestore.A_target]
                                                            },
                                                            {
                                                                type: _TEXTFIELD_,
                                                                ref: ZaRestore.A_prefix,
                                                                label: com_zimbra_backuprestore.BNR_Prefix,
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        ZaRestore.A_prefix
                                                                    ],
                                                                    [
                                                                        AccountRestoreXWizard.isAccountIdNotTheSame,
                                                                        ZaRestore.A_prefix
                                                                    ]
                                                                ],
                                                                enableDisableChecks: []
                                                            },
                                                            {
                                                                type: _DWT_ALERT_,
                                                                ref: ".",
                                                                content: com_zimbra_backuprestore.BNR_MustGivePrefix,
                                                                containerCssStyle: "padding-bottom:0px",
                                                                style: DwtAlert.WARNING,
                                                                iconVisible: true,
                                                                bmolsnr: true,
                                                                align: _CENTER_,
                                                                colSpan: 2,
                                                                width: "100%",
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupFound,
                                                                        null
                                                                    ],
                                                                    [
                                                                        AccountRestoreXWizard.isAccountIdNotTheSame,
                                                                        null
                                                                    ]
                                                                ]
                                                            },
                                                            {
                                                                type: _DWT_ALERT_,
                                                                ref: ".",
                                                                content: com_zimbra_backuprestore.BNR_NoAccBackupFound,
                                                                containerCssStyle: "padding-bottom:0px",
                                                                style: DwtAlert.CRITICAL,
                                                                iconVisible: true,
                                                                bmolsnr: true,
                                                                align: _CENTER_,
                                                                colSpan: 2,
                                                                width: "100%",
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupNotFound,
                                                                        null
                                                                    ],
                                                                    [
                                                                        XForm.checkInstanceValue,
                                                                        ZaRestore.A_restoreTo,
                                                                        ZaRestore.TO_LATEST
                                                                    ]
                                                                ]
                                                            },
                                                            {
                                                                type: _DWT_ALERT_,
                                                                ref: ".",
                                                                content: com_zimbra_backuprestore.BNR_NoAccBackupForPITFound,
                                                                containerCssStyle: "padding-bottom:0px",
                                                                style: DwtAlert.CRITICAL,
                                                                iconVisible: true,
                                                                bmolsnr: true,
                                                                align: _CENTER_,
                                                                colSpan: 2,
                                                                width: "100%",
                                                                visibilityChecks: [
                                                                    [
                                                                        AccountRestoreXWizard.accountBackupNotFound,
                                                                        null
                                                                    ],
                                                                    [
                                                                        XForm.checkInstanceValue,
                                                                        ZaRestore.A_restoreTo,
                                                                        ZaRestore.TO_PIT
                                                                    ]
                                                                ]
                                                            },
                                                            {
                                                                type: _SEPARATOR_,
                                                                colSpan: 2
                                                            }
                                                        ]
                                                    }
                                                ]
                                            }
                                        ]
                                    },
                                    {
                                        type: _ZAWIZ_TOP_GROUPER_,
                                        colSizes: ["auto"],
                                        numCols: 1,
                                        label: com_zimbra_backuprestore.BNR_MethodOptionsGrouper,
                                        id: "restore_wiz_review_server_group",
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                true
                                            ],
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_foundAnyFullBackups,
                                                true
                                            ]
                                        ],
                                        items: [
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_skipDeletedAccounts,
                                                        false
                                                    ]
                                                ],
                                                items: [
                                                    {
                                                        ref: null,
                                                        label: null,
                                                        type: _OUTPUT_,
                                                        value: com_zimbra_backuprestore.BNR_allAccountsIncludeDeleted
                                                    }
                                                ]
                                            },
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_skipDeletedAccounts,
                                                        true
                                                    ]
                                                ],
                                                items: [
                                                    {
                                                        ref: null,
                                                        label: null,
                                                        type: _OUTPUT_,
                                                        value: com_zimbra_backuprestore.BNR_allAccountsSkipDeleted
                                                    }
                                                ]
                                            }
                                        ]
                                    },
                                    {
                                        type: _ZAWIZ_TOP_GROUPER_,
                                        colSizes: ["auto"],
                                        numCols: 1,
                                        label: com_zimbra_backuprestore.BNR_NameOptionsGrouper,
                                        id: "restore_wiz_review_name_group",
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                true
                                            ],
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_foundAnyFullBackups,
                                                true
                                            ]
                                        ],
                                        visibilityChangeEventSources: [
                                            ZaRestore.A_restoreAllAcc,
                                            ZaRestore.A_foundAnyFullBackups
                                        ],
                                        items: [
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_restoreMethod,
                                                        ZaRestore.RA
                                                    ]
                                                ],
                                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                                items: [
                                                    {
                                                        ref: null,
                                                        type: _OUTPUT_,
                                                        value: com_zimbra_backuprestore.BNR_PreserveAccNames,
                                                        label: null,
                                                        labelLocation: _NONE_
                                                    }
                                                ]
                                            },
                                            {
                                                type: _GROUP_,
                                                visibilityChecks: [
                                                    [
                                                        XForm.checkInstanceValue,
                                                        ZaRestore.A_restoreMethod,
                                                        ZaRestore.CA
                                                    ]
                                                ],
                                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                                items: [
                                                    {
                                                        ref: ZaRestore.A_prefix,
                                                        type: _OUTPUT_,
                                                        valueChangeEventSources: [ZaRestore.A_prefix],
                                                        label: null,
                                                        labelLocation: _NONE_,
                                                        getDisplayValue: function (itemVal) {
                                                            return AjxMessageFormat.format (
                                                                com_zimbra_backuprestore.BNR_AddPrefix,
                                                                [itemVal]
                                                            );
                                                        }
                                                    }
                                                ]
                                            }
                                        ]
                                    },
                                    {
                                        type: _ZAWIZ_TOP_GROUPER_,
                                        label: com_zimbra_backuprestore.BNR_ServerOptionsGrouper,
                                        id: "restore_wiz_review_server_group",
                                        visibilityChecks: [
                                            [
                                                XForm.checkInstanceValue,
                                                ZaRestore.A_restoreAllAcc,
                                                true
                                            ]
                                        ],
                                        visibilityChangeEventSources: [ZaRestore.A_restoreAllAcc],
                                        items: [
                                            {
                                                ref: ZaRestore.A_toServer,
                                                type: _OUTPUT_,
                                                label: com_zimbra_backuprestore.BNR_SelectServer,
                                                valueChangeEventSources: [ZaRestore.A_toServer],
                                                nowrap: false,
                                                labelWrap: true,
                                                choices: ZaServerBackup.getBackupServerListChoices()
                                            },
                                            {
                                                ref: ZaRestore.A_target,
                                                type: _OUTPUT_,
                                                label: com_zimbra_backuprestore.BNR_SelectPath,
                                                width: "250px",
                                                labelLocation: _LEFT_,
                                                valueChangeEventSources: [ZaRestore.A_target]
                                            }
                                        ]
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        type: _CASE_,
                        numCols: 1,
                        caseKey: AccountRestoreXWizard.RESTORE_STEP,
                        items: [
                            {
                                type: _GROUP_,
                                visibilityChecks: [
                                    [
                                        XForm.checkInstanceValue,
                                        ZaRestore.A_restoreAllAcc,
                                        false
                                    ]
                                ],
                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                items: [
                                    {
                                        type: _REPEAT_,
                                        ref: ZaRestore.A_accounts,
                                        showRemoveButton: false,
                                        showAddButton: false,
                                        width: "100%",
                                        items: [
                                            {
                                                type: _ZA_PLAIN_GROUPER_,
                                                width: "90%",
                                                items: [
                                                    {
                                                        type: _DWT_ALERT_,
                                                        ref: ZaRestore.A_status_msg,
                                                        containerCssStyle: "padding-bottom:0px",
                                                        style: DwtAlert.INFO,
                                                        iconVisible: true,
                                                        bmolsnr: true,
                                                        align: _CENTER_,
                                                        colSpan: 2,
                                                        width: "100%",
                                                        visibilityChecks: [
                                                            [
                                                                AccountRestoreXWizard.isLocalStatusNormal,
                                                                ZaRestore.A_status_msg
                                                            ]
                                                        ],
                                                        visibilityChangeEventSources: [
                                                            ZaRestore.A_restoreTo,
                                                            ZaRestore.A_accounts
                                                        ]
                                                    },
                                                    {
                                                        type: _DWT_ALERT_,
                                                        ref: ZaRestore.A_status_msg,
                                                        containerCssStyle: "padding-bottom:0px",
                                                        style: DwtAlert.WARNING,
                                                        iconVisible: true,
                                                        bmolsnr: true,
                                                        align: _CENTER_,
                                                        colSpan: 2,
                                                        width: "100%",
                                                        visibilityChecks: [
                                                            [
                                                                AccountRestoreXWizard.isLocalAbortingOrInterrupted,
                                                                ZaRestore.A_status_msg
                                                            ]
                                                        ],
                                                        visibilityChangeEventSources: [
                                                            ZaRestore.A_restoreTo,
                                                            ZaRestore.A_accounts
                                                        ]
                                                    },
                                                    {
                                                        type: _DWT_ALERT_,
                                                        containerCssStyle: "padding-bottom:0px",
                                                        style: DwtAlert.CRITICAL,
                                                        iconVisible: true,
                                                        bmolsnr: true,
                                                        align: _CENTER_,
                                                        colSpan: 2,
                                                        width: "100%",
                                                        ref: ZaModel.ErrorMessage,
                                                        visibilityChecks: [
                                                            [
                                                                AccountRestoreXWizard.isLocalStatusError,
                                                                ZaModel.ErrorMessage
                                                            ]
                                                        ],
                                                        visibilityChangeEventSources: [
                                                            ZaRestore.A_restoreTo,
                                                            ZaRestore.A_accounts
                                                        ]
                                                    },
                                                    {
                                                        type: _SEPARATOR_,
                                                        colSpan: 2,
                                                        visibilityChecks: [
                                                            [
                                                                AccountRestoreXWizard.accountBackupFound,
                                                                null
                                                            ]
                                                        ],
                                                        visibilityChangeEventSources: [
                                                            ZaRestore.A_restoreTo,
                                                            ZaRestore.A_accounts
                                                        ]
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                ]
                            },
                            {
                                type: _GROUP_,
                                visibilityChecks: [
                                    [
                                        XForm.checkInstanceValue,
                                        ZaRestore.A_restoreAllAcc,
                                        true
                                    ]
                                ],
                                visibilityChangeEventSources: [ZaRestore.A_restoreTo],
                                items: [
                                    {
                                        type: _ZA_PLAIN_GROUPER_,
                                        width: "90%",
                                        items: [
                                            {
                                                type: _DWT_ALERT_,
                                                ref: ZaRestore.A_status_msg,
                                                containerCssStyle: "padding-bottom:0px",
                                                style: DwtAlert.INFO,
                                                iconVisible: true,
                                                bmolsnr: true,
                                                align: _CENTER_,
                                                colSpan: 2,
                                                width: "100%",
                                                visibilityChecks: [
                                                    [
                                                        AccountRestoreXWizard.isStatusNormal
                                                    ]
                                                ],
                                                visibilityChangeEventSources: [ZaRestore.A_status]
                                            },
                                            {
                                                type: _DWT_ALERT_,
                                                ref: ZaRestore.A_status_msg,
                                                containerCssStyle: "padding-bottom:0px",
                                                style: DwtAlert.WARNING,
                                                iconVisible: true,
                                                bmolsnr: true,
                                                align: _CENTER_,
                                                colSpan: 2,
                                                width: "100%",
                                                visibilityChecks: [AccountRestoreXWizard.isStatusAbortingOrInterrupted],
                                                visibilityChangeEventSources: [ZaRestore.A_status]
                                            },
                                            {
                                                type: _DWT_ALERT_,
                                                containerCssStyle: "padding-bottom:0px",
                                                style: DwtAlert.CRITICAL,
                                                iconVisible: true,
                                                bmolsnr: true,
                                                align: _CENTER_,
                                                colSpan: 2,
                                                width: "100%",
                                                ref: ZaModel.ErrorMessage,
                                                visibilityChecks: [AccountRestoreXWizard.isStatusError],
                                                visibilityChangeEventSources: [ZaRestore.A_status]
                                            }
                                        ]
                                    }
                                ]
                            }
                        ]
                    }
                ]
            }
        ]
    };

    return xFormObject;
};
