/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2011, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
* @class ZaAccountMove
* @contructor ZaAccountMove
* @param ZaApp app
* this class is a model for doing backup and restore operations
* @author Greg Solovyev
**/
function ZaAccountMove() {
	ZaItem.call(this, "ZaAccountMove");
	this[ZaModel.currentStep] = 1;
//	this[ZaAccountMove.A_destPort] = null;
	this[ZaAccountMove.A_source] = "";
	this[ZaAccountMove.A_destination] = "";
	this[ZaAccountMove.A_sync] = 1;
//	this[ZaAccountMove.A_switchover] = 1;
//	this[ZaAccountMove.A_overwrite] = 0;
	this[ZaAccountMove.A_status] = ZaAccountMove.A_status_PENDING;
	this[ZaAccountMove.A_originalSource] = null;
	this.account = {};
}
ZaAccountMove.A_name = "name";
ZaAccountMove.A_source = "src";
ZaAccountMove.A_originalSource = "originalSource";
ZaAccountMove.A_destination = "dest";
ZaAccountMove.A_sync = "sync";
//ZaAccountMove.A_destPort = "destPort";
//ZaAccountMove.A_switchover = "switchover";
//ZaAccountMove.A_overwrite = "overwrite";
ZaAccountMove.A_status = "status";

ZaAccountMove.A_status_msg = "status_msg";
ZaAccountMove.A_status_PENDING = 1;
ZaAccountMove.A_status_EXPORTING = 2;
ZaAccountMove.A_status_EXPORT_ERROR = 3;
ZaAccountMove.A_status_EXPORTED = 4;
ZaAccountMove.A_status_VERIFYING_LDAP = 5;
ZaAccountMove.A_status_LDAP_ERROR = 6;
ZaAccountMove.A_status_LDAP_VERIFIED = 7;
ZaAccountMove.A_status_PURGING_MAILBOX = 8;
ZaAccountMove.A_status_PURGE_ERROR = 9;
ZaAccountMove.A_status_PURGED_MAILBOX = 10;
ZaAccountMove.A_status_ERROR = -1;


ZaAccountMove.myXModel = new Object();
ZaAccountMove.myXModel.items = new Array();

ZaAccountMove.myXModel.items.push({id:ZaModel.ErrorCode, type:_STRING_});
ZaAccountMove.myXModel.items.push({id:ZaModel.ErrorMessage, type:_STRING_});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_status, type:_STRING_});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_originalSource, type:_STRING_});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_status_msg, type:_STRING_});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_name, ref:"account/"+ZaAccount.A_name,type:_STRING_, required:true, pattern:AjxUtil.EMAIL_SHORT_RE});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_source,ref:"account/attrs/"+ZaAccount.A_mailHost, type:_STRING_, required:true});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_destination, type:_STRING_, required:true});
ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_sync, type:_NUMBER_, defaultValue:0});
//ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_destPort, type:_STRING_});
//ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_switchover, type:_NUMBER_, defaultValue:1});
//ZaAccountMove.myXModel.items.push({id:ZaAccountMove.A_overwrite, type:_NUMBER_, defaultValue:0});

/**
* @method static exportMailbox
* @param obj:ZaAccountMove  
* @param callback:AjxCallback
**/
ZaAccountMove.exportMailbox = function(obj, callback) {
	if(!obj[ZaAccountMove.A_originalSource]) {
		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_originalSource]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
	}	
	if(!obj.account[ZaAccountMove.A_name]) {
		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_name]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
	}
	if(!obj[ZaAccountMove.A_destination]) {
		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_destination]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
	}	
//	if(obj[ZaAccountMove.A_switchover]==null) {
//		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_switchover]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
//	}		
//	if(obj[ZaAccountMove.A_overwrite]==null) {
//		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_overwrite]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
//	}			
	var soapDoc = AjxSoapDoc.create("MoveMailboxRequest", "urn:zimbraAdmin", null);
	var accountEl = soapDoc.set("account", "");

	accountEl.setAttribute(ZaAccountMove.A_name, obj.account[ZaAccountMove.A_name]);
	accountEl.setAttribute(ZaAccountMove.A_source, obj[ZaAccountMove.A_originalSource]);
	accountEl.setAttribute(ZaAccountMove.A_destination, obj[ZaAccountMove.A_destination]);
    //bug:62794, To use async mode, remove sync=1 from MoveMailboxReques
//	accountEl.setAttribute(ZaAccountMove.A_sync, obj[ZaAccountMove.A_sync]);
//	accountEl.setAttribute(ZaAccountMove.A_switchover, obj[ZaAccountMove.A_switchover]);
//	accountEl.setAttribute(ZaAccountMove.A_overwrite, obj[ZaAccountMove.A_overwrite]);
//
//	if(obj[ZaAccountMove.A_destPort] != null)
//		accountEl.setAttribute(ZaAccountMove.A_destPort, obj[ZaAccountMove.A_destPort]);		
		
	//var asynCommand = new ZmCsfeCommand();
	var params = new Object();
	params.soapDoc = soapDoc;	
	if(callback) {
		params.asyncMode = true;
		params.callback = callback;
	}
//	var server = ZaApp.getInstance().getServerByName(obj[ZaAccountMove.A_originalSource]);
    var server = ZaApp.getInstance().getServerByName(obj[ZaAccountMove.A_destination]);
	params.targetServer = server.id;
	var reqMgrParams = {
		controller: ZaApp.getInstance().getCurrentController(),
		busyMsg: ZaMsg.BUSY_EXPORT_MBX
	}
	ZaRequestMgr.invoke(params, reqMgrParams) ;	
}
/**
* @method static purgeMovedMailbox
* @param obj:ZaAccountMove  
* @param callback:AjxCallback
**/
ZaAccountMove.purgeMovedMailbox = 
function (obj, callback) {
	if(!obj[ZaAccountMove.A_originalSource]) {
		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_source]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
	}	
	if(!obj.account[ZaAccountMove.A_name]) {
		throw(new AjxException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_parameterCannotBeNull,[ZaAccountMove.A_name]), AjxException.INVALID_PARAM, "ZaAccountMove.exportMailbox"));
	}
	var soapDoc = AjxSoapDoc.create("PurgeMovedMailboxRequest", "urn:zimbraAdmin", null);
	var accountEl = soapDoc.set("mbox", "");

	accountEl.setAttribute(ZaAccountMove.A_name, obj.account[ZaAccountMove.A_name]);
	//var asynCommand = new ZmCsfeCommand();
	var params = new Object();
	params.soapDoc = soapDoc;	
	if(callback) {
		params.asyncMode = true;
		params.callback = callback;
	}
	var server = ZaApp.getInstance().getServerByName(obj[ZaAccountMove.A_originalSource]);
	params.targetServer = server.id;
	var reqMgrParams = {
		controller: ZaApp.getInstance().getCurrentController(),
		busyMsg: ZaMsg.BUSY_PURGE_MBX
	}
	ZaRequestMgr.invoke(params, reqMgrParams) ;
}
