/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaAccountMoveXFormView
* @contructor
* @param parent
* @author Greg Solovyev
**/
function ZaAccountMoveXFormView (parent) {
	ZaTabView.call(this, parent,"ZaAccountMoveXFormView");	
		
	this.initForm(ZaAccountMove.myXModel,this.getMyXForm());
	this._localXForm.setController(ZaApp.getInstance());
}

ZaAccountMoveXFormView.prototype = new ZaTabView();
ZaTabView.XFormModifiers["ZaAccountMoveXFormView"] = new Array();

/**
* @method getObject returns the object contained in the view
* before returning the object this updates the object attributes with 
* tha values from the form fields 
**/
ZaAccountMoveXFormView.prototype.getObject =
function() {
	return this._localXForm.getInstance();
}

ZaAccountMoveXFormView.prototype.setObject = 
function (entry) {
	this._containedObject = entry;
	this._containedObject[ZaModel.currentTab] = 1;
	this._containedObject[ZaAccountMove.A_originalSource] = this._containedObject.account.attrs[ZaAccount.A_mailHost];
	this._localXForm.setInstance(this._containedObject);		
	//update the tab
     if (!appNewUI)
	    this.updateTab();
}

ZaAccountMoveXFormView.prototype.getTitle = 
function () {
	if (this._containedObject && this._containedObject.account) {
		return AjxMessageFormat.format(com_zimbra_backuprestore.BNR_AccMoveTabName,[this._containedObject.account.name]);
	}else{
		return "";
	}
}

ZaAccountMoveXFormView.prototype.getTabTitle =
function () {
	if (this._containedObject && this._containedObject.account) {
		return AjxMessageFormat.format(com_zimbra_backuprestore.BNR_AccMoveTabName,[this._containedObject.account.name]);
	}else{
		return "";
	}
}

ZaAccountMoveXFormView.prototype.exportStatus =
function (params) {
    var instance = this._localXForm.getInstance();
    try{
    var form = this._localXForm;
    var soapDoc = AjxSoapDoc.create("QueryMailboxMoveRequest", "urn:zimbraAdmin", null);
	var accountEl = soapDoc.set("account", "");
    accountEl.setAttribute(ZaAccountMove.A_name, instance.account[ZaAccountMove.A_name]);

    var callback = new AjxCallback(form.parent, ZaAccountMoveXFormView.prototype.exportStatusCallBack);
    var params = new Object();
	params.soapDoc = soapDoc;
	if(callback) {
		params.asyncMode = true;
		params.callback = callback;
	}
   var reqMgrParams = {
		controller: ZaApp.getInstance().getCurrentController(),
		busyMsg: ZaMsg.BUSY_EXPORT_MBX
	}
	ZaRequestMgr.invoke(params, reqMgrParams) ;
    }catch(ex){
        instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_EXPORT_ERROR;
		var msg = "";
		if(ex.code) {
			msg += ZaMsg.ERROR_CODE + "  ";
		    msg += ex.code;
		    msg += "\n";
		}
		if(ex.msg) {
			msg += ZaMsg.ERROR_MESSAGE + "  ";
		    msg += ex.msg;
		    msg += "\n";
		}

		if(ex.detail) {
			msg += ZaMsg.ERROR_DETAILS;
		    msg += ex.detail;
		    msg += "\n";
		}
		instance[ZaModel.ErrorMessage] = msg;
		this._localXForm.refresh();
    }
}

ZaAccountMoveXFormView.prototype.exportStatusCallBack =
function (respObj){
    var instance = 	this._localXForm.getInstance();
	try {
        var done = false;
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaAccountMoveXFormView.prototype.exportCallBack");
			throw (ex);
		}
		if(respObj.isException && respObj.isException()) {
			var ex = respObj.getException();
			throw ex;
		} else {
			var resp;
            if(respObj.getResponse) {
                resp = respObj.getResponse().Body.QueryMailboxMoveResponse;
            } else if(respObj.Body.QueryMailboxMoveResponse) {
                resp = respObj.Body.QueryMailboxMoveResponse;
            }
			if(AjxUtil.isEmpty(resp.account)) {
				instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_EXPORTED;
				instance[ZaModel.ErrorMessage] = "";
                done=true;
			}
		}
        if(!done){
           this._pollHandler = AjxTimedAction.scheduleAction(this.pollAction, 300);
        }else{
            this.pollAction=null;
            this._pollHandler=null;
		    this.verifyLDAP();
		    this._localXForm.refresh();
		    this.setDirty(true);
        }
	} catch (ex) {
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_EXPORT_ERROR;
		var msg = "";
		if(ex.code) {
			msg += ZaMsg.ERROR_CODE + "  ";
		    msg += ex.code;
		    msg += "\n";
		}
		if(ex.msg) {
			msg += ZaMsg.ERROR_MESSAGE + "  ";
		    msg += ex.msg;
		    msg += "\n";
		}

		if(ex.detail) {
			msg += ZaMsg.ERROR_DETAILS;
		    msg += ex.detail;
		    msg += "\n";
		}
		instance[ZaModel.ErrorMessage] = msg;
		this._localXForm.refresh();
	}

}


ZaAccountMoveXFormView.prototype.exportCallBack = 
function (respObj) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaAccountMoveXFormView.prototype.exportCallBack");
			throw (ex);
		}	
		if(respObj.isException && respObj.isException()) {
			var ex = respObj.getException();
			throw ex;
		} else {
			var resp;
//			if(respObj.getResponse) {
//				resp = respObj.getResponse().Body.ExportMailboxResponse;
//			} else if(respObj.Body.ExportMailboxResponse) {
//				resp = respObj.Body.ExportMailboxResponse;
//			}	
            if(respObj.getResponse) {
                resp = respObj.getResponse().Body.MoveMailboxResponse;
            } else if(respObj.Body.MoveMailboxResponse) {
                resp = respObj.Body.MoveMailboxResponse;
            }   
			if(resp) {	
				//instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_EXPORTED;
				//instance[ZaModel.ErrorMessage] = "";
                this.pollAction = new AjxTimedAction(this, this.exportStatus);
		        this._pollHandler = AjxTimedAction.scheduleAction(this.pollAction, 300);
			}
		}
		//this.verifyLDAP();
		//this._localXForm.refresh();
		//this.setDirty(true);
	} catch (ex) {
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_EXPORT_ERROR;
		var msg = "";
		if(ex.code) {
			msg += ZaMsg.ERROR_CODE + "  ";
		    msg += ex.code;
		    msg += "\n";				
		}
		if(ex.msg) {
			msg += ZaMsg.ERROR_MESSAGE + "  ";
		    msg += ex.msg;
		    msg += "\n";	
		}
		
		if(ex.detail) {
			msg += ZaMsg.ERROR_DETAILS;
		    msg += ex.detail;
		    msg += "\n";			    
		}		
		instance[ZaModel.ErrorMessage] = msg;
		this._localXForm.refresh();		
	}
}

/**
* Callback function invoked by Asynchronous CSFE command when "SearchDirectoryRequest" call returns
**/
ZaAccountMoveXFormView.prototype.searchAccCallBack = 
function (respObj) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaAccountMoveXFormView.prototype.searchAccCallBack");
			throw(ex);
	//		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaAccountMoveXFormView.prototype.searchAccCallBack");
//			return;
		}	
		if(respObj.isException && respObj.isException()) {
			var ex = respObj.getException();
			throw ex;
		} else {
			var response = respObj.getResponse().Body.SearchDirectoryResponse;
			var list = new ZaItemList(ZaAccount, ZaApp.getInstance());	
			list.loadFromJS(response);		
			var acc = list.getArray()[0];
			if(acc) {
				if(acc.attrs[ZaAccount.A_mailHost] == instance[ZaAccountMove.A_destination]) {
					instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_LDAP_VERIFIED;					
				} else {
					var ex = new ZmCsfeException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_Error_LDAP_NotUpdated,[ZaAccount.A_mailHost,instance[ZaAccountMove.A_destination],acc.attrs[ZaAccount.A_mailHost]]),ZmCsfeException.CSFE_SVC_ERROR,"ZaAccountMoveXFormView.prototype.searchAccCallBack");
					throw(ex);					
				}
			}
			instance.account = acc;
		}
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_LDAP_VERIFIED;
	} catch (ex) {
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_LDAP_ERROR;
		var msg = "Error:\n ";
		if(ex.code) {
			msg += ZaMsg.ERROR_CODE + "  ";
		    msg += ex.code;
		    msg += "\n";				
		}
		if(ex.msg) {
			msg += ZaMsg.ERROR_MESSAGE + "  ";
		    msg += ex.msg;
		    msg += "\n";	
		}
		
		if(ex.detail) {
			msg += ZaMsg.ERROR_DETAILS;
		    msg += ex.detail;
		    msg += "\n";			    
		}		
		instance[ZaModel.ErrorMessage] = msg;
	}
	this._localXForm.refresh();
}

ZaAccountMoveXFormView.prototype.purgeCallBack = 
function (respObj) {
	var instance = 	this._localXForm.getInstance();
	try {
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaAccountMoveXFormView.prototype.purgeCallBack");
			throw (ex);
		}	
		if(respObj.isException && respObj.isException()) {
			var ex = respObj.getException();
			throw ex;
		} else {
			var resp;
			if(respObj.getResponse) {
				resp = respObj.getResponse().Body.PurgeMovedMailboxResponse;
			} else if(respObj.Body.PurgeMovedMailboxResponse) {
				resp = respObj.Body.PurgeMovedMailboxResponse;
			}	
			if(resp) {	
				instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_PURGED_MAILBOX;
				instance[ZaModel.ErrorMessage] = "";
			}
		}
	} catch (ex) {
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_PURGE_ERROR;
		var msg = "Error:\n ";
		if(ex.code) {
			msg += ZaMsg.ERROR_CODE + "  ";
		    msg += ex.code;
		    msg += "\n";				
		}
		if(ex.msg) {
			msg += ZaMsg.ERROR_MESSAGE + "  ";
		    msg += ex.msg;
		    msg += "\n";	
		}
		
		if(ex.detail) {
			msg += ZaMsg.ERROR_DETAILS;
		    msg += ex.detail;
		    msg += "\n";			    
		}		
		instance[ZaModel.ErrorMessage] = msg;
	}
	this._localXForm.refresh();
	this.setDirty(false);	
}

ZaAccountMoveXFormView.doMove = function () {
	var instance = 	this.getInstance();
	var form = this.getForm();
	instance[ZaAccountMove.A_status] =  ZaAccountMove.A_status_EXPORTING;
	instance[ZaAccountMove.A_originalSource] = instance.account.attrs[ZaAccount.A_mailHost];
	form.refresh();
	var callBack = new AjxCallback(form.parent, ZaAccountMoveXFormView.prototype.exportCallBack);
	ZaAccountMove.exportMailbox.call(form.parent,instance, callBack);
}

ZaAccountMoveXFormView.purgeAccount = function () {
	var instance = 	this.getInstance();
	var form = this.getForm();
	instance[ZaAccountMove.A_status] =  ZaAccountMove.A_status_PURGING_MAILBOX;
	form.refresh();
	var callBack = new AjxCallback(form.parent, ZaAccountMoveXFormView.prototype.purgeCallBack);
	ZaAccountMove.purgeMovedMailbox.call(form.parent,instance, callBack);	
	
}

ZaAccountMoveXFormView.viewMail = function () {
	var instance = 	this.getInstance();
	var form = this.getForm();
	ZaAccountListController._viewMailListenerLauncher.call(ZaApp.getInstance().getCurrentController(),instance.account);
}

ZaAccountMoveXFormView.prototype.verifyLDAP = function () {
	try {
		var instance = 	this._localXForm.getInstance();
		instance[ZaAccountMove.A_status] = ZaAccountMove.A_status_VERIFYING_LDAP;
		this._localXForm.refresh();
		var accountName = instance.account[ZaAccount.A_name];
		var params = new Object();
		var dataCallback = new AjxCallback(this, this.searchAccCallBack);
		params.types = [ZaSearch.ACCOUNTS,ZaSearch.RESOURCES];
		params.callback = dataCallback;
		params.sortBy = ZaAccount.A_name;
		params.query = ZaSearch.getSearchByNameQuery(accountName, params.types);
		ZaSearch.searchDirectory(params);
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaAccountMoveXFormView.prototype.verifyLDAP");		
	}	
}

ZaAccountMoveXFormView.isExportButtonEnabled = function () {
	return !AjxUtil.isEmpty(this.getInstanceValue(ZaAccountMove.A_destination)) && (this.getInstanceValue(ZaAccountMove.A_destination) != this.getInstanceValue(ZaAccountMove.A_source));	
}

ZaAccountMoveXFormView.isMoveStatusError = function() {
	return ((this.getInstanceValue(ZaAccountMove.A_status) == ZaAccountMove.A_status_EXPORT_ERROR) || (this.getInstanceValue(ZaAccountMove.A_status) == ZaAccountMove.A_status_LDAP_ERROR)) 
}

ZaAccountMoveXFormView.isIdle = function() {
	return (this.getInstanceValue(ZaAccountMove.A_status) < ZaAccountMove.A_status_VERIFYING_LDAP && this.getInstanceValue(ZaAccountMove.A_status)  >= ZaAccountMove.A_status_PENDING);
}

ZaAccountMoveXFormView.isDone = function() {
	return (this.getInstanceValue(ZaAccountMove.A_status) > ZaAccountMove.A_status_LDAP_ERROR);
}

ZaAccountMoveXFormView.isVerified = function() {
	return 	(this.getInstanceValue(ZaAccountMove.A_status)  >= ZaAccountMove.A_status_LDAP_VERIFIED);
}

ZaAccountMoveXFormView.enablePurgeButton = function() {
	return (this.getInstanceValue(ZaAccountMove.A_status) >= ZaAccountMove.A_status_LDAP_VERIFIED) && (this.getInstanceValue(ZaAccountMove.A_status) < ZaAccountMove.A_status_PURGED_MAILBOX);	
}

ZaAccountMoveXFormView.isExportingDone = function () {
	return (this.getInstanceValue(ZaAccountMove.A_status) > ZaAccountMove.A_status_EXPORT_ERROR);
}

ZaAccountMoveXFormView.isPastPending = function () {
	return (this.getInstanceValue(ZaAccountMove.A_status) >= ZaAccountMove.A_status_PENDING);
}


ZaAccountMoveXFormView.isBeforePurging = function () {
	return (this.getInstanceValue(ZaAccountMove.A_status) < ZaAccountMove.A_status_PURGING_MAILBOX);
}

ZaAccountMoveXFormView.myXFormModifier = function(xFormObject) {	
	var domainName;
	if(ZaSettings.DOMAINS_ENABLED) {
		domainName = ZaApp.getInstance().getGlobalConfig().attrs[ZaGlobalConfig.A_zimbraDefaultDomainName];
		if(!domainName)
			domainName = ZaApp.getInstance().getDomainList().getArray()[0].name;
	} else 
		domainName = ZaSettings.myDomainName;
			
	var emptyAlias = " @" + domainName;
	xFormObject.items = [
			{type:_SWITCH_,  width:530, align:_LEFT_, valign:_TOP_, items:[
				{type:_ZATABCASE_, numCols:1,  caseKey:1,id:"moveacc_wiz_main_tab", align:_LEFT_, valign:_TOP_, items:[
					{ type: _DWT_ALERT_,
					  containerCssStyle: "padding-bottom:0px",
					  style: DwtAlert.INFORMATION,
					  iconVisible: true, 
					  align:_CENTER_,
					  content: com_zimbra_backuprestore.BNR_MoveInfo1
					},
					{type:_ZA_TOP_GROUPER_, label:com_zimbra_backuprestore.BNR_ExportingMbx_group, id:"moveacc_view_export_group",
						items:[
							{ref:ZaAccountMove.A_name, type:_DYNSELECT_,editable:true,width:"200px",
							 	inputSize:30, label: com_zimbra_backuprestore.BNR_selectAccount,
								dataFetcherClass:ZaSearch,
								dataFetcherTypes:[ZaSearch.ACCOUNTS, ZaSearch.RESOURCES],
								dataFetcherAttrs:[ZaItem.A_zimbraId, ZaItem.A_cn, ZaAccount.A_name, ZaAccount.A_displayname, ZaAccount.A_mail],
								dataFetcherMethod:ZaSearch.prototype.dynSelectSearch
							},
							{ref:ZaAccountMove.A_originalSource, type:_OUTPUT_,	label:com_zimbra_backuprestore.BNR_sourceServer},
							{ref:ZaAccountMove.A_destination, type:_OSELECT1_, label:com_zimbra_backuprestore.BNR_targetServer,
								nowrap:false,labelWrap:true, choices:ZaApp.getInstance().getServerListChoices()
							},	
							{type:_SPACER_,colSpan:2},								
							{type:_CELLSPACER_},
							{type:_DWT_BUTTON_, label:com_zimbra_backuprestore.BNR_Export, name: "exportAccountButton",
								onActivate:ZaAccountMoveXFormView.doMove,
								enableDisableChecks:[ZaAccountMoveXFormView.isExportButtonEnabled],
								enableDisableChangeEventSources:[ZaAccountMove.A_destination,ZaAccountMove.A_source],
								width:100,icon:"MoveMailbox"
							},
							{type:_SPACER_,colSpan:2},								
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_ExportingMailbox,src:"QuestionMark",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_PENDING]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]														
							},						
							{type:_DWT_IMAGE_,label:com_zimbra_backuprestore.BNR_ExportingMailbox,src:"DwtWait16Icon",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_EXPORTING]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]																					
							},
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_ExportingMailbox,src:"Check",
		  					  	visibilityChecks:[ZaAccountMoveXFormView.isExportingDone],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]																													
							},	
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_ExportingMailbox,src:"Cancel",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_EXPORT_ERROR]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]																					
							},
							{type:_DWT_ALERT_,
							  containerCssStyle: "padding-bottom:0px",
							  style: DwtAlert.CRITICAL,
							  iconVisible: true, 
							  align:_CENTER_,colSpan:2,width:"100%",
		  					  ref:ZaModel.ErrorMessage,
		  					  visibilityChecks:[ZaAccountMoveXFormView.isMoveStatusError],
		  					  visibilityChangeEventSources:[ZaAccountMove.A_status]
							}							
						]
					},
					{type:_ZA_TOP_GROUPER_, label:com_zimbra_backuprestore.BNR_VerifyingMbx_group, id:"moveacc_view_verify_group",
						items:[
							{ type: _DWT_ALERT_,
							  containerCssStyle: "padding-bottom:0px",
							  style: DwtAlert.INFORMATION,
							  iconVisible: true, 
							  align:_CENTER_,colSpan:2,
							  content: null,
							  ref:ZaAccountMove.A_originalSource,
								  getDisplayValue:function(val) {
								  	var itemval = this.getInstance();
								  	return AjxMessageFormat.format(com_zimbra_backuprestore.BNR_MoveInfo2,[ZaMsg.ACTBB_ViewMail]);
								}
							},	
							{type:_SPACER_,colSpan:2},																					
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_VerifyingLDAP,src:"QuestionMark",
		  					  	visibilityChecks:[ZaAccountMoveXFormView.isIdle],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]								
							},
							{type:_DWT_IMAGE_,label:com_zimbra_backuprestore.BNR_VerifyingLDAP,src:"DwtWait16Icon",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_VERIFYING_LDAP]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_VerifyingLDAP,src:"Check",
		  					  	visibilityChecks:[ZaAccountMoveXFormView.isDone],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]								
							},
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_VerifyingLDAP,src:"Cancel",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_LDAP_ERROR]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							{type:_CELLSPACER_},	
							{type:_DWT_BUTTON_, label:ZaMsg.ACTBB_ViewMail, name: "viewMailButton",width:100,icon:"ReadMailbox",
								onActivate:ZaAccountMoveXFormView.viewMail,
								enableDisableChecks:[ZaAccountMoveXFormView.isVerified],
								enableDisableChangeEventSources:[ZaAccountMove.A_status]								
								
							}
						]
					},
					{type:_ZA_TOP_GROUPER_, label:com_zimbra_backuprestore.BNR_PurgingMbx_group, id:"moveacc_view_purge_group",
						items:[
						{ type: _DWT_ALERT_,
							  containerCssStyle: "padding-bottom:0px",
							  style: DwtAlert.INFORMATION,
							  iconVisible: true, 
							  align:_CENTER_,colSpan:2,
							  content: null,
							  ref:ZaAccountMove.A_originalSource,
								  getDisplayValue:function(val) {
								  	var itemval = this.getInstance();
								  	return AjxMessageFormat.format(com_zimbra_backuprestore.BNR_MoveInfo3,[com_zimbra_backuprestore.BNR_Purge,itemval[ZaAccountMove.A_originalSource]]);
								}
							},								
							{type:_SPACER_,colSpan:2},								
							{type:_CELLSPACER_},	
							{type:_DWT_BUTTON_, label:com_zimbra_backuprestore.BNR_Purge, name: "purgeAccountButton",
								onActivate:ZaAccountMoveXFormView.purgeAccount,
								enableDisableChecks:[ZaAccountMoveXFormView.enablePurgeButton],
								enableDisableChangeEventSources:[ZaAccountMove.A_status],
								width:100,icon:"Delete"
							},
							{type:_SPACER_,colSpan:2},						
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_PurgeOldMailbox,src:"QuestionMark",
		  					  	visibilityChecks:[ZaAccountMoveXFormView.isPastPending,ZaAccountMoveXFormView.isBeforePurging],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							{type:_DWT_IMAGE_,label:com_zimbra_backuprestore.BNR_PurgeOldMailbox,src:"DwtWait16Icon",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_PURGING_MAILBOX]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_PurgeOldMailbox,src:"Check",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_PURGED_MAILBOX]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							{type:_AJX_IMAGE_,label:com_zimbra_backuprestore.BNR_PurgeOldMailbox,src:"Cancel",
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_PURGE_ERROR]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							},
							
							{type:_DWT_ALERT_,
							  	containerCssStyle: "padding-bottom:0px",
							  	style: DwtAlert.CRITICAL,
							  	iconVisible: true, 
							  	align:_CENTER_,colSpan:2,width:"100%",
							  	ref:ZaModel.ErrorMessage,
		  					  	visibilityChecks:[[XForm.checkInstanceValue,ZaAccountMove.A_status,ZaAccountMove.A_status_PURGE_ERROR]],
		  					  	visibilityChangeEventSources:[ZaAccountMove.A_status]							
							}
						]
					}
				]}
			]}	
		];
};
ZaTabView.XFormModifiers["ZaAccountMoveXFormView"].push(ZaAccountMoveXFormView.myXFormModifier);