/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
 
 /**
 * @author Greg Solovyev
 **/
function ZaBNRAccMiniListView(parent, className, posStyle, headerList) {
    posStyle = DwtControl.ABSOLUTE_STYLE;  // Only absolute style is allowed here
	ZaAccMiniListView.call(this, parent, className, posStyle, headerList, undefined, "ZaBNRAccMiniListView", true);
}

ZaBNRAccMiniListView.prototype = new ZaAccMiniListView;
ZaBNRAccMiniListView.prototype.constructor = ZaBNRAccMiniListView;

ZaBNRAccMiniListView.prototype.toString = function() {
	return "ZaBNRAccMiniListView";
};

ZaBNRAccMiniListView.prototype._createItemHtml =
function(account, now, isDndIcon) {
	var html = new Array(50);
	var	div = document.createElement("div");
	div[DwtListView._STYLE_CLASS] = "Row";
	div[DwtListView._SELECTED_STYLE_CLASS] = div[DwtListView._STYLE_CLASS] + "-" + DwtCssStyle.SELECTED;
	div.className = div[DwtListView._STYLE_CLASS];
	this.associateItemWithElement(account, div, DwtListView.TYPE_LIST_ITEM);
	
	var idx = 0;
	html[idx++] = "<table width='100%' cellspacing='2' cellpadding='0'>";

	html[idx++] = "<tr>";
	if(this._headerList) {
		var cnt = this._headerList.length;
		for(var i = 0; i < cnt; i++) {
			var id = this._headerList[i]._field;
			if(id!=null) {			
				if(id.indexOf(ZaAccount.A_name)==0) {
					// name
					html[idx++] = "<td align='left' width=" + this._headerList[i]._width + "><nobr>";
					html[idx++] = AjxStringUtil.htmlEncode(account[ZaAccount.A_name]);
					html[idx++] = "</nobr></td>";
				} else if(id.indexOf(ZaBackupLabel.A_status)==0) {
					// status		
					html[idx++] = "<td align=left height=20px  width='";
					html[idx++] = this._headerList[i]._width 
					html[idx++] = "'>";
					
					switch(account[ZaBackupLabel.A_status]) {
						case ZaBackupLabel.STATUS_NOTSTARTED:
							html[idx++] = AjxStringUtil.htmlEncode(com_zimbra_backuprestore.BNR_NotStarted); 
						break;
						case ZaBackupLabel.STATUS_INPROGRESS:
							html[idx++] = AjxStringUtil.htmlEncode(com_zimbra_backuprestore.BNR_InProgress); 
						break;								
						case ZaBackupLabel.STATUS_COMPLETED:
							html[idx++] = AjxStringUtil.htmlEncode(com_zimbra_backuprestore.BNR_Completed); 
						break;								
						case ZaBackupLabel.STATUS_ERROR:
							html[idx++] = AjxStringUtil.htmlEncode(com_zimbra_backuprestore.BNR_Error); 
						break;													
					}
					
					html[idx++] = "</td>";
				} else if (id.indexOf(ZaBackupLabel.A_errorMessage)==0) {
					html[idx++] = "<td align='left' width=" + this._headerList[i]._width + "><nobr>";
					html[idx++] = AjxStringUtil.htmlEncode(account[ZaBackupLabel.A_errorMessage]);
					html[idx++] = "</nobr></td>";				
					
				}
			}
		}
	} else if(typeof(account)=="object") {
		html[idx++] = "<td width=100%>";
		html[idx++] = AjxStringUtil.htmlEncode(account.name);
		html[idx++] = "</td>";
	} else {
		html[idx++] = "<td width=100%>";
		html[idx++] = AjxStringUtil.htmlEncode(String(account));		
		html[idx++] = "</td>";		
	}
	
	html[idx++] = "</tr></table>";
	div.innerHTML = html.join("");
	return div;
}


ZaBNRAccMiniListView.prototype._setNoResultsHtml = function() {
	var buffer = new AjxBuffer();
	var	div = document.createElement("div");
	
	buffer.append("<table width='100%' cellspacing='0' cellpadding='1'>",
				  "<tr><td class='NoResults'><br>&nbsp",
				  "</td></tr></table>");
	
	div.innerHTML = buffer.toString();
	this._addRow(div);
};

ZaBNRAccMiniListView.prototype._sortColumn = function (columnItem, bSortAsc){
	if (bSortAsc) {
		var comparator = function (a, b) {
			return (a < b)? 1 :((a > b)? -1 : 0);
		};
		this.getList().sort(comparator);
	} else {
		this.getList().sort();
	}
};

ZaBNRAccMiniListView.prototype.setCrtInstance = function (instance, isFailedAccount) {
    this._crtInstance =  ZaUtil.deepCloneObject(instance);
    this._isFaieldAccount = isFailedAccount;
}


ZaBNRAccMiniListView.prototype._loadMsg = function(params) {

    var updateCallback = new AjxCallback(this, this.updateMoreItems);
	if(!this._isFaieldAccount) {
		this._crtInstance[ZaBackup.A_accountListOffset] = this._crtInstance[ZaBackup.A_pageNumber] * ZaSettings.RESULTSPERPAGE;
		this._crtInstance[ZaBackup.A_accountListCount] = ZaSettings.RESULTSPERPAGE;
		this._crtInstance[ZaBackup.A_pageNumber]++;
        ZaBackup.queryBackups(this._crtInstance, updateCallback);
	} else if(this._currentObject[ZaModel.currentTab]==2){
		this._crtInstance[ZaBackup.A_failedAccountListOffset] = (this._crtInstance[ZaBackup.A_failedAccPageNumber] * ZaSettings.RESULTSPERPAGE);
		this._crtInstance[ZaBackup.A_failedAccountListCount] = ZaSettings.RESULTSPERPAGE;
		this._crtInstance[ZaBackup.A_failedAccPageNumber]++;
        ZaBackup.queryFailedAccounts(this._crtInstance, updateCallback);
	}

}

ZaBNRAccMiniListView.prototype.updateMoreItems = function(resp) {
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupListView.prototype.updateMoreItems");
		return;
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.BackupQueryResponse) {
			response = resp;
		}
		var backups = null;
		var instance = new ZaBackupLabel();

		if(response.Body.BackupQueryResponse.backup) {
			backups = response.Body.BackupQueryResponse.backup;
			if(!(backups instanceof Array)) {
				instance.initFromJS(backups);
			} else {
				instance.initFromJS(backups[0]);
			}
            var newAccountArray = instance[ZaBackupLabel.A_accounts];
            this.replenish(AjxVector.fromArray(newAccountArray));
		}

        if (instance[ZaBackup.A_more]) {
            this.setScrollHasMore(true);
        }
	}
}
