/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaBackupLabelViewController controls display of a single Server's Postfix Queue
* @contructor ZaBackupLabelViewController
* @param appCtxt
* @param container
* @param abApp
* @author Greg Solovyev
**/

function ZaBackupLabelViewController(appCtxt, container) {
	ZaXFormViewController.call(this, appCtxt, container,"ZaBackupLabelViewController");
	this._UICreated = false;
	this._helpURL = location.pathname + "help/admin/html/backup_feature/starting_a_backup_session.htm?locid=" + AjxEnv.DEFAULT_LOCALE;				
	this.objType = 0;	
	this.tabConstructor	= ZaBackupLabelXFormView;
}

ZaBackupLabelViewController.prototype = new ZaXFormViewController();
ZaBackupLabelViewController.prototype.constructor = ZaBackupLabelViewController;

ZaController.initPopupMenuMethods["ZaBackupLabelViewController"] = new Array();
ZaController.setViewMethods["ZaBackupLabelViewController"] = new Array();
/**
*	@method show
*	@param entry - isntance of ZaServer class
*/
ZaBackupLabelViewController.prototype.show = 
function(entry) {
	if (! this.selectExistingTabByItemId(entry[ZaBackupLabel.A_label])){	
		this._setView(entry, true);
		this.setDirty(false);
	}
}

/**
*	@method setViewMethod 
*	@param entry - isntance of ZaBackupLabel class
*/
ZaBackupLabelViewController.setViewMethod =
function(entry) {
//	entry.load();
	if(!this._UICreated) {
		this._createUI();
	} 
	ZaApp.getInstance().pushView(this.getContentViewId());
	this._view.setDirty(false);
	this._view.setObject(entry); 	//setObject is delayed to be called after pushView in order to avoid jumping of the view	
	this._currentObject = entry;
}
ZaController.setViewMethods["ZaBackupLabelViewController"].push(ZaBackupLabelViewController.setViewMethod);

ZaBackupLabelViewController.initPopupMenuMethod =
function () {
	this._popupOperations.push(new ZaOperation(ZaOperation.MAIL_RESTORE, com_zimbra_backuprestore.BNR_TBB_RestoreMailbox, com_zimbra_backuprestore.BNR_TBB_Restore_tt, "RestoreMailbox", "RestoreMailboxDis", new AjxListener(this, ZaBackupLabelViewController.prototype._restoreMailListener)));
}
ZaController.initPopupMenuMethods["ZaBackupLabelViewController"].push(ZaBackupLabelViewController.initPopupMenuMethod);

ZaBackupLabelViewController.prototype.nextPageListener = 
function (ev) {
	this._currentObject = this._view.getObject();
	if(this._currentObject[ZaModel.currentTab]==1) {
		this._currentObject[ZaBackup.A_accountListOffset] = (this._currentObject[ZaBackup.A_pageNumber] * ZaSettings.RESULTSPERPAGE);
		this._currentObject[ZaBackup.A_accountListCount] = ZaSettings.RESULTSPERPAGE;
		this._currentObject[ZaBackup.A_pageNumber]++;
	} else if(this._currentObject[ZaModel.currentTab]==2){
		this._currentObject[ZaBackup.A_failedAccountListOffset] = (this._currentObject[ZaBackup.A_failedAccPageNumber] * ZaSettings.RESULTSPERPAGE);
		this._currentObject[ZaBackup.A_failedAccountListCount] = ZaSettings.RESULTSPERPAGE;
		this._currentObject[ZaBackup.A_failedAccPageNumber]++;
	}
	this._view.setObject(this._currentObject); 	
}

ZaBackupLabelViewController.prototype.prevPageListener = 
function (ev) {
	this._currentObject = this._view.getObject();	
	if(this._currentObject[ZaModel.currentTab]==1) {	
		if(this._currentObject[ZaBackup.A_pageNumber]<1) {
			return;
		}
		this._currentObject[ZaBackup.A_pageNumber]--;
		this._currentObject[ZaBackup.A_accountListOffset] = ((this._currentObject[ZaBackup.A_pageNumber]-1) * ZaSettings.RESULTSPERPAGE);
		this._currentObject[ZaBackup.A_accountListCount] = ZaSettings.RESULTSPERPAGE;
	} if(this._currentObject[ZaModel.currentTab]==2){
		if(this._currentObject[ZaBackup.A_failedAccPageNumber]<1) {
			return;
		}
		this._currentObject[ZaBackup.A_failedAccPageNumber]--;
		this._currentObject[ZaBackup.A_failedAccountListOffset] = ((this._currentObject[ZaBackup.A_failedAccPageNumber]-1) * ZaSettings.RESULTSPERPAGE);
		this._currentObject[ZaBackup.A_failedAccountListCount] = ZaSettings.RESULTSPERPAGE;
		
	}
	this._view.setObject(this._currentObject); 
}

ZaBackupLabelViewController.prototype._restoreMailListener = 
function (ev) {
	try {
		this._newSingleAccountRestoreWizard = new AccountRestoreXWizard(this._container);		
		var restore = new ZaRestore();
		var form = this._view.getMyForm();
		if(form) {
			var accounts = form.getItemsById("accounts_list")[0].widget.getSelection();
			var cnt = accounts.length;
			if(accounts && cnt>0) {
				restore.restoreRequest[ZaRestore.A_restoreAllAcc] = false;
				var accountNames = [];
				for(var i=0; i < cnt; i++) {
					accountNames.push(accounts[i].name);
				}
				restore.restoreRequest[ZaRestore.A_accountNames] = accountNames;
			}
			var instance = form.getInstance();
			if(instance) {
				restore.restoreRequest[ZaRestore.A_target] = instance[ZaBackup.A_target];
				restore.restoreRequest[ZaRestore.A_toServer] = instance[ZaBackup.A_serverId];				
				restore.restoreRequest[ZaRestore.A_restoreTo] = ZaRestore.TO_PIT;
				restore.restoreRequest[ZaRestore.A_restoreToTime] = new Date(instance.start);
			}
		}
		
		this._newSingleAccountRestoreWizard.setObject(restore);		
		this._newSingleAccountRestoreWizard.popup();
	} catch (ex) {
		this._handleException(ex, "ZaBackupLabelViewController.prototype._restoreMailListener", null, false);
	}
	return;
}
/**
* @method _createUI
**/
ZaBackupLabelViewController.prototype._createUI =
function () {
	this._contentView = this._view = new this.tabConstructor(this._container);

    this._initPopupMenu();
	
	var elements = new Object();
	elements[ZaAppViewMgr.C_APP_CONTENT] = this._view;
    ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
	this._UICreated = true;
	ZaApp.getInstance()._controllers[this.getContentViewId ()] = this ;
}



ZaBackupLabelViewController.prototype._handleException =
function(ex, method, params, restartOnError, obj) {
	if (ex.code && (ex.code == ZmCsfeException.INVALID_BACKUP_TARGET )) 
	{
		DBG.dumpObj(ex);
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_InvalidBackupTargetERROR, ex);
	} else if (ex.code && (ex.code == ZmCsfeException.NO_LABELS_FOUND )) {
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_NoLabelsFoundTitle, ex);
	
	} else {
		ZaController.prototype._handleException.call(this, ex, method, params, restartOnError, obj);
	}
}

ZaBackupLabelViewController.prototype.getAppBarAction = function () {
    if (AjxUtil.isEmpty(this._appbarOperation)) {
        this._appbarOperation[ZaOperation.HELP] = new ZaOperation(
            ZaOperation.HELP,
            ZaMsg.TBB_Help,
            ZaMsg.TBB_Help_tt,
            "Help",
            "Help",
            new AjxListener(
                this,
                this._helpButtonListener
            )
        );

        this._appbarOperation[ZaOperation.CLOSE] = new ZaOperation(
            ZaOperation.CLOSE,
            ZaMsg.TBB_Close,
            ZaMsg.ALTBB_Close_tt,
            "",
            "",
            new AjxListener(
                this,
                this.closeButtonListener
            )
        );
    }

    return this._appbarOperation;
}

ZaBackupLabelViewController.prototype.getAppBarOrder = function () {
    if (AjxUtil.isEmpty(this._appbarOrder)) {
        this._appbarOrder.push(ZaOperation.HELP);
        this._appbarOrder.push(ZaOperation.CLOSE);
    }

    return this._appbarOrder;
}
