/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaBackupLabelXFormView
* @contructor
* @param parent
* @author Greg Solovyev
**/
function ZaBackupLabelXFormView (parent) {
	ZaTabView.call(this, parent,"ZaBackupLabelXFormView","DwtTabView ZaXFormListView", "BACKUP_LABEL_VIEW");
		
	this.initForm(ZaBackupLabel.myXModel,this.getMyXForm());
	this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE, new AjxListener(this, ZaBackupLabelXFormView.prototype.handleXFormChange));
	this._localXForm.setController(ZaApp.getInstance());
}

ZaBackupLabelXFormView.prototype = new ZaTabView();
ZaBackupLabelXFormView.prototype.constructor = ZaBackupLabelXFormView;
ZaTabView.XFormModifiers["ZaBackupLabelXFormView"] = new Array();

ZaBackupLabelXFormView.prototype.handleXFormChange = 
function () {
	var instance = this._localXForm.getInstance();
	if(instance && instance[ZaModel.currentTab]==2) {
		if(ZaApp.getInstance().getCurrentController()) {
		}
	} else if (instance && instance[ZaModel.currentTab]==1) {
		if(ZaApp.getInstance().getCurrentController()) {
		}
	}
}

ZaBackupLabelXFormView.prototype.getTabIcon =
function () {
	return "Backup";
}

ZaBackupLabelXFormView.prototype.getTabTitle =
function () {
	if (this._containedObject) {
		return this._containedObject.label ; //return back label
	}else{
		return this.getTitle();
	}
}

ZaBackupLabelXFormView.prototype.getTabToolTip =
function () {
	if (this._containedObject) {
		return com_zimbra_backuprestore.BNR_tt_tab_backuplabel + " " + this._containedObject.label ;
	}else{
		return "";
	}
}
/**
* @method getObject returns the object contained in the view
* before returning the object this updates the object attributes with 
* tha values from the form fields 
**/
ZaBackupLabelXFormView.prototype.getObject =
function() {
	return this._localXForm.getInstance();
}

ZaBackupLabelXFormView.prototype.setObject = 
function (entry) {
	this._containedObject = entry;
	this._containedObject.id = entry[ZaBackupLabel.A_label]; //use ZaBackupLabel.A_label to distinguish the different label view. Useful when to selectExistingTab

	if(!entry[ZaModel.currentTab])
		this._containedObject[ZaModel.currentTab] = "1";
	else
		this._containedObject[ZaModel.currentTab] = entry[ZaModel.currentTab];
	
	this._containedObject[ZaBackup.A_accountListCount] = ZaSettings.RESULTSPERPAGE;	

	this._localXForm.setInstance(this._containedObject);	
	
	var callback = new AjxCallback(this, ZaBackupLabelXFormView.prototype.queryBackupsCallback, {show:true});
	ZaBackup.queryBackups(this._containedObject, callback);	
	
	var callback = new AjxCallback(this, ZaBackupLabelXFormView.prototype.queryFailedAccCallback, {show:true});
	ZaBackup.queryFailedAccounts(this._containedObject, callback);	

    if (!appNewUI)
	    this.updateTab();
}

ZaBackupLabelXFormView.prototype.getTitle = 
function () {
	return com_zimbra_backuprestore.BNR_view_title;
}

ZaBackupLabelXFormView.accSelectionListener = function (ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			var msg = "";
			if(ev.item[ZaBackupLabel.A_status] == ZaBackupLabel.STATUS_ERROR) {
				msg = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_accountErrorMsg,[ev.item.name]);
				ZaApp.getInstance().getCurrentController().popupErrorDialog(msg,{msg:ev.item[ZaBackupLabel.A_errorMessage],detail:ev.item[ZaBackupLabel.A_content]},null,DwtMessageDialog.INFO_STYLE);
			} else {
				if(ev.item[ZaBackupLabel.A_status] == ZaBackupLabel.STATUS_COMPLETED) {
					msg = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_accountCompletedMsg,[ev.item.name]);
				} else if(ev.item[ZaBackupLabel.A_status] == ZaBackupLabel.STATUS_INPROGRESS) {
					msg = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_accountInProgressMsg,[ev.item.name]);
				} else if(ev.item[ZaBackupLabel.A_status] == ZaBackupLabel.STATUS_NOTSTARTED) {
					msg = AjxMessageFormat.format (com_zimbra_backuprestore.BNR_accountNotStartedMsg,[ev.item.name]);
				}
				ZaApp.getInstance().getCurrentController().popupMsgDialog(msg);
			}
			
		}
	}
}

ZaBackupLabelXFormView.prototype.queryFailedAccCallback = 
function (params, resp) {
	if(!resp) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		return;
	}	
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		return;
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.BackupQueryResponse) {
			response = resp;
		}
		var labelArray = [];
		var backups = null;
		var instance = this._localXForm.getInstance();
		var backupLabel = new ZaBackupLabel();
		
		
		if(response.Body.BackupQueryResponse.backup) {
			backups = response.Body.BackupQueryResponse.backup;
			if(!(backups instanceof Array)) {
				backupLabel.initFromJS(backups);
			} else {			
				backupLabel.initFromJS(backups[0]);
			}
		}
		
		var oldVersion = instance[ZaBackupLabel.A_failedAccounts]._version ? instance[ZaBackupLabel.A_failedAccounts]._version : 1;
		instance[ZaBackupLabel.A_failedAccounts] = backupLabel[ZaBackupLabel.A_accounts];
		instance[ZaBackupLabel.A_failedAccounts]._version = oldVersion+1;
		instance[ZaBackup.A_moreFailedAcc] = backupLabel[ZaBackup.A_more];
		
		if(instance[ZaModel.currentTab]==2) {
			if(ZaApp.getInstance().getCurrentController()) {
			}
		}

        if(instance[ZaBackup.A_more]) {
            var failedAccountPool = this._localXForm.getItemById(this._localXForm.getId() + "_failed_accounts_list").getWidget();
            failedAccountPool.setCrtInstance(instance, true);
            failedAccountPool.setScrollHasMore(true);
        }
		this._localXForm.setInstance(instance);
	}	
}

ZaBackupLabelXFormView.prototype.queryBackupsCallback = 
function (params, resp) {
	if(!resp) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		return;
	}	
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupLabelXFormView.prototype.queryBackupsCallback");
		return;
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.BackupQueryResponse) {
			response = resp;
		}
		var labelArray = [];
		var backups = null;
		var instance = new ZaBackupLabel();
		var oldInstance = this._localXForm.getInstance();
        if(oldInstance[ZaBackupLabel.A_accounts])
		instance[ZaBackupLabel.A_accounts]._version = oldInstance[ZaBackupLabel.A_accounts]._version+1;
        else
            instance[ZaBackupLabel.A_accounts]._version = 0;
		
		if(response.Body.BackupQueryResponse.backup) {
			backups = response.Body.BackupQueryResponse.backup;
			if(!(backups instanceof Array)) {
				instance.initFromJS(backups);
			} else {			
				instance.initFromJS(backups[0]);
			}
		}

		if(oldInstance[ZaBackup.A_target])
		    instance[ZaBackup.A_target] = oldInstance[ZaBackup.A_target];
        if(oldInstance[ZaBackup.A_serverName])
		    instance[ZaBackup.A_serverName] = oldInstance[ZaBackup.A_serverName];
        if(oldInstance[ZaBackup.A_serverId])
            instance[ZaBackup.A_serverId] = oldInstance[ZaBackup.A_serverId];
		instance[ZaBackup.A_backupListOffset] = null;
		instance[ZaBackup.A_backupListCount] = null;	
		instance[ZaModel.currentTab] = oldInstance[ZaModel.currentTab];
        if(oldInstance[ZaBackup.A_accountListOffset])
		    instance[ZaBackup.A_accountListOffset] = oldInstance[ZaBackup.A_accountListOffset];
        if(oldInstance[ZaBackup.A_accountListCount])
		    instance[ZaBackup.A_accountListCount] = oldInstance[ZaBackup.A_accountListCount];
        if(oldInstance[ZaBackup.A_pageNumber])
		    instance[ZaBackup.A_pageNumber] = oldInstance[ZaBackup.A_pageNumber];
		if(instance[ZaModel.currentTab]==1) {
			if(ZaApp.getInstance().getCurrentController()) {
				
			}
		}

        if(instance[ZaBackup.A_more]) {
            var accountPool = this._localXForm.getItemById(this._localXForm.getId() + "_accounts_list").getWidget();
            accountPool.setCrtInstance(instance, false);
            accountPool.setScrollHasMore(true);
        }
		this._localXForm.setInstance(instance);
	}	
}


ZaBackupLabelXFormView.getCustomWidth = function () {
    try {
        var parentPage = this.getForm().parent.getHtmlElement();
        var totalWidth = parseInt(parentPage.style.width);
        if (isNaN(totalWidth)) {
            totalWidth =  parentPage.clientWidth? parentPage.clientWidth : parentPage.offsetWidth;
        }
        return totalWidth;
    } catch(ex) {

    }
    return "100%";
}

ZaBackupLabelXFormView.getCustomHeight = function () {
    var parentPage = this.getForm().parent;
    try {
        var totalHeight =  parentPage.getSize().y;
        var formHeaders = this.getForm().getItemsById("xform_header");
        var headHeight = 0;
        if (formHeaders) {
            var formHeader = formHeaders[0];
            if (formHeader.getContainer()) {
                formHeader = formHeader.getContainer();
            } else {
                formHeader = formHeader.getHtmlElement();
            }
            headHeight =  Dwt.getSize(formHeader).y;
        }

        var tabBarHeaders = this.getForm().getItemsById("xform_tabbar");
        var tabBarHeight = 0;
        if (tabBarHeaders) {
            var tabBarHeader = tabBarHeaders[0];
            if (tabBarHeader.getContainer()) {
                tabBarHeader = tabBarHeader.getContainer();
            } else {
                tabBarHeader = tabBarHeader.getHtmlElement();
            }
            tabBarHeight =  Dwt.getSize(tabBarHeader).y;
        }
        var containerHeight = 2;

        if (AjxEnv.isIE)
            containerHeight = 3;
        if (totalHeight > (headHeight+containerHeight + tabBarHeight)) {
            return totalHeight - headHeight -containerHeight - tabBarHeight ;
        }

    } catch(ex) {

    }
    return "100%";
}

ZaBackupLabelXFormView.myXFormModifier = function(xFormObject) {	
	
	var allAccHeaderList = new Array();
	var sortable = 1;
	allAccHeaderList[0] = new ZaListHeaderItem(ZaAccount.A_name, ZaMsg.ALV_NameAddress_col, null, "250", null, null, true, true);
	allAccHeaderList[1] = new ZaListHeaderItem(ZaBackupLabel.A_status, com_zimbra_backuprestore.BNR_Status_col, null, "250", null, null, true, true);		
	allAccHeaderList[2] = new ZaListHeaderItem(ZaBackupLabel.A_errorMessage, com_zimbra_backuprestore.BNR_ErrorMsg_col, null, "auto", null, null, true, true);		
	allAccHeaderList[3] = new ZaListHeaderItem(null, null, null, 5, null, null, false, true);

	
	xFormObject.tableCssStyle="width:100%;overflow:auto;";
	xFormObject.numCols=1;
	xFormObject.items = [
		{type:_GROUP_, cssClass:"ZmSelectedHeaderBg", colSpan: "2", id:"xform_header",
			items: [
				{type:_GROUP_,	numCols:4,colSizes:["32px","350px","100px","*"],
					items: [
						{type:_AJX_IMAGE_, src:"Server_32", label:null, rowSpan:2, visibilityChecks:[]},
						{type:_OUTPUT_, ref:ZaBackup.A_serverName, label:null,cssClass:"AdminTitle", rowSpan:2, visibilityChecks:[]},				
						{type:_OUTPUT_, ref:ZaBackupLabel.A_label, label:com_zimbra_backuprestore.BNR_Label, visibilityChecks:[]},
						{type:_OUTPUT_, ref:ZaBackupLabel.A_status, label:com_zimbra_backuprestore.BNR_Status, visibilityChecks:[]}
					]
				}
			],
			cssStyle:"padding-top:5px; padding-bottom:5px"
		},	    
		{type:_TAB_BAR_,  ref:ZaModel.currentTab,  colSpan:"2",
			choices:[
				{value:1, label:com_zimbra_backuprestore.BNR_TABT_Accounts},
				{value:2, label:com_zimbra_backuprestore.BNR_TABT_Errors},
				{value:3, label:com_zimbra_backuprestore.BNR_TABT_FailedAccounts}				
				
			],cssClass:"ZaTabBar", id:"xform_tabbar"
		},
		{type:_SWITCH_, 
			items:[
				{type:_ZATABCASE_, caseKey:1, numCols:1, width:"100%",
                    paddingStyle:"padding:0px", getCustomPaddingStyle:"return 0",
					items:[					
					    {ref:ZaBackupLabel.A_accounts, cssClass: "ZaFullPageXFormListView",
					    	onSelection:ZaBackupLabelXFormView.accSelectionListener, type:_DWT_LIST_, 
					   		forceUpdate: true,createPopupMenu:ZaBackupsXFormView.createPopupMenu,hideHeader:false,
					   		multiselect:true, widgetClass:ZaBNRAccMiniListView,headerList:allAccHeaderList,id:"accounts_list",
					   		visibilityChecks:[],enableDisableChecks:[],
                            getCustomWidth:ZaBackupLabelXFormView.getCustomWidth,
                            getCustomHeight:ZaBackupLabelXFormView.getCustomHeight
					   	}			
				   	]
				},
				{type:_ZATABCASE_, caseKey:2, numCols:1, 
					items:[					
					    {type:_SPACER_},
					    {type:_REPEAT_,showRemoveButton:false,showAddButton:false,
					    	ref:ZaBackupLabel.A_error,
					    	visibilityChecks:[[XForm.checkInstanceValueNotEmty,ZaBackupLabel.A_error]],
					    	visibilityChangeEventSources:[ZaBackupLabel.A_error],
					    	items: [ 	
								{ type: _DWT_ALERT_,
								  style: DwtAlert.CRITICAL,
								  iconVisible: true, 
								  content: null,width:"400px",
								  ref:".",
								  getDisplayValue:function(itemval) {
								  	if(itemval) {
								  		return [itemval[ZaBackupLabel.A_errorMessage],itemval[ZaBackupLabel.A_content]].join("<br/>");
								  	} else {
								  		return "";
								  	}
								  },
								  align:_CENTER_
								}
							]
					    },						
						{type:_DWT_ALERT_,style: DwtAlert.INFORMATION, 
							iconVisible: true,
							content:com_zimbra_backuprestore.BNR_AccNoErrors,width:"400px",
							ref:null,
					    	visibilityChecks:[[XForm.checkInstanceValueEmty,ZaBackupLabel.A_error]],
					    	visibilityChangeEventSources:[ZaBackupLabel.A_error],
							align:_CENTER_
						}
					]
				},
				{type:_ZATABCASE_, caseKey:3, numCols:1, width:"100%",
                    paddingStyle:"padding:0px", getCustomPaddingStyle:"return 0",
					items:[					
					    {ref:ZaBackupLabel.A_failedAccounts, cssClass: "ZaFullPageXFormListView",
					    	visibilityChecks:[],enableDisableChecks:[],
					    	onSelection:ZaBackupLabelXFormView.accSelectionListener, type:_DWT_LIST_,
					   		forceUpdate: true,createPopupMenu:ZaBackupsXFormView.createPopupMenu,hideHeader:false,
					   		multiselect:true, widgetClass:ZaBNRAccMiniListView,headerList:allAccHeaderList,id:"failed_accounts_list",
                            getCustomWidth:ZaBackupLabelXFormView.getCustomWidth,
                            getCustomHeight:ZaBackupLabelXFormView.getCustomHeight
					   	}			
				   	]
				}
			]
		}
	];
};
ZaTabView.XFormModifiers["ZaBackupLabelXFormView"].push(ZaBackupLabelXFormView.myXFormModifier);