/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaBackupListView
* @param parent
* @author Greg Solovyev
**/

function ZaBackupListView(parent, className, posStyle, headerList) {
    posStyle = DwtControl.ABSOLUTE_STYLE;  // Only absolute style is allowed here
	ZaListView.call(this, parent, className, posStyle, headerList, undefined, "ZaBackupListView", true);
	this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
}

ZaBackupListView.prototype = new ZaListView;

ZaBackupListView.prototype.toString = 
function() {
	return "ZaBackupListView";
}

ZaBackupListView.prototype.getTitle = 
function () {
	return com_zimbra_backuprestore.BNR_view_title;
}
/**
* Renders a single item as a DIV element.
*/
ZaBackupListView.prototype._createItemHtml =
function(obj, now, isDndIcon) {
	var html = new Array(50);
	var	div = document.createElement("DIV");
	div.id = Dwt.getNextId();//Row-matched
	var rowClassName = AjxBuffer.concat(this._className, "Row");	
	if(obj[ZaBackupLabel.A_errors] && obj[ZaBackupLabel.A_errors].length>0) {
		div[DwtListView._STYLE_CLASS] = AjxBuffer.concat("Row ", "-error");		
	} else {
		div[DwtListView._STYLE_CLASS] = AjxBuffer.concat("Row ", rowClassName);
	}
	div[DwtListView._SELECTED_STYLE_CLASS] = AjxBuffer.concat("Row-", DwtCssStyle.SELECTED, " ", rowClassName);
	div[DwtListView._SELECTED_DIS_STYLE_CLASS] = AjxBuffer.concat("Row-", DwtCssStyle.SELECTED, "-" , DwtCssStyle.DISABLED, " ", rowClassName);
	div.className = div[DwtListView._STYLE_CLASS];
	this.associateItemWithElement(obj, div, DwtListView.TYPE_LIST_ITEM);
	
	var idx = 0;
	html[idx++] = "<table width='100%' cellspacing='0' cellpadding='0'>";
	html[idx++] = "<tr>";
	var cnt = this._headerList.length;

    var dwtId = Dwt.getNextId();
    var rowId = "backup_list";

	for(var i = 0; i < cnt; i++) {
		var id = this._headerList[i]._field;
		if(id!=null) {
			if(id.indexOf(ZaBackupLabel.A_label)==0) {
				// name
                html[idx++] = "<td id=\"" + rowId + "_data_label_" + dwtId + "\" align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width;
				html[idx++] = "'>";
				html[idx++] = AjxStringUtil.htmlEncode(obj[ZaBackupLabel.A_label]);
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaBackupLabel.A_start)==0) {
				// description		
				html[idx++] = "<td id=\"" + rowId + "_data_start_" + dwtId + "\" align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				var date = new Date(parseInt(obj[ZaBackupLabel.A_start]));
				html[idx++] = AjxDateUtil.computeDateTimeString(date);
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaBackupLabel.A_end)==0) {
				// description		
				html[idx++] = "<td id=\"" + rowId + "_data_end_" + dwtId + "\" align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				if(obj[ZaBackupLabel.A_end]) {
					var date = new Date(parseInt(obj[ZaBackupLabel.A_end]));
					html[idx++] = AjxDateUtil.computeDateTimeString(date);
				} else if (obj[ZaBackupLabel.A_live])
					html[idx++] = com_zimbra_backuprestore.BNR_Status_Running;
				else
					html[idx++] = "&nbsp;";	
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaBackupLabel.A_status)==0) {
				// description		
				html[idx++] = "<td id=\"" + rowId + "_data_status_" + dwtId + "\" align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				html[idx++] = AjxStringUtil.htmlEncode(obj[ZaBackupLabel.A_status]); 
				html[idx++] = "</td>";
			}
		}
	}
	html[idx++] = "</tr></table>";
	div.innerHTML = html.join("");
	return div;
}

ZaBackupListView.prototype._setNoResultsHtml = function() {
	var buffer = new AjxBuffer();
	var	div = document.createElement("div");
	var text = com_zimbra_backuprestore.BNR_NOLabelsfound;
	if(this.parent && this.parent.instance) {
		text = AjxMessageFormat.format(com_zimbra_backuprestore.BNR_NOLabelsfoundIn, this.parent.instance[ZaBackup.A_target]);
	}
	buffer.append("<table width='100%' cellspacing='0' cellpadding='1'>","<tr><td class='NoResults'><br>",text,"</td></tr></table>");
	
	div.innerHTML = buffer.toString();
	this._addRow(div);
};

ZaBackupListView.prototype.setCrtInstance = function (instance) {
    this._crtInstance = ZaUtil.deepCloneObject(instance);
}

ZaBackupListView.prototype._loadMsg = function(params) {

    var updateCallback = new AjxCallback(this, this.updateMoreItems);
	this._crtInstance[ZaBackup.A_backupListOffset]= this._crtInstance[ZaBackup.A_backupListOffset] + this._crtInstance[ZaBackup.A_backupListCount];
	this._crtInstance[ZaBackup.A_pageNumber] = this._crtInstance[ZaBackup.A_pageNumber] ++;
    ZaBackup.queryBackups(this._crtInstance, updateCallback);
}

ZaBackupListView.prototype.updateMoreItems = function(resp) {
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupListView.prototype.updateMoreItems");
		return;
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.BackupQueryResponse) {
			response = resp;
		}
		var labelArray = [];
		var backup = null;


		if(response.Body.BackupQueryResponse.backup) {
			backup = response.Body.BackupQueryResponse.backup;
			if(!(backup instanceof Array)) {
				backup = [backup];
			}
			var cnt = backup.length;
			for(var i =0; i < cnt; i++) {
				var bckpLabel = new ZaBackupLabel();
				bckpLabel.initFromJS(backup[i]);
				bckpLabel[ZaBackup.A_target] = this._crtInstance[ZaBackup.A_target];
				bckpLabel[ZaBackup.A_serverId] = this._crtInstance[ZaBackup.A_serverId];
				bckpLabel[ZaBackup.A_serverName] = this._crtInstance[ZaBackup.A_serverName];
				labelArray.push(bckpLabel);
			}
            this.replenish(AjxVector.fromArray(labelArray));
		}

        if (response.Body.BackupQueryResponse[ZaBackup.A_more]) {
            var backupPool = this._localXForm.getItemById(this._localXForm.getId() + "_backupList").getWidget();
            backupPool.setScrollHasMore(true);
        }
	}
}