/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaBackupServerListController
**/
function ZaBackupServerListController(appCtxt, container) {
	ZaController.call(this, appCtxt, container,"ZaBackupServerListController");
   	this._popupOperations = new Array();			
	this.ServerPool = [];
	this._helpURL = location.pathname + "help/admin/html/backup_feature/backing_up_the_zimbra_server.htm?locid=" + AjxEnv.DEFAULT_LOCALE;					
}

ZaBackupServerListController.prototype = new ZaController();
ZaBackupServerListController.prototype.constructor = ZaBackupServerListController;

ZaController.initPopupMenuMethods["ZaBackupServerListController"] = new Array();

ZaBackupServerListController.prototype.show = 
function(list, openInNewTab) {

    if (!this._UICreated) {
		this._createUI();
	} 	

	if (list != null) {
		this._contentView.set(list.getVector());
		//start loading queue info
		this.ServerPool = [];
		var tmpList = list.getArray();
		var numServers = tmpList.length;

		for(var ix = 0; ix < numServers; ix++) {
			this.ServerPool.push(tmpList[ix]);
		}
		
		var i=0;
		var tmp = [];
		var cnt = numServers > 5 ? 5 : numServers;
		for(i = 0; i < cnt; i++) {
			tmp[i] = this.ServerPool.shift();
		}
		ZaMTA._quecountsArr = new Array();
		for(i = cnt-1; i >= 0; i--) {
			tmp[i].loadBackups();
		}
	}	
//	ZaApp.getInstance().pushView(ZaZimbraAdmin._BACKUPS_SERVER_LIST);			
	ZaApp.getInstance().pushView(this.getContentViewId());
	
	this._removeList = new Array();
	if (list != null)
		this._list = list;
		
	this.changeActionsState();		
	
	/*
	if (openInNewTab) {//when a ctrl shortcut is pressed
		
	}else{ //open in the main tab
		this.updateMainTab ("Backup") ;	
	}*/
}


ZaBackupServerListController.initPopupMenuMethod =
function () {
    if (appNewUI) {
        this._popupOperations.push(new ZaOperation(ZaOperation.VIEW, ZaMsg.TBB_View, ZaMsg.PQTBB_View_tt, "Properties", "PropertiesDis", new AjxListener(this, ZaBackupServerListController.prototype._viewButtonListener)));
        this._popupOperations.push(new ZaOperation(ZaOperation.IMMEDIATE_BACKUP, com_zimbra_backuprestore.BNRTBB_Backup, com_zimbra_backuprestore.BNRTBB_Backup_tt, "Backup", "Backup", new AjxListener(this, ZaBackupServerListController.prototype._immediateBackupListener)));
        this._popupOperations.push(new ZaOperation(ZaOperation.MAIL_RESTORE, com_zimbra_backuprestore.BNR_TBB_RestoreMailbox, com_zimbra_backuprestore.BNR_TBB_Restore_tt, "RestoreMailbox", "RestoreMailboxDis", new AjxListener(this, ZaHotBackup._restoreMailListener)));
        this._popupOperations.push(new ZaOperation(ZaOperation.CONFIGURE_BACKUP, com_zimbra_backuprestore.BNRTBB_ConfigureBackup, com_zimbra_backuprestore.BNRTBB_ConfigureBackup_tt, "AdvancedTools", "Configure", new AjxListener(this, ZaHotBackup.configureBackup)));
        this._popupOperations.push(new ZaOperation(ZaOperation.REFRESH, ZaMsg.TBB_Refresh, ZaMsg.TBB_Refresh_tt, "Refresh", "Refresh", new AjxListener(this, this.refreshListener)));
    } else {
        this._popupOperations.push(new ZaOperation(ZaOperation.VIEW, ZaMsg.TBB_View, ZaMsg.PQTBB_View_tt, "Properties", "PropertiesDis", new AjxListener(this, ZaBackupServerListController.prototype._viewButtonListener)));
    }
}
ZaController.initPopupMenuMethods["ZaBackupServerListController"].push(ZaBackupServerListController.initPopupMenuMethod);

ZaBackupServerListController.prototype._createUI = function () {
	try {
		var elements = new Object();
		this._contentView = new ZaBackupServerListView(this._container);
		this._initPopupMenu();
		if(this._popupOperations && this._popupOperations.length) {
			this._actionMenu =  new ZaPopupMenu(this._contentView, "ActionMenu", null, this._popupOperations);
		}
		elements[ZaAppViewMgr.C_APP_CONTENT] = this._contentView;
		//ZaApp.getInstance().createView(ZaZimbraAdmin._BACKUPS_SERVER_LIST, elements);
        ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
		this._contentView.addSelectionListener(new AjxListener(this, this._listSelectionListener));
		this._contentView.addActionListener(new AjxListener(this, this._listActionListener));			
	
		this._UICreated = true;
		ZaApp.getInstance()._controllers[this.getContentViewId ()] = this ;
		
	} catch (ex) {
		this._handleException(ex, "ZaBackupServerListController.prototype._createUI", null, false);
		return;
	}	
}

/**
* @return ZaItemList - the list currently displaid in the list view
**/
ZaBackupServerListController.prototype.getList = 
function() {
	return this._list;
}

/*
ZaBackupServerListController.prototype.refresh = 
function() {
	try {
		this._contentView.set(ZaApp.getInstance().getServerList(true).getVector());
	} catch (ex) {
		this._handleException(ex, ZaBackupServerListController.prototype.refresh, null, false);
	}
}
*/

ZaBackupServerListController.prototype.set = 
function(serverList) {
	this.show(serverList);
}

/**
* This listener is called when the item in the list is double clicked. It call ZaMTAController.show method
* in order to display the MailQ View
**/
ZaBackupServerListController.prototype._listSelectionListener =
function(ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			this._selectedItem = ev.item;
			ZaApp.getInstance().getBackupsViewController().show(ev.item);
		}
	} else {
		this.changeActionsState();	
	}
}

ZaBackupServerListController.prototype._immediateBackupListener = 
function (ev) {
	try {
		this._ImmediateBackupWizard = new ZaBackupXWizard(this._container);		
		var backup = new ZaBackup();
		backup[ZaBackup.A_method]=ZaBackup.METHOD_FULL;
		backup[ZaBackup.A_target] = ZaHotBackup.DEFAULT_BACKUP_TARGET;
		backup[ZaServerBackup.A_backupAll] = 0;
		var server = null;
		try {
			if(this._contentView.getSelectionCount() == 1) {
				server = this._contentView.getSelection()[0];//[ZaServer.A_ServiceHostname];
			} else {
				server = ZaApp.getInstance().getMailServers()[0];//[ZaServer.A_ServiceHostname];
			}
			if(server) {
				backup[ZaBackup.A_serverName] = server[ZaServer.A_ServiceHostname];
				if(server.attrs[ZaGlobalConfig.A_zimbraBackupTarget])
					backup[ZaBackup.A_target] = AjxStringUtil.trim(server.attrs[ZaGlobalConfig.A_zimbraBackupTarget]);
			}
		} catch (ex) {
			//smth happened but it is not significant enough to interrupt the wizard
			this._ImmediateBackupWizard.setObject(backup);		
			this._ImmediateBackupWizard.popup();
			throw(ex);
		}
		this._ImmediateBackupWizard.setObject(backup);		
		this._ImmediateBackupWizard.popup();
	} catch (ex) {
		this._handleException(ex, "ZaBackupServerListController.prototype._immediateBackupListener", null, false);
	}
	return;
}

ZaBackupServerListController.prototype._listActionListener =
function (ev) {
	this.changeActionsState();
	this._actionMenu.popup(0, ev.docX, ev.docY);
}
/**
* This listener is called when the Edit button is clicked. 
* It call ZaMTAController.show method
* in order to display the MailQ View
**/
ZaBackupServerListController.prototype._viewButtonListener =
function(ev) {
	if(this._contentView.getSelectionCount() == 1) {
		var item = this._contentView.getSelection()[0];
		ZaApp.getInstance().getBackupsViewController().show(item);
	}
}


ZaBackupServerListController.prototype.changeActionsState = 
function () {
	var cnt = this._contentView.getSelectionCount();
	if(cnt == 1) {
		var opsArray = [ZaOperation.EDIT];
		this._actionMenu.enable(opsArray, true);
	} else if (cnt > 1){
		var opsArray1 = [ZaOperation.EDIT];
		this._actionMenu.enable(opsArray1, false);
	} else {
		var opsArray = [ZaOperation.EDIT];
		this._actionMenu.enable(opsArray, false);
	}
}

/**
* @param ev
* This listener is invoked by ZaMTAController or any other controller that can change a ZaMTA object
**/
ZaBackupServerListController.prototype.handleServerChange = 
function (obj) {
	//if any of the data that is currently visible has changed - update the view
	if(obj && this._contentView) {
		this._contentView.setUI();
		//check if we have any Servers in the queue waiting to be loaded
		if(this.ServerPool.length) {
			var server = this.ServerPool.shift();
			server.loadBackups();
		}
	}
}

/**
* Asynchronously calls ZaItem.load {@link ZaItem#load}
**/
ZaBackupServerListController.prototype.getBackupStatus = function () {
	this.ServerPool = [];
	var tmpList = this._list.getArray();
	var numServers = tmpList.length;
	for(var ix = 0; ix < numServers; ix++) {
		this.ServerPool.push(tmpList[ix]);
	}
	var cnt = numServers > 5 ? 5 : numServers;
	var i=0;
	var tmp = [];
	for(i = 0; i < cnt; i++) {
		tmp[i] = this.ServerPool.shift();
	}
	for(i = cnt-1; i >= 0; i--) {
		tmp[i].loadBackups();
	}
}

ZaBackupServerListController.prototype.refresh = function () {
	this.getBackupStatus();
}

ZaBackupServerListController.prototype.refreshListener = 
function () {
	this.getBackupStatus();
}

ZaBackupServerListController.prototype.changeActionsState = 
function () {
	var cnt = this._contentView.getSelectionCount();
	if(cnt == 1) {
		var opsArray = [ZaOperation.VIEW];
		this._actionMenu.enable(opsArray, true);
	} else if (cnt > 1){
		var opsArray1 = [ZaOperation.VIEW];
		this._actionMenu.enable(opsArray1, false);
	} else {
		var opsArray = [ZaOperation.VIEW];
		this._actionMenu.enable(opsArray, false);
	}
}

ZaBackupServerListController.prototype._handleException =
function(ex, method, params, restartOnError, obj) {
	if (ex.code && (ex.code == ZmCsfeException.INVALID_BACKUP_TARGET )) 
	{
		DBG.dumpObj(ex);
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_InvalidBackupTargetERROR, ex);
	} else if (ex.code && (ex.code == ZmCsfeException.NO_LABELS_FOUND )) {
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_NoLabelsFoundTitle, ex);
	
	} else {
		ZaController.prototype._handleException.call(this, ex, method, params, restartOnError, obj);
	}
}

ZaBackupServerListController.prototype.getAppBarAction = function () {
    if (AjxUtil.isEmpty(this._appbarOperation)) {
        this._appbarOperation[ZaOperation.HELP] = new ZaOperation(
            ZaOperation.HELP,
            ZaMsg.TBB_Help,
            ZaMsg.TBB_Help_tt,
            "Help",
            "Help",
            new AjxListener(
                this,
                this._helpButtonListener
            )
        );
    }

    return this._appbarOperation;
}

ZaBackupServerListController.prototype.getAppBarOrder = function () {
    if (AjxUtil.isEmpty(this._appbarOrder)) {
        this._appbarOrder.push(ZaOperation.HELP);
    }

    return this._appbarOrder;
}
