/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaBackupServerListView
* @param parent
* @author Greg Solovyev
**/

function ZaBackupServerListView(parent) {

	var className = null;
	var posStyle = DwtControl.ABSOLUTE_STYLE;
	
	var headerList = this._getHeaderList();
	
	ZaListView.call(this, parent, className, posStyle, headerList);

	this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
	
}

ZaBackupServerListView.prototype = new ZaListView;
ZaBackupServerListView.prototype.constructor = ZaBackupServerListView;

ZaBackupServerListView.prototype.toString = 
function() {
	return "ZaBackupServerListView";
}

ZaBackupServerListView.prototype.getTitle = 
function () {
	return com_zimbra_backuprestore.BNR_view_title;
}

ZaBackupServerListView.prototype.getTabIcon = 
function () {
	return "Backup";
}
/**
* Renders a single item as a DIV element.
*/
ZaBackupServerListView.prototype._createItemHtml =
function(server, now, isDndIcon) {
	var html = new Array(50);
	var	div = document.createElement("div");
	div[DwtListView._STYLE_CLASS] = "Row";
	div[DwtListView._SELECTED_STYLE_CLASS] = div[DwtListView._STYLE_CLASS] + "-" + DwtCssStyle.SELECTED;
	div.className = div[DwtListView._STYLE_CLASS];
	this.associateItemWithElement(server, div, DwtListView.TYPE_LIST_ITEM);
	var itemId = this._getItemId(server);
	var idx = 0;
	html[idx++] = "<table width='100%' cellspacing='0' cellpadding='0'>";
	html[idx++] = "<tr>";
	var cnt = this._headerList.length;

    var dwtId = Dwt.getNextId();
    var rowId = "backup_server_list";

	for(var i = 0; i < cnt; i++) {
		var id = this._headerList[i]._field;
		if(id!=null) {
			if(id.indexOf(ZaServer.A_ServiceHostname)==0) {	
				// name
				html[idx++] = "<td id=\"" + rowId + "_data_host_" + dwtId + "\" width=" + this._headerList[i]._width + ">";
				html[idx++] = AjxStringUtil.htmlEncode(server.attrs[ZaServer.A_ServiceHostname]);
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaServerBackup.A_freeSpace)==0){
				html[idx++] = "<td id=\"" + rowId + "_data_free_space_" + dwtId + "\" width=" + this._headerList[i]._width + ">";
 				if(server[ZaServerBackup.A_freeSpace])
                                	html[idx++] = AjxStringUtil.htmlEncode(String(server[ZaServerBackup.A_freeSpace]));
                                else
					html[idx++] = "&nbsp;";
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaServerBackup.A_totalSpace)==0){
                                html[idx++] = "<td id=\"" + rowId + "_data_total_space_" + dwtId + "\" width=" + this._headerList[i]._width + ">";
                                if(server[ZaServerBackup.A_totalSpace])
                                        html[idx++] = AjxStringUtil.htmlEncode(String(server[ZaServerBackup.A_totalSpace]));
                                else
                                        html[idx++] = "&nbsp;";
                                html[idx++] = "</td>";
                        } else if(id.indexOf(ZaServerBackup.A_latestLabel)==0) {	
				html[idx++] = "<td id=\"" + rowId + "_data_latest_label_" + dwtId + "\" width=" + this._headerList[i]._width + ">";
				if(server[ZaServerBackup.A_latestLabel])
					html[idx++] = AjxStringUtil.htmlEncode(server[ZaServerBackup.A_latestLabel][ZaBackupLabel.A_label]);
				else
					html[idx++] = "&nbsp;";	
				html[idx++] = "</td>";			
			} else if(id.indexOf(ZaServerBackup.A_latestLabelStatus)==0) {	
				html[idx++] = "<td id=\"" + rowId + "_data_latest_label_status_" + dwtId + "\" width=" + this._headerList[i]._width + ">";
				if(server[ZaServerBackup.A_latestLabel])
					html[idx++] = AjxStringUtil.htmlEncode(server[ZaServerBackup.A_latestLabel][ZaBackupLabel.A_status]);
				else
					html[idx++] = "&nbsp;";
				html[idx++] = "</td>";			
			}
		}
	}
	html[idx++] = "</tr></table>";
	div.innerHTML = html.join("");
	return div;
}

ZaBackupServerListView.prototype._getHeaderList =
function() {

	var headerList = new Array();
//idPrefix, label, iconInfo, width, sortable, sortField, resizeable, visible

	headerList[0] = new ZaListHeaderItem(ZaServer.A_ServiceHostname, com_zimbra_backuprestore.BNR_Server_col, null, 195, null, null, true, true);
        
        headerList[1] = new ZaListHeaderItem(ZaServerBackup.A_freeSpace, com_zimbra_backuprestore.BNR_FreeSpace_col, null, 195, null, null, true, true);

	headerList[2] = new ZaListHeaderItem(ZaServerBackup.A_totalSpace, com_zimbra_backuprestore.BNR_TotalSpace_col, null, 195, null, null, true, true);

	headerList[3] = new ZaListHeaderItem(ZaServerBackup.A_latestLabel, com_zimbra_backuprestore.BNR_LatestLabel_col, null, 195, null, null, true, true);

	headerList[4] = new ZaListHeaderItem(ZaServerBackup.A_latestLabelStatus, com_zimbra_backuprestore.BNR_Status_col, null, "auto", null, null, true, true);		
	
	return headerList;
}


