/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
* @class ZaBackupXWizard
* @contructor ZaBackupXWizard
* @param ZaApp parent
* this is the wizard dialog for making an immediate backup
* @author Greg Solovyev
**/
function ZaBackupXWizard(parent) {
	ZaXWizardDialog.call(this, parent, null, com_zimbra_backuprestore.BNR_BackupWizardTitle, "550px", "400px","ZaBackupXWizard");
	this.stepChoices = [
		{label:com_zimbra_backuprestore.BNR_SelectDestinationStep_Title, value:1},
		{label:com_zimbra_backuprestore.BNR_BackupSelectAccountsStep_Title, value:2},
		{label:com_zimbra_backuprestore.BNR_BackupBackingUp_Title, value:3}
	];	
	this.backupMethodChoices = [
		{value:ZaBackup.METHOD_FULL, label:com_zimbra_backuprestore.BNR_BackupMethodFull}, 
		{value:ZaBackup.METHOD_INCREMENTAL, label:com_zimbra_backuprestore.BNR_BackupMethodIncremental}
	];	

	this.initForm(ZaServerBackup.myXModel,this.getMyXForm());	
	this._helpURL = ZaBackupXWizard.helpURL;		
}
ZaBackupXWizard.helpURL = location.pathname + "help/admin/html/backup_feature/starting_a_backup_session.htm?locid=" + AjxEnv.DEFAULT_LOCALE;
ZaBackupXWizard.prototype = new ZaXWizardDialog;
ZaBackupXWizard.prototype.constructor = ZaBackupXWizard;
ZaBackupXWizard.prototype.miniType = 2;
ZaXDialog.XFormModifiers["ZaBackupXWizard"] = new Array();

ZaBackupXWizard.prototype.handleXFormChange =
function () {

       if (this._containedObject[ZaModel.currentStep] == 2) {
           this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
           if((this._containedObject[ZaBackup.A_Accounts] && this._containedObject[ZaBackup.A_Accounts].length>0) || this._containedObject[ZaServerBackup.A_backupAll]==1)
				this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
			else
				this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
           this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
       }
       if (this._containedObject[ZaModel.currentStep] == 3) {
           this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
           this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
           this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
       }
}


/**
* @method setObject sets the object contained in the view
* @param entry - ZaRestore object to display
**/
ZaBackupXWizard.prototype.setObject =
function(entry) {
	this._containedObject = entry;
	this._containedObject[ZaModel.currentStep] = entry[ZaModel.currentStep] || 1;
	this._containedObject["activateSearch"] = entry["activateSearch"] || 1;
	this._containedObject["poolNumPages"] = entry["poolNumPages"] || 1;
	this._containedObject["poolPagenum"] = entry["poolPagenum"] || 0;

    if(entry._uuid) {
        this._containedObject._uuid = entry._uuid;
    }
	this._localXForm.setInstance(this._containedObject);		
}
/**
* Overwritten methods that control wizard's flow (open, go next,go previous, finish)
**/
ZaBackupXWizard.prototype.popup = 
function (loc) {
	ZaXWizardDialog.prototype.popup.call(this, loc);
	this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
	this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
	this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);	
}

ZaBackupXWizard.prototype.goNext = 
function() {
	var instance = this._localXForm.getInstance();
	if (instance[ZaModel.currentStep] == 2 ||
		(instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL && instance[ZaModel.currentStep] == 1 )
	) {
		var server = ZaApp.getInstance().getServerByName(instance[ZaBackup.A_serverName]);
		//check server backup mode
		var isAutoGroup=false;
		if (
				(server.attrs[ZaServer.A_zimbraBackupMode]==ZaHotBackup.BACKUP_MODE_AUTOGROUP) || 
				(!server.attrs[ZaServer.A_zimbraBackupMode] && ZaApp.getInstance().getGlobalConfig().attrs[ZaGlobalConfig.A_zimbraBackupMode]==ZaHotBackup.BACKUP_MODE_AUTOGROUP) 
			)
		{	
			isAutoGroup = true;
		
			if(instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL) {
				ZaApp.getInstance().getCurrentController().popupErrorDialog(com_zimbra_backuprestore.ERROR_NO_INCREMENTALS_IN_AUTOGROUPED_MODE);
				return;
			}
		}
		
		this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);	
		this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(true);
		
		var accounts = [];
		if((instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL)) {
				//
		} else if (instance[ZaServerBackup.A_backupAll]==1) {
			accounts = ["all"];
		} else {
			accounts = instance[ZaBackup.A_Accounts];
		}
		if(instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL && this._containedObject[ZaModel.currentStep] == 1 ) {
			this._containedObject[ZaModel.currentStep]++;
			instance.doneMsg = com_zimbra_backuprestore.BNR_BackupStarting;
		}

        ZaBackup.runBackupQuery(server.id,instance[ZaBackup.A_method], (isAutoGroup ? null : instance[ZaBackup.A_target]),
        	accounts,null,new AjxCallback(this,ZaBackupXWizard.prototype.backupQueryCallback));		
		
	} else if (instance[ZaModel.currentStep] == 1) {
		//check the path
		if(this.checkPath(instance)) {
			this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(true);
			this._button[DwtWizardDialog.NEXT_BUTTON].setText(com_zimbra_backuprestore.BNR_BtnRunbackup);		
	
			if((instance[ZaBackup.A_Accounts] && instance[ZaBackup.A_Accounts].length>0) || instance[ZaServerBackup.A_backupAll]==1)
				this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);		
			else
				this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);		
		} else {
			return;
		}

	}

	ZaXWizardDialog.prototype.goNext.call(this);
}

ZaBackupXWizard.prototype.goPrev = 
function() {
	var instance = this._localXForm.getInstance();
	if (this._containedObject[ZaModel.currentStep] == 2 || 
	(this._containedObject[ZaModel.currentStep] == 3 &&
	(instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL) )) {
		this._button[DwtWizardDialog.PREV_BUTTON].setEnabled(false);	
		this._button[DwtWizardDialog.NEXT_BUTTON].setText(AjxMsg._next);
		if(this._containedObject[ZaModel.currentStep] == 3 && (instance[ZaBackup.A_method] == ZaBackup.METHOD_INCREMENTAL) ) {
			this._containedObject[ZaModel.currentStep]--;					
		}
	} 


	this._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
	this._button[DwtWizardDialog.FINISH_BUTTON].setEnabled(false);
	ZaXWizardDialog.prototype.goPrev.call(this);
}

ZaBackupXWizard.prototype.createPathCallback = function (respObj) {
	try {
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.createPathCallback");
			throw(ex);
		}	
		if(respObj.isException && respObj.isException()) {
			throw(respObj.getException());
		} else {
			var responce;
			if(respObj.getResponse) {
				var responce = respObj.getResponse();
				if(responce.Body.CheckDirectoryResponse.directory[0].exists) {
					this.goNext();
				} else {
					if(responce.Body.CheckDirectoryResponse.directory[0].path) {
						var parts = responce.Body.CheckDirectoryResponse.directory[0].path.split("/");
						if((parts.length = 4 && parts[1]=="opt" && parts[2]=="zimbra") || (parts.length = 5 && parts[1]=="opt" && parts[2]=="zimbra" && parts[4]=="") ){
							throw(new ZmCsfeException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_FailedToCreateTargetFolderOptZimbra,[responce.Body.CheckDirectoryResponse.directory[0].path,"/opt/zimbra"]),ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.createPathCallback"));
						}
					} 
					throw(new ZmCsfeException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_FailedToCreateTargetFolder,[responce.Body.CheckDirectoryResponse.directory[0].path]),ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.createPathCallback"));
				}
			}
		}
		if (ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"]) {
			ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"].popdown();		
		}
	} catch (ex) {
		if (ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"]) {
			ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"].popdown();		
		}
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupXWizard.prototype.createPathCallback");
	}

}

ZaBackupXWizard.prototype.createPath = function(instance) {
	try {
		instance[ZaBackup.A_create] = true;
		var respObj = ZaBackup.queryTargetPath(instance, new AjxCallback(this,ZaBackupXWizard.prototype.createPathCallback));
	} catch(ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupXWizard.prototype.createPath");		
	}
}

ZaBackupXWizard.prototype.checkPath = function(instance, create) {
	try {
		var respObj = ZaBackup.queryTargetPath(instance, null);
		if(respObj && respObj.Body && respObj.Body.CheckDirectoryResponse && respObj.Body.CheckDirectoryResponse.directory && respObj.Body.CheckDirectoryResponse.directory[0]) {
			if(respObj.Body.CheckDirectoryResponse.directory[0].exists && respObj.Body.CheckDirectoryResponse.directory[0].readable && respObj.Body.CheckDirectoryResponse.directory[0].writable) {
				return true;
			} else {
				if(!respObj.Body.CheckDirectoryResponse.directory[0].exists) {
					var instance = this._localXForm.getInstance();
					var isForcedToCreate = instance[ZaBackup.A_create];
					if (!isForcedToCreate) {
						if(!ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"]) {
							ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"] = new ZaMsgDialog(ZaApp.getInstance().getAppCtxt().getShell(), null, [DwtDialog.YES_BUTTON,DwtDialog.NO_BUTTON,DwtDialog.CANCEL_BUTTON]);
						}
						ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"].setMessage(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_CreateTargetFolderQ, instance[ZaBackup.A_target]), DwtMessageDialog.WARNING_STYLE);	
						ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"].registerCallback(
								DwtDialog.YES_BUTTON, new AjxCallback(this,ZaBackupXWizard.prototype.createPath,[instance]));

						ZaApp.getInstance().dialogs["BackupTargetPathWarningDialog"].popup();

					} else {
						ZaBackupXWizard.prototype.createPath.call(this, instance);
					}
				} else if (!respObj.Body.CheckDirectoryResponse.directory[0].writable) {
					var ex = new ZmCsfeException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_TargetFolderNotWritable,[respObj.Body.CheckDirectoryResponse.directory[0].path]),ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.createPath");
					throw(ex);			
				}  else if (!respObj.Body.CheckDirectoryResponse.directory[0].readable) {
					var ex = new ZmCsfeException(AjxMessageFormat.format(com_zimbra_backuprestore.BNR_TargetFolderNotReadable,[respObj.Body.CheckDirectoryResponse.directory[0].path]),ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.createPath");
					throw(ex);			
				}		
			}
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupXWizard.prototype.checkPath");		
	}
	return false;
}

ZaBackupXWizard.prototype.backupQueryCallback = function (respObj) {
	try {
		if(!respObj) {
			var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupXWizard.prototype.backupQueryCallback");
			throw(ex);
		}	
		if(respObj.isException && respObj.isException()) {
			throw(respObj.getException());
		} else {
			var resp;
			if(respObj.getResponse) {
				resp = respObj.getResponse();
				if(resp) {
					var instance = this._localXForm.getInstance();
					instance[ZaBackup.A_label] = resp.Body.BackupResponse.backup[0][ZaBackup.A_label];
					instance[ZaBackup.A_incrLabel] = resp.Body.BackupResponse.backup[0][ZaBackup.A_incrLabel];					
					var msgBuf = [com_zimbra_backuprestore.BNR_BackupStarted,"<br/>"];
					if(instance[ZaBackup.A_label]) {
						msgBuf.push(com_zimbra_backuprestore.BNR_FullLabel);
						msgBuf.push(" ");
						msgBuf.push(instance[ZaBackup.A_label]);
						msgBuf.push("<br/>");						
					}
					if (instance[ZaBackup.A_incrLabel]) {
						msgBuf.push(com_zimbra_backuprestore.BNR_IncrementalLabel);
						msgBuf.push(" ");
						msgBuf.push(instance[ZaBackup.A_incrLabel]);						
						
					}
					instance.doneMsg = msgBuf.join("");
					
					this._localXForm.setInstance(instance);

					var otherController = ZaApp.getInstance().getCurrentController();
                    if (otherController instanceof ZaBackupsViewController && otherController._view && 
                        otherController._view._localXForm ) {

                    	var otherXForm = otherController._view._localXForm;
						//update the target path in the ZaBackupsViewController
						otherXForm.setInstanceValue(instance[ZaBackup.A_target], ZaBackup.A_target);
                    }

					if(otherController.refresh) {
						otherController.refresh();
					}
				}
			}
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupXWizard.prototype.backupQueryCallback");
		this.goPrev();
		return;
	}
}

ZaBackupXWizard.removeAllButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();

	instance[ZaBackup.A_Accounts] = new Array();
	var currentTargetList = instance[ZaBackup.A_Accounts] ? instance[ZaBackup.A_Accounts] : [];		
	if(currentTargetList.length > 0) {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);		
	} else {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);		
	}	
	this.getForm().setInstance(instance);	
}


ZaBackupXWizard.addAllButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var oldArr = instance[ZaBackup.A_Accounts] ? instance[ZaBackup.A_Accounts]  : [];
	var arr = instance[ZaBackup.A_accountPool];
	var arr2 = new Array();
	if(arr) {
		var cnt = arr.length;
		var oldCnt = oldArr.length;
		for(var ix=0; ix< cnt; ix++) {
			var found = false;
			for(var j = oldCnt-1;j>=0;j-- ) {
				if(oldArr[j].id == arr[ix].id) {
					found = true;
					break;
				}
			}
			if(!found)
				arr2.push(arr[ix]);
		}
	}
	arr2 = arr2.concat(oldArr);
	instance[ZaBackup.A_Accounts] = arr2;
	instance[ZaBackup.A_accountPool] = new Array();
	var currentTargetList = instance[ZaBackup.A_Accounts] ? instance[ZaBackup.A_Accounts] : [];	
	if(currentTargetList.length > 0) {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);		
	} else {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);		
	}	
	this.getForm().setInstance(instance);	
}

ZaBackupXWizard.addButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var sourceListItems = form.getItemsById(ZaBackup.A_accountPool);
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[0] && sourceListItems[0].widget) {
		var selection = sourceListItems[0].widget.getSelection();
		var currentTargetList = instance[ZaBackup.A_Accounts] ? instance[ZaBackup.A_Accounts] : [];
		var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
		if(list) {
			list.sort(ZaItem.compareNamesDesc);		
			//currentTargetList.sort(ZaItem.compareNamesDesc);
			var tmpTargetList = [];
			var cnt2 = currentTargetList.length;			
			for(var i=0;i<cnt2;i++)				
				tmpTargetList.push(currentTargetList[i]);
				
			tmpTargetList.sort(ZaItem.compareNamesDesc);
			
			var tmpList = [];
			var cnt = list.length;
			for(var i=cnt-1; i>=0; i--) {
				var dup = false;
				cnt2 = tmpTargetList.length;				
				for(var j = cnt2-1; j >=0; j--) {
					if(list[i].name==tmpTargetList[j].name) {
						dup=true;
						tmpTargetList.splice(j,cnt2-j);
						break;
					}
				}
				if(!dup) {
					currentTargetList.push(list[i])
				}
			}

			instance[ZaBackup.A_Accounts] = currentTargetList;				
		}
	}
	if(currentTargetList.length > 0) {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);		
	} else {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);		
	}
	this.getForm().setInstance(instance);	
}

ZaBackupXWizard.removeButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var targetListItems = form.getItemsById(ZaBackup.A_Accounts);
	if(targetListItems && (targetListItems instanceof Array) && targetListItems[0] && targetListItems[0].widget) {
		var selection = targetListItems[0].widget.getSelection();
		
		var currentTargetList = instance[ZaBackup.A_Accounts] ? instance[ZaBackup.A_Accounts] : [];
		currentTargetList.sort(ZaItem.compareNamesDesc);
		var tmpTargetList = [];
		var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
		if(list) {
			list.sort(ZaItem.compareNamesDesc);
			var cnt = list.length;
			var cnt2 = currentTargetList.length;
			for(var i=0;i<cnt2;i++)				
				tmpTargetList.push(currentTargetList[i]);
			
			for(var i=cnt-1; i>=0; i--) {
				var cnt2 = tmpTargetList.length;
				for(var j = cnt2-1; j >=0; j--) {
					if(list[i].name==tmpTargetList[j].name) {
						currentTargetList.splice(j,1);
						tmpTargetList.splice(j,cnt2-j);
					}
				}
			}
			instance[ZaBackup.A_Accounts] = currentTargetList;
		}
	}
	if(currentTargetList.length > 0) {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);		
	} else {
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);		
	}
	this.getForm().setInstance(instance);	
}


ZaBackupXWizard.addAllBtnEnabled = function () {
	return !AjxUtil.isEmpty(this.getInstanceValue(ZaBackup.A_accountPool));
}

ZaBackupXWizard.removeAllBtnEnabled = function () {
	return (this.instance && this.instance[ZaBackup.A_Accounts] && this.instance[ZaBackup.A_Accounts].length>0);
}

/**
* method of the XForm
**/
ZaBackupXWizard.forwardBtnEnabled = function () {
	return (parseInt(this.getInstanceValue(ZaServerBackup.A_poolPagenum)) < (parseInt(this.getInstanceValue(ZaServerBackup.A_poolNumPages))-1) && this.getInstanceValue(ZaServerBackup.A_activateSearch)==1);
};

/**
* method of the XForm
**/
ZaBackupXWizard.backBtnEnabled = function () {
	return (parseInt(this.getInstanceValue(ZaServerBackup.A_poolPagenum)) > 0 && this.getInstanceValue(ZaServerBackup.A_activateSearch)==1);
};

ZaBackupXWizard.accPoolSelectionListener = function() {
	var arr = this.widget.getSelection();
	if(arr && arr.length) {
		arr.sort();
		this.getModel().setInstanceValue(this.getInstance(), ZaServerBackup.A_src_acct_selection_pool, arr);
	} else {
		this.getModel().setInstanceValue(this.getInstance(), ZaServerBackup.A_src_acct_selection_pool, null);
	}	
}

ZaBackupXWizard.accTargetSelectionListener = function() {
	var arr = this.widget.getSelection();
	if(arr && arr.length) {
		arr.sort();
		this.getModel().setInstanceValue(this.getInstance(), ZaServerBackup.A_tgt_acct_selection_pool, arr);
	} else {
		this.getModel().setInstanceValue(this.getInstance(), ZaServerBackup.A_tgt_acct_selection_pool, null);
	}	
}

/*
ZaBackupXWizard.addBtnEnabled = function () {
	//scope is form
	var retVal = false;
	var sourceListItems = this.getItemsById(ZaBackup.A_accountPool);
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[0] && sourceListItems[0].widget) {
		var selection = sourceListItems[0].widget.getSelection();
		var selectionArray = (selection instanceof AjxVector) ? selection.getArray() : selection;
		if(selection && selection.length) {
			retVal = true;
		} 
	}
	return retVal;
}*/

/*
ZaBackupXWizard.removeBtnEnabled = function () {
	//scope is form
	var retVal = false;
	var sourceListItems = this.getItemsById(ZaBackup.A_Accounts);
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[0] && sourceListItems[0].widget) {
		var selection = sourceListItems[0].widget.getSelection();
		var selectionArray = (selection instanceof AjxVector) ? selection.getArray() : selection;
		if(selection && selection.length) {
			retVal = true;
		} 
	}
	return retVal;
}
*/
ZaBackupXWizard.isAutogroupedMode = function() {
	var instance = this.getInstance();
	
	if(instance) {
		var server = ZaApp.getInstance().getServerByName(instance[ZaBackup.A_serverName]);
		if (server.attrs[ZaServer.A_zimbraBackupMode]==ZaHotBackup.BACKUP_MODE_AUTOGROUP)
			return true;
			
		if(!server.attrs[ZaServer.A_zimbraBackupMode] && ZaApp.getInstance().getGlobalConfig().attrs[ZaGlobalConfig.A_zimbraBackupMode]==ZaHotBackup.BACKUP_MODE_AUTOGROUP)
			return true;
		
	}
	return false;
}

ZaBackupXWizard.isNotAutogroupedMode = function() {
	return !ZaBackupXWizard.isAutogroupedMode.call(this);
}
/**
* method of an XFormItem
**/
ZaBackupXWizard.fwdPoolButtonHndlr = 
function(evt) {
	var currentPageNum = parseInt(this.getInstanceValue("/poolPagenum"));
	this.setInstanceValue(currentPageNum+1,"/poolPagenum");
	ZaBackupXWizard.srchButtonHndlr.call(this, evt);
}

/**
* method of an XFormItem
**/
ZaBackupXWizard.backPoolButtonHndlr = 
function(evt) {
	var currentPageNum = parseInt(this.getInstanceValue("/poolPagenum"))-1;
	this.setInstanceValue(currentPageNum,"/poolPagenum");
	ZaBackupXWizard.srchButtonHndlr.call(this, evt);
}

/**
* method of an XFormItem
**/
ZaBackupXWizard.srchButtonHndlr = function (ev) {
	var instance = this.getForm().getInstance();
	var formParent = this.getForm().parent;
	var params = {};
	params.types = [ZaSearch.ACCOUNTS,ZaSearch.RESOURCES];
	params.query = ["(","&",ZaSearch.getSearchByNameQuery (instance[ZaSearch.A_query]),"(zimbraMailHost=",instance[ZaBackup.A_serverName],"))"].join("");
	params.limit = ZaSettings.RESULTSPERPAGE;
	params.offset = instance.poolPagenum*ZaSettings.RESULTSPERPAGE;
	params.attrs = [ZaItem.A_zimbraId,ZaAccount.A_uid,ZaAccount.A_displayname,ZaResource.A_zimbraCalResType].join(",");
	params.callback = new AjxCallback(formParent,ZaBackupXWizard.srchAccountsClbck);
	instance.activateSearch = 0;
	this.getForm().setInstance(instance);
	ZaSearch.searchDirectory(params);
}

ZaBackupXWizard.srchAccountsClbck = function (resp) {
	var instance = this._localXForm.getInstance();
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "ZaBackupXWizard.srchAccountsClbck"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse().Body.SearchDirectoryResponse;
			var list = new ZaItemList(null);	
			list.loadFromJS(response);	
			instance[ZaBackup.A_accountPool] = list.getArray();
			var poolNumPages = Math.ceil(response.searchTotal/ZaSettings.RESULTSPERPAGE);
			instance.poolNumPages = poolNumPages;
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupXWizard.srchAccountsClbck");	
	}	
	
	instance.activateSearch = 1;
	this._localXForm.setInstance(instance);	
}

ZaBackupXWizard.switchNextButton = 
function (value, event, form) {
	if(value==1 || (form.getInstance()[ZaBackup.A_Accounts] && form.getInstance()[ZaBackup.A_Accounts].length>0))
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(true);
	else 
		form.parent._button[DwtWizardDialog.NEXT_BUTTON].setEnabled(false);
	this.setInstanceValue(value);
}

ZaBackupXWizard.myXFormModifier = function(xFormObject) {
    var sourceHeaderList = new Array();
    sourceHeaderList[0] = new ZaListHeaderItem(
        "type",
        ZaMsg.ALV_Type_col,
        null,
        "34px",
        null,
        "objectClass",
        true,
        true
    );
    sourceHeaderList[1] = new ZaListHeaderItem(
        ZaAccount.A_name,
        ZaMsg.ALV_Name_col,
        null,
        "auto",
        null,
        ZaAccount.A_name,
        true,
        true
    );

    var case1 = {
        type: _CASE_,
        numCols: 2,
        caseKey: 1,
        align: _LEFT_,
        valign: _TOP_,
        items: [
            {
                ref: ZaBackup.A_serverName,
                type: _OSELECT1_,
                label: com_zimbra_backuprestore.BNR_BackupSourceServer,
                labelLocation: _LEFT_,
                choices: ZaApp.getInstance().getServerListChoices(),
                visibilityChecks: [],
                enableDisableChecks: []
            },
            {
                ref: ZaBackup.A_method,
                type: _OSELECT1_,
                label: com_zimbra_backuprestore.BNR_BackupMethod,
                labelLocation: _LEFT_,
                choices: this.backupMethodChoices,
                visibilityChecks: [],
                enableDisableChecks: []
            },
            {
                ref: ZaBackup.A_target,
                type: _TEXTFIELD_,
                label: com_zimbra_backuprestore.BNR_BackupTargetPath,
                labelLocation: _LEFT_,
                width: 250,
                visibilityChecks: [
                    ZaBackupXWizard.isNotAutogroupedMode,
                    [
                        XForm.checkInstanceValue,
                        ZaBackup.A_method,
                        ZaBackup.METHOD_FULL
                    ]
                ],
                visibilityChangeEventSources: [ZaBackup.A_method],
                enableDisableChecks: []
            },
            {
                ref: ZaBackup.A_create,
                type: _CHECKBOX_,
                label: com_zimbra_backuprestore.BNR_CreateFolderIfDoesnotExist,
                visibilityChecks: [
                    ZaBackupXWizard.isNotAutogroupedMode,
                    [
                        XForm.checkInstanceValue,
                        ZaBackup.A_method,
                        ZaBackup.METHOD_FULL
                    ]
                ],
                visibilityChangeEventSources: [ZaBackup.A_method],
                enableDisableChecks: [],
                trueValue: true,
                falseValue: false
            },
			{
                type: _DWT_ALERT_,
                style: DwtAlert.WARNING,
                colSpan: 2,
                iconVisible: true,
                content: com_zimbra_backuprestore.WARNING_MANUAL_AUTOGROUP,
                width: "500px",
                visibilityChecks: [ZaBackupXWizard.isAutogroupedMode]
            }
        ]
    };

    var case2 = {
        type: _CASE_,
        numCols: 1,
        caseKey: 2,
        width: "100%",
        items: [
            {
                type: _GROUP_,
                numCols: 2,
                items: [
                    {
                        ref: ZaServerBackup.A_backupAll,
                        type: _CHECKBOX_,
                        label: com_zimbra_backuprestore.BNR_BackupAllAccounts,
                        align: _LEFT_,
                        labelLocation: _LEFT_,
                        trueValue: 1,
                        falseValue: 0,
                        labelCssClass: "xform_label",
                        onChange: ZaBackupXWizard.switchNextButton,
                        visibilityChecks: [],
                        enableDisableChecks: []
                    }
                ]
            },
            {
                type: _SPACER_
            },
            {
                type: _GROUP_,
                numCols: 3,
                width: "100%",
                colSizes: ["170px", "85px", "170px"],
                cellspacing: "5px",
                visibilityChecks: [
                    [
                        XForm.checkInstanceValue,
                        ZaServerBackup.A_backupAll,
                        0
                    ]
                ],
                visibilityChangeEventSources: [ZaServerBackup.A_backupAll],
                items: [
                    {
                        type: _TEXTFIELD_,
                        cssClass: "admin_xform_name_input",
                        width: "160px",
                        ref: ZaSearch.A_query,
                        label: null,
                        elementChanged: function (elementValue, instanceValue, event) {
                            var charCode = event.charCode;
                            if (charCode == 13 || charCode == 3) {
                                ZaBackupXWizard.srchButtonHndlr.call(this);
                            } else {
                                this.getForm().itemChanged(this, elementValue, event);
                            }
                        },
                        visibilityChecks: [],
                        enableDisableChecks: []
                    },
                    {
                        type: _DWT_BUTTON_,
                        label: ZaMsg.DLXV_ButtonSearch,
                        width: "80px",
                        onActivate: ZaBackupXWizard.srchButtonHndlr,
                        align: _CENTER_,
                        enableDisableChecks: [
                            [
                                XForm.checkInstanceValue,
                                'activateSearch',
                                1
                            ]
                        ],
                        enableDisableChangeEventSources: ['activateSearch']
                    },
                    {
                        type: _OUTPUT_,
                        value: com_zimbra_backuprestore.BNR_AccountsTobackup,
                        visibilityChecks: []
                    },
                    {
                        type: _SPACER_,
                        colSpan: 3
                    },
                    {
                        ref: ZaBackup.A_accountPool,
                        type: _DWT_LIST_,
                        height: "200px",
                        width: "170px",
                        cssClass: "DLSource",
                        widgetClass: ZaAccMiniListView,
                        headerList: sourceHeaderList,
                        rowSpan: 5,
                        hideHeader: true,
                        onSelection: ZaBackupXWizard.accPoolSelectionListener,
                        visibilityChecks: [],
                        enableDisableChecks: []
                    },
                    {
                        type: _DWT_BUTTON_,
                        label: AjxMsg.addAll,
                        width: "80px",
                        onActivate: ZaBackupXWizard.addAllButtonHndlr,
                        enableDisableChecks: [
                            [
                                XForm.checkInstanceValueNotEmty,
                                ZaBackup.A_accountPool
                            ]
                        ],
                        enableDisableChangeEventSources: [ZaBackup.A_accountPool]
                    },
                    {
                        ref: ZaBackup.A_Accounts,
                        type: _DWT_LIST_,
                        height: "200px",
                        width: "170px",
                        cssClass: "DLSource",
                        widgetClass: ZaAccMiniListView,
                        headerList: sourceHeaderList,
                        rowSpan: 5,
                        hideHeader: true,
                        onSelection: ZaBackupXWizard.accTargetSelectionListener,
                        visibilityChecks: [],
                        enableDisableChecks: []
                    },
                    {
                        type: _DWT_BUTTON_,
                        label: AjxMsg.add,
                        width: "80px",
                        onActivate: ZaBackupXWizard.addButtonHndlr,
                        enableDisableChecks: [
                            [
                                XForm.checkInstanceValueNotEmty,
                                ZaServerBackup.A_src_acct_selection_pool
                            ]
                        ],
                        enableDisableChangeEventSources: [ZaServerBackup.A_src_acct_selection_pool]
                    },
                    {
                        type: _DWT_BUTTON_,
                        label: AjxMsg.remove,
                        width: "80px",
                        onActivate: ZaBackupXWizard.removeButtonHndlr,
                        enableDisableChecks: [
                            [
                                XForm.checkInstanceValueNotEmty,
                                ZaServerBackup.A_tgt_acct_selection_pool
                            ]
                        ],
                        enableDisableChangeEventSources: [ZaServerBackup.A_tgt_acct_selection_pool]
                    },
                    {
                        type: _DWT_BUTTON_,
                        label: AjxMsg.removeAll,
                        width: "80px",
                        onActivate: ZaBackupXWizard.removeAllButtonHndlr,
                        enableDisableChecks: [
                            [
                                XForm.checkInstanceValueNotEmty,
                                ZaBackup.A_Accounts
                            ]
                        ],
                        enableDisableChangeEventSources: [ZaBackup.A_Accounts]
                    },
                    {
                        type: _SPACER_
                    },
                    {
                        type: _GROUP_,
                        numCols: 3,
                        colSizes: ["80px", "*", "80px"],
                        items: [
                            {
                                type: _SPACER_,
                                colSpan: 3
                            },
                            {
                                type: _DWT_BUTTON_,
                                label: ZaMsg.Previous,
                                width: 75,
                                id: "backButton",
                                icon: "LeftArrow",
                                disIcon: "LeftArrowDis",
                                onActivate: ZaBackupXWizard.backPoolButtonHndlr,
                                align: _CENTER_,
                                enableDisableChecks: [ZaBackupXWizard.backBtnEnabled],
                                enableDisableChangeEventSources: [
                                    ZaServerBackup.A_poolPagenum,
                                    ZaServerBackup.A_activateSearch
                                ]
                            },
                            {
                                type: _CELLSPACER_
                            },
                            {
                                type: _DWT_BUTTON_,
                                label: ZaMsg.Next,
                                width: 75,
                                id: "fwdButton",
                                icon: "RightArrow",
                                disIcon: "RightArrowDis",
                                onActivate: ZaBackupXWizard.fwdPoolButtonHndlr,
                                align: _CENTER_,
                                enableDisableChecks: [ZaBackupXWizard.forwardBtnEnabled],
                                enableDisableChangeEventSources: [
                                    ZaServerBackup.A_poolPagenum,
                                    ZaServerBackup.A_activateSearch
                                ]
                            }
                        ]
                    },
                    {
                        type: _CELLSPACER_
                    },
                    {
                        type: _DWT_ALERT_,
                        style: DwtAlert.INFORMATION,
                        iconVisible: true,
                        content: ZaMsg.pleaseWaitSearching,
                        width: "200px",
                        align: _CENTER_,
                        cssStyle: "position: relative; top: -200; left: 125; opacity: 0.8",
                        visibilityChecks: [
                            [
                                XForm.checkInstanceValue,
                                ZaServerBackup.A_activateSearch,
                                0
                            ]
                        ],
                        visibilityChangeEventSources: [ZaServerBackup.A_activateSearch]
                    }
                ]
            }
        ]
    };

    var case3 = {
        type: _CASE_,
        numCols: 1,
        caseKey: 3,
        width: "100%",
        items: [
            {
                ref: "doneMsg",
                style: DwtAlert.INFORMATION,
                type: _DWT_ALERT_,
                width: "420px",
                iconVisible: true,
                content: null
            }
        ]
    };

    xFormObject.items = [
        {
            type: _OUTPUT_,
            colSpan: 2,
            valign: _TOP_,
            cssStyle: "white-space: normal",
            ref: ZaModel.currentStep,
            choices: this.stepChoices,
            visibilityChecks: [],
            valueChangeEventSources: [ZaModel.currentStep]
        },
        {
            type: _SEPARATOR_,
            align: _CENTER_,
            valign: _TOP_
        },
        {
            type: _SPACER_,
            align: _CENTER_,
            valign: _TOP_
        },
        {
            type: _SWITCH_,
            width: 500,
            align: _LEFT_,
            valign: _TOP_,
            cssStyle: "white-space: normal",
            items: [
                case1,
                case2,
                case3
            ]
        }
    ];
}
ZaXDialog.XFormModifiers["ZaBackupXWizard"].push(ZaBackupXWizard.myXFormModifier);
