/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaBackupsViewController
* This is a singleton object that controls all the user interaction with the list of ZaDomain objects
**/
function ZaBackupsViewController(appCtxt, container) {
	ZaXFormViewController.call(this, appCtxt, container,"ZaBackupsViewController");
	this._UICreated = false;	
	this._helpURL = ZaBackupsViewController.helpURL;
	this.tabConstructor = ZaBackupsXFormView;
}

ZaBackupsViewController.prototype = new ZaXFormViewController();
ZaBackupsViewController.prototype.constructor = ZaBackupsViewController;
ZaBackupsViewController.helpURL =  location.pathname + "help/admin/html/backup_feature/backing_up_the_zimbra_server.htm?locid=" + AjxEnv.DEFAULT_LOCALE;
ZaController.initPopupMenuMethods["ZaBackupsViewController"] = new Array();
ZaController.setViewMethods["ZaBackupsViewController"] = new Array();
ZaController.changeActionsStateMethods["ZaBackupsViewController"] = new Array();
/**
*	@method show
*	@param entry - isntance of ZaServer class
*/
ZaBackupsViewController.prototype.show =
function(entry) {
	if (! this.selectExistingTabByItemId(entry.id)){
		this._setView(entry, true);
		this.setDirty(false);
	}
}


ZaBackupsViewController.initPopupMenuMethod =
function () {
    this._popupOperations[ZaOperation.IMMEDIATE_BACKUP] = new ZaOperation(ZaOperation.IMMEDIATE_BACKUP, com_zimbra_backuprestore.BNRTBB_Backup, com_zimbra_backuprestore.BNRTBB_Backup_tt, "Backup", "Backup", new AjxListener(this, ZaBackupsViewController.prototype._immediateBackupListener));
    this._popupOrder.push(ZaOperation.IMMEDIATE_BACKUP);

    this._popupOperations[ZaOperation.MAIL_RESTORE] = new ZaOperation(ZaOperation.MAIL_RESTORE, com_zimbra_backuprestore.BNR_TBB_RestoreMailbox, com_zimbra_backuprestore.BNR_TBB_Restore_tt, "RestoreMailbox", "RestoreMailboxDis", new AjxListener(this, this.restoreMailListener));
    this._popupOrder.push(ZaOperation.MAIL_RESTORE);

    this._popupOperations[ZaOperation.ABORT_BACKUP] = new ZaOperation(ZaOperation.ABORT_BACKUP, com_zimbra_backuprestore.BNRTBB_AbortBackup, com_zimbra_backuprestore.BNRTBB_AbortBackup_tt, "Cancel", "Cancel", new AjxListener(this, this.abortBackupListener));
    this._popupOrder.push(ZaOperation.ABORT_BACKUP);

    this._popupOperations[ZaOperation.CONFIGURE_BACKUP] = new ZaOperation(ZaOperation.CONFIGURE_BACKUP, com_zimbra_backuprestore.BNRTBB_ConfigureBackup, com_zimbra_backuprestore.BNRTBB_ConfigureBackup_tt, "AdvancedTools", "Configure", new AjxListener(this, ZaHotBackup.configureBackup));
    this._popupOrder.push(ZaOperation.CONFIGURE_BACKUP);

    this._popupOperations[ZaOperation.REFRESH] = new ZaOperation(ZaOperation.REFRESH, ZaMsg.TBB_Refresh, ZaMsg.TBB_Refresh_tt, "Refresh", "Refresh", new AjxListener(this, this.refreshListener));
    this._popupOrder.push(ZaOperation.REFRESH);
}
ZaController.initPopupMenuMethods["ZaBackupsViewController"].push(ZaBackupsViewController.initPopupMenuMethod);
/**
*	@method setViewMethod 
*	@param entry - isntance of ZaBackup class
*/
ZaBackupsViewController.setViewMethod =
function(entry) {
	if(!this._UICreated) {
		this._createUI();
	}

	entry.refresh();
	ZaApp.getInstance().pushView(this.getContentViewId());
	this._view.setDirty(false);
	this._view.setObject(entry); 	//setObject is delayed to be called after pushView in order to avoid jumping of the view	
	this._currentObject = entry;

}
ZaController.setViewMethods["ZaBackupsViewController"].push(ZaBackupsViewController.setViewMethod);

ZaBackupsViewController.prototype.nextPageListener = 
function (ev) {
	this._currentObject[ZaBackup.A_backupListOffset] = (this._currentObject[ZaBackup.A_pageNumber] * ZaSettings.RESULTSPERPAGE);
	this._currentObject[ZaBackup.A_backupListCount] = ZaSettings.RESULTSPERPAGE;
	this._currentObject[ZaBackup.A_pageNumber]++;
	this._view.setObject(this._currentObject); 	
}

ZaBackupsViewController.prototype.prevPageListener = 
function (ev) {
	if(this._currentObject[ZaBackup.A_pageNumber]<1) {
		return;
	}
	this._currentObject[ZaBackup.A_pageNumber]--;
	this._currentObject[ZaBackup.A_backupListOffset] = ((this._currentObject[ZaBackup.A_pageNumber]-1) * ZaSettings.RESULTSPERPAGE);
	this._currentObject[ZaBackup.A_backupListCount] = ZaSettings.RESULTSPERPAGE;

	this._view.setObject(this._currentObject); 
}

ZaBackupsViewController.prototype.listActionListener =
function (ev) {
	this.changeActionsState();
	this._actionMenu.popup(0, ev.docX, ev.docY);
}

ZaBackupsViewController.prototype.restoreMailListener = 
function (ev) {
	try {
		this._accountRestoreWizard = new AccountRestoreXWizard(this._container);		
		var restore = new ZaRestore();
		var form = this._view.getMyForm();
		if(form) {
			var instance = form.getInstance();
			if(instance) {
				restore.restoreRequest[ZaRestore.A_target] = instance[ZaBackup.A_target];
				restore.restoreRequest[ZaRestore.A_toServer] = instance[ZaBackup.A_serverId];
			}
			var listFormItems = form.getItemsById(ZaBackup.A_backupLabelList);
			if(listFormItems && listFormItems.length) {
				var listWidget = listFormItems[0].widget;
				var selection = listWidget.getSelection();
				var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
				if(list && list.length) {
					var selectedLabel = list[0];
					if(selectedLabel) {
						restore.restoreRequest[ZaRestore.A_restoreToTime] = new Date(selectedLabel.start);
						restore.restoreRequest[ZaRestore.A_restoreTo] = ZaRestore.TO_PIT;
					}
				}
			}
		}
		
		this._accountRestoreWizard.setObject(restore);		
		this._accountRestoreWizard.popup();
	} catch (ex) {
		this._handleException(ex, "ZaBackupsViewController.prototype.restoreMailListener", null, false);
	}
	return;
}

ZaBackupsViewController.prototype.abortBackupListener = function (ev) {
	if(this._view) {
		var form = this._view.getMyForm();
		if(form) {
			var listFormItems = form.getItemsById(ZaBackup.A_backupLabelList);
			var instance = form.getInstance();
			var target = instance[ZaBackup.A_target];
			if(target && listFormItems && listFormItems.length && listFormItems[0]) {
				var listWidget = listFormItems[0].widget;
				var selection = listWidget.getSelection();
				var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
				if(list && list.length) {
					var selectedLabel = list[0];
					var callback = new AjxCallback(this,this.abortBackupCallback)
					ZaBackup.runBackupQuery(this._currentObject.id,ZaBackup.METHOD_ABORT,target,null,selectedLabel[ZaBackupLabel.A_label],callback);

					instance.activeOperation += ZaHotBackup.ABORT_OPERATION;
					form.setInstance(instance);

					if (this._popupOperations[ZaOperation.ABORT_BACKUP]){
						this._popupOperations[ZaOperation.ABORT_BACKUP].enabled = false;
					}
				}
			}
		}
	}
}

ZaBackupsViewController.prototype.abortBackupCallback = function(resp) {
	
	var form = this._view.getMyForm();
	var instance = form.getInstance();
	instance.activeOperation -= ZaHotBackup.ABORT_OPERATION;
	form.setInstance(instance);
	
	if(resp.isException && resp.isException()) {
		var ex = resp.getException();
		if(ex.code == ZmCsfeException.CANNOT_ABORT_COMPLETED_BACKUP) {
			this.popupErrorDialog(com_zimbra_backuprestore.BNR_CannotAbortCompletedBckup,ex,null,DwtMessageDialog.WARNING_STYLE);
		} else {
			ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupsViewController.prototype.abortBackupCallback");
		}
	} else {
		this.popupMsgDialog(com_zimbra_backuprestore.BNR_Aborting);
	}		
	this._view.refresh();
}

ZaBackupsViewController.prototype.refresh = function () {
	this._view.refresh();
}

ZaBackupsViewController.prototype.refreshListener = function(ev) {
	this._view.refresh();
}

ZaBackupsViewController.prototype._immediateBackupListener = 
function (ev) {
	try {
		this._ImmediateBackupWizard = new ZaBackupXWizard(this._container);		
		var backup = new ZaBackup();
		backup[ZaBackup.A_method]=ZaBackup.METHOD_FULL;
		if(this._currentObject.attrs[ZaGlobalConfig.A_zimbraBackupTarget])
			backup[ZaBackup.A_target] = AjxStringUtil.trim(this._currentObject.attrs[ZaGlobalConfig.A_zimbraBackupTarget]);
		else	
			backup[ZaBackup.A_target] = ZaHotBackup.DEFAULT_BACKUP_TARGET;
			
		backup[ZaServerBackup.A_backupAll] = 0;
		backup[ZaBackup.A_serverName] = this._currentObject[ZaServer.A_ServiceHostname];
		this._ImmediateBackupWizard.setObject(backup);		
		this._ImmediateBackupWizard.popup();
	} catch (ex) {
		this._handleException(ex, "ZaBackupServerListController.prototype._immediateBackupListener", null, false);
	}
	return;
}
/**
* @method _createUI
**/
ZaBackupsViewController.prototype._createUI =
function () {
	this._contentView = this._view = new this.tabConstructor(this._container);

    this._initPopupMenu();
	
	var elements = new Object();
	elements[ZaAppViewMgr.C_APP_CONTENT] = this._view;
    ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
	this._UICreated = true;
	ZaApp.getInstance()._controllers[this.getContentViewId ()] = this ;
}

ZaBackupsViewController.prototype.searchCallback =
function(params, resp) {
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "ZaListViewController.prototype.searchCallback"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse().Body.BackupQueryResponse;
			this._list = new ZaItemList(ZaBackupLabel);	
			this._list.loadFromJS(response);	
			if(params.show)
				this._show(this._list);			
			else
				this._updateUI(this._list);
		}
	} catch (ex) {
		if (ex.code != ZmCsfeException.MAIL_QUERY_PARSE_ERROR) {
			this._handleException(ex, "ZaBackupsViewController.prototype.searchCallback");	
		} else {
			this.popupErrorDialog(ZaMsg.queryParseError, ex);
			if(this._searchField)
				this._searchField.setEnabled(true);	
		}		
	}
}

ZaBackupsViewController.prototype._handleException =
function(ex, method, params, restartOnError, obj) {
	if (ex.code && (ex.code == ZmCsfeException.INVALID_BACKUP_TARGET )) 
	{
		DBG.dumpObj(ex);
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_InvalidBackupTargetERROR, ex);
	} else if (ex.code && (ex.code == ZmCsfeException.NO_LABELS_FOUND )) {
		this._execFrame = {obj: obj, func: method, args: params, restartOnError: restartOnError};
		this._errorDialog.registerCallback(DwtDialog.OK_BUTTON, this._errorDialogCallback, this);
		this.popupErrorDialog(com_zimbra_backuprestore.BNR_NoLabelsFoundTitle, ex);
	
	} else {
		ZaController.prototype._handleException.call(this, ex, method, params, restartOnError, obj);
	}
}

ZaBackupsViewController.changeActionsStateMethod = function () {
	var form = this._view.getMyForm();
	if(form) {
		var listFormItems = form.getItemsById(ZaBackup.A_backupLabelList);
		var targetData = form.getInstance()[ZaBackup.A_target];
		if(targetData && listFormItems && listFormItems.length && listFormItems[0]) {
			var listWidget = listFormItems[0].widget;
			var selection = listWidget.getSelection();
			var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
			var cnt = listWidget.getSelectionCount();
			if(list && list.length && cnt == 1) {
				var selectedLabel = list[0];

				var isRunning = selectedLabel[ZaBackupLabel.A_live];
				var enabled = (isRunning == 1 || isRunning == true);
				//only isRunning is the right indicator, it means INPROGRESS status
				//while selectedLabel[ZaBackupLabel.A_status] is not the proper indicator indeed,
				//it will return INPROGRESS/NOTSTARTED/COMPLETED/ERROR text in localized language
				if (this._popupOperations[ZaOperation.ABORT_BACKUP]){
					this._popupOperations[ZaOperation.ABORT_BACKUP].enabled = enabled;
				}
			}else {
				//if no select or multi-selected, disable it
				if (this._popupOperations[ZaOperation.ABORT_BACKUP]){
					this._popupOperations[ZaOperation.ABORT_BACKUP].enabled = false;
				}
			}
		}
	}
}
ZaController.changeActionsStateMethods["ZaBackupsViewController"].push(ZaBackupsViewController.changeActionsStateMethod);

ZaBackupsViewController.prototype.getAppBarAction = function () {
    if (AjxUtil.isEmpty(this._appbarOperation)) {
        this._appbarOperation[ZaOperation.HELP] = new ZaOperation(
            ZaOperation.HELP,
            ZaMsg.TBB_Help,
            ZaMsg.TBB_Help_tt,
            "Help",
            "Help",
            new AjxListener(
                this,
                this._helpButtonListener
            )
        );

        this._appbarOperation[ZaOperation.CLOSE] = new ZaOperation(
            ZaOperation.CLOSE,
            ZaMsg.TBB_Close,
            ZaMsg.ALTBB_Close_tt,
            "",
            "",
            new AjxListener(
                this,
                this.closeButtonListener
            )
        );
    }

    return this._appbarOperation;
}

ZaBackupsViewController.prototype.getAppBarOrder = function () {
    if (AjxUtil.isEmpty(this._appbarOrder)) {
        this._appbarOrder.push(ZaOperation.HELP);
        this._appbarOrder.push(ZaOperation.CLOSE);
    }

    return this._appbarOrder;
}
