/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaBackupsXFormView
* @contructor
* @param parent
* @author Greg Solovyev
**/
function ZaBackupsXFormView (parent) {
	ZaTabView.call(this, parent,"ZaBackupsXFormView", "DwtTabView ZaXFormListView", "BACKUP_VIEW");
	this.initForm(ZaServerBackup.myXModel,this.getMyXForm());
	this._localXForm.removeListener(DwtEvent.XFORMS_VALUE_CHANGED, this.formChangeListener);
//	this._localXForm.addListener(DwtEvent.XFORMS_FORM_DIRTY_CHANGE, new AjxListener(this, ZaBackupsXFormView.prototype.handleXFormChange));	
	this._localXForm.setController(ZaApp.getInstance());
}

ZaBackupsXFormView.prototype = new ZaTabView();
ZaBackupsXFormView.prototype.constructor = ZaBackupsXFormView;
ZaTabView.XFormModifiers["ZaBackupsXFormView"] = new Array();

ZaBackupsXFormView.prototype.getTabIcon =
function () {
	return "Backup";
}

ZaBackupsXFormView.prototype.getTabToolTip =
function () {
	if (this._containedObject) {
		return com_zimbra_backuprestore.BNR_tt_tab_backup + " " + this._containedObject.type + " " + this._containedObject.name ;
	}else{
		return "";
	}
}

ZaBackupsXFormView.prototype.setObject = 
function (entry) {
	//this._localXForm.setInstance(new ZaBackup());
	this._containedObject = new ZaBackup();
	this._containedObject.type = entry.type ;
	this._containedObject.name = entry.name ;
	this._containedObject.id = entry.id ;
	this._containedObject[ZaBackup.A_serverId] = entry.id;
	this._containedObject[ZaBackup.A_target] = entry.attrs[ZaGlobalConfig.A_zimbraBackupTarget];
	this._containedObject[ZaBackup.A_pageNumber] = entry[ZaBackup.A_pageNumber];	
	this._containedObject[ZaBackup.A_serverName] = entry.attrs[ZaServer.A_ServiceHostname];
	this._containedObject[ZaBackup.A_backupLabelList] = entry[ZaBackup.A_backupLabelList];
	this._containedObject[ZaBackup.A_accountListStatus] = entry[ZaBackup.A_accountListStatus];
	this._containedObject[ZaBackup.A_backupListOffset] = entry[ZaBackup.A_backupListOffset];
	this._containedObject[ZaBackup.A_backupListCount] = ZaSettings.RESULTSPERPAGE;
	this._containedObject.activeOperation +=ZaHotBackup.SEARCH_OPERATION;
	this._localXForm.setInstance(this._containedObject);
	var callback = new AjxCallback(this, ZaBackupsXFormView.prototype.queryBackupsCallback, {show:true});
	ZaBackup.queryBackups(this._containedObject, callback);
    if (!appNewUI)
	    this.updateTab();
}

ZaBackupsXFormView.prototype.getTitle = 
function () {
	return com_zimbra_backuprestore.BNR_view_title;
}

ZaBackupsXFormView.prototype.refresh = function () {
	this._localXForm.getInstance().activeOperation +=ZaHotBackup.SEARCH_OPERATION;
	var callback = new AjxCallback(this, ZaBackupsXFormView.prototype.queryBackupsCallback, {show:true});
	ZaBackup.queryBackups(this._localXForm.getInstance(),callback);	
	
}

ZaBackupsXFormView._listObjects = {};

ZaBackupsXFormView.labelSelectionListener = function (ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			this._selectedItem = ev.item;
			ZaApp.getInstance().getBackupLabelViewController().show(ev.item);
		}
	} else {
		ZaApp.getInstance().getCurrentController().changeActionsState();
	}
}

ZaBackupsXFormView.createPopupMenu = function (listWidget) {
	ZaApp.getInstance().getCurrentController()._actionMenu = listWidget.actionMenu = new ZaPopupMenu(listWidget, "ActionMenu", null, ZaApp.getInstance().getCurrentController()._popupOperations);
	listWidget.addActionListener(new AjxListener(ZaApp.getInstance().getCurrentController(), ZaApp.getInstance().getCurrentController().listActionListener));
	listWidget.xFormItem = this;
}

ZaBackupsXFormView.queryBackups = function (ev) {
	var instance = this.getInstance();
	//var target = instance[ZaBackup.A_target];
//	var serverId = instance[ZaBackup.A_serverId];
	this.getInstance().activeOperation +=ZaHotBackup.SEARCH_OPERATION;
	var callback = new AjxCallback(this.getForm().parent, ZaBackupsXFormView.prototype.queryBackupsCallback, {show:true});
	ZaBackup.queryBackups(instance, callback);	
}

ZaBackupsXFormView.prototype.queryBackupsCallback = 
function (params, resp) {
	this._localXForm.getInstance().activeOperation -= ZaHotBackup.SEARCH_OPERATION;
	this._localXForm.refresh();
	if(!resp) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaBackupsXFormView.prototype.queryBackupsCallback");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaBackupsXFormView.prototype.queryBackupsCallback");
		return;
	}	
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaBackupsXFormView.prototype.queryBackupsCallback");
		return;
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.BackupQueryResponse) {
			response = resp;
		}
		var labelArray = [];
		var backup = null;
		var instance = new ZaBackup();
		var oldInstance = this._localXForm.getInstance();

		instance[ZaBackup.A_target] = oldInstance[ZaBackup.A_target];
		instance[ZaBackup.A_serverName] = oldInstance[ZaBackup.A_serverName];
		instance[ZaBackup.A_serverId] = oldInstance[ZaBackup.A_serverId];
		instance[ZaBackup.A_backupListOffset] = oldInstance[ZaBackup.A_backupListOffset];
		instance[ZaBackup.A_backupListCount] = oldInstance[ZaBackup.A_backupListCount];
		instance[ZaBackup.A_pageNumber] = oldInstance[ZaBackup.A_pageNumber];
		instance[ZaBackup.A_more] = (response.Body.BackupQueryResponse[ZaBackup.A_more] != null);
		instance.activeOperation = oldInstance.activeOperation - ZaHotBackup.SEARCH_OPERATION;
        if(response.Body.BackupQueryResponse.freeSpace) {
            instance[ZaServerBackup.A_freeSpace] = Math.floor(response.Body.BackupQueryResponse.freeSpace/(1024*1024));
        } else {
            instance[ZaServerBackup.A_freeSpace] = 0;
        }
        if(response.Body.BackupQueryResponse.totalSpace) {
            instance[ZaServerBackup.A_totalSpace] = Math.floor(response.Body.BackupQueryResponse.totalSpace/(1024*1024));
        } else {
            instance[ZaServerBackup.A_totalSpace] = 0;
        }
		if(response.Body.BackupQueryResponse.backup) {
			backup = response.Body.BackupQueryResponse.backup;
			if(!(backup instanceof Array)) {
				backup = [backup];
			}			
			var cnt = backup.length;
			for(var i =0; i < cnt; i++) {
				var bckpLabel = new ZaBackupLabel();
				bckpLabel.initFromJS(backup[i]);
				bckpLabel[ZaBackup.A_target] = instance[ZaBackup.A_target];
				bckpLabel[ZaBackup.A_serverId] = instance[ZaBackup.A_serverId];
				bckpLabel[ZaBackup.A_serverName] = instance[ZaBackup.A_serverName];				
				labelArray.push(bckpLabel);
			}
		}
		instance[ZaBackup.A_backupLabelList] =  labelArray;

		var backupsViewController = ZaApp.getInstance().getCurrentController();
		if(backupsViewController) {
			var popupMenu = backupsViewController.getPopUpMenu();
			if (popupMenu != null){
				popupMenu.enable([ZaOperation.ABORT_BACKUP], false);
			}
		}
		this._localXForm.setInstance(instance);
        if (instance[ZaBackup.A_more]) {
            var backupPool = this._localXForm.getItemById(ZaBackup.A_backupLabelList).getWidget();
            backupPool.setCrtInstance(instance);
            backupPool.setScrollHasMore(true);
        }
	}	
}

ZaBackupsXFormView.isPleaseWaitVisible = function () {
	return (this.getInstanceValue(ZaServerBackup.A_activateOperation) > 0);
}

ZaBackupsXFormView.getCustomWidth = function () {
    try {
        var parentPage = this.getForm().parent.getHtmlElement();
        var totalWidth = parseInt(parentPage.style.width);
        if (isNaN(totalWidth)) {
            totalWidth =  parentPage.clientWidth? parentPage.clientWidth : parentPage.offsetWidth;
        }
        return totalWidth;
    } catch(ex) {

    }
    return "100%";
}

ZaBackupsXFormView.getCustomHeight = function () {
    var parentPage = this.getForm().parent;
    try {
        var totalHeight =  parentPage.getSize().y;
        var formHeaders = this.getForm().getItemsById("header_group");
        var headHeight = 0;
        if (formHeaders) {
            var formHeader = formHeaders[0];
            if (formHeader.getContainer()) {
                formHeader = formHeader.getContainer();
            } else {
                formHeader = formHeader.getHtmlElement();
            }
            headHeight =  Dwt.getSize(formHeader).y;
        }
        var containerHeight = 2;

        if (AjxEnv.isIE)
            containerHeight = 3;
        if (totalHeight > (headHeight+containerHeight)) {
            return totalHeight - headHeight -containerHeight ;
        }

    } catch(ex) {

    }
    return "100%";
}

ZaBackupsXFormView.myXFormModifier = function(xFormObject) {	
	xFormObject.tableCssStyle="width:100%;overflow:auto;";
	
	
	var headerList = new Array();
	headerList[0] = new ZaListHeaderItem(ZaBackupLabel.A_label, com_zimbra_backuprestore.BNR_Label_col, null, "160px", null, null, true, true);
	headerList[1] = new ZaListHeaderItem(ZaBackupLabel.A_start, com_zimbra_backuprestore.BNR_StartTime_col, null, "200px", null, null, true, true);
	headerList[2] = new ZaListHeaderItem(ZaBackupLabel.A_end, com_zimbra_backuprestore.BNR_EndTime_col, null, "200px", null, null, true, true);	
	headerList[3] = new ZaListHeaderItem(ZaBackupLabel.A_status, com_zimbra_backuprestore.BNR_Status_col, null, "auto", null, null, true, true);		
	//headerList[4] = new ZaListHeaderItem(null, null, null, "auto", null, null, false, true);
	xFormObject.items = [
        {type:_GROUP_, id:"header_group", colSizes:["auto"], numCols:1, colSpan:2,  width:"100%", items:[
            {type:_GROUP_, visibilityChecks:[], colSizes:["250px","*","150px"], colSpan:2, numCols:3, width:"100%", id:"xform_header",
                items:[
                    {type:_TEXTFIELD_, width:"100%", ref:ZaBackup.A_target,
                        containerCssClass:"search_field_container", label:com_zimbra_backuprestore.BNR_TargetPath,
                        cssClass:"search_input", visibilityChecks:[], enableDisableChecks:[]
                    },
                    {type:_DWT_BUTTON_, label:com_zimbra_backuprestore.BNR_FindBackups, name: "backupsSearchButton",
                            onActivate:ZaBackupsXFormView.queryBackups, visibilityChecks:[], enableDisableChecks:[]}
                ]
            },
            {type:_GROUP_, colSpan: 2, numCols:4, colSizes:["25%", "25%","15%", "35%"], width:"100%",
                items:[
                    {type:_OUTPUT_, label:com_zimbra_backuprestore.BNR_FreeSpace, ref:ZaServerBackup.A_freeSpace},
                    {type:_OUTPUT_, label:com_zimbra_backuprestore.BNR_TotalSpace, ref:ZaServerBackup.A_totalSpace}
                ]
            },
            {type:_SPACER_, colSpan:2}
            ]
        },
		{type:_DWT_ALERT_,style: DwtAlert.INFORMATION, 
			iconVisible: true,
			content: ZaMsg.pleaseWait,width:"200px",
			align:_CENTER_,//cssStyle:"position:absolute;top:100;left:125;opacity: 0.8",
			visibilityChecks:[ZaBackupsXFormView.isPleaseWaitVisible],
			visibilityChangeEventSources:[ZaServerBackup.A_activateOperation]
		},
	    {ref:ZaBackup.A_backupLabelList, colSpan:2,cssClass: "ZaFullPageXFormListView",
	    	onSelection:ZaBackupsXFormView.labelSelectionListener, type:_DWT_LIST_, 
	   		forceUpdate: true,createPopupMenu:ZaBackupsXFormView.createPopupMenu,
	   		multiselect:false, widgetClass:ZaBackupListView,headerList:headerList,
	   		visibilityChecks:[], enableDisableChecks:[],
            getCustomWidth:ZaBackupsXFormView.getCustomWidth,
            getCustomHeight:ZaBackupsXFormView.getCustomHeight
	   	}
	];
};
ZaTabView.XFormModifiers["ZaBackupsXFormView"].push(ZaBackupsXFormView.myXFormModifier);