/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Server
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014 Zimbra, Inc.
 * 
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at: http://www.zimbra.com/license
 * The License is based on the Mozilla Public License Version 1.1 but Sections 14 and 15 
 * have been added to cover use of software over a computer network and provide for limited attribution 
 * for the Original Developer. In addition, Exhibit A has been modified to be consistent with Exhibit B. 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. 
 * See the License for the specific language governing rights and limitations under the License. 
 * The Original Code is Zimbra Open Source Web Client. 
 * The Initial Developer of the Original Code is Zimbra, Inc. 
 * All portions of the code are Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014 Zimbra, Inc. All Rights Reserved. 
 * ***** END LICENSE BLOCK *****
 */
if(window.console && window.console.log) console.log("Loaded ZaCertView.js");

function ZaCertView (parent,className) {
	if (arguments.length == 0) return ;
	className = className || "ZaCertView" ;
	var posStyle = DwtControl.ABSOLUTE_STYLE;
	DwtComposite.call (this, parent, className, posStyle) ;
	
	this._certInstallStatus = new DwtAlert (this) ;
	this._certInstallStatus.setIconVisible(false) ;
	this.setScrollStyle (Dwt.SCROLL);
	this._app = ZaApp.getInstance() ;
}

ZaCertView.prototype = new DwtComposite ;
ZaCertView.prototype.constructor = ZaCertView ; 

ZaCertView.prototype.toString = 
function() {
	return "ZaCertView";
}

ZaCertView.prototype.getTitle = 
function () {
	return com_zimbra_cert_manager.Cert_view_title;
}

ZaCertView.prototype.getTabIcon = 
function () {
	return "ViewCertificate" ;
}

ZaCertView.prototype.getTabTitle = 
function () {
	return com_zimbra_cert_manager.Cert_view_title + ": "  + this.getTargetServerName();
}

ZaCertView.prototype.getTabToolTip =
function () {
	return	this.getTitle ();
}

ZaCertView.prototype._setUI = function (certs) {
	if (!this._certContent) {
		this._certContent = new DwtComposite (this, null, DwtControl.ABSOLUTE_STYLE) ;
	}
	//Cert Install Status
	if (ZaCertWizard.INSTALL_STATUS < 0) {
		this._certInstallStatus.setDisplay (Dwt.DISPLAY_NONE) ;
	}else {
		this._certInstallStatus.setDisplay (Dwt.DISPLAY_BLOCK) ;
	}
	
	//Cert Content
	var html = [] ;
	
	html.push("<div style='padding-left:10px;'>") ;
	html.push("<h2>" + com_zimbra_cert_manager.CERT_server_name + " " + this.getTargetServerName() + "</h2>");

	if (certs && certs.cert) {
		for (var i=0; i < certs.cert.length; i ++) {
			var currentCert = certs.cert[i];
			var certType = currentCert.type ;
			var certInfo = this.getCertTable(currentCert) ;
			var title = AjxMessageFormat.format(com_zimbra_cert_manager.Cert_Service_title, certType) ;
			html.push("<h3>" + title + "</h3>") ;
			html.push(certInfo) ;
		}
	}else{
		html.push(com_zimbra_cert_manager.Cert_Info_Unavailable);
	}
	html.push("</div>") ;
	this._certContent.getHtmlElement().innerHTML = html.join("") ;	
}

ZaCertView.prototype.getCertTable = function (cert) {
	var html = [] ;	
	if (cert) {
		html.push("<table><colgroup><col width=160 /><col width='*' /></colgroup>") ;
		if (cert[ZaCert.A_subject] && cert[ZaCert.A_subject][0]) {
			html.push("<tr><td><strong>" + com_zimbra_cert_manager.CERT_INFO_SUBJECT + "</strong> " 
				+ "</td><td>" + cert[ZaCert.A_subject][0]._content 
				+ "</td></tr>") ;
		}
		if (cert.issuer && cert.issuer[0]) {
			html.push("<tr><td><strong>" + com_zimbra_cert_manager.CERT_INFO_ISSUER+ "</strong>" + "</td><td>" 
			+ cert.issuer[0]._content + "</td></tr>") ;
		}
		
		if (cert.notBefore && cert.notBefore[0] && cert.notAfter && cert.notAfter[0]) {
			html.push("<tr><td><strong>" + com_zimbra_cert_manager.CERT_INFO_VALIDATION_DAYS +"</strong>" 
					+ "</td><td> " + cert.notBefore[0]._content + " - " + cert.notAfter[0]._content + "</td></tr>") ;
		}
		
		if (cert[ZaCert.A_subject_alt] && cert[ZaCert.A_subject_alt][0]) {
			html.push("<tr><td><strong>" + com_zimbra_cert_manager.CERT_INFO_SubjectAltName + " </strong>" + "</td><td> " 
			+ cert[ZaCert.A_subject_alt][0]._content +  "</td></tr>") ;
		}
		html.push("</table>") ;
	}else{
		html.push (com_zimbra_cert_manager.Cert_Service_Unavailable);
	}
	return html.join("");
}                             

ZaCertView.prototype.set = function (certs, targetServerId) {
	if(window.console && window.console.log) console.log ("Set the certs") ;
    //this._containedObject is used to check if the item tab exists already
    if (!this._containedObject) this._containedObject = {} ;
    this._containedObject.certs = certs ;
    this._containedObject.id = targetServerId ;

	this.setTargetServerId(targetServerId);
	this._setUI (certs);
}

ZaCertView.prototype.setTargetServerId = function (targetServerId) {
	if (!targetServerId) 
		throw new AjxException (com_zimbra_cert_manager.NO_TARGET_SERVER_ERROR, "ZaCertView.prototype.setTargetServerId");
	this._targetServerId = targetServerId ;	
}

ZaCertView.prototype.getTargetServerId = function () {
	return this._targetServerId ;	
}

ZaCertView.prototype.getTargetServerName = function() {
	for (var i=0; i < ZaCert.TARGET_SERVER_CHOICES.length; i ++) {
		if (this.getTargetServerId() == ZaCert.TARGET_SERVER_CHOICES[i].value){
			return ZaCert.TARGET_SERVER_CHOICES[i].label ;
		}
	}
	
	return this.getTargetServerId() ;
}

ZaCertView.prototype.getTabChoices =
function() {
    this.tabChoices = new Array();
    this.tabChoices.push({value:1, label:ZaMsg.TABT_GeneralPage});
    return this.tabChoices;
}

