/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2009, 2010, 2011, 2012, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
 * Contains most of the domain admin functions
 */
if(ZaSettings && ZaSettings.EnabledZimlet["com_zimbra_delegatedadmin"]){

ZaDelegatedAdmin = function () {};

ZaDelegatedAdmin.isDelegatedAdmin = function (obj) {
    var isAdminAccount = (obj.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount] &&
                            (obj.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount] == "TRUE")) ;
    var isAdminGroup = (obj.attrs[ZaDistributionList.A_isAdminGroup] &&
                            (obj.attrs[ZaDistributionList.A_isAdminGroup] == "TRUE")) ;

    return isAdminAccount || isAdminGroup ;
}

if (ZaAccount) {
    ZaAccount.A2_adminRoles = "adminRoles" ;
    ZaAccount.adminRolesModelItem = {ref: ZaAccount.A2_adminRoles ,id: ZaAccount.A2_adminRoles,
                                    type: _LIST_, listItem:{type:_EMAIL_ADDRESS_}} ;
    ZaAccount.adminAccountModelItem = {id:ZaAccount.A_zimbraIsDelegatedAdminAccount, type:_ENUM_,
            choices:ZaModel.BOOLEAN_CHOICES,
            ref:"attrs/"+ZaAccount.A_zimbraIsDelegatedAdminAccount} ;
    ZaAccount.myXModel.items.push (ZaAccount.adminAccountModelItem) ;

    //admin roles model item
    ZaAccount.myXModel.items.push (ZaAccount.adminRolesModelItem) ;

    ZaAccount.A2_isAssignDefaultDARights = "isAssignDefaultDARights" ;
    ZaAccount.myXModel.items.push ({
        id: ZaAccount.A2_isAssignDefaultDARights, type: _ENUM_, choices:ZaModel.BOOLEAN_CHOICES,
        ref: ZaAccount.A2_isAssignDefaultDARights
    }) ;

    ZaAccount._deleteOldDirectMember = function (item, oldVal, form) {
    	if(!item.getInstance() [ZaAccount.A2_memberOf]) {
	   item.getInstance() [ZaAccount.A2_memberOf] = {};
	}

        if(!item.getInstance() [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList]) {
	   item.getInstance() [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList] = [];
	}
        
	var directMemberOfList = item.getInstance()[ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList];
        var i = ZaUtil.findValueInObjArrByPropertyName(directMemberOfList, oldVal, "name");
 	if (i >= 0){
           directMemberOfList.splice (i, 1);
           form.getModel().setInstanceValue(item.getInstance(), ZaAccount.A2_directMemberList, directMemberOfList);
           if (form.parent.setDirty){
		form.parent.setDirty (true) ;
           }
	}    
    } 

    ZaAccount.changeAdminRoles = function (value, event, form) {
        var oldVal = this.getInstanceValue();
        if(oldVal == value)
            return;

        this.setInstanceValue(value);

        //check if the value is valid admin group
        var adminGroupChoices = this.getChoices () ;
        if (adminGroupChoices && adminGroupChoices.getChoiceByValue(value) != null) {
             var adminGroupId = adminGroupChoices.getChoiceByValue(value).id ;
        } else {
	    // at this point, we don't think this is error
            ZaAccount._deleteOldDirectMember(this, oldVal, form);
            if (value == ""){
		if(this.hasError()){
                    this.clearError();	
		}
            }
            else{
              this.setError(com_zimbra_delegatedadmin.ERROR_INVALID_ADMIN_ROLE);
              var event = new DwtXFormsEvent(form, this, value);
              form.notifyListeners(DwtEvent.XFORMS_VALUE_ERROR, event);
            }
            return ;
        }
        
         //add this value to the  direct member
        if (!this.getInstance () [ZaAccount.A2_memberOf]) {
            this.getInstance () [ZaAccount.A2_memberOf] = {} ;          
        }

        if (!this.getInstance () [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList]) {
            this.getInstance () [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList] = [];   
        }

        var directMemberOfList = this.getInstance () [ZaAccount.A2_memberOf] [ZaAccount.A2_directMemberList] ;
        if (value && value.length > 0 && ZaUtil.findValueInObjArrByPropertyName(directMemberOfList, value, "name") < 0){
            directMemberOfList.push ({
                id: adminGroupId,
                name: value
            }) ;

            form.getModel().setInstanceValue(this.getInstance(), ZaAccount.A2_directMemberList, directMemberOfList) ;
        }
	if (ZaAccount.getDomain(form.getModel().getInstanceValue(this.getInstance(), ZaAccount.A_name))!=ZaAccount.getDomain(value)){
        ZaApp.getInstance().getCurrentController().popupWarningDialog(com_zimbra_delegatedadmin.Alert_SingleDomain);
        }

    }

    ZaAccount.onAdminRoleRemove = function (index, form) {
        var value = this.getInstanceValue () [index] ;
        var path = this.getRefPath();
		this.getModel().removeRow(this.getInstance(), path, index);
        this.items[index].clearError();

        //update the memberOf instance value
        if (!this.getInstance () [ZaAccount.A2_memberOf]) {
            this.getInstance () [ZaAccount.A2_memberOf] = {} ;
        }

        if (!this.getInstance () [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList]) {
            this.getInstance () [ZaAccount.A2_memberOf][ZaAccount.A2_directMemberList] = [];
        }
        
        var directMemberOfList = this.getInstance () [ZaAccount.A2_memberOf] [ZaAccount.A2_directMemberList] ;
        var i = ZaUtil.findValueInObjArrByPropertyName(directMemberOfList, value, "name")  ; 
        if (i >= 0){
            directMemberOfList.splice (i, 1) ;
            form.getModel().setInstanceValue(this.getInstance(), ZaAccount.A2_directMemberList, directMemberOfList) ;
            if (form.parent.setDirty) form.parent.setDirty (true) ;
        }
    }

    ZaController.prototype.assignDefaultDARights = function (item, value, ev) {
        try {
            var isAssignDefaultDARights = (value && value == "TRUE");
            var isDelegatedAdminValue = item.getInstanceValue(ZaAccount.A_zimbraIsDelegatedAdminAccount) ;
            var isDelegatedAdmin = (isDelegatedAdminValue && isDelegatedAdminValue == "TRUE") ;
            var isDelegatedGroupValue = item.getInstanceValue(ZaDistributionList.A_isAdminGroup) ;
            var isDelegatedGroup = (isDelegatedGroupValue && isDelegatedGroupValue == "TRUE") ;

            if ((isDelegatedAdmin || isDelegatedGroup) && isAssignDefaultDARights)  {
    //            console.log("Adding the DA Rights and views ...") ;
//                var tmpObject = item.getInstance() ;
                  var tmpObject = ev.getDetails (); //it should be the object after the event occured, especially useful to get the newly created account id
                //1. Modify the admin view
                var defaultAdminView = ZaUtil.cloneArray (ZaNewAdminWizard.LEGACY_DA_VIEW) ;
                if (isDelegatedAdmin) {
                    tmpObject [ZaNewAdmin.A_admin_type] = ZaItem.ACCOUNT ;
                }else if (isDelegatedGroup) {
                    tmpObject [ZaNewAdmin.A_admin_type] = ZaItem.DL ;
                }

                var uiComponents = tmpObject.attrs[ZaAccount.A_zimbraAdminConsoleUIComponents] || [] ;
                tmpObject.attrs[ZaAccount.A_zimbraAdminConsoleUIComponents] = AjxUtil.mergeArrays(uiComponents, defaultAdminView);
                if (ZaNewAdmin.modifyAdmin (tmpObject)) {
                    if (this._currentObject && this._currentObject.attrs) { //it is not applicable to the new account creation
                        this._currentObject [ZaAccount.A2_isAssignDefaultDARights] = "TRUE" ;
                        this._currentObject.attrs [ZaAccount.A_zimbraAdminConsoleUIComponents]
                                    = tmpObject.attrs [ZaAccount.A_zimbraAdminConsoleUIComponents] ;
                    }
                }

                //2. Modify the account right
                var defaultRights = ZaNewAdminWizard.getDefaultDARights (tmpObject) ;

                ZaGrant.assignMultiGrants.call(this, {
                        currentIndex: 0,
                        rightsArr: defaultRights,
                        showStatus: -1
                }) ;
            }else {
    //            console.log("No need to add the DA Rights ...") ;
            }
        }catch (ex) {
            this.popupErrorDialog(com_zimbra_delegatedadmin.ERROR_ASSIGN_DEFAULT_DA_RIGHTS,ex);	
        } finally {
            //we should remove the event change handler if everything runs fine.
            this.removeChangeListener (this._assignDefaultDARightslistener) ;
            this.removeCreationListener (this._assignDefaultDARightslistener) ;
        }
    }

    ZaAccount.getAssignDefaultDARightsChkBoxItem = function (w) {
        if (w == null) w = "300px" ;
        var ckb = { type: _GROUP_, colSpan: "*", numCols:2, colSizes: [w, "*"], items: [
                {
                    ref: ZaAccount.A2_isAssignDefaultDARights ,
                    type: _CHECKBOX_ ,
                    label: com_zimbra_delegatedadmin.NAD_IsAssignDefaultDARights,
                    bmolsnr: true ,
                    visibilityChangeEventSources: [ZaAccount.A_zimbraIsDelegatedAdminAccount, ZaDistributionList.A_isAdminGroup] ,
                    enableDisableChecks: [],
                    onChange :
                        function(value, event, form) {
                            var controller = ZaApp.getInstance ().getCurrentController () ;

                            var isDelegatedAdmin = this.getInstanceValue(ZaAccount.A_zimbraIsDelegatedAdminAccount) ;
                            var isAdminGroup = this.getInstanceValue (ZaDistributionList.A_isAdminGroup);

                            if (!controller._assignDefaultDARightslistener) {
                                controller._assignDefaultDARightslistener =  new AjxListener (controller, ZaController.prototype.assignDefaultDARights, [this, value]) ;
                            }
                            if(value == "TRUE" &&
                                ((isDelegatedAdmin && isDelegatedAdmin == "TRUE")
                                        || (isAdminGroup && isAdminGroup == "TRUE"))) {
                                controller.addChangeListener (controller._assignDefaultDARightslistener) ;
                                controller.addCreationListener (controller._assignDefaultDARightslistener)  ;
                            }else {
                                controller.removeChangeListener (controller._assignDefaultDARightslistener) ;
                                controller.removeCreationListener (controller._assignDefaultDARightslistener) ;
                            }

                            this.setInstanceValue(value);
                    },
                    trueValue:"TRUE", falseValue:"FALSE"
                }
            ]
        };
        return ckb ;
    }

    ZaAccount.getAdminChkBoxItem = function () {
        var adminChkBox = {
            ref:ZaAccount.A_zimbraIsDelegatedAdminAccount,type:_CHECKBOX_,
            label: com_zimbra_delegatedadmin.NAD_IsAdmin,
            bmolsnr:true,
            elementChanged :
            function(elementValue,instanceValue, event) {
                if(elementValue == "TRUE") {
                    var isSystemAdmin = this.getInstanceValue(ZaAccount.A_zimbraIsAdminAccount) ;
                    if ( isSystemAdmin && isSystemAdmin == "TRUE" )
                        this.setInstanceValue("FALSE", ZaAccount.A_zimbraIsAdminAccount);
                }
                    this.getForm().itemChanged(this, elementValue, event);
            },
            trueValue:"TRUE", falseValue:"FALSE"
        };

        return adminChkBox;
    }

    ZaAccount.getAdminRolesItem = function () {
       var adminRoleField = {
            ref: ".", type:_DYNSELECT_,
            dataFetcherMethod:ZaSearch.prototype.dynSelectSearchAdminGroups,
            onChange: ZaAccount.changeAdminRoles ,
            emptyText:com_zimbra_delegatedadmin.searchTermAdminGroup,
            enableDisableChecks:[],
            visibilityChecks: [],
			dataFetcherClass:ZaSearch,editable:true,
			autoCompleteEnabled:true
       }

       var adminRolesItem = {
           ref: ZaAccount.A2_adminRoles , type: _REPEAT_,
           label: com_zimbra_delegatedadmin.Label_AssignAdminRole, labelLocation:_LEFT_ ,
           labelCssStyle:"vertical-align: top; padding-top: 3px;",
           align:_LEFT_,
           repeatInstance:"",
           showAddButton:true, showAddOnNextRow:true,
            //addButtonWidth: 100, removeButtonWidth: 100,
           addButtonLabel:com_zimbra_delegatedadmin.NAD_Add,
           showRemoveButton:true, removeButtonLabel:com_zimbra_delegatedadmin.NAD_Remove,
           enableDisableChecks: [],
           visibilityChecks:[],
           visibilityChangeEventSources: [ZaAccount.A_zimbraIsDelegatedAdminAccount, ZaDistributionList.A_isAdminGroup] ,

           onRemove:ZaAccount.onAdminRoleRemove,
           items:[adminRoleField]
       }

        return adminRolesItem ;

    }
}

ZaDelegatedAdmin.accountObjectModifer = function () {
    var directMemberOfList = this._containedObject [ZaAccount.A2_memberOf] [ZaAccount.A2_directMemberList] ;
    if (! this._containedObject [ZaAccount.A2_adminRoles]) this._containedObject [ZaAccount.A2_adminRoles] = [];
    if(directMemberOfList && directMemberOfList.length) {
	    for (var i = 0; i < directMemberOfList.length; i ++) {
	    // TODO: enable it when GetAccountMembershipRequest returns isAdminGroup
			if (directMemberOfList[i][ZaDistributionList.A_isAdminGroup] == "TRUE") {
	        	this._containedObject [ZaAccount.A2_adminRoles].push (directMemberOfList[i].name) ;
			}
	    }
    }
}

if (ZaTabView.ObjectModifiers["ZaAccountXFormView"]){
    ZaTabView.ObjectModifiers["ZaAccountXFormView"].push(ZaDelegatedAdmin.accountObjectModifer) ;
}


if (ZaTabView.XFormModifiers["ZaAccountXFormView"]) {
   ZaDelegatedAdmin.AccountXFormModifier = function (xFormObject) {
       var adminChkBox = ZaAccount.getAdminChkBoxItem ();
       var adminRolesItem = ZaAccount.getAdminRolesItem () ;
       adminChkBox.visibilityChecks = [[ZaItem.hasReadPermission,ZaAccount.A_zimbraIsDelegatedAdminAccount],[XForm.checkInstanceValueNot,ZaAccount.A_zimbraIsExternalVirtualAccount,"TRUE"]] ;
       adminChkBox.enableDisableChecks = [[ZaItem.hasWritePermission,ZaAccount.A_zimbraIsDelegatedAdminAccount]] ;
       adminRolesItem.visibilityChecks = [[XForm.checkInstanceValue, ZaAccount.A_zimbraIsDelegatedAdminAccount, "TRUE"],
       [ZaItem.hasRight, ZaAccount.GET_ACCOUNT_MEMBERSHIP_RIGHT],[XForm.checkInstanceValueNot,ZaAccount.A_zimbraIsExternalVirtualAccount,"TRUE"]] ;
       adminRolesItem.enableDisableChecks = [[ZaItem.hasWritePermission,ZaAccount.A_zimbraIsDelegatedAdminAccount]] ;

       var assignDefaultDARightsItem = ZaAccount.getAssignDefaultDARightsChkBoxItem() ;

       //we should only allow it to show up for global admin now
       assignDefaultDARightsItem.items[0].visibilityChecks = [
           /*[ZaZimbraAdmin.isGlobalAdmin],*/
           [XForm.checkInstanceValue, ZaAccount.A_zimbraIsDelegatedAdminAccount, "TRUE"],
           [ZaItem.hasWritePermission,ZaAccount.A_zimbraAdminConsoleUIComponents]
       ];
      assignDefaultDARightsItem= assignDefaultDARightsItem.items[0];

        var tabs = xFormObject.items[2].items;
        var tmpItems = tabs[0].items;
        var cnt = tmpItems.length;
        for(var i = 0; i < cnt; i ++) {
           if(tmpItems[i].id == "account_form_setup_group" && tmpItems[i].items) {
               var tmpGrouperItems = tmpItems[i].items;
               var cnt2 = tmpGrouperItems.length;
               for(var j=0;j<cnt2;j++) {
                   if(tmpGrouperItems[j] && tmpGrouperItems[j].ref == ZaAccount.A_zimbraIsAdminAccount) {
                       //add  Admin checkbox
                       xFormObject.items[2].items[0].items[i].items.splice(j+1,0,
                               adminChkBox, assignDefaultDARightsItem, adminRolesItem);
                       
                       //add the mutual exclusive action to global admin 
                       tmpGrouperItems[j].elementChanged =
								function(elementValue,instanceValue, event) {
									if(elementValue == "TRUE") {
                                        var isAdmin = this.getInstanceValue(ZaAccount.A_zimbraIsDelegatedAdminAccount) ;
                                        if ( isAdmin && isAdmin == "TRUE" )
                                            this.setInstanceValue("FALSE", ZaAccount.A_zimbraIsDelegatedAdminAccount);
								    }
								    this.getForm().itemChanged(this, elementValue, event);
								};
                       break;
                   }
               }
               break;
           }
       }
   }

    ZaTabView.XFormModifiers["ZaAccountXFormView"].push(ZaDelegatedAdmin.AccountXFormModifier);
}

//for new account wizard
/*
if (ZaTabView.ObjectModifiers["ZaNewAccountXWizard"]){
    ZaTabView.ObjectModifiers["ZaNewAccountXWizard"].push(ZaDelegatedAdmin.accountObjectModifer) ;
} */


if (ZaXDialog.XFormModifiers["ZaNewAccountXWizard"]) {
   ZaDelegatedAdmin.newAccountModifier = function (xFormObject) {
       var adminChkBox = ZaAccount.getAdminChkBoxItem ();
       adminChkBox.labelLocation = _RIGHT_;
       adminChkBox.align = _RIGHT_;
       adminChkBox.subLabel = "";
       var adminRolesItem = ZaAccount.getAdminRolesItem () ;
       adminChkBox.visibilityChecks = [[ZaItem.hasWritePermission,ZaAccount.A_zimbraIsDelegatedAdminAccount]] ;
       adminRolesItem.visibilityChecks = [[XForm.checkInstanceValue, ZaAccount.A_zimbraIsDelegatedAdminAccount, "TRUE"]] ;
       adminRolesItem.enableDisableChecks = [[ZaItem.hasWritePermission,ZaAccount.A_zimbraIsDelegatedAdminAccount]] ;

       var assignDefaultDARightsItem = ZaAccount.getAssignDefaultDARightsChkBoxItem("220px") ;
       assignDefaultDARightsItem.items[0].labelLocation = _RIGHT_;
       assignDefaultDARightsItem.items[0].align = _RIGHT_;
       assignDefaultDARightsItem.items[0].subLabel = "";
       assignDefaultDARightsItem.items[0].visibilityChecks = [
           /*[ZaZimbraAdmin.isGlobalAdmin],*/    
           [XForm.checkInstanceValue, ZaAccount.A_zimbraIsDelegatedAdminAccount, "TRUE"],
           [ZaItem.hasWritePermission,ZaAccount.A_zimbraAdminConsoleUIComponents]
       ];

        var tabs = xFormObject.items[3].items;
        var tmpItems = tabs[0].items;
        var cnt = tmpItems.length;
        for(var i = 0; i < cnt; i ++) {
           if(tmpItems[i].id == "account_wiz_setup_group" && tmpItems[i].items) {
               var tmpGrouperItems = tmpItems[i].items;
               var cnt2 = tmpGrouperItems.length;
               for(var j=0;j<cnt2;j++) {
                   if(tmpGrouperItems[j] && tmpGrouperItems[j].ref == ZaAccount.A_zimbraIsAdminAccount) {
                       //add  Admin checkbox
                      tmpItems[i].items.splice(j+1,0, adminChkBox, assignDefaultDARightsItem, adminRolesItem);

                       //add the mutual exclusive action to global admin
                       tmpGrouperItems[j].elementChanged =
								function(elementValue,instanceValue, event) {
									if(elementValue == "TRUE") {
                                        var isAdmin = this.getInstanceValue(ZaAccount.A_zimbraIsDelegatedAdminAccount) ;
                                        if ( isAdmin && isAdmin == "TRUE" )
                                            this.setInstanceValue("FALSE", ZaAccount.A_zimbraIsDelegatedAdminAccount);
								    }
								    this.getForm().itemChanged(this, elementValue, event);
								};
                       break;
                   }
               }
               break;
           }
       }
   }

    ZaXDialog.XFormModifiers["ZaNewAccountXWizard"].push(ZaDelegatedAdmin.newAccountModifier);
}

ZaDelegatedAdmin.accountViewMethod =
function (entry) {
    if (entry.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount]
            && entry.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount] == "TRUE"
            && ZaItem.hasRight (ZaAccount.GET_ACCOUNT_MEMBERSHIP_RIGHT, entry) ) {
        this._view._containedObject[ZaAccount.A2_adminRoles] = [] ;
        //Get the isAdminAccount DLs from the directMemberList
        var allDirectMemberOfs = this._view._containedObject [ZaAccount.A2_memberOf] [ZaAccount.A2_directMemberList] ;
        if (allDirectMemberOfs != null) {
            for (var i = 0; i < allDirectMemberOfs.length; i ++) {
            //enable it when GetAccountMembershipRequest returns isAdminGroup
            if (allDirectMemberOfs[i][ZaDistributionList.A_isAdminGroup] == "TRUE")
                    this._view._containedObject[ZaAccount.A2_adminRoles].push (allDirectMemberOfs[i].name) ;
            }
        }

        var xform = this._view._localXForm ;
        var instance  = xform.getInstance ();
        xform.getModel().setInstanceValue(instance,ZaAccount.A2_adminRoles,
                 this._view._containedObject[ZaAccount.A2_adminRoles]);
    }
}

if (ZaController.setViewMethods["ZaAccountViewController"]) {
	ZaController.setViewMethods["ZaAccountViewController"].push(ZaDelegatedAdmin.accountViewMethod);
}

if (ZaDistributionList) {
    ZaDistributionList.myXModel.items.push (
        {id:ZaDistributionList.A_isAdminGroup, type:_ENUM_, choices:ZaModel.BOOLEAN_CHOICES,
            ref:"attrs/"+ZaDistributionList.A_isAdminGroup}) ;

    ZaDistributionList.myXModel.items.push ({
        id: ZaAccount.A2_isAssignDefaultDARights, type: _ENUM_, choices:ZaModel.BOOLEAN_CHOICES,
        ref: ZaAccount.A2_isAssignDefaultDARights
    }) ;
}
     

if (ZaTabView.XFormModifiers["ZaDLXFormView"]) {
   ZaDelegatedAdmin.DLXFormModifier = function (xFormObject) {
       var adminGroupChkBx =
            {
                ref: ZaDistributionList.A_isAdminGroup,type:_CHECKBOX_,
                label:com_zimbra_delegatedadmin.LBL_IsAdminGroup,
                labelLocation:_LEFT_,  align:_LEFT_,
				labelCssClass:"xform_label", cssStyle:"padding-left:0px",
                nowrap:false,labelWrap:true,
                trueValue:"TRUE", falseValue:"FALSE",
                visibilityChecks: [[XForm.checkInstanceValueNot, ZaDistributionList.A_zimbraIsACLGroup, "FALSE"],[XForm.checkInstanceValueNot, ZaDistributionList.A2_dlType, ZaDistributionList.DYNAMIC_DL_TYPE]]
                //if ZaDistributionList.A_zimbraIsACLGroup is TRUE or no value, we should show it
            }  ;

       var assignDefaultDARightsItem = ZaAccount.getAssignDefaultDARightsChkBoxItem("130px") ;
       assignDefaultDARightsItem.items[0].labelLocation = _RIGHT_;
       assignDefaultDARightsItem.items[0].align = _RIGHT_;
       assignDefaultDARightsItem.items[0].subLabel = "";
       assignDefaultDARightsItem.items[0].visibilityChecks = [
           /*[ZaZimbraAdmin.isGlobalAdmin],*/
           [XForm.checkInstanceValue, ZaDistributionList.A_isAdminGroup, "TRUE"],
           [ZaItem.hasWritePermission,ZaAccount.A_zimbraAdminConsoleUIComponents]
       ];

       var switchGroupItems ;
        for (var i=0; i < xFormObject.items.length; i ++) {
            if (xFormObject.items[i].type == _SWITCH_) {
                switchGroupItems = xFormObject.items[i].items ;
                break ;
            }
        }

        var membersView, tmpGroup;
        for (var j=0; j < switchGroupItems.length; j ++) {
            if ((switchGroupItems[j].type == _ZATABCASE_ )
                    && (switchGroupItems[j].id == "dl_form_properties")) {
                membersView = switchGroupItems[j].items ;
                for (var m=0; m < membersView.length; m ++) {
                    if (membersView[m].id == "dl_properties") {
                        for (var n=0; n < membersView[m].items.length; n ++ ) {
                            if (membersView[m].items[n].ref == "zimbraMailStatus") {
                                membersView[m].items.splice (n,0, adminGroupChkBx, assignDefaultDARightsItem) ;
                                break;
                            }
                        }
                        break ;
                    }
                }
                break ;
            }
        }

//        permissionView.items.splice(0, 0, adminGroupChkBx);
   }

   ZaTabView.XFormModifiers["ZaDLXFormView"].push(ZaDelegatedAdmin.DLXFormModifier);

}

if (ZaSearch) {
	/**
 	* @argument callArgs {value, event, callback}
 	*/
    ZaSearch.prototype.dynSelectSearchAdminGroups =  function (callArgs) {
        var extraLdapQuery = "(zimbraIsAdminGroup=TRUE)" ;
		var value = callArgs["value"];
		var event = callArgs["event"];
		var callback = callArgs["callback"];
		        
        ZaSearch.prototype.dynSelectSearchGroups.call (this, {value:value, event:event, callback:callback, extraLdapQuery:extraLdapQuery}) ;
    }
}

}




