/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
* @class ZaRestore
* @contructor ZaRestore
* this class is a model for doing backup and restore operations
* @author Greg Solovyev
**/
function ZaRestore() {
	ZaItem.call(this, "ZaRestore");
	this[ZaModel.currentStep] = 1;
	this.restoreRequest = new Object();
	this.accounts = new Array();
	this.accounts._version = 1;
	this.restoreRequest[ZaRestore.A_accountName] = "";
	this.restoreRequest[ZaRestore.A_accountNames] = new Array();
	this.restoreRequest[ZaRestore.A_restoreMethod] = ZaRestore.RA;
	this.restoreRequest[ZaRestore.A_includeIncrementals] = "FALSE";	//soap servlet uses 'TRUE'/'FALSE' literal values everywhere else
	this.restoreRequest[ZaRestore.A_prefix] = "restored_";		
	this.restoreRequest[ZaRestore.A_toServer] = "";	
	this.restoreRequest[ZaRestore.A_target] = ZaHotBackup.DEFAULT_BACKUP_TARGET;				
	this.restoreRequest[ZaRestore.A_restoreAllAcc] = true;
	this.restoreRequest[ZaRestore.A_restoreTo] = ZaRestore.TO_LATEST;
	this.restoreRequest[ZaRestore.A_skipDeletedAccounts] = false;
    this.restoreRequest[ZaRestore.A_ignoreRedoErrors] = false;
    this.restoreRequest[ZaRestore.A_skipDeleteOps] = false;
	this.restoreRequest[ZaRestore.A_restoreToTime] = new Date();
}

ZaRestore.prototype = new ZaItem;
ZaRestore.prototype.constructor = ZaRestore;

ZaRestore.MB = "mb";
ZaRestore.RA = "ra";
ZaRestore.CA = "ca";

ZaRestore.TO_LABEL = "toLabel";
ZaRestore.TO_PIT = "toPIT";
ZaRestore.TO_LATEST = "toLatest";

ZaRestore.RESTORE_CREATE_CHOICES = [ZaRestore.RA, ZaRestore.CA];
ZaRestore.A_searchStatusMsg = "searchStatusMsg";
ZaRestore.A_accountWarning = "accountWarning";
ZaRestore.A_accountId = "accountId";
ZaRestore.A_currentAccountId = "currentAccountId";
ZaRestore.A_accountNames = "accountNames";
ZaRestore.A_accountName = "accountName";
ZaRestore.A_accounts = "accounts";
ZaRestore.A_label = "label";
ZaRestore.A_prefix = "prefix";
ZaRestore.A_target="target";
ZaRestore.A_restoreMethod = "method";
ZaRestore.A_restoreTo = "restoreTo";
ZaRestore.A_toServer="toServer";
ZaRestore.A_includeIncrementals = "includeIncrementals";
ZaRestore.A_status = "status";
ZaRestore.A_status_msg = "status_msg";
ZaRestore.A_status_PENDING = "pending";
ZaRestore.A_status_RESTORING = "restoring";
ZaRestore.A_status_OK = "ok";
ZaRestore.A_status_INTERRUPTED = "interrupted";
ZaRestore.A_status_ABORTING = "aborting";
ZaRestore.A_status_ERROR = "err";
ZaRestore.A_restoreAllAcc = "restoreAllAcc";
ZaRestore.A_skipDeletedAccounts = "skipDeletedAccounts";
ZaRestore.A_restoreToTime = "restoreToTime";
ZaRestore.A_stop = "stop";
ZaRestore.A_currentAccount = "currentAccount";
ZaRestore.A_foundAnyBackups = "foundAnyBackups";
ZaRestore.A_ignoreRedoErrors = "ignoreRedoErrors";
ZaRestore.A_skipDeleteOps = "skipDeleteOps";
ZaRestore.A_foundAnyFullBackups = "foundAnyFullBackups";
			
BNRAccountSummary_XModelItem = function (){}
XModelItemFactory.createItemType("_BNR_ACCOUNTSUMMARY_", "bnr_accountsummary", BNRAccountSummary_XModelItem);
BNRAccountSummary_XModelItem.prototype.items = [
	{id:ZaRestore.A_accountName, type:_STRING_},
	{id:ZaRestore.A_accountId, type:_STRING_},
	{id:ZaRestore.A_currentAccountId, type:_STRING_},			
	{id:ZaRestore.A_prefix, type:_STRING_},
	{id:ZaRestore.A_toServer, type:_STRING_},
	{id:ZaRestore.A_restoreToTime, type:_DATETIME_},
	{id:ZaRestore.A_target, type:_STRING_},
	{id:ZaRestore.A_accountWarning,type:_STRING_},
	{id:ZaRestore.A_status, type:_STRING_},
	{id:ZaRestore.A_status_msg, type:_STRING_},	
	{id:ZaModel.ErrorCode, type:_STRING_},
	{id:ZaModel.ErrorMessage, type:_STRING_},
	{id:ZaRestore.A_restoreMethod, type:_ENUM_, ref:ZaRestore.A_restoreMethod, choices:ZaRestore.RESTORE_CREATE_CHOICES}
];
				
ZaRestore.myXModel = new Object();
ZaRestore.myXModel.items = new Array();

ZaRestore.myXModel.items.push({id:ZaModel.ErrorCode, type:_STRING_});
ZaRestore.myXModel.items.push({id:ZaModel.ErrorMessage, type:_STRING_});
ZaRestore.myXModel.items.push({id:ZaRestore.A_status, type:_STRING_});
ZaRestore.myXModel.items.push({id:ZaRestore.A_status_msg, type:_STRING_});
ZaRestore.myXModel.items.push({id:ZaRestore.A_accounts, type:_LIST_, listItem:{type:_BNR_ACCOUNTSUMMARY_}});
ZaRestore.myXModel.items.push({id:ZaRestore.A_searchStatusMsg, type:_STRING_});
ZaRestore.myXModel.items.push({id:ZaRestore.A_includeIncrementals, ref:"restoreRequest/" + ZaRestore.A_includeIncrementals, type:_ENUM_, choices:ZaModel.BOOLEAN_CHOICES});
ZaRestore.myXModel.items.push({id:ZaRestore.A_accountName, ref:"restoreRequest/" + ZaRestore.A_accountName, type:_EMAIL_ADDRESS_/*pattern:AjxUtil.EMAIL_SHORT_RE, pattern:/^([a-zA-Z0-9_\-])+((\.)?([a-zA-Z0-9_\-])+)*@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/*/});
ZaRestore.myXModel.items.push({id:ZaRestore.A_accountNames, type:_LIST_, ref:"restoreRequest/" + ZaRestore.A_accountNames, listItem:{type:_EMAIL_ADDRESS_ /*pattern:AjxUtil.EMAIL_SHORT_RE*/}});
ZaRestore.myXModel.items.push({id:ZaRestore.A_restoreMethod, type:_ENUM_, ref:"restoreRequest/" + ZaRestore.A_restoreMethod, choices:ZaRestore.RESTORE_CREATE_CHOICES});
ZaRestore.myXModel.items.push({id:ZaRestore.A_target, type:_STRING_, ref:"restoreRequest/"+ZaRestore.A_target});
ZaRestore.myXModel.items.push({id:ZaRestore.A_prefix, type:_STRING_, ref:"restoreRequest/"+ZaRestore.A_prefix,
				 constraints: {type:"method", value:
                        		function (value, form, formItem, instance) {
                                		if(value && AjxEmailAddress.accountPat.test(value)){
                                                	return value;
                                        	} else if(!value) {
							throw com_zimbra_backuprestore.BNR_NullPrefix;
						}
						else{
                                                	throw com_zimbra_backuprestore.BNR_InvalidPrefix;
                                        	}
                        		}
                		}
        			});
ZaRestore.myXModel.items.push({id:ZaRestore.A_label, type:_STRING_, ref:"restoreRequest/"+ZaRestore.A_label});
ZaRestore.myXModel.items.push({id:ZaRestore.A_toServer, type:_STRING_, ref:"restoreRequest/"+ZaRestore.A_toServer});
ZaRestore.myXModel.items.push({id:ZaRestore.A_restoreToTime, type:_DATETIME_, ref:"restoreRequest/"+ZaRestore.A_restoreToTime});
ZaRestore.myXModel.items.push({id:ZaRestore.A_restoreTo, type:_STRING_, ref:"restoreRequest/"+ZaRestore.A_restoreTo});
ZaRestore.myXModel.items.push({id:ZaModel.currentStep, type:_NUMBER_, ref:ZaModel.currentStep});
ZaRestore.myXModel.items.push({id:ZaRestore.A_restoreAllAcc, type:_ENUM_, choices:[true,false], ref:"restoreRequest/"+ZaRestore.A_restoreAllAcc});
ZaRestore.myXModel.items.push({id:ZaRestore.A_skipDeletedAccounts, type:_ENUM_, choices:[true,false], ref:"restoreRequest/"+ZaRestore.A_skipDeletedAccounts});
ZaRestore.myXModel.items.push({id:ZaRestore.A_foundAnyBackups, type:_ENUM_, choices:[true,false]});
ZaRestore.myXModel.items.push({id:ZaRestore.A_foundAnyFullBackups, type:_ENUM_, choices:[true,false], defaultValue:false});
ZaRestore.myXModel.items.push({id:ZaRestore.A_ignoreRedoErrors, type:_ENUM_, choices:[true,false], ref:"restoreRequest/"+ZaRestore.A_ignoreRedoErrors});
ZaRestore.myXModel.items.push({id:ZaRestore.A_skipDeleteOps, type:_ENUM_, choices:[true,false], ref:"restoreRequest/"+ZaRestore.A_skipDeleteOps});
ZaRestore.abortRestore = function(restoreRequest, callback) {
	var soapDoc = AjxSoapDoc.create("RestoreRequest", "urn:zimbraAdmin", null);
	var restoreEl = soapDoc.set("restore", "");
	restoreEl.setAttribute("method", "abort");
	this.asynCommand = new ZmCsfeCommand();
	var params = new Object();
	params.soapDoc = soapDoc;
	params.noAuthToken = true;	
	if(callback) {
		params.asyncMode = true;
		params.callback = callback;
		params.targetServer = restoreRequest[ZaRestore.A_toServer];
	}
	this.asynCommand.invoke(params)	
}
/**
* @method static restoreAccount
* @param restoreAccount:object  
* @param callback:AjxCallback
**/
ZaRestore.restoreAccount = 
function (restoreRequest, callback) {
	var soapDoc = AjxSoapDoc.create("RestoreRequest", "urn:zimbraAdmin", null);
	var restoreEl = soapDoc.set("restore", "");
	if(!restoreRequest[ZaRestore.A_restoreMethod]) {
		throw(new AjxException("method parameter cannot be null", AjxException.INVALID_PARAM, "ZaRestore.restoreAccount", com_zimbra_backuprestore.ERROR_RESTORE_3));
	}

	if(restoreRequest[ZaRestore.A_restoreMethod] == ZaRestore.CA && (!restoreRequest[ZaRestore.A_prefix] || restoreRequest[ZaRestore.A_prefix].length < 1)) {
		throw(new AjxException(com_zimbra_backuprestore.BNR_MustGivePrefix, AjxException.INVALID_PARAM, "ZaRestore.restoreAccount", com_zimbra_backuprestore.ERROR_RESTORE_1));
	}
	restoreEl.setAttribute("method", restoreRequest[ZaRestore.A_restoreMethod]);
	/*if(label) {
		restoreEl.setAttribute("label", label);
	}*/
	if(restoreRequest[ZaRestore.A_target]) {
		restoreEl.setAttribute("target", restoreRequest[ZaRestore.A_target]);
	}
	if(restoreRequest[ZaRestore.A_prefix] && restoreRequest[ZaRestore.A_restoreMethod] == ZaRestore.CA ) {
		restoreEl.setAttribute("prefix", restoreRequest[ZaRestore.A_prefix]);
	}
	
	if(restoreRequest[ZaRestore.A_restoreAllAcc]) {
		el = soapDoc.set("account", "", restoreEl);
		el.setAttribute("name", "all");
		if(restoreRequest[ZaRestore.A_skipDeletedAccounts]) {
			restoreEl.setAttribute(ZaRestore.A_skipDeletedAccounts, "1");
		} else {
			restoreEl.setAttribute(ZaRestore.A_skipDeletedAccounts, "0");
		}
	} else {
		if(restoreRequest[ZaRestore.A_accountNames]) {
			if(!restoreRequest[ZaRestore.A_accountNames] instanceof Array)
				restoreRequest[ZaRestore.A_accountNames] = [restoreRequest[ZaRestore.A_accountNames]];
				
			var cnt = restoreRequest[ZaRestore.A_accountNames].length;
			var el = null;
			for(var i = 0; i < cnt; i ++) {
				el = soapDoc.set("account", "", restoreEl);
				el.setAttribute("name", restoreRequest[ZaRestore.A_accountNames][i]);
			}
		}
		restoreEl.setAttribute(ZaRestore.A_skipDeletedAccounts, "0");	
	}
    if(restoreRequest[ZaRestore.A_ignoreRedoErrors]) {
         restoreEl.setAttribute(ZaRestore.A_ignoreRedoErrors, "1");
    } else {
         restoreEl.setAttribute(ZaRestore.A_ignoreRedoErrors, "0");
    }
    if(restoreRequest[ZaRestore.A_skipDeleteOps]) {
         restoreEl.setAttribute(ZaRestore.A_skipDeleteOps, "1");
    } else {
         restoreEl.setAttribute(ZaRestore.A_skipDeleteOps, "0");
    }

	if(restoreRequest[ZaRestore.A_restoreTo] == ZaRestore.TO_PIT) {
		restoreEl.setAttribute(ZaRestore.A_restoreToTime, restoreRequest[ZaRestore.A_restoreToTime].getTime());	
	}
	this.asynCommand = new ZmCsfeCommand();
	var params = new Object();
	params.soapDoc = soapDoc;
	params.noAuthToken = true;	
	if(callback) {
		params.asyncMode = true;
		params.callback = callback;
		params.targetServer = restoreRequest[ZaRestore.A_toServer];
	}
	this.asynCommand.invoke(params);
}
