/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

ZaRightsListViewController = function(appCtxt, container) {
	ZaListViewController.call(this, appCtxt, container,"ZaRightsListViewController");
   	this._toolbarOperations = new Array();
   	this._popupOperations = new Array();

	this._helpURL = location.pathname + ZaUtil.HELP_URL + "da_process/rights.htm?locid="+AjxEnv.DEFAULT_LOCALE;
}

ZaRightsListViewController.prototype = new ZaXFormViewController();
ZaRightsListViewController.prototype.constructor = ZaRightsListViewController;

ZaController.initPopupMenuMethods["ZaRightsListViewController"] = new Array();
ZaController.changeActionsStateMethods["ZaRightsListViewController"] = new Array();
ZaController.setViewMethods["ZaRightsListViewController"] = new Array();

/**
* @param list {ZaItemList} a list of ZaRight {@link ZaRight} objects
**/
ZaRightsListViewController.prototype.show =
function(list, openInNewTab) {
    this._setView(list.getVector());
    this.setDirty(false);
    this.changeActionsState();
}

ZaRightsListViewController.initPopupMenuMethod =
function () {

    this._popupOperations[ZaOperation.VIEW] = new ZaOperation(ZaOperation.VIEW, ZaMsg.TBB_View, com_zimbra_delegatedadmin.RIGHT_View_tt, "Properties", "PropertiesDis", new AjxListener(this, ZaRightsListViewController.prototype._editButtonListener));
    
}
ZaController.initPopupMenuMethods["ZaRightsListViewController"].push(ZaRightsListViewController.initPopupMenuMethod);

ZaRightsListViewController.prototype._createUI = function () {
	try {
		var elements = new Object();

        this._contentView = new ZaRightsSearchXFormView(this._container);
		this._initPopupMenu();
		this._actionMenu =  new ZaPopupMenu(this._contentView, "ActionMenu", null, this._popupOperations);
		elements[ZaAppViewMgr.C_APP_CONTENT] = this._contentView;
//		ZaApp.getInstance().createView(ZaZimbraAdmin._RIGHTS_LIST_VIEW, elements);
        ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
		this._removeConfirmMessageDialog = new ZaMsgDialog(ZaApp.getInstance().getAppCtxt().getShell(), null, [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON]);

		this._UICreated = true;
		ZaApp.getInstance()._controllers[this.getContentViewId ()] = this ;
	} catch (ex) {
		this._handleException(ex, "ZaRightsListViewController.prototype._createUI", null, false);
		return;
	}
}


ZaRightsListViewController.setViewMethod =
function(entry) {
	if(!this._UICreated) {
		this._createUI();
	}

	ZaApp.getInstance().pushView(this.getContentViewId());
	this._contentView.setDirty(false);
	this._contentView.setObject(entry);
	this._currentObject = entry;
}
ZaController.setViewMethods["ZaRightsListViewController"].push(ZaRightsListViewController.setViewMethod);

/*
ZaRightsListViewController.prototype.refresh =
function() {
	try {
		this._contentView.set(ZaApp.getInstance().getRightList(true).getVector());
	} catch (ex) {
		this._handleException(ex, ZaRightsListViewController.prototype.refresh, null, false);
	}
}
*/

ZaRightsListViewController.prototype.set =
function(rightsList) {
	this.show(rightsList);
}

ZaRightsListViewController.prototype.getAppBarAction =
function () {
    if (AjxUtil.isEmpty(this._appbarOperation)) {
    	this._appbarOperation[ZaOperation.HELP]=new ZaOperation(ZaOperation.HELP,ZaMsg.TBB_Help, ZaMsg.TBB_Help_tt, "Help", "Help", new AjxListener(this, this._helpButtonListener));
    }

    return this._appbarOperation;
}

ZaRightsListViewController.prototype.getAppBarOrder =
function () {
    if (AjxUtil.isEmpty(this._appbarOrder)) {
    	this._appbarOrder.push(ZaOperation.HELP);
    }
    return this._appbarOrder;
}
/**
* @param ev
* This listener is invoked by  any controller that can change an ZaRight object
**/
ZaRightsListViewController.prototype.handleRightChange =
function (ev) {
	//if any of the data that is currently visible has changed - update the view
	if(ev) {
		var details = ev.getDetails();
		//if(details["modFields"] && (details["modFields"][ZaRight.A_description] )) {
		if (details) {
			if (this._list) this._list.replace (details);
			if (this._contentView) this._contentView.setUI();
			if(ZaApp.getInstance().getCurrentController() == this) {
				this.show();
			}
			this.changeActionsState();
		}
	}
}

/**
* @param ev
* This listener is invoked by ZaRightController or any other controller that can create an ZaRight object
**/
ZaRightsListViewController.prototype.handleRightCreation =
function (ev) {
	if(ev) {
		//add the new ZaRight to the controlled list
		if(ev.getDetails()) {
			if (this._list) this._list.add(ev.getDetails());
			if (this._contentView) this._contentView.setUI();
			if(ZaApp.getInstance().getCurrentController() == this) {
				this.show();
			}
		}
	}
}

/**
* @param ev
* This listener is invoked by ZaRightController or any other controller that can remove an ZaRight object
**/
ZaRightsListViewController.prototype.handleRightRemoval =
function (ev) {
	if(ev) {
		//add the new ZaAccount to the controlled list
		if(ev.getDetails()) {
			if (this._list) this._list.remove(ev.getDetails());
			if (this._contentView ) this._contentView.setUI();
			if(ZaApp.getInstance().getCurrentController() == this) {
				this.show();
			}
		}
	}
}

/**
* Adds listener to removal of an ZaRight
* @param listener
**/
ZaRightsListViewController.prototype.addRightRemovalListener =
function(listener) {
	this._evtMgr.addListener(ZaEvent.E_REMOVE, listener);
}

/*
// refresh button was pressed
ZaRightsListViewController.prototype._refreshButtonListener =
function(ev) {
	this.refresh();
}
*/

/**
*	Private method that notifies listeners to that the controlled ZaRight (are) removed
* 	@param details
*/
ZaRightsListViewController.prototype._fireRightRemovalEvent =
function(details) {
	try {
		if (this._evtMgr.isListenerRegistered(ZaEvent.E_REMOVE)) {
			var evt = new ZaEvent(ZaEvent.S_SERVER);
			evt.set(ZaEvent.E_REMOVE, this);
			evt.setDetails(details);
			this._evtMgr.notifyListeners(ZaEvent.E_REMOVE, evt);
		}
	} catch (ex) {
		this._handleException(ex, ZaRightsListViewController.prototype._fireRightRemovalEvent, details, false);
	}
}


// new button was pressed
ZaRightsListViewController.prototype._newButtonListener =
function(ev) {
	var newRight = new ZaRight();
	ZaApp.getInstance().getRightViewController().show(newRight);
}

/**
* This listener is called when the item in the list is double clicked. It call ZaRightController.show method
* in order to display the Right View
**/
ZaRightsListViewController.prototype._listSelectionListener =
function(ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			this._selectedItem = ev.item;
			ZaApp.getInstance().getRightViewController().show(ev.item);
		}
	} else {
		this.changeActionsState();
	}
}

ZaRightsListViewController.prototype._listActionListener =
function (ev) {
	this.changeActionsState();
	this._actionMenu.popup(0, ev.docX, ev.docY);
}
/**
* This listener is called when the Edit button is clicked.
* It call ZaRightController.show method
* in order to display the Right View
**/
ZaRightsListViewController.prototype._editButtonListener =
function(ev) {
    var form = this._contentView.getMyForm();
	if(form) {
        var listFormItems = form.getItemsById(ZaRight.A2_filteredRightsList);
        var listWidget = listFormItems[0].widget;
        var selection = listWidget.getSelection();
        if(selection.length > 0){
            ZaApp.getInstance().getRightViewController().show(selection[0]);
            var parentPath = ZaTree.getPathByArray([ZaMsg.OVP_home, ZaMsg.OVP_configure, com_zimbra_delegatedadmin.OVP_rights]);
            ZaZimbraAdmin.getInstance().getOverviewPanelController().addObjectItem(parentPath, selection[0].name, null, false, false, selection[0]);
        }

    }
}

/**
 * This listener is called when the Delete button is clicked.
 * Currently, the codes of delete operation is obsolete.
**/
ZaRightsListViewController.prototype._deleteButtonListener =
function(ev) {
	this._removeList = new Array();
    var form = this._contentView.getMyForm();
	if(!form) return;
    var listFormItems = form.getItemsById(ZaRight.A2_filteredRightsList);
    var listWidget = listFormItems[0].widget;
    var selection = listWidget.getSelection();

	if(selection.length > 0) {
		var arrItems = selection;
		var cnt = arrItems.length;
		for(var key =0; key < cnt; key++) {
			if(arrItems[key]) {
				this._removeList.push(arrItems[key]);
			}
		}
	}
	if(this._removeList.length) {
		dlgMsg = ZaMsg.Q_DELETE_SERVERS;
		dlgMsg += "<br>";
		for(var key in this._removeList) {
			if(i > 19) {
				dlgMsg += "<li>...</li>";
				break;
			}
			dlgMsg += "<li>";
			if(this._removeList[key].name.length > 50) {
				//split it
				var endIx = 49;
				var beginIx = 0; //
				while(endIx < this._removeList[key].name.length) { //
					dlgMsg +=  this._removeList[key].name.slice(beginIx, endIx); //
					beginIx = endIx + 1; //
					if(beginIx >= (this._removeList[key].name.length) ) //
						break;

					endIx = ( this._removeList[key].name.length <= (endIx + 50) ) ? this._removeList[key].name.length-1 : (endIx + 50);
					dlgMsg +=  "<br>";
				}
			} else {
				dlgMsg += this._removeList[key].name;
			}
			dlgMsg += "</li>";
			i++;
		}
		this._removeConfirmMessageDialog.setMessage(dlgMsg,DwtMessageDialog.INFO_STYLE);
		this._removeConfirmMessageDialog.registerCallback(DwtDialog.YES_BUTTON, ZaRightsListViewController.prototype._deleteRightsCallback, this);
		this._removeConfirmMessageDialog.registerCallback(DwtDialog.NO_BUTTON, ZaRightsListViewController.prototype._donotDeleteRightsCallback, this);
		this._removeConfirmMessageDialog.popup();
	}
}

ZaRightsListViewController.prototype._deleteRightsCallback =
function () {
	var successRemList=new Array();
	for(var key in this._removeList) {
		if(this._removeList[key]) {
			try {
				this._removeList[key].remove();
				successRemList.push(this._removeList[key]);
			} catch (ex) {
				this._removeConfirmMessageDialog.popdown();
				this._handleException(ex, ZaRightsListViewController.prototype._deleteRightsCallback, null, false);
				return;
			}
		}
		if (this._list) this._list.remove(this._removeList[key]); //remove from the list
	}
	this._fireRightRemovalEvent(successRemList);
	this._removeConfirmMessageDialog.popdown();
	if (this._contentView) this._contentView.setUI();
	this.show();
}

ZaRightsListViewController.prototype._donotDeleteRightsCallback =
function () {
	this._removeList = new Array();
	this._removeConfirmMessageDialog.popdown();
}

ZaRightsListViewController.changeActionsStateMethod =
function () {
	if(this._contentView) {
        var form = this._contentView.getMyForm();
        if(!form) return;
        var listFormItems = form.getItemsById(ZaRight.A2_filteredRightsList);
        var listWidget = listFormItems[0].widget;
        var cnt = listWidget.getSelection().length;
		 if (cnt > 1){
			if(this._popupOperations[ZaOperation.VIEW])
				this._popupOperations[ZaOperation.VIEW].enabled = false;
		} else if (cnt <1) {
			if(this._popupOperations[ZaOperation.VIEW])
				this._popupOperations[ZaOperation.VIEW].enabled = false;

		}
	}
}
ZaController.changeActionsStateMethods["ZaRightsListViewController"].push(ZaRightsListViewController.changeActionsStateMethod);
