/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2009, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

function ZaXMBxBasicPicker(parent) {
	ZaXMBxPicker.call(this, parent, ZaXMBxPicker.BASIC);
};

ZaXMBxBasicPicker.prototype = new ZaXMBxPicker;
ZaXMBxBasicPicker.prototype.constructor = ZaXMBxBasicPicker;

ZaXMBxPicker.CTOR[ZaXMBxPicker.BASIC] = ZaXMBxBasicPicker;

ZaXMBxBasicPicker.prototype.toString = 
function() {
	return "ZaXMBxBasicPicker";
};

ZaXMBxBasicPicker.prototype._makeRow =
function(text, id) {
    var size = 15;
    var html = [];
    var i = 0;
    html[i++] = "<tr valign='middle'>";
    html[i++] = "<td align='right' nowrap>";
	html[i++] = text;
	html[i++] = ":</td>";
    html[i++] = "<td align='left' nowrap><input type='text' autocomplete='off' nowrap size='";
	html[i++] = size;
	html[i++] = "' id='";
	html[i++] = id;
	html[i++] = "'/></td>";
    html[i++] = "</tr>";

	return html.join("");		
};

// TODO: if we really wanted, we could add a prefs listener to update the "also search" checkboxes
ZaXMBxBasicPicker.prototype._setupPicker =
function(parent) {
    var picker = new DwtComposite(parent);

    var fromId = Dwt.getNextId();
    var toId = Dwt.getNextId();
    var subjectId = Dwt.getNextId();
    var contentId = Dwt.getNextId();
    var inTrashId = Dwt.getNextId();
    var inSpamId, checked;
    
	var html = [];
	var i = 0;
	html[i++] = "<table cellpadding='5' cellspacing='0' border='0'>";
	html[i++] = this._makeRow(com_zimbra_xmbxsearch.XMBX_from, fromId);
	html[i++] = this._makeRow(com_zimbra_xmbxsearch.XMBX_toCc, toId);
	html[i++] = this._makeRow(com_zimbra_xmbxsearch.XMBX_subject, subjectId);
	html[i++] = this._makeRow(com_zimbra_xmbxsearch.XMBX_content, contentId);
	
	inSpamId = Dwt.getNextId();
	checked = " checked" ;
	html[i++] = "<tr valign='middle'>";
	html[i++] = "<td align='right'><input type='checkbox'";
	html[i++] = checked;
	html[i++] = " id='";
	html[i++] = inSpamId;
	html[i++] = "' /></td>";
	html[i++] = "<td align='left' nowrap>";
	html[i++] = com_zimbra_xmbxsearch.XMBX_includeJunk;
	html[i++] = "</td>";
	html[i++] = "</tr>";

	checked = " checked" ;
	html[i++] = "<tr valign='middle'>";
	html[i++] = "<td align='right'><input type='checkbox'";
	html[i++] = checked;
	html[i++] = " id='";
	html[i++] = inTrashId;
	html[i++] = "' /></td>";
	html[i++] = "<td align='left' nowrap>";
	html[i++] = com_zimbra_xmbxsearch.XMBX_includeTrash;
	html[i++] = "</td>";
	html[i++] = "</tr>";
	html[i++] = "</table>";
	picker.getHtmlElement().innerHTML = html.join("");

	this._from = this._setupField(fromId);
	this._to = this._setupField(toId);
	this._subject= this._setupField(subjectId);
	this._content = this._setupField(contentId);
	this._inSpam = this._setupCheckbox(inSpamId);
	this._inTrash = this._setupCheckbox(inTrashId);
};

ZaXMBxBasicPicker.prototype.setFrom =
function(from) {
	this._from.value = from;
	this._updateQuery();
};

ZaXMBxBasicPicker.prototype.setTo =
function(to) {
	this._to.value = to;
	this._updateQuery();
};

ZaXMBxBasicPicker.prototype.setSubject =
function(subject) {
	this._subject.value = subject;
	this._updateQuery();
};

ZaXMBxBasicPicker.prototype.setContent =
function(content) {
	this._content.value = content;
	this._updateQuery();
};

ZaXMBxBasicPicker.prototype._setupField = 
function(id) {
	var f = document.getElementById(id);
	Dwt.setHandler(f, DwtEvent.ONKEYUP, ZaXMBxBasicPicker._onChange);
	Dwt.associateElementWithObject(f, this);
	return f;
};

ZaXMBxBasicPicker.prototype._setupCheckbox = 
function(id) {
	var f = document.getElementById(id);
	Dwt.setHandler(f, DwtEvent.ONCLICK, ZaXMBxBasicPicker._onChange);
	Dwt.associateElementWithObject(f, this);
	return f;
};

ZaXMBxBasicPicker._onChange =
function(ev) {
	var element = DwtUiEvent.getTarget(ev);
	var picker = Dwt.getObjectFromElement(element);

	var charCode = DwtKeyEvent.getCharCode(ev);
	if (charCode == 13 || charCode == 3 || charCode == 9) {
		picker._updateQuery();
	    return false;
	} else {
		picker._updateQuery();
		return true;
	}
};

ZaXMBxBasicPicker.prototype._updateQuery = 
function() {
	var query1 = [];
	var from = AjxStringUtil.trim(this._from.value, true);
	if (from.length)
		query1.push("from:(" + from + ")");
	var to = AjxStringUtil.trim(this._to.value, true);
	if (to.length)
		query1.push("(to:(" + to + ")" + " OR cc:(" + to + "))");
	var subject = AjxStringUtil.trim(this._subject.value, true);
	if (subject.length)
		query1.push("subject:(" + subject + ")");
	var content = AjxStringUtil.trim(this._content.value, true);
	if (content.length)
		query1.push("content:(" + content + ")");
	
	// Sort out "Check Trash/Spam" pref vs checkbox
	var query2 = [];
	var checkSpamCheckbox = (this._inSpam && this._inSpam.checked);
	var checkTrashCheckbox = (this._inTrash && this._inTrash.checked);
	
	if (checkSpamCheckbox && checkTrashCheckbox) {
		query2.push("is:anywhere");
	} else {
		if (!checkTrashCheckbox)
			query2.push("not in:trash");

		if (!checkSpamCheckbox)
			query2.push("not in:junk");
	}
	
	var cbQuery = query2.length ? query2.join(" ") : null;

	// if the "search Trash/Spam" checkboxes changed, run the query
	var cbChange = (query1.length && ((this._cbQuery || cbQuery) && (this._cbQuery != cbQuery)));
	var query = query1.concat(query2);
	this.setQuery(query.length ? query.join(" ") : "");
/*	if (cbChange) {
		this.execute();
	}*/
	this._cbQuery = cbQuery;
};


