/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2009, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

function ZaXMBxDatePicker(parent) {
	ZaXMBxPicker.call(this, parent, ZaXMBxPicker.DATE);
};

ZaXMBxDatePicker.prototype = new ZaXMBxPicker;
ZaXMBxDatePicker.prototype.constructor = ZaXMBxDatePicker;

ZaXMBxPicker.CTOR[ZaXMBxPicker.DATE] = ZaXMBxDatePicker;

ZaXMBxDatePicker.SELECT_OPTIONS = [
	{ label: com_zimbra_xmbxsearch.XMBX_isAfter, 	value: "after", 	selected: false },
	{ label: com_zimbra_xmbxsearch.XMBX_isBefore, 	value: "before", 	selected: true  },
	{ label: com_zimbra_xmbxsearch.XMBX_isOn, 		value: "date", 		selected: false }];

ZaXMBxDatePicker.prototype.toString = 
function() {
	return "ZaXMBxDatePicker";
};

ZaXMBxDatePicker.prototype._setupPicker =
function(parent) {
	var picker = new DwtComposite(parent);
	var selectId = Dwt.getNextId();
	var calId = Dwt.getNextId();
    
    var html = new Array(11);
    var i = 0;
    html[i++] = "<table cellpadding=3 cellspacing=0 border=0 style='width:100%;'>";
    html[i++] = "<tr align='center' valign='middle'><td align='right' nowrap>";
    html[i++] = com_zimbra_xmbxsearch.XMBX_date;
    html[i++] = ":</td><td align='left' id='";
    html[i++] = selectId;
    html[i++] = "'></td></tr>";
    html[i++] = "<tr valign='left'>";
    html[i++] = "<td nowrap align='center' colspan='2' id='";
    html[i++] = calId;
    html[i++] = "'></td></tr></table>";
	picker.getHtmlElement().innerHTML = html.join("");

	// create and add DwtSelect
	this._select = new DwtSelect(this);
	this._select.addChangeListener(new AjxListener(this, this._dateChangeListener));
	for (var i = 0; i < ZaXMBxDatePicker.SELECT_OPTIONS.length; i++) {
		var option = ZaXMBxDatePicker.SELECT_OPTIONS[i];
		this._select.addOption(option.label, option.selected, option.value);
	}
	this._select.reparentHtmlElement(selectId);

	// create and add DwtCalendar
	var firstDay = 0;
	var cal = this._cal = new DwtCalendar(picker, null, null, firstDay);
	cal.setDate(new Date());
	cal.addSelectionListener(new AjxListener(this, this._calSelectionListener));
	cal.reparentHtmlElement(calId);

	this._updateQuery();
};

// Set date for second instance of date picker to 3 months back, select "after"
ZaXMBxDatePicker.prototype.secondDate =
function() {
	this._select.setSelected(0);
	var date = new Date(this._cal.getDate().getTime());
	AjxDateUtil.roll(date, AjxDateUtil.MONTH, -3);
	this._cal.setDate(date);
	this._updateQuery();
};

ZaXMBxDatePicker.prototype._updateQuery = 
function() {
	var d = this._cal.getDate();
	if (d) {
		var date = (d.getMonth() + 1) + "/" + d.getDate() + "/" + d.getFullYear();
		this.setQuery(this._select.getValue() + ":" + date);
	} else {
		this.setQuery("");
	}
//	this.execute();
};

ZaXMBxDatePicker.prototype._calSelectionListener =
function(ev) {
	this._updateQuery();
};

ZaXMBxDatePicker.prototype._dateChangeListener = 
function(ev) {
	this._updateQuery();
};
