/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaXMBxListView
* @param parent
* @author Greg Solovyev
**/

function ZaXMBxListView(parent, className, posStyle, headerList) {
	ZaListView.call(this, parent, className, posStyle, headerList);
	this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
}

ZaXMBxListView.prototype = new ZaListView;
ZaXMBxListView.prototype.constructor = ZaXMBxListView;

ZaXMBxListView.prototype.toString = 
function() {
	return "ZaXMBxListView";
}

ZaXMBxListView.prototype.getTitle = 
function () {
	return ZaMsg.BNR_view_title;
}
/**
* Renders a single item as a DIV element.
*/
ZaXMBxListView.prototype._createItemHtml =
function(obj, now, isDndIcon) {
	var html = new Array(50);
	var	div = document.createElement("DIV");
	div.id = Dwt.getNextId();//Row-matched
	var rowClassName = AjxBuffer.concat(this._className, "Row");	
	div[DwtListView._STYLE_CLASS] = AjxBuffer.concat("Row ", rowClassName);
	div[DwtListView._SELECTED_STYLE_CLASS] = AjxBuffer.concat("Row-", DwtCssStyle.SELECTED, " ", rowClassName);
	div[DwtListView._SELECTED_DIS_STYLE_CLASS] = AjxBuffer.concat("Row-", DwtCssStyle.SELECTED, "-" , DwtCssStyle.DISABLED, " ", rowClassName);
	div.className = div[DwtListView._STYLE_CLASS];
	this.associateItemWithElement(obj, div, DwtListView.TYPE_LIST_ITEM);
	
	var idx = 0;
	html[idx++] = "<table width='100%' cellspacing='0' cellpadding='0'>";
	html[idx++] = "<tr>";
	var cnt = this._headerList.length;
	for(var i = 0; i < cnt; i++) {
		var id = this._headerList[i]._field;
		if(id!=null) {
			if(id.indexOf(ZaXMBxSearch.A_serverName)==0) {
				html[idx++] = "<td align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width;
				html[idx++] = "'>";
				html[idx++] = obj.attrs[ZaXMBxSearch.A_serverName];
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaXMBxSearch.A_status)==0) {
				html[idx++] = "<td align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				if(!obj.attrs[ZaXMBxSearch.A_status]) {
					html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusIdle;				
				} else if (obj.attrs[ZaXMBxSearch.A_status]==1) {
					html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusError;						
				} else {
					var status = parseInt(obj.attrs[ZaXMBxSearch.A_status]);
					if ((status & ZaXMBxSearch.iSTATUS_FINISHED) == ZaXMBxSearch.iSTATUS_FINISHED) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusFinished;
					} else if ((status & ZaXMBxSearch.iSTATUS_ABORTING) == ZaXMBxSearch.iSTATUS_ABORTING) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusAborting;
					} else if ((status & ZaXMBxSearch.iSTATUS_ABORTED) == ZaXMBxSearch.iSTATUS_ABORTED) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusAborted;
					} else if ((status & ZaXMBxSearch.iSTATUS_SENDING_NOTIFICATIONS) == ZaXMBxSearch.iSTATUS_SENDING_NOTIFICATIONS) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusSendingNotifications;
					} else if ((status & ZaXMBxSearch.iSTATUS_IMPORTING) == ZaXMBxSearch.iSTATUS_IMPORTING) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusImporting;
					} else if ((status & ZaXMBxSearch.iSTATUS_SEARCHING) == ZaXMBxSearch.iSTATUS_SEARCHING) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusSearching;
					} else if ((status & ZaXMBxSearch.iSTATUS_STARTED) == ZaXMBxSearch.iSTATUS_STARTED) {
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusStarted;
					}
						
					if ((status & ZaXMBxSearch.iSTATUS_ERROR) == ZaXMBxSearch.iSTATUS_ERROR) {
						html[idx++] = " ";
						html[idx++] = com_zimbra_xmbxsearch.XMBX_StatusWithErrors;
					}																			
				}
	
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaXMBxSearch.A_targetMbx)==0) {
				html[idx++] = "<td align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				html[idx++] = obj.attrs[ZaXMBxSearch.A_targetMbx];
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaXMBxSearch.A_searchID)==0) {
				html[idx++] = "<td align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				html[idx++] = obj.attrs[ZaXMBxSearch.A_searchID]; 
				html[idx++] = "</td>";
			} else if(id.indexOf(ZaXMBxSearch.A_numMsgs)==0) {
				html[idx++] = "<td align=left height=20px  width='";
				html[idx++] = this._headerList[i]._width 
				html[idx++] = "'>";
				html[idx++] = obj.attrs[ZaXMBxSearch.A_numMsgs]; 
				html[idx++] = "</td>";
			}
		}
	}
	html[idx++] = "</tr></table>";
	div.innerHTML = html.join("");
	return div;
}

ZaXMBxListView.prototype._setNoResultsHtml = function() {
	var buffer = new AjxBuffer();
	var	div = document.createElement("div");
	var text = com_zimbra_xmbxsearch.XMBX_NoSearchesFound;
	if(this.parent && this.parent.instance) {
		text = com_zimbra_xmbxsearch.XMBX_NOSearchTasksFound;
	}
	buffer.append("<table width='100%' cellspacing='0' cellpadding='1'>","<tr><td class='NoResults'><br>",text,"</td></tr></table>");
	
	div.innerHTML = buffer.toString();
	this._addRow(div);
};