/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2009, 2011, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

function ZaXMBxPicker(parent, id) {
	if (arguments.length == 0) return;
	DwtComposite.call(this, parent, "ZaSearchOptionView", DwtControl.ABSOLUTE_STYLE);

	this._appCtxt = this.shell.getData(ZaAppCtxt.LABEL);
	this._header = new DwtToolBar(this);
	this._label = new DwtLabel(this._header, DwtLabel.IMAGE_LEFT | DwtLabel.ALIGN_LEFT, "ZaXMBxPickerLabel");
	this._header.addFiller();
	this._close = new DwtButton(this._header, DwtLabel.IMAGE_LEFT, "DwtToolbarButton");
	this._close.setImage("Close");
	this._close.setToolTipContent(ZaMsg.close);
	this._picker = new DwtComposite(this, "ZaSearchOptionOverview");
	this._picker.setSize(Dwt.DEFAULT, parent.getH() - this._header.getH());
	this._picker.setScrollStyle(DwtControl.SCROLL);
	this._pickerEvent = new ZaEvent(ZaEvent.S_PICKER);
	this._pickerEvent.set(ZaEvent.E_MODIFY, this);
	this.id = id;
    this.setTitle(ZaMsg[ZaXMBxPicker.T_MSG_KEY[id]]);
    this.setImage(ZaXMBxPicker.IMAGE[id]);
	this._setupPicker(this._picker);
};

ZaXMBxPicker.prototype = new DwtComposite;
ZaXMBxPicker.prototype.constructor = ZaXMBxPicker;

var i = 1;
ZaXMBxPicker.ATTACHMENT	= i++;
ZaXMBxPicker.BASIC		= i++;
ZaXMBxPicker.CUSTOM		= i++;
ZaXMBxPicker.DATE		= i++;
ZaXMBxPicker.DOMAIN		= i++;
ZaXMBxPicker.FLAG		= i++;
ZaXMBxPicker.FOLDER		= i++;
ZaXMBxPicker.ZIMLET		= i++;
ZaXMBxPicker.SEARCH		= i++;
ZaXMBxPicker.SIZE		= i++;
ZaXMBxPicker.TAG		= i++;
ZaXMBxPicker.TIME		= i++;
ZaXMBxPicker.RESET		= i++;	// not really a picker
ZaXMBxPicker.CLOSE		= i++;	// not really a picker

ZaXMBxPicker.NEXT_ID	= i;

ZaXMBxPicker.DEFAULT_PICKERS=[ZaXMBxPicker.ATTACHMENT,
				ZaXMBxPicker.BASIC,
				ZaXMBxPicker.DATE,
				ZaXMBxPicker.DOMAIN,
				ZaXMBxPicker.FOLDER];

// Button labels
ZaXMBxPicker.MSG_KEY = new Object();
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.ATTACHMENT]	= "XMBX_attachment";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.BASIC]		= "XMBX_basic";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.CUSTOM]		= "XMBX_custom";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.DATE]			= "XMBX_date";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.DOMAIN]		= "XMBX_domain";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.FLAG]			= "XMBX_status";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.FOLDER]		= "XMBX_folder";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.ZIMLET]		= "XMBX_zimlets";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.SEARCH]		= "XMBX_savedSearch";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.SIZE]			= "XMBX_size";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.TAG]			= "XMBX_tag";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.TIME]			= "XMBX_time";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.RESET]		= "XMBX_removeAll";
ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.CLOSE]		= "XMBX_close";

// Button and picker icons
ZaXMBxPicker.IMAGE = new Object();
ZaXMBxPicker.IMAGE[ZaXMBxPicker.ATTACHMENT]	= "Attachment";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.BASIC]		= "Message";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.CUSTOM]		= "Search";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.DATE]		= "Date";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.DOMAIN]		= "URL";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.FLAG]		= "FlagRed";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.FOLDER]		= "Folder";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.ZIMLET]		= "Zimlet";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.SEARCH]		= "SearchFolder";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.SIZE]		= "Folder";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.TAG]		= "Tag";
ZaXMBxPicker.IMAGE[ZaXMBxPicker.TIME]		= "Time";
//ZaXMBxPicker.IMAGE[ZaXMBxPicker.RESET]		= "Close";	//MOW: no image for reset button
ZaXMBxPicker.IMAGE[ZaXMBxPicker.CLOSE]		= "Close";

// Button tooltips
ZaXMBxPicker.TT_MSG_KEY = new Object();
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.ATTACHMENT]	= "XMBX_searchByAttachment";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.BASIC]			= "XMBX_searchByBasic";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.CUSTOM]		= "XMBX_searchByCustom";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.DATE]			= "XMBX_searchByDate";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.DOMAIN]		= "XMBX_searchByDomain";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.FLAG]			= "XMBX_searchByFlag";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.FOLDER]		= "XMBX_searchByFolder";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.ZIMLET]		= "XMBX_searchByZimlet";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.SEARCH]		= "XMBX_searchBySavedSearch";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.SIZE]			= "XMBX_searchBySize";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.TAG]			= "XMBX_searchByTag";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.TIME]			= "XMBX_searchByTime";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.RESET]			= "XMBX_clearAdvSearch";
ZaXMBxPicker.TT_MSG_KEY[ZaXMBxPicker.CLOSE]			= "XMBX_closeSearchBuilder";

// Picker titles
ZaXMBxPicker.T_MSG_KEY = new Object();
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.ATTACHMENT]	= "XMBX_attachments";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.BASIC]		= "XMBX_basicSearch";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.CUSTOM]		= "XMBX_custom";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.DATE]		= "XMBX_date";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.DOMAIN]		= "XMBX_domains";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.FLAG]		= "XMBX_status";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.FOLDER]		= "XMBX_folders";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.ZIMLET]		= "XMBX_zimlets";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.SEARCH]		= "XMBX_savedSearches";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.SIZE]		= "XMBX_size";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.TAG]		= "XMBX_tags";
ZaXMBxPicker.T_MSG_KEY[ZaXMBxPicker.TIME]		= "XMBX_time";

// Max number of instances for each picker
// -1 means no limit
ZaXMBxPicker.LIMIT = new Object();
ZaXMBxPicker.LIMIT[ZaXMBxPicker.ATTACHMENT]	= 1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.BASIC]		= -1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.CUSTOM]		= 1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.DATE]		= 2;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.DOMAIN]		= 2;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.FLAG]		= 4;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.FOLDER]		= 1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.ZIMLET]		= -1; 
ZaXMBxPicker.LIMIT[ZaXMBxPicker.SEARCH]		= -1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.SIZE]		= 2;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.TAG]		= -1;
ZaXMBxPicker.LIMIT[ZaXMBxPicker.TIME]		= 1;

ZaXMBxPicker.MULTI_JOIN = new Object();
for (var i = 1; i <= ZaXMBxPicker.CLOSE; i++)
	ZaXMBxPicker.MULTI_JOIN[i] = " ";
ZaXMBxPicker.MULTI_JOIN[ZaXMBxPicker.BASIC] = " OR ";
ZaXMBxPicker.MULTI_JOIN[ZaXMBxPicker.FLAG] = " OR ";

ZaXMBxPicker.CTOR = new Object();

ZaXMBxPicker.DEFAULT_PICKER = ZaXMBxPicker.BASIC;

ZaXMBxPicker.KEY_ID = "_id_";
ZaXMBxPicker.KEY_CTOR = "_ctor_";
ZaXMBxPicker.KEY_PICKER = "_picker_";

function ZaXMBxPicker_Descriptor(id, label, image, toolTip, ctor) {
	this.id = id;
	this.label = label || ZaMsg[ZaXMBxPicker.MSG_KEY[id]];
	this.image = image || ZaXMBxPicker.IMAGE[id];
	this.toolTip = toolTip || ZaMsg[ZaXMBxPicker.TT_MSG_KEY[id]] || this.label;
	this.ctor = ctor;
}

ZaXMBxPicker.prototype.toString = 
function() {
	return "ZaXMBxPicker";
}

ZaXMBxPicker.prototype._setupPicker  = function() {}
ZaXMBxPicker.prototype._updateQuery  = function() {}

ZaXMBxPicker.prototype.setTitle =
function(text) {
    this._label.setText(text);
}

ZaXMBxPicker.prototype.setImage =
function(imageInfo) {
    this._label.setImage(imageInfo);
}

ZaXMBxPicker.prototype.getCloseButton = 
function() {
	return this._close;
}

ZaXMBxPicker.prototype.setEnabled =
function(enabled) {
   DwtControl.prototype.setEnabled.call(this, enabled);
    this._label.setEnabled(enabled);
    if (this._picker.setEnabled)
	    this._picker.setEnabled(enabled);
}

ZaXMBxPicker.prototype.addPickerListener =
function(listener) {
	this.addListener(ZaEvent.L_PICKER, listener);
}

ZaXMBxPicker.prototype.removePickerListener =
function(listener) {
	this.removeListener(ZaEvent.L_PICKER, listener);
}

ZaXMBxPicker.prototype.execute =
function() {
/*	if (this.isListenerRegistered(DwtEvent.SELECTION)) {
		this._pickerEvent.set(ZaEvent.E_LOAD, this);
		this.notifyListeners(ZaEvent.L_PICKER, this._pickerEvent);
	}*/
}

ZaXMBxPicker.prototype.setQuery =
function(query) {
	this._query = query;
	if (this.isListenerRegistered(ZaEvent.L_PICKER)) {
		this._pickerEvent.set(ZaEvent.E_MODIFY, this);
		this.notifyListeners(ZaEvent.L_PICKER, this._pickerEvent);
	}
}

ZaXMBxPicker.prototype.dispose =
function() {
	DwtComposite.prototype.dispose.call(this);
	/*if (this._treeView) {
		var opc = this._appCtxt.getOverviewController();
		opc.clearOverview(this._overviewId);
	}*/
}
/*
ZaXMBxPicker.prototype._setOverview =
function(overviewId, parent, types) {
	this._overviewId = overviewId;
	this._picker.setScrollStyle(Dwt.CLIP);
	var opc = this._appCtxt.getOverviewController();
	opc.createOverview({overviewId: overviewId, parent: parent, //scroll: Dwt.VISIBLE,
						headerClass: "DwtTreeItem",
						treeStyle: DwtTree.CHECKEDITEM_STYLE});
	opc.set(overviewId, types);
	this._treeView = new Object();
	for (var i = 0; i < types.length; i++) {
		var treeView = this._treeView[types[i]] = opc.getTreeView(overviewId, types[i]);
		treeView.addSelectionListener(new AjxListener(this, this._treeListener));
	}
	if (types.length == 1)
		this._hideRoot(types[0]);
}

ZaXMBxPicker.prototype._hideRoot =
function(type) {
	var ti = this._treeView[type].getTreeItemById(ZmOrganizer.ID_ROOT);
	ti.showCheckBox(false);
	ti.setExpanded(true);
	ti.setVisible(false, true);
}*/
