/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */
/**
* XFormItem class: XMBX_SRCH_BUILDER
* @class ZaXMBxSearchBuilder
* @constructor ZaXMBxSearchBuilder
* @author Greg Solovyev
**/
function ZaXMBxSearchBuilder() {}
XFormItemFactory.createItemType("_XMBX_SRCH_BUILDER_", "xmbx_srch_builder", ZaXMBxSearchBuilder, Composite_XFormItem);
ZaXMBxSearchBuilder.prototype.numCols = 1;
ZaXMBxSearchBuilder.prototype.pickers = [];
ZaXMBxSearchBuilder.prototype._numPickers=0;
ZaXMBxSearchBuilder.prototype._buttonIds = {};
ZaXMBxSearchBuilder.prototype.tableCssStyle="width:100%;";
ZaXMBxSearchBuilder.prototype.initializeItems = function () {
	this.pickers = [];
	this._numPickers = 0;
	var btnChoices = this.getChoices();
	if(btnChoices.constructor == XFormChoices) {
		btnChoices = btnChoices.getChoiceObject();
	} 
	this.btnGridId = Dwt.getNextId();
	
	var btnGrid =	{	
		type:_GROUP_, numCols:btnChoices.length+1, align:_TOP_,
		id:this.btnGridId,items:[],cellpadding:5
	};
	
	for (var ix in btnChoices) {
		var buttonId = Dwt.getNextId();
		var w;
         if(ZaZimbraAdmin.LOCALE=="ja"||ZaZimbraAdmin.LOCALE=="ko"||ZaZimbraAdmin.LOCALE=="zh_CN"||ZaZimbraAdmin.LOCALE=="zh_HK")
            w = String(btnChoices[ix].label).length*6*2+40
         else
            w = String(btnChoices[ix].label).length*6+40;

		if(isNaN(w))
			w = 100; 
			
		this._buttonIds[btnChoices[ix].value] = buttonId;
		this.pickers[btnChoices[ix].value] = new AjxVector();
		var onActivate = (btnChoices[ix].value != ZaXMBxPicker.RESET) ?
				(["ZaXMBxSearchBuilder.buttonClickHandler.call(this,",btnChoices[ix].value,")"].join("") ) :
				("ZaXMBxSearchBuilder.resetPickersHandler.call(this)") ;

		var button  = {type:_DWT_BUTTON_, label:btnChoices[ix].label, icon:btnChoices[ix].icon,
		onActivate:onActivate,
		width:(w+"px"), id:buttonId, pickerId:btnChoices[ix].value,
		enableDisableChecks:[ZaXMBxSearchBuilder.isPickerButtonEnabled],
		enableDisableChangeEventSources:[ZaXMBxSearch.A_query]
		//relevant:("ZaXMBxSearchBuilder.isPickerButtonEnabled(this, instance, item)"), relevantBehavior:_DISABLE_
		};
		btnGrid.items.push(button);
	}

	this.pickerGridId = Dwt.getNextId();
	var pickerGrid =	{	
		type:_XMBX_PICKER_GRID_, align:_LEFT_,id:this.pickerGridId,height:250,width:"98%"
	};	
	
	this.items = [btnGrid,{type:_SPACER_},pickerGrid];
	Composite_XFormItem.prototype.initializeItems.call(this);
}

ZaXMBxSearchBuilder.isPickerButtonEnabled = function () {
	var pickerId = this.getInheritedProperty("pickerId");
	var builder = this.getParentItem().getParentItem();
	return (!(ZaXMBxPicker.LIMIT[pickerId] != -1 && builder.pickers[pickerId].size() == ZaXMBxPicker.LIMIT[pickerId]));
}

ZaXMBxSearchBuilder.buttonClickHandler = function (id, ev) {
	var builder = this.getParentItem().getParentItem();
	if(builder)
		ZaXMBxSearchBuilder.onButtonClick.call(builder, this, id, ev);
}

ZaXMBxSearchBuilder.onButtonClick = function (source, id, ev) {
	var form = source.getForm();
	var pickerGridXFI = form.getItemById([form.getId(),"_",this.pickerGridId].join(""));
	if(pickerGridXFI) {
		var pickerGrid = pickerGridXFI.getWidget();
		if(pickerGrid) {
			if(!this.pickers[id])
				this.pickers[id] = new AjxVector();
				
			if (ZaXMBxPicker.LIMIT[id] != -1 && this.pickers[id].size() >= ZaXMBxPicker.LIMIT[id]) {return;}
		
			var ctor = ZaXMBxPicker.CTOR[id];
			var picker = new ctor(pickerGrid);
			pickerGrid.addPicker(picker, id);			
			this.pickers[id].add(picker);
			this._numPickers++;
			if (id == ZaXMBxPicker.DATE && this.pickers[id].size() == ZaXMBxPicker.LIMIT[id]) {
				picker.secondDate();
			}
		    var cb = picker.getCloseButton();
		    cb.setData(ZaXMBxPicker.KEY_ID, id);
		    cb.setData(ZaXMBxPicker.KEY_PICKER, picker);
		    cb.addSelectionListener(new AjxListener(this, this._pickerCloseListener));
		    picker.addPickerListener(new AjxListener(this, this._pickerListener));
			this._updateQuery(ev);
			return picker;
	
		}
	}
}

ZaXMBxSearchBuilder.prototype.items = [];

/*
ZaXMBxSearchBuilder.prototype._addPickerListener =
function(ev) {
	try {
		var id = ev.item.getData(ZaXMBxPicker.KEY_ID);
		this.addPicker(id);
	} catch (ex) {
		this._handleException(ex, this._addPickerListener, ev, false);
	}
};*/

ZaXMBxSearchBuilder.prototype._pickerCloseListener =
function(ev) {
	var form = this.getForm();
	var pickerGridXFI = form.getItemById([form.getId(),"_",this.pickerGridId].join(""));
	if(pickerGridXFI) {
		var pickerGrid = pickerGridXFI.getWidget();	
	    var b = ev.item;
		var picker = b.getData(ZaXMBxPicker.KEY_PICKER);
		pickerGrid.removePicker(picker);
		var id = b.getData(ZaXMBxPicker.KEY_ID);
		this.pickers[id].remove(picker);
		this._numPickers--;
		if (this._numPickers == 0) {
			this._clearQuery(ev);
		} else {
			this._updateQuery(ev);
		}
		// enable picker button if max instances of this picker has not been reached
/*		if (ZaXMBxPicker.LIMIT[id] != -1 && this.pickers[id].size() < ZaXMBxPicker.LIMIT[id]) {
			this._toolbar.enable(id, true);
		}
*/		
	}
};

ZaXMBxSearchBuilder.prototype._clearQuery =
function(event) {
	this._query = "";
	var elemChanged = this.getElementChangedMethod();
	elemChanged.call(this, "", this.getInstanceValue(), event);	
	form = this.getForm();
	form.refresh();		
};

ZaXMBxSearchBuilder.prototype._pickerListener =
function(ev) {
	if (ev.type != ZaEvent.S_PICKER) {return;}
	if (ev.event == ZaEvent.E_MODIFY) {
		this._updateQuery(ev);
	} else if (ev.event == ZaEvent.E_LOAD) {
		this._updateQuery(ev);
	}
};

ZaXMBxSearchBuilder.resetPickersHandler = function (id,ev) {
	var builder = this.getParentItem().getParentItem();
	if(builder)
		builder.resetPickers(ev);
}

ZaXMBxSearchBuilder.prototype.resetPickers =
function(ev) {
	var form = this.getForm();
	var pickerGridXFI = form.getItemById([form.getId(),"_",this.pickerGridId].join(""));
	if(pickerGridXFI && this.pickers) {
		var pickerGrid = pickerGridXFI.getWidget();	
		for (var id in this.pickers) {
			var cnt = this.pickers[id].size();		
			for(var i = 0; i < cnt; i++) {
				pickerGrid.removePicker(this.pickers[id].get(i));				
			}
			this.pickers[id] = new AjxVector();
		}
	}		
	this._numPickers = 0;
	this._clearQuery(ev);
};


/*
* The point of the mess below is to safely construct queries. Some pickers (right now, just
* the basic picker) may have multiple instances that join their queries with OR instead of
* AND (represented as a space). In that case, we need parens around each of the picker
* queries as well as around the query for the type of picker, in order to protect the OR.
*/
ZaXMBxSearchBuilder.prototype._updateQuery =
function(event) {
	var queries = {};
	var numPickers = 0;
	for (var id in this.pickers) {
		var a = this.pickers[id].getArray();
		for (var i = 0; i < a.length; i++) {
			var pq = a[i]._query;
			if (pq && pq.length) {
				if (!queries[id]) {
					queries[id] = [];
					numPickers++;
				}
				queries[id].push(pq);
			}
		}
	}
	var queryStr = [];
	for (var id in queries) {
		var a = queries[id];
		if (a && a.length) {
			if (a.length > 1) {
				var q = [];
				var j = ZaXMBxPicker.MULTI_JOIN[id];
				var needParen = (j != " ");
				for (var i = 0; i < a.length; i++)
					q.push((needParen && a.length > 1) ? "(" + a[i] + ")" : a[i]);
				var query = q.join(j);
				if (needParen && numPickers > 1)
					query = "(" + query + ")";
				queryStr.push(query);
			} else {
				queryStr.push(a[0]);
			}
		}
	}
	
	var newQuery = queryStr.join(" ");
	//if (newQuery != this._query) {
		this._query = newQuery;
		//var value = newQuery;
		this.setInstanceValue(newQuery);
		//var elemChanged = this.getElementChangedMethod();
		//elemChanged.call(this, value, this.getInstanceValue(), event);		
	//}
	/*else {
		form = this.getForm();
		form.refresh();
	}*/
}
