/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @constructor
* @class ZaXMBxSearchListController
**/
function ZaXMBxSearchListController(appCtxt, container) {
	ZaXFormViewController.call(this, appCtxt, container, "ZaXMBxSearchListController");
	this._UICreated = false;	
	this._helpURL = ZaXMBxSearchListController.helpURL;
	this._popupOperations = new Array();
}

ZaXMBxSearchListController.prototype = new ZaXFormViewController();
ZaXMBxSearchListController.prototype.constructor = ZaXMBxSearchListController;
ZaXMBxSearchListController.helpURL = location.pathname + "help/admin/html/search/cross_mailbox_search.htm?locid=" + AjxEnv.DEFAULT_LOCALE;
//ZaController.initToolbarMethods["ZaXMBxSearchListController"] = new Array();
ZaController.initPopupMenuMethods["ZaXMBxSearchListController"] = new Array();
ZaController.setViewMethods["ZaXMBxSearchListController"] = new Array();
ZaController.changeActionsStateMethods["ZaXMBxSearchListController"] = new Array();
/**
*	@method show
*	@param entry - isntance of ZaServer class
*/
ZaXMBxSearchListController.prototype.show = 
function(list) {
	this._setView(list);
	this.setDirty(false);
}

ZaXMBxSearchListController.prototype.getAppBarAction =
function () {
    if (AjxUtil.isEmpty(this._appbarOperation)) {
    	this._appbarOperation[ZaOperation.HELP]=new ZaOperation(ZaOperation.HELP,ZaMsg.TBB_Help, ZaMsg.TBB_Help_tt, "Help", "Help", new AjxListener(this, this._helpButtonListener));
    }

    return this._appbarOperation;
}

ZaXMBxSearchListController.prototype.getAppBarOrder =
function () {
    if (AjxUtil.isEmpty(this._appbarOrder)) {
    	this._appbarOrder.push(ZaOperation.HELP);
    }

    return this._appbarOrder;
}

ZaXMBxSearchListController.changeActionsStateMethod = function () {
	var selectionCount = 0;
	var form;
	var listFormItems;
	var listWidget;
	if(this._contentView) {
		form = this._contentView.getMyForm();
		if(form) {
			listFormItems = form.getItemsById(ZaXMBxSearch.A_searchesList);
			if(listFormItems && listFormItems.length && listFormItems[0]) {
				listWidget = listFormItems[0].widget;
                if(listWidget)
				    selectionCount = listWidget.getSelectionCount();
			}
		}
	}
  
 	if(selectionCount < 1) {

		if(this._popupOperations[ZaOperation.EDIT])
			this._popupOperations[ZaOperation.EDIT].enabled = false;


		if(this._popupOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS])
			this._popupOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS].enabled = false;


		if(this._popupOperations[ZaOperation.ABORT_SEARCH])
			this._popupOperations[ZaOperation.ABORT_SEARCH].enabled = false;


		if(this._popupOperations[ZaOperation.DELETE])
			this._popupOperations[ZaOperation.DELETE].enabled = false;

		return; //no select, thus no need to check "abort" button status

	} else if (selectionCount == 1) {
		//only check "abort" button status as below

	} else if(selectionCount > 1) {

        if(this._popupOperations[ZaOperation.EDIT])
			this._popupOperations[ZaOperation.EDIT].enabled = false;

		if(this._popupOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS])
			this._popupOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS].enabled = false;

	}

	var selection = listWidget.getSelection();
	var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
	if(list) {
		for (var i = 0; i < selectionCount; i++){
			var selectedSearch = list[i];
			var status = parseInt(selectedSearch.attrs[ZaXMBxSearch.A_status]);
			if(selectedSearch) {
				if( ((status & ZaXMBxSearch.iSTATUS_FINISHED) == ZaXMBxSearch.iSTATUS_FINISHED)
					|| ((status & ZaXMBxSearch.iSTATUS_ABORTED) == ZaXMBxSearch.iSTATUS_ABORTED)
					|| ((status & ZaXMBxSearch.iSTATUS_ABORTING) == ZaXMBxSearch.iSTATUS_ABORTING)
					|| ((status == ZaXMBxSearch.iSTATUS_IDLE))
					) {

						if(this._popupOperations[ZaOperation.ABORT_SEARCH])
							this._popupOperations[ZaOperation.ABORT_SEARCH].enabled = false;

						break;
				}
			}
		}
	}
  
}
ZaController.changeActionsStateMethods["ZaXMBxSearchListController"].push(ZaXMBxSearchListController.changeActionsStateMethod);

ZaXMBxSearchListController.initPopupMenuMethod =
function () {
    if (appNewUI) {
        this._popupOperations[ZaOperation.NEW] = new ZaOperation(ZaOperation.NEW, ZaMsg.TBB_New, com_zimbra_xmbxsearch.XMBXTBB_New_tt,"CrossMailboxSearch","CrossMailboxSearch", new AjxListener(this, this.newButtonListener));
    }
    this._popupOperations[ZaOperation.EDIT] = new ZaOperation(ZaOperation.EDIT, com_zimbra_xmbxsearch.XMBXTBB_View, com_zimbra_xmbxsearch.XMBXTBB_View_tt, "Properties", "PropertiesDis", new AjxListener(this, this.viewButtonListener));
	this._popupOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS] = new ZaOperation(ZaOperation.VIEW_XMBXSEARCH_RESULTS, com_zimbra_xmbxsearch.XMBX_ViewResults, com_zimbra_xmbxsearch.XMBX_ViewResults_tt, "ReadMailbox", "ReadMailboxDis", new AjxListener(this, ZaXMBxSearchListController.prototype.viewResultsListener));
    this._popupOperations[ZaOperation.ABORT_SEARCH] = new ZaOperation(ZaOperation.ABORT_SEARCH, com_zimbra_xmbxsearch.XMBXTBB_Abort, com_zimbra_xmbxsearch.XMBXTBB_Abort_tt, "Cancel", "Cancel", new AjxListener(this, this.abortButtonListener));
    this._popupOperations[ZaOperation.DELETE] = new ZaOperation(ZaOperation.DELETE, ZaMsg.TBB_Delete, com_zimbra_xmbxsearch.XMBXTBB_Delete_tt, "Delete", "DeleteDis", new AjxListener(this, this.deleteButtonListener));
    
}
ZaController.initPopupMenuMethods["ZaXMBxSearchListController"].push(ZaXMBxSearchListController.initPopupMenuMethod);

/*ZaXMBxSearchListController.initToolbarMethod =
function () {
	this._toolbarOperations[ZaOperation.REFRESH] = new ZaOperation(ZaOperation.REFRESH, ZaMsg.TBB_Refresh, ZaMsg.TBB_Refresh_tt, "Refresh", "Refresh", new AjxListener(this, this.refreshListener));
	
	var serverList = ZaApp.getInstance().getXMbxServerList();
	this._toolbarOperations[ZaOperation.NEW] = new ZaOperation(ZaOperation.NEW, ZaMsg.TBB_New, com_zimbra_xmbxsearch.XMBXTBB_New_tt,"CrossMailboxSearch","CrossMailboxSearch", new AjxListener(this, this.newButtonListener));
	this._toolbarOperations[ZaOperation.EDIT] = new ZaOperation(ZaOperation.EDIT, com_zimbra_xmbxsearch.XMBXTBB_View, com_zimbra_xmbxsearch.XMBXTBB_View_tt,"Properties","PropertiesDis", new AjxListener(this, this.viewButtonListener));	
	this._toolbarOperations[ZaOperation.VIEW_XMBXSEARCH_RESULTS] =new ZaOperation (ZaOperation.VIEW_XMBXSEARCH_RESULTS, com_zimbra_xmbxsearch.XMBX_ViewResults, com_zimbra_xmbxsearch.XMBX_ViewResults_tt, "ReadMailbox", "ReadMailboxDis", new AjxListener(this, ZaXMBxSearchListController.prototype.viewResultsListener));
	this._toolbarOperations[ZaOperation.ABORT_SEARCH] = new ZaOperation(ZaOperation.ABORT_SEARCH, com_zimbra_xmbxsearch.XMBXTBB_Abort, com_zimbra_xmbxsearch.XMBXTBB_Abort_tt,"Cancel","Cancel", new AjxListener(this, this.abortButtonListener));		
	this._toolbarOperations[ZaOperation.DELETE] = new ZaOperation(ZaOperation.DELETE, ZaMsg.TBB_Delete, com_zimbra_xmbxsearch.XMBXTBB_Delete_tt,"Delete","DeleteDis", new AjxListener(this, this.deleteButtonListener));			

	this._toolbarOrder.push(ZaOperation.REFRESH);
	this._toolbarOrder.push(ZaOperation.NEW);
    this._toolbarOrder.push(ZaOperation.EDIT);
    this._toolbarOrder.push(ZaOperation.VIEW_XMBXSEARCH_RESULTS);
    this._toolbarOrder.push(ZaOperation.ABORT_SEARCH);
    this._toolbarOrder.push(ZaOperation.DELETE);        
}
ZaController.initToolbarMethods["ZaXMBxSearchListController"].push(ZaXMBxSearchListController.initToolbarMethod);*/
/**
*	@method setViewMethod 
*	@param list - list of ZaServer objects
*/
ZaXMBxSearchListController.setViewMethod =
function(list) {
	if(!this._UICreated) {
		this._createUI();
	} 
	ZaApp.getInstance().pushView(this.getContentViewId());
	this._contentView.setDirty(false);
	this._contentView.setObject(list); 	//setObject is delayed to be called after pushView in order to avoid jumping of the view	
	//this.changeActionsState();
}
ZaController.setViewMethods["ZaXMBxSearchListController"].push(ZaXMBxSearchListController.setViewMethod);

ZaXMBxSearchListController.prototype.viewResultsListener =
function(ev) {
	try {
		if(this._contentView) {
			var form = this._contentView.getMyForm();
			if(form) {
				var listFormItems = form.getItemsById(ZaXMBxSearch.A_searchesList);
				if(listFormItems && listFormItems.length && listFormItems[0]) {
					var listWidget = listFormItems[0].widget;
					var selection = listWidget.getSelection();
					var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
					if(list) {
						var selectedSearch = list[0];
						if(selectedSearch) {
							var accountName = selectedSearch.attrs[ZaXMBxSearch.A_targetMbx];
							if(accountName) {
								var account = ZaSearch.findAccount (ZaAccount.A_mail,accountName);
								if(account) {
									ZaAccountListController._viewMailListenerLauncher.call(this, account);
								}
							}
						}
					}
				}
			}
		}		

	} catch (ex) {
		this._handleException(ex, "ZaXMBxSearchListController.prototype.viewResultsListener", null, false);			
	}
}

ZaXMBxSearchListController.prototype.refreshListener = function(ev) {
	this._contentView.refresh(ZaApp.getInstance().getMailServers(true));
	this.changeActionsState();
}


ZaXMBxSearchListController.prototype.refresh = function() {
	this._contentView.refresh(ZaApp.getInstance().getMailServers(true));
}

ZaXMBxSearchListController.prototype.getTitle = 
function () {
	return com_zimbra_xmbxsearch.OVP_xmbxsrch;	
}

/**
* @method _createUI
**/
ZaXMBxSearchListController.prototype._createUI =
function () {

	this._initPopupMenu();
	this._contentView = this._view = new ZaXMBxSearchListView(this._container);

	//always add Help button at the end of the toolbar
	//this._initToolbar();
	
	var elements = new Object();
	elements[ZaAppViewMgr.C_APP_CONTENT] = this._contentView;
    ZaApp.getInstance().getAppViewMgr().createView(this.getContentViewId(), elements);
    this._UICreated = true;
	ZaApp.getInstance()._controllers[this.getContentViewId ()] = this ;
}

ZaXMBxSearchListController.prototype.getPopUpOperation =
function() {
    return this._popupOperations;
}

// new button was pressed
ZaXMBxSearchListController.prototype.newButtonListener =
function(ev) {
	var newSrch = new ZaXMBxSearch();
	ZaApp.getInstance().getXMbxSearchViewController().show(newSrch);
    if (appNewUI) {
        var parentPath = ZaTree.getPathByArray([ZaMsg.OVP_home, ZaMsg.OVP_toolMig, com_zimbra_xmbxsearch.OVP_xmbxsrch]);
        ZaZimbraAdmin.getInstance().getOverviewPanelController().addObjectItem(parentPath, newSrch.name, null, false, false, newSrch);
    }
}

ZaXMBxSearchListController.prototype.listActionListener =
function (ev) {
	this.changeActionsState();
	this._actionMenu.popup(0, ev.docX, ev.docY);
}

ZaXMBxSearchListController.prototype.viewButtonListener =
function(ev) {
	if(this._contentView) {
		var form = this._contentView.getMyForm();
		if(form) {
			var listFormItems = form.getItemsById(ZaXMBxSearch.A_searchesList);
			if(listFormItems && listFormItems.length && listFormItems[0]) {
				var listWidget = listFormItems[0].widget;
				var selection = listWidget.getSelection();
				var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
				if(list) {
					var selectedSearch = list[0];
					ZaApp.getInstance().getXMbxSearchViewController().show(selectedSearch);
                    if (appNewUI) {
                        var parentPath = ZaTree.getPathByArray([ZaMsg.OVP_home, ZaMsg.OVP_toolMig, com_zimbra_xmbxsearch.OVP_xmbxsrch]);
                        ZaZimbraAdmin.getInstance().getOverviewPanelController().addObjectItem(parentPath, selectedSearch.name, null, false, false, selectedSearch);
                    }
				}
			}
		}
	}
}

ZaXMBxSearchListController.prototype.abortSearchCallback = function(resp) {
	this._contentView.refresh(ZaApp.getInstance().getMailServers(true));
	this.changeActionsState();
	/*this._toolbar.enable([ZaOperation.ABORT_SEARCH],false);
	this._toolbar.enable([ZaOperation.EDIT],false);
	this._toolbar.enable([ZaOperation.DELETE],false);
	this._toolbar.enable([ZaOperation.VIEW_XMBXSEARCH_RESULTS],false);*/	
}

ZaXMBxSearchListController.prototype.abortButtonListener =
function(ev) {
	if(this._contentView) {
		var form = this._contentView.getMyForm();
		if(form) {
			var listFormItems = form.getItemsById(ZaXMBxSearch.A_searchesList);
			if(listFormItems && listFormItems.length && listFormItems[0]) {
				var listWidget = listFormItems[0].widget;
				var cnt = listWidget.getSelectionCount();
				if (cnt > 0){
					var selection = listWidget.getSelection();
					var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
					if(list) {
						for (var i = 0; i < cnt; i++){
							var selectedLabel = list[i];
							var callback = new AjxCallback(this, ZaXMBxSearchListController.prototype.abortSearchCallback);
							ZaXMBxSearch.abortXMbxSearch(selectedLabel, callback);
						}
					}
				}
			}
		}
	}
}

ZaXMBxSearchListController.prototype.deleteButtonListener =
function(ev) {
	if(this._contentView) {
		var form = this._contentView.getMyForm();
		if(form) {
			var listFormItems = form.getItemsById(ZaXMBxSearch.A_searchesList);
			if(listFormItems && listFormItems.length && listFormItems[0]) {
				var listWidget = listFormItems[0].widget;
				var cnt = listWidget.getSelectionCount();
				if (cnt > 0){
					var selection = listWidget.getSelection();
					var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
					if(list) {
						for (var i = 0; i < cnt; i++){
							var selectedLabel = list[i];
							var callback = new AjxCallback(this, ZaXMBxSearchListController.prototype.abortSearchCallback);
							ZaXMBxSearch.deleteXMbxSearch(selectedLabel, callback);
						}
					}
				}

			}
		}
	}
}
