/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaXMBxSearchListView
* @contructor
* @param parent
* @param app
* @author Greg Solovyev
**/
function ZaXMBxSearchListView (parent) {
	ZaTabView.call(this, parent, "ZaXMBxSearchListView");	
		
	this.initForm(ZaXMBxSearch.listXModel,this.getMyXForm());
	this._localXForm.setController(ZaApp.getInstance());
}

ZaXMBxSearchListView.prototype = new ZaTabView();
ZaXMBxSearchListView.prototype.constructor = ZaXMBxSearchListView;
ZaTabView.XFormModifiers["ZaXMBxSearchListView"] = new Array();


ZaXMBxSearchListView.prototype.setObject = 
function (list) {
	this._containedObject = list;
	var obj = {};
	obj[ZaXMBxSearch.A_searchesList] = new Array();
	obj[ZaXMBxSearch.A_searchesList]._version = 1;
	this._localXForm.setInstance(obj);
	this.doSearchQuery(0);
}

ZaXMBxSearchListView.prototype.doSearchQuery = 
function (ix) {
	if(this._containedObject && this._containedObject[ix]) {
		var callback = new AjxCallback(this, ZaXMBxSearchListView.prototype.querySearchesCallback, ix);
		ZaXMBxSearch.getXMbxSearchesList(this._containedObject[ix], callback);	
	}
}

ZaXMBxSearchListView.prototype.getTabTitle =
function () {
	return com_zimbra_xmbxsearch.XMBX_list_view_title;
}

ZaXMBxSearchListView.prototype.getTabIcon =
function () {
	return "CrossMailboxSearch";
}

ZaXMBxSearchListView.prototype.refresh = function (list) {
	this._containedObject = list;
	var obj = {};
	obj[ZaXMBxSearch.A_searchesList] = new Array();
	obj[ZaXMBxSearch.A_searchesList]._version = 1;
	this._localXForm.setInstance(obj);
	this.doSearchQuery(0);
}

ZaXMBxSearchListView._listObjects = {};

ZaXMBxSearchListView.labelSelectionListener = function (ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			this._selectedItem = ev.item;
			ZaApp.getInstance().getBackupLabelViewController().show(ev.item);
		}
	} else {
		if(ev.item && (ev.item[ZaBackupLabel.A_live]=="1" || ev.item[ZaBackupLabel.A_live]==true)) {
			ZaApp.getInstance().getBackupsViewController().getToolBar().enable([ZaOperation.ABORT_SEARCH],true);
		}
	}
}

ZaXMBxSearchListView.prototype.querySearchesCallback = 
function (ix, resp) {
	if(!resp) {
		var ex = new ZmCsfeException(ZMsg.errorEmptyResponse,ZmCsfeException.CSFE_SVC_ERROR,"ZaXMBxSearchListView.prototype.querySearchesCallback");
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaXMBxSearchListView.prototype.querySearchesCallback");
		this.doSearchQuery(++ix);
	}	
	if(resp.isException && resp.isException()) {
		ZaApp.getInstance().getCurrentController()._handleException(resp.getException(), "ZaXMBxSearchListView.prototype.querySearchesCallback");
		this.doSearchQuery(++ix);
	} else {
		var response;
		if(resp.getResponse) {
			response = resp.getResponse();
		} else if(resp.Body.GetXMbxSearchesListResponse) {
			response = resp;
		}
		var instance = this._localXForm.getInstance();

		if(response.Body.GetXMbxSearchesListResponse[ZaXMBxSearch.E_searchtask]) {
			var searchTaskElement = response.Body.GetXMbxSearchesListResponse[ZaXMBxSearch.E_searchtask];
			if(!(searchTaskElement instanceof Array)) {
				searchTaskElement = [searchTaskElement];
			}			
			var cnt = searchTaskElement.length;
			var searchesList = instance[ZaXMBxSearch.A_searchesList] || [];
			for(var i =0; i < cnt; i++) {
				var searchTask = new ZaXMBxSearch();
				searchTask.initFromJS(searchTaskElement[i]);
				for ( var j=0; ; j++ ) {
					if (j >= searchesList.length) {
						searchesList.push(searchTask); //no found, then add it
						break;
					}

					if (searchesList[j].id == searchTask.id) {
						searchesList[j] = searchTask; //replace it
						break;
					}
				}
				searchesList._version++;
			}
		}
		this._localXForm.setInstance(instance);
		this._localXForm.refresh();
	}
	this.doSearchQuery(++ix);	
}


ZaXMBxSearchListView.labelSelectionListener = function (ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		if(ev.item) {
			this._selectedItem = ev.item;
			ZaApp.getInstance().getXMbxSearchViewController().show(ev.item);
            if (appNewUI) {
                var parentPath = ZaTree.getPathByArray([ZaMsg.OVP_home, ZaMsg.OVP_toolMig, com_zimbra_xmbxsearch.OVP_xmbxsrch]);
                ZaZimbraAdmin.getInstance().getOverviewPanelController().addObjectItem(parentPath, ev.item.name, null, false, false, ev.item);
            }
		}
	} else {
		/*if(ev.item) {
			ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.EDIT],true);
			ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.VIEW_XMBXSEARCH_RESULTS],true);			
			if (((parseInt(ev.item.attrs[ZaXMBxSearch.A_status]) & ZaXMBxSearch.iSTATUS_FINISHED) == ZaXMBxSearch.iSTATUS_FINISHED) ||
				 ((parseInt(ev.item.attrs[ZaXMBxSearch.A_status]) & ZaXMBxSearch.iSTATUS_ABORTED) == ZaXMBxSearch.iSTATUS_ABORTED)
				){
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.ABORT_SEARCH],false);
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.DELETE],true);
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.VIEW_XMBXSEARCH_RESULTS],true);				
			} else if ( parseInt(ev.item.attrs[ZaXMBxSearch.A_status]) == ZaXMBxSearch.iSTATUS_IDLE) {
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.ABORT_SEARCH],false);
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.DELETE],true);
			} else {
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.ABORT_SEARCH],true);
				ZaApp.getInstance().getXMbxSearchListController().getToolBar().enable([ZaOperation.DELETE],false);
			}
		}*/
		ZaApp.getInstance().getXMbxSearchListController().changeActionsState();
	}
}

ZaXMBxSearchListView.createPopupMenu = function (listWidget) {
	ZaApp.getInstance().getCurrentController()._actionMenu = listWidget.actionMenu = new ZaPopupMenu(listWidget, "ActionMenu", null, ZaApp.getInstance().getCurrentController()._popupOperations);
	listWidget.addActionListener(new AjxListener(ZaApp.getInstance().getCurrentController(), ZaApp.getInstance().getCurrentController().listActionListener));
	listWidget.xFormItem = this;
}

ZaXMBxSearchListView.getCustomHeight = function () {
	try {
		var form = this.getForm();
		var formParentElement = this.getForm().parent.getHtmlElement();
		var totalHeight = parseInt(formParentElement.style.height);
		if(isNaN(totalHeight)) {
			totalHeight = formParentElement.clientHeight ? formParentElement.clientHeight : formParentElement.offsetHeight;
		}
		if(totalHeight<=0)
			return "100%";
		else
			return totalHeight - 2;
	} catch (ex) {
        
	}
	return "100%";  					
};

ZaXMBxSearchListView.getCustomWidth = function () {
	try {

		var form = this.getForm();
		var formParentElement = this.getForm().parent.getHtmlElement();
		var totalWidth = parseInt(formParentElement.style.width);
		if(isNaN(totalWidth)) {
			totalWidth = formParentElement.clientWidth ? formParentElement.clientWidth : formParentElement.offsetWidth;
		}
		if(totalWidth<=0)
			return "100%";
		else
			return totalWidth;
	} catch (ex) {
        
	}
	return "100%";  					
};

ZaXMBxSearchListView.myXFormModifier = function(xFormObject) {	
	xFormObject.tableCssStyle="width:100%;overflow:auto;";
	
	
	var headerList = new Array();
	headerList[0] = new ZaListHeaderItem(ZaXMBxSearch.A_serverName, com_zimbra_xmbxsearch.XMBX_serverName_col, null, "150px", null, null, true, true);
	headerList[1] = new ZaListHeaderItem(ZaXMBxSearch.A_searchID, com_zimbra_xmbxsearch.XMBX_searchID_col, null, "150px", null, null, true, true);
	headerList[2] = new ZaListHeaderItem(ZaXMBxSearch.A_targetMbx, com_zimbra_xmbxsearch.XMBX_targetMbx_col, null, "200px", null, null, true, true);		
	headerList[3] = new ZaListHeaderItem(ZaXMBxSearch.A_status, com_zimbra_xmbxsearch.XMBX_status_col, null, "100px", null, null, true, true);	
	headerList[4] = new ZaListHeaderItem(ZaXMBxSearch.A_numMsgs, com_zimbra_xmbxsearch.XMBX_numMsgs_col, null, "100px", null, null, true, true);			
	xFormObject.items = [
	    {ref:ZaXMBxSearch.A_searchesList,  cssClass: "ZaFullPageXFormListView",
			type:_DWT_LIST_, forceUpdate: true,multiselect:true,
            createPopupMenu:ZaXMBxSearchListView.createPopupMenu,
			onSelection:ZaXMBxSearchListView.labelSelectionListener,
			widgetClass:ZaXMBxListView,headerList:headerList,
			getCustomHeight:ZaXMBxSearchListView.getCustomHeight,
	   		getCustomWidth:ZaXMBxSearchListView.getCustomWidth
	   	}
	];
};
ZaTabView.XFormModifiers["ZaXMBxSearchListView"].push(ZaXMBxSearchListView.myXFormModifier);
