/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaXMBxSearchXFormView
* @contructor
* @param parent
* @param app
* @author Greg Solovyev
**/
function ZaXMBxSearchXFormView (parent) {
	ZaTabView.call(this, parent, "ZaXMBxSearchXFormView");	

	this.initForm(ZaXMBxSearch.myXModel,this.getMyXForm());
	this._localXForm.setController(ZaApp.getInstance());
	ZaApp.getInstance().getAccountListController().addCreationListener(new AjxListener(this._localXForm, ZaXMBxSearchXFormView.handleAccountCreation));		
}

ZaXMBxSearchXFormView.prototype = new ZaTabView();
ZaXMBxSearchXFormView.prototype.constructor = ZaXMBxSearchXFormView;
ZaTabView.XFormModifiers["ZaXMBxSearchXFormView"] = new Array();

ZaXMBxSearchXFormView.prototype.initForm = 
function (xModelMetaData, xFormMetaData) {
	if(xModelMetaData == null || xFormMetaData == null)
		throw new AjxException(ZaMsg.ERROR_METADATA_NOT_DEFINED, AjxException.INVALID_PARAM, "ZaXMBxSearchXFormView.prototype.initForm");

	this._localXModel = new XModel(xModelMetaData);
	this._localXForm = new XForm(xFormMetaData, this._localXModel, null, this);
	this._localXForm.setController(ZaApp.getInstance());
	this._localXForm.draw();
	this._drawn = true;
}
/**
 * @param item ZaXMBxSearch
 */
ZaXMBxSearchXFormView.prototype.setObject = 
function (item) {
	this._containedObject = item;
	this._localXForm.setInstance(this._containedObject);
	//this._localXForm.refresh();
	var searchBuilderBtn = this._localXForm.getItemById(this._localXForm.getId()+"_searchBuilderBtn");
	if(searchBuilderBtn && searchBuilderBtn.getWidget()) {
		searchBuilderBtn.getWidget().setSelected(item[ZaXMBxSearch.A_advancedSearchOn]);
	}
	if(!item.attrs[ZaXMBxSearch.A_searchID]) {
		var srchBuilder = this._localXForm.getItemById(this._localXForm.getId()+"_searchBuilder");
		if(srchBuilder)
			srchBuilder.resetPickers();
	}
    if (!appNewUI)
	    this.updateTab();
}

ZaXMBxSearchXFormView.toggleAdvancedSearch = function () {
	var form = this.getForm();
	var instance = this.getInstance();
	instance[ZaXMBxSearch.A_advancedSearchOn] = !instance[ZaXMBxSearch.A_advancedSearchOn];
	form.refresh();	
}

ZaXMBxSearchXFormView.clearSearch = function () {
	var form = this.getForm();
	var instance = this.getInstance();
	var srchBuilder = form.getItemById(form.getId()+"_searchBuilder");
	instance[ZaXMBxSearch.A_query] = "";
	if(srchBuilder)
		srchBuilder.resetPickers();

	form.setInstance(instance);	
}

ZaXMBxSearchXFormView.removeAllButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_Accounts,[]);
	this.getModel().setInstanceValue(this.getInstance(), ZaXMBxSearch.A_tgt_acct_selection_pool, null);
	//instance.attrs[ZaXMBxSearch.A_Accounts] = new Array();
	//var currentTargetList = instance.attrs[ZaXMBxSearch.A_Accounts] ? instance.attrs[ZaXMBxSearch.A_Accounts] : [];		

	//this.getForm().setInstance(instance);	
}


ZaXMBxSearchXFormView.addAllButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var sourceList = instance[ZaXMBxSearch.A_accountPool];
	var currentTargetList = instance.attrs[ZaXMBxSearch.A_Accounts] ? instance.attrs[ZaXMBxSearch.A_Accounts] : [];
	var tmpTarget = new Array();

	//put the current target elements in the temporary array
	if(currentTargetList) {
		var cnt = currentTargetList.length;
		for(var ix=0; ix< cnt; ix++) {
			tmpTarget.push(currentTargetList[ix]);
		}
	}

	tmpTarget.sort(ZaItem.compareNamesDesc);
	var cnt = sourceList.length;
	for(var ix=cnt-1; ix>=0; ix--) {
		//run through source items
		var dup = false;
		var cnt2 = tmpTarget.length;				
		for(var jx = cnt2-1; jx >=0; jx--) {
			//find out if this 	source item is in the target list already
			if(sourceList[ix].name==tmpTarget[jx].name) {
				dup=true;
				tmpTarget.splice(jx,1); //remove it, so we do not compare it anymore
				break;
			}
		}
		if(!dup) {
			//if this item was not already in the target list add it to the target list
			currentTargetList.push(sourceList[ix])
		}
	}
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_Accounts,currentTargetList);
	//this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_accountPool,[]);
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_src_acct_selection_pool,[]);
	//instance.attrs[ZaXMBxSearch.A_Accounts] = currentTargetList;
	//instance[ZaXMBxSearch.A_accountPool] = new Array();
	//this.getForm().setInstance(instance);	
}

ZaXMBxSearchXFormView.addButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var sourceListItems = form.getItemsById(ZaXMBxSearch.A_accountPool);
	var currentTargetList=null;
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[0] && sourceListItems[0].widget) {
		var selection = sourceListItems[0].widget.getSelection();
		currentTargetList = instance.attrs[ZaXMBxSearch.A_Accounts] ? instance.attrs[ZaXMBxSearch.A_Accounts] : [];
		var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
		if(list) {
			list.sort(ZaItem.compareNamesDesc);		
			var tmpTargetList = [];
			var cnt2 = currentTargetList.length;			
			for(var i=0;i<cnt2;i++)				
				tmpTargetList.push(currentTargetList[i]);
				
			tmpTargetList.sort(ZaItem.compareNamesDesc);
			
			var cnt = list.length;
			for(var i=cnt-1; i>=0; i--) {
				var dup = false;
				var cnt2 = tmpTargetList.length;				
				for(var j = cnt2-1; j >=0; j--) {
					if(list[i].name==tmpTargetList[j].name) {
						dup=true;
						tmpTargetList.splice(j,1);
						break;
					}
				}
				if(!dup) {
					currentTargetList.push(list[i])
				}
			}

			//instance.attrs[ZaXMBxSearch.A_Accounts] = currentTargetList;	
			sourceListItems[0].widget.deselectAll();			
		}
	}
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_Accounts,currentTargetList);
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_src_acct_selection_pool,[]);	
	//this.getForm().setInstance(instance);	
}

ZaXMBxSearchXFormView.removeButtonHndlr = function (ev) {
	var form = this.getForm();
	var instance = form.getInstance();
	var targetListItems = form.getItemsById(ZaXMBxSearch.A_Accounts);
	var currentTargetList = null;
	if(targetListItems && (targetListItems instanceof Array) && targetListItems[1] && targetListItems[1].widget) {
		var selection = targetListItems[1].widget.getSelection();
		
		currentTargetList = instance.attrs[ZaXMBxSearch.A_Accounts] ? instance.attrs[ZaXMBxSearch.A_Accounts] : [];
		currentTargetList.sort(ZaItem.compareNamesDesc);
		var tmpTargetList = [];
		var list = (selection instanceof AjxVector) ? selection.getArray() : (selection instanceof Array) ? selection : [selection];
		if(list) {
			list.sort(ZaItem.compareNamesDesc);
			var cnt = list.length;
			var cnt2 = currentTargetList.length;
			for(var i=0;i<cnt2;i++)				
				tmpTargetList.push(currentTargetList[i]);
			
			for(var i=cnt-1; i>=0; i--) {
				var cnt2 = tmpTargetList.length;
				for(var j = cnt2-1; j >=0; j--) {
					if(list[i].name==tmpTargetList[j].name) {
						currentTargetList.splice(j,1);
						tmpTargetList.splice(j,cnt2-j);
					}
				}
			}
			//instance.attrs[ZaXMBxSearch.A_Accounts] = currentTargetList;
		}
	}
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_Accounts,currentTargetList);
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_tgt_acct_selection_pool,[]);	
	
	//this.getForm().setInstance(instance);	
}


ZaXMBxSearchXFormView.addAllBtnEnabled = function () {
	return (this.instance && this.instance[ZaXMBxSearch.A_accountPool] && this.instance[ZaXMBxSearch.A_accountPool].length>0);
}

ZaXMBxSearchXFormView.removeAllBtnEnabled = function () {
	return (this.instance && this.instance.attrs[ZaXMBxSearch.A_Accounts] && this.instance.attrs[ZaXMBxSearch.A_Accounts].length>0);
}

/**
* method of the XForm
**/
ZaXMBxSearchXFormView.forwardBtnEnabled = function () {
	return (this.getInstance()[ZaXMBxSearch.A_poolPagenum] < (this.getInstance()[ZaXMBxSearch.A_poolNumPages]-1) && this.getInstance()[ZaXMBxSearch.A_activateSearch]==1);
};

/**
* method of the XForm
**/
ZaXMBxSearchXFormView.backBtnEnabled = function () {
	return (this.getInstance()[ZaXMBxSearch.A_poolPagenum] > 0 && this.getInstance()[ZaXMBxSearch.A_activateSearch]==1);
};

ZaXMBxSearchXFormView.addBtnEnabled = function () {
	//scope is form
	var retVal = false;
	var sourceListItems = this.getItemsById(ZaXMBxSearch.A_accountPool);
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[0] && sourceListItems[0].widget) {
		var selection = sourceListItems[0].widget.getSelection();
		var selectionArray = (selection instanceof AjxVector) ? selection.getArray() : selection;
		if(selection && selection.length) {
			retVal = true;
		} 
	}
	return retVal;
}

ZaXMBxSearchXFormView.removeBtnEnabled = function () {
	//scope is form
	var retVal = false;
	var sourceListItems = this.getItemsById(ZaXMBxSearch.A_Accounts);
	if(sourceListItems && (sourceListItems instanceof Array) && sourceListItems[1] && sourceListItems[1].widget) {
		var selection = sourceListItems[1].widget.getSelection();
		var selectionArray = (selection instanceof AjxVector) ? selection.getArray() : selection;
		if(selection && selection.length) {
			retVal = true;
		} 
	}
	return retVal;
}

/**
* method of an XFormItem
**/
ZaXMBxSearchXFormView.fwdPoolButtonHndlr = 
function(evt) {
	var fieldObj = this.getForm().parent;
	var currentPageNum = parseInt(this.getInstanceValue(ZaXMBxSearch.A_poolPagenum));
	this.setInstanceValue(currentPageNum+1,ZaXMBxSearch.A_poolPagenum);
	ZaXMBxSearchXFormView.srchButtonHndlr.call(this, evt);
}

/**
* method of an XFormItem
**/
ZaXMBxSearchXFormView.backPoolButtonHndlr = 
function(evt) {
	var fieldObj = this.getForm().parent;
	var currentPageNum = parseInt(this.getInstanceValue(ZaXMBxSearch.A_poolPagenum))-1;
	this.setInstanceValue(currentPageNum,ZaXMBxSearch.A_poolPagenum);
	ZaXMBxSearchXFormView.srchButtonHndlr.call(this, evt);
}

/**
* method of an XFormItem
**/
ZaXMBxSearchXFormView.srchButtonHndlr = function (ev) {
	var instance = this.getForm().getInstance();
	var formParent = this.getForm().parent;
	var params = {};
	params.types = [ZaSearch.ACCOUNTS,ZaSearch.RESOURCES];
	var queryElements = ["(","&",ZaSearch.getSearchByNameQuery (instance[ZaXMBxSearch.A_accountQuery])];
	if(!instance[ZaXMBxSearch.A_hideArchivesAndSystemRes]) {
		queryElements.push("(!(");
		queryElements.push(ZaAccount.A_zimbraIsSystemResource);
		queryElements.push("=*))");
	}
	queryElements.push("(|(!(");
	queryElements.push(ZaAccount.A_zimbraExcludeFromCMBSearch);
	queryElements.push("=*))(");
	queryElements.push(ZaAccount.A_zimbraExcludeFromCMBSearch);
	queryElements.push("=FALSE)))");	
	params.query = queryElements.join("");
	params.limit = ZaSettings.RESULTSPERPAGE;
	params.offset = instance.poolPagenum*ZaSettings.RESULTSPERPAGE;
	params.attrs = [ZaItem.A_zimbraId,ZaAccount.A_uid,ZaAccount.A_displayname,ZaResource.A_zimbraCalResType].join(",");
	params.callback = new AjxCallback(formParent,ZaXMBxSearchXFormView.srchAccountsClbck);
	this.getForm().getModel().setInstanceValue(instance,ZaXMBxSearch.A_activateSearch,0);
	//instance.activateSearch = 0;
	//this.getForm().setInstance(instance);
	ZaSearch.searchDirectory(params);
}

ZaXMBxSearchXFormView.srchAccountsClbck = function (resp) {
	var instance = this._localXForm.getInstance();
	try {
		if(!resp) {
			throw(new AjxException(ZaMsg.ERROR_EMPTY_RESPONSE_ARG, AjxException.UNKNOWN, "ZaXMBxSearchXFormView.srchAccountsClbck"));
		}
		if(resp.isException()) {
			throw(resp.getException());
		} else {
			var response = resp.getResponse().Body.SearchDirectoryResponse;
			var list = new ZaItemList(null);	
			list.loadFromJS(response);	
			this._localXForm.getModel().setInstanceValue(instance,ZaXMBxSearch.A_accountPool,list.getArray());
			//instance[ZaXMBxSearch.A_accountPool] = list.getArray();
			var poolNumPages = Math.ceil(response.searchTotal/ZaSettings.RESULTSPERPAGE);
			this._localXForm.getModel().setInstanceValue(instance,ZaXMBxSearch.A_poolNumPages,poolNumPages);
			//instance.poolNumPages = poolNumPages;
		}
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaXMBxSearchXFormView.srchAccountsClbck");	
	}	
	this._localXForm.getModel().setInstanceValue(instance,ZaXMBxSearch.A_activateSearch,1);
	//instance[ZaXMBxSearch.A_activateSearch] = 1;
	//this._localXForm.setInstance(instance);	
}

ZaXMBxSearchXFormView.launchNewAccWiz = function (ev) {
	var controller = ZaApp.getInstance().getCurrentController();
	try {
		var newAccount = new ZaAccount();
		newAccount.loadNewObjectDefaults("name", ZaSettings.myDomainName);
		newAccount.getAttrs = {all:true};
		newAccount.attrs[ZaAccount.A_zimbraHideInGal] = "TRUE";
		newAccount.attrs[ZaAccount.A_zimbraIsSystemResource] = "TRUE";	
		newAccount.attrs[ZaAccount.A_zimbraExcludeFromCMBSearch] = "TRUE";			
		newAccount.attrs[ZaAccount.A_lastName] = com_zimbra_xmbxsearch.XMBX_TargetAcctName;
		newAccount.attrs[ZaAccount.A_displayname] = com_zimbra_xmbxsearch.XMBX_TargetAcctName;
		newAccount.attrs[ZaAccount.A_description] = com_zimbra_xmbxsearch.XMBX_TargetAcctDescription;		
		newAccount.attrs[ZaAccount.A_notes] = com_zimbra_xmbxsearch.XMBX_TargetAcctNotes;				
		if(!ZaApp.getInstance()._newAccountWizard2)
			ZaApp.getInstance()._newAccountWizard2 = new ZaNewAccountXWizard(controller._container);	
			
		ZaApp.getInstance()._newAccountWizard2.setObject(newAccount);
		ZaApp.getInstance()._newAccountWizard2.popup();
	} catch (ex) {
		ZaApp.getInstance().getCurrentController()._handleException(ex, "ZaXMBxSearchXFormView.launchNewAccWiz", null, false);
	}	
}

ZaXMBxSearchXFormView.handleAccountCreation = function (ev) {
	if(ev) {
		if(ev.getDetails()) {
			var instance = this.getInstance();
			this.getModel().setInstanceValue(instance,ZaXMBxSearch.A_targetMbx,ev.getDetails().name);
		}
	}
}

ZaXMBxSearchXFormView.isNewTask = 
function (inst) {
	return (!(inst.attrs[ZaXMBxSearch.A_searchID]>0));
}

ZaXMBxSearchXFormView.accPoolSelectionListener = function() {
	var arr = this.widget.getSelection();
	if(arr && arr.length) {
		arr.sort();
		this.getModel().setInstanceValue(this.getInstance(), ZaXMBxSearch.A_src_acct_selection_pool, arr);
	} else {
		this.getModel().setInstanceValue(this.getInstance(), ZaXMBxSearch.A_src_acct_selection_pool, null);
	}	
}

ZaXMBxSearchXFormView.accSelectionListener = function() {
	var arr = this.widget.getSelection();
	if(arr && arr.length) {
		arr.sort();
		this.getModel().setInstanceValue(this.getInstance(), ZaXMBxSearch.A_tgt_acct_selection_pool, arr);
	} else {
		this.getModel().setInstanceValue(this.getInstance(), ZaXMBxSearch.A_tgt_acct_selection_pooll, null);
	}	
}


ZaXMBxSearchXFormView.myXFormModifier = function(xFormObject) {	
	xFormObject.tableCssStyle="width:100%;overflow:auto;";
	xFormObject.numCols = 1;
	var buttonChoices = new Array();
	buttonChoices.push(
		{value:ZaXMBxPicker.BASIC, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.BASIC]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.BASIC]},
		{value:ZaXMBxPicker.DATE, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.DATE]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.DATE]}, 
		{value:ZaXMBxPicker.TIME, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.TIME]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.TIME]},
		{value:ZaXMBxPicker.ZIMLET, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.ZIMLET]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.ZIMLET]},
		{value:ZaXMBxPicker.FLAG, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.FLAG]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.FLAG]},
		{value:ZaXMBxPicker.ATTACHMENT, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.ATTACHMENT]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.ATTACHMENT]},
		{value:ZaXMBxPicker.SIZE, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.SIZE]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.SIZE]},
		{value:ZaXMBxPicker.DOMAIN, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.DOMAIN]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.DOMAIN]},
		{value:ZaXMBxPicker.RESET, label:com_zimbra_xmbxsearch[ZaXMBxPicker.MSG_KEY[ZaXMBxPicker.RESET]],icon:ZaXMBxPicker.IMAGE[ZaXMBxPicker.CLOSE]}
	);
	
	var sourceHeaderList = new Array();
	sourceHeaderList[0] = new ZaListHeaderItem("type", ZaMsg.ALV_Type_col, null, "34px", null, "objectClass", true, true);
	sourceHeaderList[1] = new ZaListHeaderItem(ZaAccount.A_name, ZaMsg.ALV_Name_col, null, "116px", null, ZaAccount.A_name, true, true);


	var case1 = {type:_ZATABCASE_, visibilityChecks:[[XForm.checkInstanceValueNotEmty,ZaXMBxSearch.A_searchID]],numCols:1};
	var case2 = {type:_ZATABCASE_, visibilityChecks:[[XForm.checkInstanceValueEmty,ZaXMBxSearch.A_searchID]], numCols:1};	
	var cases = [case1,case2];
	xFormObject.items = [{type:_SWITCH_, align:_LEFT_, valign:_TOP_, items:cases}];
	case1.items=[
		{type:_ZA_TOP_GROUPER_, numCols:2,colSizes:["275px","*"], width: "100%",label:com_zimbra_xmbxsearch.XMBX_BuildSearchQuery,
			items: [	
				{height:Dwt.DEFAULT, width:Dwt.DEFAULT, type:_OUTPUT_,width: "100%",label:com_zimbra_xmbxsearch.XMBX_Query, ref:ZaXMBxSearch.A_query}
			 ]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Results, numCols:2,colSizes:["275px","*"], width:"100%", 
		   items:[
			    {type:_OUTPUT_, ref:ZaXMBxSearch.A_status,width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_status+":",
					getDisplayValue:function(itemVal) {
						var retVal = "";
						var status = parseInt(itemVal);
						if(!itemVal) {
							retVal = com_zimbra_xmbxsearch.XMBX_StatusIdle;				
						} else if (itemVal==1) {
							retVal= com_zimbra_xmbxsearch.XMBX_StatusError;						
						} else {
							if ((status & ZaXMBxSearch.iSTATUS_FINISHED) == ZaXMBxSearch.iSTATUS_FINISHED) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusFinished;
							} else if ((status & ZaXMBxSearch.iSTATUS_ABORTING) == ZaXMBxSearch.iSTATUS_ABORTING) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusAborting;
							} else if ((status & ZaXMBxSearch.iSTATUS_ABORTED) == ZaXMBxSearch.iSTATUS_ABORTED) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusAborted;
							} else if ((status & ZaXMBxSearch.iSTATUS_SENDING_NOTIFICATIONS) == ZaXMBxSearch.iSTATUS_SENDING_NOTIFICATIONS) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusSendingNotifications;
							} else if ((status & ZaXMBxSearch.iSTATUS_IMPORTING) == ZaXMBxSearch.iSTATUS_IMPORTING) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusImporting;
							} else if ((status & ZaXMBxSearch.iSTATUS_SEARCHING) == ZaXMBxSearch.iSTATUS_SEARCHING) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusSearching;
							} else if ((status & ZaXMBxSearch.iSTATUS_STARTED) == ZaXMBxSearch.iSTATUS_STARTED) {
								retVal = com_zimbra_xmbxsearch.XMBX_StatusStarted;
							}
												
							if ((parseInt(itemVal) & ZaXMBxSearch.iSTATUS_ERROR) == ZaXMBxSearch.iSTATUS_ERROR) {
								retVal += " ";
								retVal += com_zimbra_xmbxsearch.XMBX_StatusWithErrors;
							}							
						}
						return retVal;
					}
				},
			    {type:_OUTPUT_, ref:ZaXMBxSearch.A_serverName,width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_ServerName
				},					
			    {type:_OUTPUT_, ref:ZaXMBxSearch.A_numMsgs,width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_MessagesFound
				},				
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_targetMbx,width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_ResultsTargetMbx
				},
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_targetFolder,width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_ResultsTargetFolder					
				},
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_limit,width:"50px",label:com_zimbra_xmbxsearch.XMBX_maximumNumberOfMessages,
					visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_doLimit,true]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_doLimit],
					nowrap:false,labelWrap:true	
				}			]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Accounts, numCols:2, colSizes:["275px","*"], width:"100%", 
			items:[
				{type:_OUTPUT_,label:com_zimbra_xmbxsearch.XMBX_AccToSearch, value:com_zimbra_xmbxsearch.XMBX_AllAccToSearch,
				 	visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_searchAll,1]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_searchAll]
				},
				{type:_REPEAT_,label:com_zimbra_xmbxsearch.XMBX_AccToSearch,ref:ZaXMBxSearch.A_Accounts,
				 	visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_searchAll,0]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_searchAll],						 
					showAddButton:false,showRemoveButton:false,showMoveUpButton:false,showMoveDownButton:false,alwaysShowAddButton:false,
					items:[
						{type:_OUTPUT_, ref:".", label:null,labelLocation:_NONE_}
					]
				}						

			]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_SearchScope, numCols:2, colSizes:["275px","*"], width:"100%", 
		   items:[
				{type:_CHECKBOX_, ref:ZaXMBxSearch.A_searchArchives,label:com_zimbra_xmbxsearch.XMBX_SearchArchives,
					trueValue:1,falseValue:0,readOnly:true,enableDisableChecks:[function() {return false; }],visibilityChecks:[]
				},
				{type:_CHECKBOX_, ref:ZaXMBxSearch.A_searchLive,label:com_zimbra_xmbxsearch.XMBX_SearchLive,
					trueValue:1,falseValue:0,readOnly:true,enableDisableChecks:[function() {return false; }],visibilityChecks:[]
				},
				{type:_CHECKBOX_, ref:ZaXMBxSearch.A_inDumpster,label:com_zimbra_xmbxsearch.XMBX_InDumpster,
					trueValue:1,falseValue:0, readOnly:true,enableDisableChecks:[function() {return false; }], visibilityChecks:[]
				}
			]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Notification, numCols:2, colSizes:["275px","*"], width:"100%", 
			items: [
				{type:_CHECKBOX_, ref:ZaXMBxSearch.A_sendNotification,label:com_zimbra_xmbxsearch.XMBX_SendSearchNotification,
					trueValue:1,falseValue:0, readOnly:true,
					enableDisableChecks:[function() {return false; }],
					visibilityChecks:[]
				},
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_notificationReceivers,label:com_zimbra_xmbxsearch.XMBX_notificationReceivers,
					visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],
					width:"60em"
				},
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_notificationSubj,label:com_zimbra_xmbxsearch.XMBX_notificationSubj,
					visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],
					width:"60em"
				},
				{type:_OUTPUT_, ref:ZaXMBxSearch.A_notificationMessage,label:com_zimbra_xmbxsearch.XMBX_notificationMessage,
					visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],
					width:"60em"
				}
			]
		}
	];
	case2.items = [
		{type:_ZA_TOP_GROUPER_, numCols:1,colSizes:["*"], width: "100%",label:com_zimbra_xmbxsearch.XMBX_BuildSearchQuery,
			items: [
				{type:_GROUP_, numCols:6, colSizes:["470px","10px","90px","10px","90px","100%"], //use 100% to make the last one use the blank fully.
					cssStyle:"padding:7px",
					items:[
						{type:_OUTPUT_, height:"20px", value:com_zimbra_xmbxsearch.XMBX_EnterQuery,colSpan:6},
						{type:_TEXTFIELD_, height:"23px", width:"470px", cssStyle:"font-size:13px", align:_CENTER_, label:null, ref:ZaXMBxSearch.A_query,visibilityChecks:[],enableDisableChecks:[],bmolsnr:true},
						{type:_CELLSPACER_},
						{type:_DWT_BUTTON_,label:com_zimbra_xmbxsearch.XMBX_searchBuilder,onActivate:"ZaXMBxSearchXFormView.toggleAdvancedSearch.call(this)", width:"80px",
						id:"searchBuilderBtn",labelLocation:(DwtLabel.IMAGE_LEFT | DwtLabel.ALIGN_CENTER | DwtButton.TOGGLE_STYLE)},
						{type:_CELLSPACER_},
						{type:_DWT_BUTTON_,label:com_zimbra_xmbxsearch.XMBX_clearQuery,onActivate:"ZaXMBxSearchXFormView.clearSearch.call(this)", width:"80px"},				
						{type:_CELLSPACER_},
						{type:_SPACER_, colSpan:6, height:"5px"},
					    {height:Dwt.DEFAULT, width:"100%", type:_XMBX_SRCH_BUILDER_, choices:buttonChoices, ref:ZaXMBxSearch.A_query, 
							visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_advancedSearchOn,true]],
							visibilityChangeEventSources:[ZaXMBxSearch.A_advancedSearchOn],bmolsnr:true,											
					    	colSpan:6, id:"searchBuilder"
					    }
				    ]
				}
			 ]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Results, numCols:2,colSizes:["275px","100%"], width:"100%",
		   items:[
	   			{type:_DWT_ALERT_,style: DwtAlert.INFORMATION, width:"95%",
	 				iconVisible: true,
					content: com_zimbra_xmbxsearch.XMBX_ResultsExplanation,							
					align:_LEFT_,colSpan:2
				},
			    {type:_OSELECT1_, ref:ZaXMBxSearch.A_serverId, width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_ServerName,
					choices:ZaApp.getInstance().getMbxServerIdListChoices(),
					visibilityChecks:[],
					enableDisableChecks:[]
				},	
				{type:_GROUP_, width:"100%", label:com_zimbra_xmbxsearch.XMBX_ResultsTargetMbx,
					nowrap:true, numCols:4,colSizes:["210px","30px","100%"], //use 100% to make the last one use the blank fully.
					items:[
						{type:_DYNSELECT_, ref:ZaXMBxSearch.A_targetMbx,editable:true, 
						inputWidth:"200px",
						dataFetcherClass:ZaSearch,dataFetcherMethod:ZaSearch.prototype.dynSelectSearch,
						dataFetcherTypes:[ZaSearch.ACCOUNTS],
						dataFetcherAttrs:[ZaItem.A_zimbraId, ZaItem.A_cn, ZaAccount.A_name, ZaAccount.A_displayname, ZaAccount.A_mail],
						enableDisableChecks:[],visibilityChecks:[]
						},
						{type:_OUTPUT_,value:(" " + com_zimbra_xmbxsearch.XMBX_OR + " "), width:"15px"},
						{type:_DWT_BUTTON_, label:com_zimbra_xmbxsearch.XMBX_NewAccountWiz, width:"180px",
						   onActivate:ZaXMBxSearchXFormView.launchNewAccWiz,align:_LEFT_,icon:"Account",
						   enableDisableChecks:[],visibilityChecks:[]
						}
					]
				},
				{type:_TEXTFIELD_, ref:ZaXMBxSearch.A_targetFolder, width:"200px", 
					label:com_zimbra_xmbxsearch.XMBX_ResultsTargetFolder,visibilityChecks:[],enableDisableChecks:[]
				},
				{type:_CHECKBOX_, ref:ZaXMBxSearch.A_doLimit,label:com_zimbra_xmbxsearch.XMBX_LimitResultSet,
					trueValue:true,falseValue:false,
					visibilityChecks:[],enableDisableChecks:[]
				},
				{type:_TEXTFIELD_, ref:ZaXMBxSearch.A_limit,label:com_zimbra_xmbxsearch.XMBX_maximumNumberOfMessages,
					visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_doLimit,true]],
					visibilityChangeEventSources:[ZaXMBxSearch.A_doLimit],
					enableDisableChecks:[],
					nowrap:false,labelWrap:true, width:"100px"
				}
		   ]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Accounts, colSizes:["*"], numCols:1, width:"100%",
			   items:[
				{type:_GROUP_, numCols:1, colSizes:["100%"], cssStyle:"padding:7px",
					items:[
					{type:_GROUP_, numCols:2,  colSizes:["275","*"],
						items: [
							{ref:ZaXMBxSearch.A_searchAll,type:_RADIO_,groupname:"accounts_radio_group",
								label:com_zimbra_xmbxsearch.XMBX_SearchAllAccounts, labelLocation:_LEFT_, align: _LEFT_,
								elementChanged:function(elementValue,instanceValue, event) {
									if(elementValue) {
										this.getForm().itemChanged(this, 1, event);	
										this.getForm().getInstance().attrs[ZaXMBxSearch.A_Accounts]=[];
									}
								},
								updateElement:function(value) {
									this.getElement().checked = (value==1);
								},
								visibilityChecks:[],enableDisableChecks:[]								
							},
							{ref:ZaXMBxSearch.A_searchAll,type:_RADIO_,groupname:"accounts_radio_group",
								label:com_zimbra_xmbxsearch.XMBX_SelectMailboxes, labelLocation:_LEFT_, align: _LEFT_,
								elementChanged:function(elementValue,instanceValue, event) {
									if(elementValue) {
										this.getForm().itemChanged(this, 0, event);	
									}
								},
								updateElement:function(value) {
									this.getElement().checked = (value !=1 );
								},
								visibilityChecks:[],enableDisableChecks:[]							
							},
							{ref:ZaXMBxSearch.A_hideArchivesAndSystemRes, type:_CHECKBOX_,
								label:com_zimbra_xmbxsearch.XMBX_HideArchivesAndSystemResources,
								visibilityChecks:[[XForm.checkInstanceValueNot,ZaXMBxSearch.A_searchAll,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_searchAll],
								enableDisableChecks:[],
								trueValue:true, falseValue:false
							}
						]
					},
					{type:_GROUP_, numCols:3, width:"720px", colSizes:["275px","130px","275px"],cellspacing:"10px",
						visibilityChecks:[[XForm.checkInstanceValueNot,ZaXMBxSearch.A_searchAll,1]],
						visibilityChangeEventSources:[ZaXMBxSearch.A_searchAll],											
						items:[
						{type:_TEXTFIELD_, cssClass:"admin_xform_name_input",width:"275px", ref:ZaXMBxSearch.A_accountQuery, label:null,
					      elementChanged: function(elementValue,instanceValue, event) {
							  var charCode = event.charCode;
							  if (charCode == 13 || charCode == 3) {
							      ZaXMBxSearchXFormView.srchButtonHndlr.call(this);
							  } else {
							      this.getForm().itemChanged(this, elementValue, event);
							  }
				      		},
				      		enableDisableChecks:[],visibilityChecks:[]
						},
						{type:_DWT_BUTTON_, label:com_zimbra_xmbxsearch.XMBX_ButtonLookup, width:"120px",
						   onActivate:ZaXMBxSearchXFormView.srchButtonHndlr,align:_CENTER_,
							enableDisableChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_activateSearch,1]],
							enableDisableChangeEventSources:[ZaXMBxSearch.A_activateSearch]										   
						},
						{type:_ZACENTER_GROUPER_, label:com_zimbra_xmbxsearch.XMBX_AccountsTosearch,rowSpan:6, 
							cssStyle:"width:255px;",borderCssClass:"CenterGrouperBorder ZaXMBxAccountsListGrouper",
							items:[
		 				    	{ref: ZaXMBxSearch.A_Accounts, type:_DWT_LIST_, height:"240px", width:"255px",
		 				    		cssClass: "DLSource", forceUpdate: true, align:_CENTER_,
		 				    		widgetClass:ZaAccMiniListView, headerList:sourceHeaderList,
		 				    		hideHeader:true,
 				    				onSelection:ZaXMBxSearchXFormView.accSelectionListener,
 				    				visibilityChecks:[],enableDisableChecks:[]		 				    	
		 				    	}
		 					]
		 				}, 						
	 				    {ref: ZaXMBxSearch.A_accountPool, type:_DWT_LIST_, height:"200px", width:"275px",
	 				    	cssClass: "DLSource", forceUpdate: true, align:_CENTER_,
	 				    	widgetClass:ZaAccMiniListView, headerList:sourceHeaderList,
	 				    	rowSpan:4,hideHeader:true,
	 				    	onSelection:ZaXMBxSearchXFormView.accPoolSelectionListener,
 					    	visibilityChecks:[],enableDisableChecks:[]
	 				    	
	 				    },
						{type:_DWT_BUTTON_, label:AjxMsg.addAll, width:"120px",align:_CENTER_,
						   	onActivate:ZaXMBxSearchXFormView.addAllButtonHndlr,
					   		enableDisableChecks:[[XForm.checkInstanceValueNotEmty,ZaXMBxSearch.A_accountPool]],
					   		enableDisableChangeEventSources:[ZaXMBxSearch.A_accountPool]						   
						},
						{type:_DWT_BUTTON_, label:AjxMsg.add, width:"120px",align:_CENTER_,
						   onActivate:ZaXMBxSearchXFormView.addButtonHndlr,
					   		enableDisableChecks:[[XForm.checkInstanceValueNotEmty,ZaXMBxSearch.A_src_acct_selection_pool]],
					   		enableDisableChangeEventSources:[ZaXMBxSearch.A_src_acct_selection_pool]						   
						}, 
						{type:_DWT_BUTTON_, label:AjxMsg.remove, width:"120px",align:_CENTER_,
						   	onActivate:ZaXMBxSearchXFormView.removeButtonHndlr,
					   		enableDisableChecks:[[XForm.checkInstanceValueNotEmty,ZaXMBxSearch.A_tgt_acct_selection_pool]],
					   		enableDisableChangeEventSources:[ZaXMBxSearch.A_tgt_acct_selection_pool]						   
						}, 
						{type:_DWT_BUTTON_, label:AjxMsg.removeAll, width:"120px",align:_CENTER_,
						   	onActivate:ZaXMBxSearchXFormView.removeAllButtonHndlr,
						   	enableDisableChecks:[[XForm.checkInstanceValueNotEmty,ZaXMBxSearch.A_Accounts]],
					   		enableDisableChangeEventSources:[ZaXMBxSearch.A_Accounts]						   
						},
						{type:_GROUP_,numCols:3,colSizes:["80px","*","80px"], 
							items:[
								{type:_SPACER_, colSpan:3},
								{type:_DWT_BUTTON_, label:ZaMsg.Previous, width:75, 
								   id:"backButton", icon:"LeftArrow", disIcon:"LeftArrowDis",
								   onActivate:ZaXMBxSearchXFormView.backPoolButtonHndlr,align:_CENTER_,
								   enableDisableChecks:[ZaXMBxSearchXFormView.backBtnEnabled],
								   enableDisableChangeEventSources:[ZaXMBxSearch.A_poolPagenum,ZaXMBxSearch.A_activateSearch]
								},
								{type:_CELLSPACER_},
								{type:_DWT_BUTTON_, label:ZaMsg.Next, width:75,
								   id:"fwdButton", icon:"RightArrow", disIcon:"RightArrowDis", 
								   onActivate:ZaXMBxSearchXFormView.fwdPoolButtonHndlr,align:_CENTER_,
								   enableDisableChecks:[ZaXMBxSearchXFormView.forwardBtnEnabled],
								   enableDisableChangeEventSources:[ZaXMBxSearch.A_poolPagenum,ZaXMBxSearch.A_activateSearch,ZaXMBxSearch.A_poolNumPages]
								}													
							]
						},
						{type:_CELLSPACER_},
						{type:_DWT_ALERT_,style: DwtAlert.INFORMATION, 
			 				iconVisible: true,
							content: ZaMsg.pleaseWaitSearching,width:"200px",				
							align:_LEFT_,cssStyle:"position:relative;top:-200;left:125;opacity: 0.8",
							visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_activateSearch,0]],
							visibilityChangeEventSources:[ZaXMBxSearch.A_activateSearch]																		
						}
					]}
				]
			}
			]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_SearchScope, numCols:2, colSizes:["275px","*"], width:"100%", 
		   items:[
					{type:_CHECKBOX_, ref:ZaXMBxSearch.A_searchArchives,label:com_zimbra_xmbxsearch.XMBX_SearchArchives,
						trueValue:1,falseValue:0,enableDisableChecks:[],visibilityChecks:[]
					},
					{type:_CHECKBOX_, ref:ZaXMBxSearch.A_searchLive,label:com_zimbra_xmbxsearch.XMBX_SearchLive,
						trueValue:1,falseValue:0,enableDisableChecks:[],visibilityChecks:[]
					},
					{type:_CHECKBOX_, ref:ZaXMBxSearch.A_inDumpster,label:com_zimbra_xmbxsearch.XMBX_InDumpster,
						trueValue:1,falseValue:0, enableDisableChecks:[], visibilityChecks:[]
					}
			]
		},
		{type:_ZA_TOP_GROUPER_,label:com_zimbra_xmbxsearch.XMBX_Notification, numCols:2, colSizes:["275px","100%"], width:"100%",
			   items:[
					{type:_CHECKBOX_, ref:ZaXMBxSearch.A_sendNotification,label:com_zimbra_xmbxsearch.XMBX_SendSearchNotification,
						trueValue:1,falseValue:0,visibilityChecks:[],enableDisableChecks:[]},		
					{type:_DWT_ALERT_,style: DwtAlert.INFORMATION, width:"90%",
		 				iconVisible: true,
						content: com_zimbra_xmbxsearch.XMBX_notificationMessageNote,
							
						align:_LEFT_, colSpan:2,
						visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
						visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification]				
						
					},
					{type:_GROUP_, numCols:1, colSizes:["700px"], cssStyle:"padding:7px", 
						visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
						visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],	
						colSpan:2,
						items:[
							{type:_OUTPUT_,value:com_zimbra_xmbxsearch.XMBX_notificationReceivers,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],															
								colSpan:2
							},
							{type:_TEXTAREA_, ref:ZaXMBxSearch.A_notificationReceivers,label:null,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],	enableDisableChecks:[],									
								colSpan:2,labelLocation:_NONE_,width:"60em"},
							{type:_SPACER_,colSpan:2,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification]									
							},						
							{type:_OUTPUT_,value:com_zimbra_xmbxsearch.XMBX_notificationSubj,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],									
								colSpan:2
							},						
							{type:_TEXTFIELD_, ref:ZaXMBxSearch.A_notificationSubj,label:null,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],	enableDisableChecks:[],								
								colSpan:2,labelLocation:_NONE_,width:"60em"
							},					
							{type:_SPACER_, colSpan:2,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification]
							},
							{type:_OUTPUT_,value:com_zimbra_xmbxsearch.XMBX_notificationMessage,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],									
								colSpan:2
							},						
							{type:_TEXTAREA_, ref:ZaXMBxSearch.A_notificationMessage,label:null,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification],	enableDisableChecks:[],							
								colSpan:2,labelLocation:_NONE_,width:"60em"
							},
							{type:_SPACER_,colSpan:2,
								visibilityChecks:[[XForm.checkInstanceValue,ZaXMBxSearch.A_sendNotification,1]],
								visibilityChangeEventSources:[ZaXMBxSearch.A_sendNotification]	
							}
						]
					}
			   ]
		}
	];
};
ZaTabView.XFormModifiers["ZaXMBxSearchXFormView"].push(ZaXMBxSearchXFormView.myXFormModifier);

ZaXMBxSearchXFormView.prototype.getTabChoices =
function() {
    this.tabChoices = new Array();
    this.tabChoices.push({value:1, label:ZaMsg.TABT_GeneralPage});
    return this.tabChoices;
}