/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2007, 2009, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

function ZaXMBxSizePicker(parent) {
	ZaXMBxPicker.call(this, parent, ZaXMBxPicker.SIZE);
};

ZaXMBxSizePicker.prototype = new ZaXMBxPicker;
ZaXMBxSizePicker.prototype.constructor = ZaXMBxSizePicker;

ZaXMBxPicker.CTOR[ZaXMBxPicker.SIZE] = ZaXMBxSizePicker;

ZaXMBxSizePicker.OP_SELECT_OPTIONS = [
	{ label: com_zimbra_xmbxsearch.XMBX_larger, 	value: "larger", 	selected: true  },
	{ label: com_zimbra_xmbxsearch.XMBX_smaller, value: "smaller", 	selected: false }];

ZaXMBxSizePicker.UNIT_SELECT_OPTIONS = [
	{ label: com_zimbra_xmbxsearch.XMBX_bytes, 		value: "b", 	selected: false },
	{ label: com_zimbra_xmbxsearch.XMBX_kilobytes, 	value: "kb", 	selected: true  },
	{ label: com_zimbra_xmbxsearch.XMBX_megabytes, 	value: "mb", 	selected: false }];

ZaXMBxSizePicker.prototype.toString = 
function() {
	return "ZaXMBxSizePicker";
};

ZaXMBxSizePicker.prototype._setupPicker =
function(parent) {
	var picker = new DwtComposite(parent);

    var size = 16;
	var opId = Dwt.getNextId();
	var unitsId = Dwt.getNextId();    
	var fieldId = Dwt.getNextId();

	var html = new Array(21);
	var i = 0;
	html[i++] = "<center><table cellpadding=5 cellspacing=0 border=0>";
	html[i++] = "<tr valign='middle'><td align='right' nowrap>";
	html[i++] = com_zimbra_xmbxsearch.XMBX_size;
	html[i++] = ":</td><td align='left' nowrap colspan=1 id='";
	html[i++] = opId;
	html[i++] = "'></td></tr>";
	html[i++] = "<tr valign='middle'><td align='right' nowrap>";
	html[i++] = com_zimbra_xmbxsearch.XMBX_value;
	html[i++] = ":</td>";
	html[i++] = "<td align='left' nowrap><input type='text' autocomplete='off' nowrap size='";
	html[i++] = size;
	html[i++] = "' id='";
	html[i++] = fieldId;
	html[i++] = "'/></td></tr>";
	html[i++] = "<tr valign='middle'><td align='right' nowrap>";
	html[i++] = com_zimbra_xmbxsearch.XMBX_units;
	html[i++] = ":</td><td align='left' nowrap id='";
	html[i++] = unitsId;
	html[i++] = "'></td></tr>";
	html[i++] = "</table></center>";
	picker.getHtmlElement().innerHTML = html.join("");

	// set up DwtSelects
	var selectChangeListener = new AjxListener(this, this._selectChangeListener);
	this._op = new DwtSelect(this);
	this._op.addChangeListener(selectChangeListener);
	for (var i = 0; i < ZaXMBxSizePicker.OP_SELECT_OPTIONS.length; i++) {
		var option = ZaXMBxSizePicker.OP_SELECT_OPTIONS[i];
		this._op.addOption(option.label, option.selected, option.value);
	}
	this._op.reparentHtmlElement(opId);

	this._units = new DwtSelect(this);
	this._units.addChangeListener(selectChangeListener);
	for (var i = 0; i < ZaXMBxSizePicker.UNIT_SELECT_OPTIONS.length; i++) {
		var option = ZaXMBxSizePicker.UNIT_SELECT_OPTIONS[i];
		this._units.addOption(option.label, option.selected, option.value);
	}
	this._units.reparentHtmlElement(unitsId);

	// set up input field
	this._size = document.getElementById(fieldId);
	Dwt.setHandler(this._size, DwtEvent.ONKEYUP, ZaXMBxSizePicker._onKeyUp);
	Dwt.associateElementWithObject(this._size, this);
};

ZaXMBxSizePicker.prototype._updateQuery = 
function() {
	if (this._size.value && this._size.value.match(/^[1-9][0-9]*$/)) {
		this.setQuery(this._op.getValue() + ":" + this._size.value + this._units.getValue());
	} else {
		this.setQuery("");
	}
};

ZaXMBxSizePicker.prototype._selectChangeListener = 
function(ev) {
	this._updateQuery();
//	this.execute();
};

ZaXMBxSizePicker._onKeyUp =
function(ev) {
	var element = DwtUiEvent.getTarget(ev);
	var picker = Dwt.getObjectFromElement(element);
	var charCode = DwtKeyEvent.getCharCode(ev);
	if (charCode == 13 || charCode == 3) {
		picker._updateQuery();
	    return false;
	} else {
		picker._updateQuery();
		return true;
	}

};
