/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite, Network Edition.
 * Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Zimbra, Inc.  All Rights Reserved.
 * ***** END LICENSE BLOCK *****
 */

if (window.console && window.console.log) {
    console.log ("Starting loading domainadmin.js");
}

function ZaDomainAdmin() {}

var adminName = ZaZimbraAdmin.currentUserLogin;

if (adminName) {
    var emailChunks = adminName.split("@");
    var tmpDomain = new ZaDomain();
    if (emailChunks.length > 1 ) {
        tmpDomain.name = emailChunks[1];
        ZaSettings.myDomainName = emailChunks[1];
        EmailAddr_XFormItem.domainChoices.setChoices([tmpDomain]);
        EmailAddr_XFormItem.domainChoices.dirtyChoices();
    }
}

if(ZaSettings && ZaSettings.EnabledZimlet["com_zimbra_delegatedadmin"]) {

    // Modify Account Model
    if (ZaAccount) {
        ZaAccount.A_zimbraDomainAdminMaxMailQuota = "zimbraDomainAdminMaxMailQuota";

        var domainAdminMaxMailQuotaItem = {
            id: ZaAccount.A_zimbraDomainAdminMaxMailQuota,
            type: _COS_MAILQUOTA_,
            minInclusive: -1,
            ref: "attrs/" + ZaAccount.A_zimbraDomainAdminMaxMailQuota
        };

        ZaAccount.myXModel.items.push(domainAdminMaxMailQuotaItem);
    }

    if (ZaSearchOption) {
        ZaSearchOption.A_objTypeAccountDomainAdmin = ZaAccount.A_zimbraIsDelegatedAdminAccount;
    }

    // Modify COS model
    if (ZaCos) {
        ZaCos.A_zimbraDomainAdminMaxMailQuota = "zimbraDomainAdminMaxMailQuota";

        ZaCos.myXModel.items.push(
            {
                id: ZaCos.A_zimbraDomainAdminMaxMailQuota,
                type: _MAILQUOTA_,
                ref: "attrs/" + ZaCos.A_zimbraDomainAdminMaxMailQuota,
                minInclusive: -1
            }
        );
    }

    if (ZaSettings) {
        ZaDomainAdmin.initSettings = function() {
            if (window.console && window.console.log) {
                console.log("domainadmin.js is modifying ZaSettings");
            }
        }

        if (ZaSettings.initMethods) {
            ZaSettings.initMethods.push(ZaDomainAdmin.initSettings);
        }
    }

    // Modify Account Edit Form
    if (ZaTabView.XFormModifiers["ZaAccountXFormView"]) {
        ZaDomainAdmin.AccountXFormModifier = function (xFormObject) {
            var accountDomainAdminMaxQuotaField = {
                ref: ZaAccount.A_zimbraDomainAdminMaxMailQuota,
                colSpan: "*",
                type: _SUPER_TEXTFIELD_,
                resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
                msgName: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                txtBoxLabel: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                toolTipContent: com_zimbra_delegatedadmin.tt_DomainAdminMaxMailQuota,
                visibilityChecks: [
                    [
                        XForm.checkInstanceValue,
                        ZaAccount.A_zimbraIsDelegatedAdminAccount,
                        "TRUE"
                    ],
                    [
                        ZaItem.hasReadPermission,
                        ZaAccount.A_zimbraDomainAdminMaxMailQuota
                    ]
                ],
                visibilityChangeEventSources: [ZaAccount.A_zimbraIsDelegatedAdminAccount],
                enableDisableChecks: [
                    ZaItem.hasWritePermission,
                    ZaAccount.A_zimbraDomainAdminMaxMailQuota
                ],
                enableDisableChangeEventSources: [ZaAccount.A_zimbraIsDelegatedAdminAccount],
                getDisplayValue: function (newValue) {
                    if (newValue < 0) {
                        return "";
                    } else {
                        return newValue;
                    }
                },
                bmolsnr:true
            };

            var tabs = xFormObject.items[2].items;

            // Change General tab
            var tmpItems = tabs[0].items;
            var cnt = tmpItems.length;
            for (var i = 0; i < cnt; i ++) {
                if (tmpItems[i].id == "account_form_setup_group" && tmpItems[i].items) {
                    var tmpGrouperItems = tmpItems[i].items;
                    var cnt2 = tmpGrouperItems.length;
                    for (var j = 0; j < cnt2; j++) {
                        if (tmpGrouperItems[j] && tmpGrouperItems[j].ref == ZaAccount.A2_adminRoles ) {
                            //add domain admin quota field
                            xFormObject.items[2].items[0].items[i].items.splice(j + 1, 0, accountDomainAdminMaxQuotaField);
                            break;
                        }
                    }
                    break;
                }
            }

            // Change timeout tab
            for (var i = 1; i < tabs.length; i++) {
                if (tabs[i].id == "account_form_advanced_tab") {
                    var tmpItems = tabs[i].items;
                    var cnt = tmpItems.length;
                    for (var j = 0; j < cnt; j++) {
                        if (tmpItems[j].id == "timeout_settings" && tmpItems[j].items) {
                            var tmpGrouperItems = tmpItems[j].items;
                            var cnt2 = tmpGrouperItems.length;
                            for (var k = 0; k < cnt2; k++) {
                                if (tmpGrouperItems[k] && tmpGrouperItems[k].ref == ZaAccount.A_zimbraAdminAuthTokenLifetime) {
                                    tmpGrouperItems[k].visibilityChecks = [
                                        "instance.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount]==\'TRUE\' || instance.attrs[ZaAccount.A_zimbraIsAdminAccount]==\'TRUE\'"
                                    ];
                                    break;
                                }
                            }
                            break;
                        }
                    }
                    break;
                }
            }
        }

        ZaTabView.XFormModifiers["ZaAccountXFormView"].push(ZaDomainAdmin.AccountXFormModifier);
    }

    // Modify Account Creation wizard
    if (ZaXDialog.XFormModifiers["ZaNewAccountXWizard"]) {
        ZaDomainAdmin.NewAccountWizXFormModifier = function(xFormObject) {
            var accountDomainAdminMaxQuotaField = {
                ref: ZaAccount.A_zimbraDomainAdminMaxMailQuota,
                type: _SUPERWIZ_TEXTFIELD_,
                resetToSuperLabel: ZaMsg.NAD_ResetToCOS,
                msgName: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                textFieldCssClass: "admin_xform_number_input",
                txtBoxLabel: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                toolTipContent: com_zimbra_delegatedadmin.tt_DomainAdminMaxMailQuota,
                visibilityChecks: [
                    [
                        XForm.checkInstanceValue,
                        ZaAccount.A_zimbraIsDelegatedAdminAccount,
                        "TRUE"
                    ],
                    [
                        ZaItem.hasWritePermission,
                        ZaAccount.A_zimbraDomainAdminMaxMailQuota
                    ]
                ],
                visibilityChangeEventSources: [ZaAccount.A_zimbraIsDelegatedAdminAccount],
                getDisplayValue: function (newValue) {
                    if (newValue < 0) {
                        return "";
                    } else {
                        return newValue;
                    }
                },
                bmolsnr:true
            };

            var steps = xFormObject.items[3].items;
            var tmpItems = steps[0].items;
            var cnt = tmpItems.length;
            for (var i = 0; i < cnt; i++) {
                if (tmpItems[i].id == "account_wiz_setup_group" && tmpItems[i].items) {
                    var tmpGrouperItems = tmpItems[i].items;
                    var cnt2 = tmpGrouperItems.length;
                    for (var j = 0; j < cnt2; j++) {
                        if (tmpGrouperItems[j] && tmpGrouperItems[j].ref == ZaAccount.A2_adminRoles) {
                            xFormObject.items[3].items[0].items[i].items.splice(j + 1, 0, accountDomainAdminMaxQuotaField);
                            break;
                        }
                    }
                    break;
                }
            }

            for (var i = 0; i < steps.length; i++) {
                if (steps[i] && steps[i].id == "account_form_advanced_step" && steps[i].items) {
                    var stepItems = steps[i].items;
                    var cnt = stepItems.length;
                    for (var j = 0; j < cnt; j++) {
                        if (stepItems[j] && stepItems[j].items && stepItems[j].id=="timeout_settings") {
                            var tmpGrouperItems = stepItems[j].items;
                            var cnt2 = tmpGrouperItems.length;
                            for (var k = 0; k < cnt2; k++) {
                                if (tmpGrouperItems[k] && tmpGrouperItems[k].ref == ZaAccount.A_zimbraAdminAuthTokenLifetime) {
                                    tmpGrouperItems[k].visibilityChecks = [
                                        "instance.attrs[ZaAccount.A_zimbraIsDelegatedAdminAccount]==\'TRUE\' || instance.attrs[ZaAccount.A_zimbraIsAdminAccount]==\'TRUE\'"
                                    ];
                                    break;
                                }
                            }
                            break;
                        }
                    }
                    break;
                }
            }
        }

        ZaXDialog.XFormModifiers["ZaNewAccountXWizard"].push(ZaDomainAdmin.NewAccountWizXFormModifier);
    }

    if (ZaTabView.XFormModifiers["ZaMigrationWizView"]) {
        ZaDomainAdmin.MigViewXFormModifier = function(xFormObject) {
            var cnt = xFormObject.items.length;

            for (var ix = 0; ix < cnt; ix++) {
                if (xFormObject.items[ix].zName == "DownloadsForAdmin") {
                    xFormObject.items[ix].items = xFormObject.items[ix].items.concat(
                        [
                            //MSI Customizer for ZCO
                            {
                                type: _GROUP_,
                                numCols: 2,
                                items: [
                                    {
                                        type: _OUTPUT_,
                                        value: AjxImg.getImageHtml("MigrationWiz")
                                    },
                                    {
                                        type: _OUTPUT_,
                                        cssStyle: "font-size: 12px;",
                                        labelLocation: _NONE_,
                                        label: null,
                                        id: "zco_customizer_msi_link",
                                        value: ZaMigrationWizView.getDownloadLink(
                                            ZaMsg.CONNECTOR_MSI_DOWNLOAD_LINK,
                                            ZaMsg.CONNECTOR_MSI_DOWNLOAD_LINK_MSG
                                        )
                                    }
                                ]
                            },
                            {
                                type: _OUTPUT_,
                                cssClass: "ZaDownloadText",
                                label: null,
                                value: com_zimbra_delegatedadmin.CONNECTOR_MSI_DOWNLOAD_TEXT
                            },
                            //Branding MSI Customer for ZCO
                            {
                                type: _GROUP_,
                                numCols: 2,
                                items: [
                                    {
                                        type: _OUTPUT_,
                                        value: AjxImg.getImageHtml("MigrationWiz")
                                    },
                                    {
                                        type: _OUTPUT_,
                                        cssStyle: "font-size: 12px;",
                                        labelLocation: _NONE_,
                                        label: null,
                                        id: "zco_branding_msi_link",
                                        value: ZaMigrationWizView.getDownloadLink(
                                            ZaMsg.ZCO_BRANDING_DOWNLOAD_LINK,
                                            ZaMsg.ZCO_BRANDING_DOWNLOAD_LINK_MSG
                                        )
                                    }
                                ]
                            },
                            {
                                type: _OUTPUT_,
                                cssClass: "ZaDownloadText",
                                label: null,
                                value: com_zimbra_delegatedadmin.ZCO_BRANDING_DOWNLOAD_TEXT
                            }
                        ]
                    );
                }

                if (xFormObject.items[ix].zName == "DownloadsForUser") {
                    xFormObject.items[ix].items = xFormObject.items[ix].items.concat(
                        [
                            {
                                type: _GROUP_,
                                numCols: 5,
                                items: [
                                    {
                                        type: _OUTPUT_,
                                        value: AjxImg.getImageHtml("MigrationWiz")
                                    },
                                    {
                                        type: _OUTPUT_,
                                        cssStyle: "font-size: 12px;",
                                        labelLocation: _NONE_,
                                        label: null,
                                        id: "zco_32bit_link",
                                        value: ZaMigrationWizView.getDownloadLink(
                                            ZaMsg.CONNECTOR_32_DOWNLOAD_LINK,
                                            ZaMsg.CONNECTOR_32_DOWNLOAD_LINK_MSG
                                        )
                                    },
                                    {
                                        type: _CELL_SPACER_
                                    },
                                    {
                                        type: _OUTPUT_,
                                        cssStyle: "font-size: 12px;",
                                        labelLocation: _NONE_,
                                        label: null,
                                        id: "zco_64bit_link",
                                        value: ZaMigrationWizView.getDownloadLink(
                                            ZaMsg.CONNECTOR_64_DOWNLOAD_LINK,
                                            ZaMsg.CONNECTOR_64_DOWNLOAD_LINK_MSG
                                        )
                                    },
                                    {
                                        type: _OUTPUT_,
                                        cssStyle: "font-size: 12px;",
                                        labelLocation: _NONE_,
                                        label: null,
                                        id: "zco_help_link",
                                        value: [
                                            "(<A target='_blank' onclick='ZaZimbraAdmin.unloadHackCallback();' HREF='",
                                            location.pathname,
                                            "help/admin/pdf/User%20Instructions%20Connector%20for%20Outlook.pdf?locid=",
                                            AjxEnv.DEFAULT_LOCALE,
                                            "'>",
                                            ZaMsg.CONNECTOR_DOWNLOAD_HELP,
                                            "</a>)"
                                        ].join("")
                                    }
                                ]
                            },
                            {
                                type: _OUTPUT_,
                                cssClass: "ZaDownloadText",
                                label: null,
                                value: com_zimbra_delegatedadmin.CONNECTOR_DOWNLOAD_TEXT
                            }
                        ]
                    );
                }
            }
        }

        ZaTabView.XFormModifiers["ZaMigrationWizView"].push(ZaDomainAdmin.MigViewXFormModifier);
    }

    // Modify COS UI
    if (ZaTabView.XFormModifiers["ZaCosXFormView"]) {
        ZaDomainAdmin.CosXFormModifier = function(xFormObject) {
            //add the zimbraDomainAdminMaxMailQuota
            var cosDomainAdminMaxQuotaField = {
                ref: ZaCos.A_zimbraDomainAdminMaxMailQuota,
                type: _TEXTFIELD_,
                msgName: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                label: com_zimbra_delegatedadmin.NAD_DomainAdminMaxMailQuota,
                labelLocation: _LEFT_,
                cssClass: "admin_xform_number_input",
                bmolsnr: true,
                getDisplayValue: function (newValue) {
                    if(newValue == -1) {
                        return "";
                    } else {
                        return newValue;
                    }
                }
            };

            var tabCases = xFormObject.items[2].items;
            var advCase = null;
            var cnt = tabCases.length;
            for (var i = 0; i < cnt; i++) {
                if (tabCases[i].id == "cos_form_advanced_tab") {
                    advCase = tabCases[i];
                    break;
                }
            }
            if (advCase) {
                var tmpItems = advCase.items;
                var cnt2 = tmpItems.length;
                for (var i = 0; i < cnt2; i++) {
                    if (tmpItems[i].id == "cos_quota_settings" && tmpItems[i].items) {
                        var tmpGrouperItems = tmpItems[i].items;
                        var cnt2 = tmpGrouperItems.length;
                        for (var j = 0; j < cnt2; j++) {
                            if (tmpGrouperItems[j] && tmpGrouperItems[j].ref == ZaCos.A_zimbraMailQuota) {
                                tmpGrouperItems.splice(j + 1, 0, cosDomainAdminMaxQuotaField);
                                break;
                            }
                        }
                        break;
                    }
                }
            }
        }

        ZaTabView.XFormModifiers["ZaCosXFormView"].push(ZaDomainAdmin.CosXFormModifier);
    }

    // Modify Domain UI
    if (ZaTabView.XFormModifiers["ZaDomainXFormView"]) {

        ZaAL = function() {
            ZaItem.call(this, "ZaAL");
        }

        ZaAL.prototype = new ZaItem;
        ZaAL.prototype.constructor = ZaAL;

        ZaAL.getXModel = function () {
            var model = {
                items: [
                    {
                        id: "cos",
                        type: _STRING_,
                        ref: "cos"
                    },
                    {
                        id: "limits",
                        type: _INT_,
                        ref: "limits",
                        minInclusive: 0
                    }
                ]
            };

            return model;
        }

        ZaAL.addAl = function (alInstance, domainInstance) {
            var newAl =  alInstance.cos + ":" + alInstance.limits;

            if (!domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts]) {
                domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts] = [];
            }

            var done = false;
            for (var i = 0; i < domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts].length; i++) {
                if (alInstance.cos == domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts][i].split(":")[0]) {
                    domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts][i] = newAl;
                    done = true ;
                }
            }

            if (!done) {
                domainInstance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts].push(newAl);
            }
        }

        ZaEditAlXDialog = function (parent, app, title) {
            if (arguments.length == 0) {
                return;
            }
            this._standardButtons = [
                DwtDialog.CANCEL_BUTTON,
                DwtDialog.OK_BUTTON
            ];
            ZaXDialog.call(
                this,
                parent,
                null,
                title,
                "350px",
                "100px"
            );
            this._containedObject = {};
            this.cosChoices = new XFormChoices(
                [],
                XFormChoices.OBJECT_LIST,
                "id",
                "name"
            );
            this.initForm(
                ZaAL.getXModel(),
                this.getMyXForm()
            );
            this._helpURL = ZaEditAlXDialog.helpURL;
        }

        ZaEditAlXDialog.prototype = new ZaXDialog;
        ZaEditAlXDialog.prototype.constructor = ZaEditAlXDialog;
        ZaEditAlXDialog.helpURL =
            location.pathname +
            ZaUtil.HELP_URL +
            "managing_domains/setting_account_limits_and_types_for_a_domain.htm?locid=" +
            AjxEnv.DEFAULT_LOCALE;

        ZaEditAlXDialog.prototype.getMyXForm = function() {
            var xFormObject = {
                numCols: 1,
                items: [
                    {
                        type: _GROUP_,
                        isTabGroup: true,
                        items: [
                            {
                                type: _SPACER_,
                                height: "5px"
                            },
                            {
                                ref: "cos",
                                visibilityChecks: [],
                                enableDisableChecks: true,
                                type: _INPUT_,
                                label: ZaMsg.NAD_ClassOfService
                            },
                            {
                                ref: "limits",
                                visibilityChecks: [],
                                enableDisableChecks: true,
                                type: _INPUT_,
                                label:ZaMsg.Col_Limit
                            }
                        ]
                    }
                ]
            };

            return xFormObject;
        }

        ZaDomainXFormView.isEditAlEnabled = function (attrName) {
            return (
                this.getInstance().al_selection_cache != null &&
                this.getInstance().al_selection_cache.length == 1 &&
                this.getInstance().attrs[attrName].length > 0
            );
        }

        ZaDomainXFormView.isDeleteAlEnabled = function (attrName) {
            return (
                this.getInstance().al_selection_cache != null &&
                this.getInstance().al_selection_cache.length > 0 &&
                this.getInstance().attrs[attrName].length > 0
            );
        }

        ZaDomainXFormView.alSelectionListener = function (ev) {
            var instance = this.getInstance();
            var arr = this.widget.getSelection();

            if(arr && arr.length) {
                arr.sort();
                instance.al_selection_cache = arr;
            } else {
                instance.al_selection_cache = null;
            }

            this.getForm().refresh();
            if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
                ZaDomainXFormView.editAlButtonListener.call(this);
            }
        };

        ZaDomainXFormView.deleteAlButtonListener = function (attrName) {
            var instance = this.getInstance();
            if (instance.al_selection_cache != null) {
                var cnt = instance.al_selection_cache.length;
                if (cnt && instance.attrs[attrName]) {
                    for (var i = 0; i < cnt; i++) {
                        var cnt2 = instance.attrs[attrName].length - 1;
                        for (var k = cnt2; k >= 0; k--) {
                            if (instance.attrs[attrName][k] == instance.al_selection_cache[i]) {
                                instance.attrs[attrName].splice(k, 1);
                                break;
                            }
                        }
                    }
                }
            }
            this.getForm().parent.setDirty(true);
            this.getForm().refresh();
        }

        ZaDomainXFormView.editAlButtonListener = function () {
            var instance = this.getInstance();
            if (instance.al_selection_cache && instance.al_selection_cache[0]) {
                var formPage = this.getForm().parent;
                if (!formPage.addAlDlg) {
                    formPage.addAlDlg = new ZaEditAlXDialog(
                        ZaApp.getInstance().getAppCtxt().getShell(),
                        ZaApp.getInstance(),
                        com_zimbra_delegatedadmin.Edit_Al_Title
                    );
                    formPage.addAlDlg.registerCallback(
                        DwtDialog.OK_BUTTON,
                        ZaDomainXFormView.addAl,
                        this.getForm(),
                        null
                    );
                }  else {
                    formPage.addAlDlg.setTitle (com_zimbra_delegatedadmin.Edit_Al_Title);
                }
                var obj = {} ;
                obj.cos = instance.al_selection_cache[0].split(":")[0];
                obj.limits = instance.al_selection_cache[0].split(":")[1];

                formPage.addAlDlg.setObject(obj);
                formPage.addAlDlg.popup();
            }
        }

        ZaDomainXFormView.addAlButtonListener = function () {
            var instance = this.getInstance();
            var formPage = this.getForm().parent;

            if (!formPage.addAlDlg) {
                formPage.addAlDlg = new ZaEditAlXDialog(
                    ZaApp.getInstance().getAppCtxt().getShell(),
                    ZaApp.getInstance(),
                    com_zimbra_delegatedadmin.Add_Al_Title
                );
                formPage.addAlDlg.registerCallback(
                    DwtDialog.OK_BUTTON,
                    ZaDomainXFormView.addAl,
                    this.getForm(),
                    null
                );
            }  else {
                formPage.addAlDlg.setTitle (com_zimbra_delegatedadmin.Add_Al_Title);
            }

            var obj = {
                cos: "",
                limits: ""
            }

            formPage.addAlDlg.setObject(obj);
            formPage.addAlDlg.popup();
        }

        ZaDomainXFormView.addAl  = function () {
            if (this.parent.addAlDlg) {
                var app = ZaApp.getInstance();
                var obj = this.parent.addAlDlg._localXForm.getInstance();

                if (obj.cos == null) {
                    app.getCurrentController().popupErrorDialog(
                        AjxMessageFormat.format(
                            ZaMsg.ERROR_INVALID_VALUE_FOR,
                            [
                                ZaMsg.NAD_COS
                            ]
                        )
                    );
                    return;
                }

                var cosByID = ZaCos.getCosById(obj.cos, app);
                var cosByName = ZaCos.getCosByName(obj.cos, app);

                if (cosByID == null && cosByName == null) {
                    app.getCurrentController().popupErrorDialog(
                        AjxMessageFormat.format(
                            ZaMsg.ERROR_INVALID_VALUE_FOR,
                            [
                                ZaMsg.NAD_COS
                            ]
                        )
                    );
                    return;
                }

                if (parseInt(obj.limits) !=  obj.limits) {
                    app.getCurrentController().popupErrorDialog(
                        AjxMessageFormat.format(
                            ZaMsg.ERROR_INVALID_VALUE_FOR,
                            [
                                ZaMsg.msgCol_Limit
                            ]
                        )
                    );
                    return;
                }

                obj.cos = (cosByID != null) ? obj.cos : cosByName.id;

                var instance = this.getInstance();
                ZaAL.addAl (obj, instance);

                if (this.parent.addAlDlg) {
                    this.parent.addAlDlg.popdown();
                }

                this.parent.setDirty(true);
                this.refresh();
            }
        }

        ZaDomainXFormView.isAccountLimitWarningEnabled = function () {
            var domainMax = this.getInstance().attrs[ZaDomain.A_domainMaxAccounts];
            var cosMax = ZaDomain.getTotalLimitsPerAccountTypes(this.getInstance().attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts]);

            if (domainMax && cosMax && (domainMax < cosMax)) {
                // Customisation of the message doesn't work!
                return true;
            }
            return false;
        }

        // Edit Feature Account Limits
        ZaFeatureAL = function() {
            ZaItem.call(this, "ZaFeatureAL");
        }

        ZaFeatureAL.prototype = new ZaItem;
        ZaFeatureAL.prototype.constructor = ZaFeatureAL;

        ZaFeatureAL.getXModel = function () {
            var model = {
                items: [
                    {
                        id: "feature",
                        type: _STRING_,
                        ref:"feature",
                        choices: ZaCos.MAJOR_FEATURES_CHOICES
                    },
                    {
                        id: "limits",
                        type: _INT_,
                        ref: "limits",
                        minInclusive: 0
                    }
                ]
            };

            return model;
        }

        ZaFeatureAL.addAl = function (alInstance, domainInstance) {
            var newAl =  alInstance.feature + ":" + alInstance.limits;

            if (!domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts]) {
                domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts] = [];
            }

            var done = false;
            for (var i = 0; i < domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts].length; i++) {
                if (alInstance.feature == domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts][i].split(":")[0]) {
                    domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts][i] = newAl;
                    done = true;
                }
            }

            if (!done) {
                domainInstance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts].push (newAl);
            }
        }

        ZaEditFeatureAlXDialog = function(parent, app, title) {
            if (arguments.length == 0) {
                return;
            }

            this._standardButtons = [
                DwtDialog.CANCEL_BUTTON,
                DwtDialog.OK_BUTTON
            ];

            ZaXDialog.call(
                this,
                parent,
                null,
                title,
                "300px",
                "100px"
            );

            this._containedObject = {};

            this.initForm(
                ZaFeatureAL.getXModel(),
                this.getMyXForm()
            );
        }

        ZaEditFeatureAlXDialog.prototype = new ZaXDialog;
        ZaEditFeatureAlXDialog.prototype.constructor = ZaEditFeatureAlXDialog;

        ZaEditFeatureAlXDialog.prototype.getMyXForm = function() {
            var xFormObject = {
                numCols: 1,
                items: [
                    {
                        type: _GROUP_,
                        isTabGroup: true,
                        items: [
                            {
                                type: _SPACER_,
                                height: "5px"
                            },
                            {
                                ref: "feature",
                                type: _OSELECT1_,
                                label: ZaMsg.NAD_Feature,
                                width: "200px"
                            },
                            {
                                ref: "limits",
                                type: _INPUT_,
                                label: ZaMsg.Col_Limit,
                                width: "200px"
                            }
                        ]
                    }
                ]
            };

            return xFormObject;
        }

        ZaDomainXFormView.alFeatureSelectionListener = function (ev) {
            var instance = this.getInstance();

            var arr = this.widget.getSelection();
            if (arr && arr.length) {
                arr.sort();
                instance.feature_al_selection_cache = arr;
            } else {
                instance.feature_al_selection_cache = null;
            }

            this.getForm().refresh();

            if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
                ZaDomainXFormView.editFeatureAlButtonListener.call(this);
            }
        };

        ZaDomainXFormView.isEditFeatureAlEnabled = function (attrName) {
            return (
                this.instance.feature_al_selection_cache != null &&
                this.instance.feature_al_selection_cache.length == 1 &&
                this.instance.attrs[attrName].length > 0
            );
        }

        ZaDomainXFormView.isDeleteFeatureAlEnabled = function (attrName) {
            return (
                this.instance.feature_al_selection_cache != null &&
                this.instance.feature_al_selection_cache.length > 0 &&
                this.instance.attrs[attrName].length > 0
            );
        }

        ZaDomainXFormView.editFeatureAlButtonListener = function () {
            var instance = this.getInstance();
            if (instance.feature_al_selection_cache && instance.feature_al_selection_cache[0]) {
                var formPage = this.getForm().parent;
                if (!formPage.addFeatureAlDlg) {
                    formPage.addFeatureAlDlg = new ZaEditFeatureAlXDialog(
                        ZaApp.getInstance().getAppCtxt().getShell(),
                        ZaApp.getInstance(),
                        com_zimbra_delegatedadmin.Edit_FeatureAl_Title
                    );
                    formPage.addFeatureAlDlg.registerCallback(
                        DwtDialog.OK_BUTTON,
                        ZaDomainXFormView.addFeatureAl,
                        this.getForm(),
                        null
                    );
                } else {
                    formPage.addFeatureAlDlg.setTitle (com_zimbra_delegatedadmin.Edit_FeatureAl_Title);
                }

                var obj = {};
                obj.feature = instance.feature_al_selection_cache[0].split(":")[0];
                obj.limits = instance.feature_al_selection_cache[0].split(":")[1];

                formPage.addFeatureAlDlg.setObject(obj);
                formPage.addFeatureAlDlg.popup();
            }
        }

        ZaDomainXFormView.deleteFeatureAlButtonListener = function (attrName) {
            var instance = this.getInstance();
            if (instance.feature_al_selection_cache != null) {
                var cnt = instance.feature_al_selection_cache.length;
                if (cnt && instance.attrs[attrName]) {
                    for(var i = 0; i < cnt; i++) {
                        var cnt2 = instance.attrs[attrName].length - 1;
                        for (var k = cnt2; k >= 0; k--) {
                            if (instance.attrs[attrName][k] == instance.feature_al_selection_cache[i]) {
                                instance.attrs[attrName].splice(k, 1);
                                break;
                            }
                        }
                    }
                }
            }

            this.getForm().parent.setDirty(true);
            this.getForm().refresh();
        }

        ZaDomainXFormView.addFeatureAlButtonListener = function () {
            var instance = this.getInstance();
            var formPage = this.getForm().parent;

            if (!formPage.addFeatureAlDlg) {
                formPage.addFeatureAlDlg = new ZaEditFeatureAlXDialog(
                    ZaApp.getInstance().getAppCtxt().getShell(),
                    ZaApp.getInstance(),
                    com_zimbra_delegatedadmin.Add_FeatureAl_Title
                );
                formPage.addFeatureAlDlg.registerCallback(
                    DwtDialog.OK_BUTTON,
                    ZaDomainXFormView.addFeatureAl,
                    this.getForm(),
                    null
                );
            } else {
                formPage.addFeatureAlDlg.setTitle (com_zimbra_delegatedadmin.Add_FeatureAl_Title);
            }

            var obj = {
                feature: "",
                limits: ""
            }

            formPage.addFeatureAlDlg.setObject(obj);
            formPage.addFeatureAlDlg.popup();
        }

        ZaDomainXFormView.addFeatureAl  = function () {
            if (this.parent.addFeatureAlDlg ) {
                var app = ZaApp.getInstance();
                var obj = this.parent.addFeatureAlDlg._localXForm.getInstance();

                if (obj.feature == null || obj.feature.length <= 0 ) {
                    app.getCurrentController().popupErrorDialog(
                        AjxMessageFormat.format(
                            ZaMsg.ERROR_INVALID_VALUE_FOR,
                            [
                                ZaMsg.TABT_Feature
                            ]
                        )
                    );

                    return;
                }

                if (parseInt(obj.limits) !=  obj.limits) {
                    app.getCurrentController().popupErrorDialog(
                        AjxMessageFormat.format(
                            ZaMsg.ERROR_INVALID_VALUE_FOR,
                            [
                                ZaMsg.msgCol_Limit
                            ]
                        )
                    );

                    return;
                }

                var instance = this.getInstance();
                ZaFeatureAL.addAl (obj, instance);

                if (this.parent.addFeatureAlDlg) {
                    this.parent.addFeatureAlDlg.popdown();
                }

                this.parent.setDirty(true);
                this.refresh();
            }
        }

        //skin logo properties
        ZaDomainXFormView.LogoAppBannerPreviewId = Dwt.getNextId();

        ZaDomainXFormView.onAppLogoURLChange = function (value, event, form) {
            this.setInstanceValue(value);
            var appBannerPreviewItem = form.getItemsById(ZaDomainXFormView.LogoAppBannerPreviewId)[0];
            appBannerPreviewItem.updateElement(ZaDomainXFormView.getAppLogoPreview.call(this));
        }

        ZaDomainXFormView.getAppLogoPreview = function () {
            var width =  "200px";
            var height = "28px";

            var form = this.getForm();
            var instance = form.getInstance();

            var src = instance.attrs[ZaDomain.A_zimbraSkinLogoAppBanner];
            if (src == null && instance.cos != null) {
                var src = instance.cos.attrs [ZaDomain.A_zimbraSkinLogoAppBanner];
            }

            var out = AjxBuffer.concat(
                "<div style='border: inset 1px; height:",
                height,
                "; width:",
                width,
                ";",
                "background-image:url(",
                src ,
                ")",
                "'></div>"
            );

            return out;
        }

        ZaDomainXFormView.LogoLoginBannerPreviewId = Dwt.getNextId();

        ZaDomainXFormView.onLoginLogoURLChange = function (value, event, form) {
            this.setInstanceValue(value);
            var loginBannerPreviewItem = form.getItemsById(ZaDomainXFormView.LogoLoginBannerPreviewId)[0];
            loginBannerPreviewItem.updateElement(ZaDomainXFormView.getLoginLogoPreview.call(this));
        }

        ZaDomainXFormView.getLoginLogoPreview = function () {
            var width =  "300px";
            var height = "48px";

            var form = this.getForm();
            var instance = form.getInstance();

            var src = instance.attrs[ZaDomain.A_zimbraSkinLogoLoginBanner];
            if (src == null && instance.cos != null) {
                var src =  instance.cos.attrs [ZaDomain.A_zimbraSkinLogoLoginBanner];
            }

            var out = AjxBuffer.concat(
                "<div style='border: inset 1px; height:",
                height,
                "; width:",
                width,
                ";",
                "background-image:url(",
                src,
                ")",
                "'></div>"
            );

            return out;
        }

        ZaDomainAdmin.domainXFormModifer = function (xFormObject, entry) {
            var enableMXCheckCheckbox = {
                ref: ZaDomain.A_zimbraAdminConsoleDNSCheckEnabled,
                type: _SUPER_CHECKBOX_,
                resetToSuperLabel: ZaMsg.NAD_ResetToGlobal,
                checkBoxLabel: com_zimbra_delegatedadmin.ENABLE_DOM_CONSOLE_DNS_CHECK,
                trueValue: "TRUE",
                falseValue: "FALSE",
                onChange: ZaDomainXFormView.onFormFieldChanged
            };

            var simpleCount = 0;
            var tempItems = xFormObject.items[2].items[0].items;
            for (var i = 0; i < tempItems.length; i++) {
                if (tempItems[i].id == "dns_check_group") {
                    if (!ZaSettings.DOMAINS_ARE_READONLY) {
                        tempItems[i].items.push(enableMXCheckCheckbox);
                    }

                    simpleCount++;
                }
                if (simpleCount == 1) {
                    break;
                }
            }

            //add account limits to a new Tab
            var tabBar, switchGroup;
            for (var i = 0; i < xFormObject.items.length; i++) {
                if (xFormObject.items[i].type == _TAB_BAR_) {
                    tabBar = xFormObject.items[i];
                }

                if (xFormObject.items[i].type == _SWITCH_) {
                    switchGroup = xFormObject.items[i];
                }
            }

            var domainXformMaxAccountItem , accountLimitsItems, domainFeatureMaxAccountItem;
            var acctLimitsHeaderList = new Array();
            var featureMaxHeaderList = [];
            var acct_type_col_display = ZaMsg.Col_Cos;

            if (ZaSettings.isDomainAdmin) {
                acct_type_col_display = com_zimbra_delegatedadmin.Col_account_type;
            }

            acctLimitsHeaderList[0] = new ZaListHeaderItem(
                "cos",
                acct_type_col_display,
                null,
                "200px",
                null,
                null,
                true,
                true
            );
            acctLimitsHeaderList[1] = new ZaListHeaderItem(
                "limits",
                ZaMsg.headerCol_Limit,
                null,
                "auto",
                null,
                null,
                true,
                true
            );

            featureMaxHeaderList[0] = new ZaListHeaderItem(
                "feature",
                ZaMsg.TABT_Features,
                null,
                "200px",
                null,
                null,
                true,
                true
            );
            featureMaxHeaderList[1] = new ZaListHeaderItem(
                "limits",
                ZaMsg.headerCol_Limit,
                null,
                "auto",
                null,
                null,
                true,
                true
            );

            if (ZaSettings.isDomainAdmin) {
                domainXformMaxAccountItem = {
                    ref: ZaDomain.A_domainMaxAccounts,
                    type: _OUTPUT_,
                    label: com_zimbra_delegatedadmin.NAD_DomainMaxAccounts,
                    width: "50px",
                    getDisplayValue: function (v) {
                        if (!v) {
                            v = ZaMsg.Unlimited;
                        }

                        return v;
                    }
                };

                accountLimitsItems = {
                    type: _GROUP_,
                    colSpan: "*",
                    colSizes: ["275px", "*"],
                    numCols: 2,
                    width: "100%",
                    visibilityChecks: [
                        "((instance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts] != null) && " +
                        " (instance.attrs[ZaDomain.A_zimbraDomainCOSMaxAccounts].length > 0)) "
                    ],
                    items: [
                        {
                            type: _OUTPUT_,
                            value: com_zimbra_delegatedadmin.accountLimitsByAccountType,
                            align: _RIGHT_
                        },
                        {
                            ref: ZaDomain.A_zimbraDomainCOSMaxAccounts,
                            type: _DWT_LIST_,
                            height: "200px",
                            width: "300px",
                            forceUpdate: false,
                            cssClass: "DLSource",
                            widgetClass: ZaDomainCOSMaxAccountsListView,
                            headerList: acctLimitsHeaderList,
                            hideHeader: false
                        }
                    ]
                };

                domainFeatureMaxAccountItem = {
                    type: _GROUP_,
                    colSpan: "*",
                    colSizes: ["275px", "*"],
                    numCols: 2,
                    width: "100%",
                    visibilityChecks: [
                        "((instance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts] != null) && " +
                        " (instance.attrs[ZaDomain.A_zimbraDomainFeatureMaxAccounts].length > 0)) "
                    ],
                    items: [
                        {
                            type: _OUTPUT_,
                            value: com_zimbra_delegatedadmin.accountLimitsByFeature,
                            align: _RIGHT_
                        },
                        {
                            ref: ZaDomain.A_zimbraDomainFeatureMaxAccounts,
                            type: _DWT_LIST_,
                            height: "200px",
                            width: "300px",
                            forceUpdate: true,
                            cssClass: "DLSource",
                            widgetClass: ZaDomainFeatureMaxAccountsListView,
                            headerList: featureMaxHeaderList,
                            hideHeader: false
                        }
                    ]
                };
            } else {
                domainXformMaxAccountItem = {
                    ref: ZaDomain.A_domainMaxAccounts,
                    type: _INPUT_,
                    label: com_zimbra_delegatedadmin.NAD_DomainMaxAccounts,
                    width: "50px",
                    onChange: ZaDomainXFormView.onFormFieldChanged
                };

                var warningItem = {
                    type: _DWT_ALERT_,
                    id: "_domain_cos_account_limits_alert_",
                    visibilityChecks: [ZaDomainXFormView.isAccountLimitWarningEnabled],
                    visibilityChangeEventSources: [ZaDomain.A_domainMaxAccounts],
                    containerCssStyle: "width: 400px;",
                    style: DwtAlert.WARNING,
                    iconVisible: true,
                    content: com_zimbra_delegatedadmin.WARNING_TOTAL_LIMIT_OUT_OF_SYNC
                }

                accountLimitsItems = {
                    type: _GROUP_,
                    colSpan: "*",
                    colSizes: ["275px", "*"],
                    numCols: 2,
                    width: "100%",
                    items: [
                        {
                            type: _OUTPUT_,
                            value: com_zimbra_delegatedadmin.accountLimitsByCos,
                            containerCssClass: "gridGroupBodyLabel",
                            containerCssStyle: "text-align: left; border-right: 1px solid;"
                        },
                        {
                            ref: ZaDomain.A_zimbraDomainCOSMaxAccounts,
                            type: _DWT_LIST_,
                            height: "200px",
                            width: "300px",
                            forceUpdate: false,
                            preserveSelection: false,
                            multiselect: true,
                            cssClass: "DLSource",
                            cssStyle: "border: 2px solid #A4A4A4; ",
                            widgetClass: ZaDomainCOSMaxAccountsListView,
                            onSelection: ZaDomainXFormView.alSelectionListener,
                            headerList: acctLimitsHeaderList,
                            hideHeader: false
                        },
                        {
                            type: _CELLSPACER_,
                            height: "5px"
                        },
                        {
                            type: _GROUP_,
                            numCols: 5,
                            width: "270px",
                            cssStyle: "margin: 10px; padding-bottom: 0px; ",
                            items: [
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.TBB_Delete,
                                    width: "80px",
                                    onActivate: "ZaDomainXFormView.deleteAlButtonListener.call(this, ZaDomain.A_zimbraDomainCOSMaxAccounts);",
                                    enableDisableChecks: [
                                        [
                                            ZaDomainXFormView.isDeleteAlEnabled,
                                            ZaDomain.A_zimbraDomainCOSMaxAccounts
                                        ]
                                    ]
                                },
                                {
                                    type: _CELLSPACER_,
                                    width: "10px"
                                },
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.TBB_Edit,
                                    width: "80px",
                                    onActivate: "ZaDomainXFormView.editAlButtonListener.call(this);",
                                    enableDisableChecks: [
                                        [
                                            ZaDomainXFormView.isEditAlEnabled,
                                            ZaDomain.A_zimbraDomainCOSMaxAccounts
                                        ]
                                    ]
                                },
                                {
                                    type: _CELLSPACER_,
                                    width: "10px"
                                },
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.NAD_Add,
                                    width: "80px",
                                    onActivate: "ZaDomainXFormView.addAlButtonListener.call(this);"
                                }
                            ]
                        }
                    ]
                };

                domainFeatureMaxAccountItem = {
                    type: _GROUP_,
                    colSpan: "*",
                    colSizes: ["275px", "*"],
                    numCols: 2,
                    width: "100%",
                    items: [
                        {
                            type: _CELLSPACER_,
                            height: "5px"
                        },
                        {
                            type: _DWT_ALERT_,
                            containerCssStyle: "width: 300px;",
                            style: DwtAlert.WARNING,
                            iconVisible: true,
                            content: com_zimbra_delegatedadmin.WARNING_FEATURE_MAX_ACCOUNTS
                        },
                        {
                            type: _OUTPUT_,
                            value: com_zimbra_delegatedadmin.accountLimitsByFeature,
                            align: _RIGHT_
                        },
                        {
                            ref: ZaDomain.A_zimbraDomainFeatureMaxAccounts,
                            type: _DWT_LIST_,
                            height: "200px",
                            width: "300px",
                            forceUpdate: true,
                            preserveSelection: false,
                            multiselect: true,
                            cssClass: "DLSource",
                            widgetClass: ZaDomainFeatureMaxAccountsListView,
                            onSelection: ZaDomainXFormView.alFeatureSelectionListener,
                            headerList: featureMaxHeaderList,
                            hideHeader: false
                        },
                        {
                            type: _CELLSPACER_,
                            height: "5px"
                        },
                        {
                            type: _GROUP_,
                            numCols: 5,
                            width: "300px",
                            colSizes: ["80px", "auto", "80px", "auto", "80px"],
                            cssStyle: "margin: 10px; padding-bottom: 0px; ",
                            items: [
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.TBB_Delete,
                                    width: "100px",
                                    onActivate: "ZaDomainXFormView.deleteFeatureAlButtonListener.call(this, ZaDomain.A_zimbraDomainFeatureMaxAccounts);",
                                    enableDisableChecks: [
                                        [
                                            ZaDomainXFormView.isDeleteFeatureAlEnabled,
                                            ZaDomain.A_zimbraDomainFeatureMaxAccounts
                                        ]
                                    ]
                                },
                                {
                                    type: _CELLSPACER_,
                                    height: "5px"
                                },
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.TBB_Edit,
                                    width: "100px",
                                    onActivate: "ZaDomainXFormView.editFeatureAlButtonListener.call(this);",
                                    enableDisableChecks: [
                                        [
                                            ZaDomainXFormView.isEditFeatureAlEnabled,
                                            ZaDomain.A_zimbraDomainFeatureMaxAccounts
                                        ]
                                    ]
                                },
                                {
                                    type: _CELLSPACER_,
                                    height: "5px"
                                },
                                {
                                    type: _DWT_BUTTON_,
                                    label: ZaMsg.NAD_Add,
                                    width: "100px",
                                    onActivate:"ZaDomainXFormView.addFeatureAlButtonListener.call(this);"
                                }
                            ]
                        }
                    ]
                };
            }

            if (tabBar && switchGroup) {
                //load the grant list and check for account limit view grants.
                ZaGrant.load.call(
                    entry,
                    {
                        target: {
                            "by": "id",
                            "type": entry.type,
                            "val": entry.id
                        }
                    }
                );
                if (
                    ZaGrant.getGrant(
                        entry.grantsList,
                        ZaZimbraAdmin.currentUserLogin,
                        entry.id,
                        ZaSettings.DOMAIN_ACCT_LIMIT_TAB
                    ) === "grant" ||
                    ZaSettings.ENABLED_UI_COMPONENTS[ZaSettings.CARTE_BLANCHE_UI]) {

                    var tabIx = ++this.TAB_INDEX;

                    tabBar.choices.push(
                        {
                            value: tabIx,
                            label: com_zimbra_delegatedadmin.TT_account_limits
                        }
                    );

                    this.helpMap[tabIx] = [
                        location.pathname,
                        ZaUtil.HELP_URL,
                        "managing_domains_account_limits.htm",
                        "?locid=",
                        AjxEnv.DEFAULT_LOCALE
                    ].join("");

                    var caseItem = {
                        type: _ZATABCASE_,
                        id: "account_form_account_limits_tab",
                        numCols: 1,
                        colSizes: ["*"],
                        caseKey: tabIx,
                        items: [
                            warningItem,
                            {
                                type: _ZA_TOP_GROUPER_,
                                id: "account_form_account_limits_group",
                                items: [
                                    domainXformMaxAccountItem,
                                    accountLimitsItems
                                ]
                            }
                        ]
                    }

                    switchGroup.items.push(caseItem);
                }

                //add the Skin Logo properties
                var skinTab = null;

                for (var i = 0; i < switchGroup.items.length; i++) {
                    if (switchGroup.items[i].id == "domain_form_skin_tab") {
                        skinTab =  switchGroup.items[i];
                        break;
                    }
                }

                if (skinTab) {
                    var domainSkinLogoItem = {
                        type: _ZA_TOP_GROUPER_,
                        label: com_zimbra_delegatedadmin.NAD_Skin_Logo_Settings,
                        colSizes: ["275px", "100%"],
                        items: [
                            {
                                ref: ZaDomain.A_zimbraSkinLogoURL,
                                type: ZaSettings.isDomainAdmin ? _TEXTFIELD_ : _SUPER_TEXTFIELD_,
                                textFieldWidth: "200px",
                                label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoURL,
                                labelLocation: _LEFT_,
                                resetToSuperLabel: ZaMsg.NAD_ResetToGlobal
                            },
                            {
                                ref: ZaDomain.A_zimbraSkinLogoAppBanner,
                                type: ZaSettings.isDomainAdmin ? _TEXTFIELD_ : _SUPER_TEXTFIELD_,
                                textFieldWidth: "200px",
                                label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoAppBanner,
                                labelLocation: _LEFT_,
                                resetToSuperLabel: ZaMsg.NAD_ResetToGlobal,
                                onChange: ZaDomainXFormView.onAppLogoURLChange
                            },
                            {
                                type: _OUTPUT_,
                                id: ZaDomainXFormView.LogoAppBannerPreviewId,
                                labelWrap: true,
                                cssStyle: "padding: 0px",
                                label: com_zimbra_delegatedadmin.NAD_zimbraLogoAppBannerPreview,
                                getDisplayValue: ZaDomainXFormView.getAppLogoPreview,
                                labelLocation: _LEFT_
                            },
                            {
                                ref: ZaDomain.A_zimbraSkinLogoLoginBanner,
                                type: ZaSettings.isDomainAdmin ? _TEXTFIELD_ : _SUPER_TEXTFIELD_,
                                textFieldWidth: "200px",
                                label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoLoginBanner,
                                labelLocation: _LEFT_,
                                resetToSuperLabel: ZaMsg.NAD_ResetToGlobal,
                                onChange: ZaDomainXFormView.onLoginLogoURLChange
                            },
                            {
                                type: _OUTPUT_,
                                id: ZaDomainXFormView.LogoLoginBannerPreviewId,
                                labelWrap: true,
                                cssStyle: "padding: 0px",
                                label: com_zimbra_delegatedadmin.NAD_zimbraLogoLoginBannerPreview,
                                getDisplayValue: ZaDomainXFormView.getLoginLogoPreview,
                                labelLocation: _LEFT_
                            },
                            {
                                type: _GROUP_,
                                colSpan: 2,
                                cssStyle: "margin-top: 10px; margin-left: 200px",
                                items: [
                                    {
                                        type: _DWT_BUTTON_,
                                        label: com_zimbra_delegatedadmin.bt_ResetAllSkinLogo,
                                        onActivate: ZaDomainXFormView.resetAllLogoThemes
                                    }
                                ]
                            }
                        ]
                    };

                    skinTab.items.push(domainSkinLogoItem);
                }
            }
        }

        ZaTabView.XFormModifiers["ZaDomainXFormView"].push(ZaDomainAdmin.domainXFormModifer);

        ZaDomainXFormView.resetAllLogoThemes = function () {
            var form = this.getForm();

            this.setInstanceValue("", ZaDomain.A_zimbraSkinLogoURL);
            this.setInstanceValue("", ZaDomain.A_zimbraSkinLogoAppBanner);
            this.setInstanceValue("", ZaDomain.A_zimbraSkinLogoLoginBanner);

            var appBannerPreviewItem = form.getItemsById(ZaDomainXFormView.LogoAppBannerPreviewId)[0];
            appBannerPreviewItem.updateElement(ZaDomainXFormView.getAppLogoPreview.call(this));

            var loginBannerPreviewItem = form.getItemsById(ZaDomainXFormView.LogoLoginBannerPreviewId)[0];
            loginBannerPreviewItem.updateElement(ZaDomainXFormView.getLoginLogoPreview.call(this));

            form.parent.setDirty(true);
            form.refresh();
        }
    }

    if (ZaXDialog.XFormModifiers["ZaNewDomainXWizard"]) {
        ZaDomainAdmin.domainWizardModifer = function (xFormObject) {
            var domainWizardMaxAccountItem = {
                ref: ZaDomain.A_domainMaxAccounts,
                type: _TEXTFIELD_,
                label: com_zimbra_delegatedadmin.NAD_DomainMaxAccounts,
                labelLocation: _LEFT_,
                width: "50px"
            };

            var enableMXCheckCheckbox = {
                ref: ZaDomain.A_zimbraAdminConsoleDNSCheckEnabled,
                type: _SUPER_WIZ_CHECKBOX_,
                resetToSuperLabel: ZaMsg.NAD_ResetToGlobal,
                colSizes: ["200px", "250px", "*"],
                checkBoxLabel: com_zimbra_delegatedadmin.ENABLE_DOM_CONSOLE_DNS_CHECK,
                trueValue: "TRUE",
                falseValue: "FALSE"
            };

            var tempItems = xFormObject.items[3].items[0].items;

            var simpleCount = 0;

            if (ZaSettings.DOMAINS_ARE_READONLY) {
                simpleCount++;
            }

            for (var i = 0; i < tempItems.length; i++) {
                if (tempItems[i].ref == ZaDomain.A_description) {
                    tempItems.splice(i + 1, 0, domainWizardMaxAccountItem);
                    simpleCount++;
                } else if (tempItems[i].id=="dns_check_group") {
                    if (!ZaSettings.DOMAINS_ARE_READONLY) {
                        tempItems[i].items.push(enableMXCheckCheckbox);
                    }

                    simpleCount++;
                }

                if (simpleCount == 2) {
                    break;
                }
            }
        }

        ZaXDialog.XFormModifiers["ZaNewDomainXWizard"].push(ZaDomainAdmin.domainWizardModifer);
    }

    if (GlobalConfigXFormView) {
        GlobalConfigXFormView.LogoAppBannerPreviewId = Dwt.getNextId();

        GlobalConfigXFormView.onAppLogoURLChange = function (value, event, form) {
            this.setInstanceValue(value);

            var appBannerPreviewItem = form.getItemsById(GlobalConfigXFormView.LogoAppBannerPreviewId)[0];
            appBannerPreviewItem.updateElement(ZaDomainXFormView.getAppLogoPreview.call(this));
        }

        GlobalConfigXFormView.LogoLoginBannerPreviewId = Dwt.getNextId();

        GlobalConfigXFormView.onLoginLogoURLChange = function (value, event, form) {
            this.setInstanceValue(value);

            var loginBannerPreviewItem = form.getItemsById(GlobalConfigXFormView.LogoLoginBannerPreviewId)[0];
            loginBannerPreviewItem.updateElement(ZaDomainXFormView.getLoginLogoPreview.call(this));
        }
    }

    ZaDomainAdmin.GlobalConfigXFormModifier = function (xFormObject) {
        var enableMXCheckCheckbox = {
            ref: ZaGlobalConfig.A_zimbraAdminConsoleDNSCheckEnabled,
            type: _CHECKBOX_,
            label: com_zimbra_delegatedadmin.ENABLE_GLOB_CONSOLE_DNS_CHECK,
            trueValue: "TRUE",
            falseValue: "FALSE"
        };

        var tabCases = xFormObject.items[2].items;
        var cnt = tabCases.length;

        var mtaCase = null;
        var skinTab = null ;

        for (var i = 0; i < cnt; i++) {
            if (tabCases[i].id == "global_mta_tab") {
                mtaCase = tabCases[i];
            } else if (tabCases[i].id == "global_skin_tab") {
                skinTab = tabCases[i];
            }
        }

        if (mtaCase) {
            var tmpItems = mtaCase.items;
            var cnt2 = tmpItems.length;
            for (var i = 0; i < cnt2; i++) {
                if (tmpItems[i].id == "mta_network_group" && tmpItems[i].items) {
                    tmpItems[i].items.push(enableMXCheckCheckbox);
                    break;
                }
            }
        }

        //add skin logo item
        if (skinTab) {
            var globalSkinLogoItems = {
                type: _ZA_TOP_GROUPER_,
                label: com_zimbra_delegatedadmin.NAD_Skin_Logo_Settings,
                colSizes: ["275px", "100%"],
                items: [
                    {
                        ref: ZaGlobalConfig.A_zimbraSkinLogoURL,
                        type: _TEXTFIELD_,
                        label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoURL,
                        labelLocation: _LEFT_,
                        width: "200px"
                    },
                    {
                        ref: ZaGlobalConfig.A_zimbraSkinLogoAppBanner,
                        type: _TEXTFIELD_,
                        label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoAppBanner,
                        labelLocation: _LEFT_,
                        width: "200px",
                        onChange: GlobalConfigXFormView.onAppLogoURLChange
                    },
                    {
                        type: _OUTPUT_,
                        id: GlobalConfigXFormView.LogoAppBannerPreviewId,
                        cssStyle: "padding: 0px",
                        label: com_zimbra_delegatedadmin.NAD_zimbraLogoAppBannerPreview,
                        labelWrap: true,
                        getDisplayValue: ZaDomainXFormView.getAppLogoPreview,
                        labelLocation: _LEFT_
                    },
                    {
                        ref: ZaGlobalConfig.A_zimbraSkinLogoLoginBanner,
                        type: _TEXTFIELD_,
                        label: com_zimbra_delegatedadmin.NAD_zimbraSkinLogoLoginBanner,
                        labelLocation: _LEFT_,
                        width: "200px",
                        onChange: GlobalConfigXFormView.onLoginLogoURLChange
                    },
                    {
                        type: _OUTPUT_,
                        id: GlobalConfigXFormView.LogoLoginBannerPreviewId,
                        labelWrap: true,
                        cssStyle: "padding: 0px",
                        label: com_zimbra_delegatedadmin.NAD_zimbraLogoLoginBannerPreview,
                        getDisplayValue: ZaDomainXFormView.getLoginLogoPreview,
                        labelLocation: _LEFT_
                    }
                ]
            };

            skinTab.items.push(globalSkinLogoItems);
        }
    }

    if(ZaTabView.XFormModifiers["GlobalConfigXFormView"]) {
        ZaTabView.XFormModifiers["GlobalConfigXFormView"].push(ZaDomainAdmin.GlobalConfigXFormModifier);
    }
}

if(window.console && window.console.log) {
    console.log ("Loaded domainadmin.js");
}
